// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Hello.h>

#include <stdlib.h>
#ifndef WIN32
# include <unistd.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get "hello" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Hello.ref");
    if(CORBA::is_nil(obj))
    {
	cerr << argv[0] << ": cannot read IOR from Hello.ref" << endl;
	return EXIT_FAILURE;
    }
    
    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));
    
    //
    // Main loop
    //
    cout << "Enter number of invocations:\n";
    cout << "> ";
    int n;
    cin >> n;
    while (n-- > 0)
    {
        CORBA::String_var msg = hello -> say_hello();
        cout << msg << endl;
#ifdef WIN32
        Sleep(1000);
#else
        sleep(1);
#endif

    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
