// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Library.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get "library" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Library.ref");
    if(CORBA::is_nil(obj))
    {
	cerr << argv[0] << ": cannot read IOR from Library.ref" << endl;
	return EXIT_FAILURE;
    }
    
    Library_var library = Library::_narrow(obj);
    assert(!CORBA::is_nil(library));

    //
    // Get the list of library books
    //
    BookTitles_var titles = library -> get_titles();

    //
    // For each title, get the book and its info
    //
    for(CORBA::ULong i = 0 ; i < titles -> length() ; i++)
    {
	Book_var book = library -> get_book(titles[i]);
	BookInfo_var info = book -> info();

	cout << "Book " << i + 1 << ":" << endl;
	cout << "\ttitle:  " << info -> title << endl;
	cout << "\tauthor: " << info -> author << endl;
	cout << "\tisbn:   " << info -> isbn << endl;
    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
