// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Library_impl.h>

//
// IDL:Book:1.0
//
Book_impl::Book_impl(PortableServer::POA_ptr poa,
		     PortableServer::Current_ptr current,
		     const BookInfoSeq& bookInfos)
    : poa_(PortableServer::POA::_duplicate(poa)),
      current_(PortableServer::Current::_duplicate(current)),
      bookInfos_(bookInfos)
{
}

Book_impl::~Book_impl()
{
}

PortableServer::POA_ptr
Book_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

//
// IDL:Book/info:1.0
//
BookInfo*
Book_impl::info()
    throw(CORBA::SystemException)
{
    //
    // Get the requested title.
    //
    PortableServer::ObjectId_var oid = current_ -> get_object_id();
    CORBA::String_var title = PortableServer::ObjectId_to_string(oid);

    //
    // Find the book info.
    //
    for(CORBA::ULong i = 0 ; i < bookInfos_.length() ; i++)
    {
	if(strcmp(title.in(), bookInfos_[i].title) == 0)
	{
	    BookInfo_var bookInfo = new BookInfo(bookInfos_[i]);
	    return bookInfo._retn();
	}
    }

    throw CORBA::OBJECT_NOT_EXIST();
    return 0; // Some compilers need this
}

//
// IDL:Library:1.0
//
Library_impl::Library_impl(PortableServer::POA_ptr poa,
			   const BookInfoSeq& bookInfos)
    : poa_(PortableServer::POA::_duplicate(poa)),
      bookInfos_(bookInfos)
{
}

Library_impl::~Library_impl()
{
}

PortableServer::POA_ptr
Library_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

//
// IDL:Library/get_book:1.0
//
Book_ptr
Library_impl::get_book(const char* title)
    throw(CORBA::SystemException)
{
    PortableServer::ObjectId_var oid =
	PortableServer::string_to_ObjectId(title);

    CORBA::Object_var obj =
	poa_ -> create_reference_with_id(oid, "IDL:Book:1.0");

    return Book::_narrow(obj);
}

//
// IDL:Library/get_titles:1.0
//
BookTitles*
Library_impl::get_titles()
    throw(CORBA::SystemException)
{
    BookTitles_var titles = new BookTitles;
    for(CORBA::ULong i = 0 ; i < bookInfos_.length() ; i++)
    {
	CORBA::ULong len = titles -> length();
	titles -> length(len + 1);
	titles[len] = bookInfos_[i].title;
    }

    return titles._retn();
}
