// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>

#include <BalancerLogger.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(LoadBalancing::Logger_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(LoadBalancing::Logger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// Logger constructor/destructor
// ----------------------------------------------------------------------

LoadBalancing::Logger::Logger(OB::Logger_ptr logger,
			     OB::Properties_ptr properties)
    : logger_(OB::Logger::_duplicate(logger)),
      traceLifecycle_(0),
      traceLoadBalance_(0),
      traceDatabase_(0)
{
#ifdef LB_DEBUG
    OB_CERR << "Logger::Logger()" << OB_ENDL;
#endif

    OB::Properties::KeySeq keys = properties -> getKeys("ooc.balancer.trace.");
    for(CORBA::ULong i = 0; i < keys.length(); ++i)
    {
        const char* key = keys[i];
	const char* value = properties -> getProperty(key);
	
	if(strcmp(key, "ooc.balancer.trace.lifecycle") == 0)
	{
	    traceLifecycle_ = atol(value);
	}
	else if(strcmp(key, "ooc.balancer.trace.load_balance") == 0)
	{
	    traceLoadBalance_ = atol(value);
	}
	else if(strcmp(key, "ooc.balancer.trace.database") == 0)
	{
	    traceDatabase_ = atol(value);
	}
	else
	{
	    String_var msg = string_dup("`");
	    msg += key;
	    msg += "': unknown property";

	    logger_ -> warning(msg);
	}
    }
}

LoadBalancing::Logger::~Logger()
{
#ifdef LB_DEBUG
    OB_CERR << "Logger::~Logger()" << OB_ENDL;
#endif
}

// ----------------------------------------------------------------------
// Logger trace level accessors
// ----------------------------------------------------------------------

CORBA::ULong
LoadBalancing::Logger::traceLifecycle()
{
    return traceLifecycle_;
}

CORBA::ULong
LoadBalancing::Logger::traceLoadBalance()
{
    return traceLoadBalance_;
}

CORBA::ULong
LoadBalancing::Logger::traceDatabase()
{
    return traceDatabase_;
}

// ----------------------------------------------------------------------
// Logger logging methods
// ----------------------------------------------------------------------

void
LoadBalancing::Logger::info(const char* msg)
{
    logger_ -> info(msg);
}

void
LoadBalancing::Logger::error(const char* msg)
{
    logger_ -> error(msg);
}

void
LoadBalancing::Logger::warning(const char* msg)
{
    logger_ -> warning(msg);
}

void
LoadBalancing::Logger::trace(const char* category, const char* msg)
{
    logger_ -> trace(category, msg);
}
