# **********************************************************************
#
# Copyright (c) 2002
# IONA Technologies, Inc.
# Waltham, MA, USA
#
# All Rights Reserved
#
# **********************************************************************

top_srcdir	= ..\..

#
# Two libraries are created. Balancer is for end-users and
# BalancerService is for applications that need to actually include an
# implementation of the service.
#
LIB_NAME	= $(top_srcdir)\lib\Balancer$(LIBSUFFIX).lib
SERVLIB_NAME	= $(top_srcdir)\lib\BalancerService$(LIBSUFFIX).lib

LIB_PDB_NAME	= $(top_srcdir)\lib\Balancer$(LIBSUFFIX).pdb
SERVLIB_PDB_NAME= $(top_srcdir)\lib\BalancerService$(LIBSUFFIX).pdb

BALANCER_NAME	= $(top_srcdir)\bin\balancer.exe

NTSERVICE_NAME	= $(top_srcdir)\bin\ntbalancerservice.exe

UTILS		= $(BALANCER_NAME) \
		  $(NTSERVICE_NAME)

#
# Work around the inference rules problem with nmake.
#
!if !defined(BUILD_STAGE)

PDB_FLAGS       = /Fd$(LIB_PDB_NAME)
TARGETS		= $(LIB_NAME)

!elseif $(BUILD_STAGE) == 1

PDB_FLAGS   	= /Fd$(SERVLIB_PDB_NAME)
TARGETS		= $(SERVLIB_NAME)

!elseif $(BUILD_STAGE) == 2

TARGETS 	= $(UTILS)
PDB_FILES       = $(UTILS:.exe=.pdb) \
		  $(LIB_PDB_NAME) \
		  $(SERVLIB_PDB_NAME)

!endif

!include $(top_srcdir)\config\Make.rules.mak

IDLDIR		= $(top_srcdir)\idl\OB
HDIR		= $(top_srcdir)\include\OB

LIB_OBJS	= Balancer.obj \
		  Balancer_skel.obj \
		  BalancerInternal.obj \
		  BalancerInternal_skel.obj \
		  BalancerTypes.obj \
		  BalancerTypes_skel.obj \
		  BalancerPolicyTypes.obj \
		  BalancerPolicies.obj \
		  BalancerPolicies_impl.obj \
		  GroupORT.obj \
		  GroupORT_impl.obj \
		  BalancerInitializer.obj

SERVLIB_OBJS	= BalancerDB.obj \
		  BalancerServer.obj \
		  Database.obj \
		  BalancerDatabase.obj \
		  BalancerLogger.obj \
		  Group_impl.obj \
		  GroupFactory_impl.obj \
		  ReplicaLocator.obj \
		  StrategyProxy_impl.obj \
		  StrategyFactory.obj \
		  StrategyBase.obj \
		  StrategyAdaptiveBase.obj \
		  StrategyRR_impl.obj \
		  StrategyRand_impl.obj \
		  StrategyLL_impl.obj \
		  StrategyMinDisp_impl.obj

BALANCER_OBJS	= BalancerMain.obj

NTSERVICE_OBJS	= NTBalancerMain.obj \
		  NTBalancerService.obj \
		  NTBalancerMsg.res

#
# When not in DLL mode these symbols are also needed for clean target
#
LIB_DEF_NAME	= Balancer$(LIBSUFFIX).def
LIB_DLL_NAME    = $(top_srcdir)\lib\Balancer$(OB_LIBVER)$(LIBSUFFIX).dll
LIB_EXP_NAME    = $(top_srcdir)\lib\Balancer$(LIBSUFFIX).exp

SERVLIB_DEF_NAME= BalancerService$(LIBSUFFIX).def
SERVLIB_DLL_NAME= $(top_srcdir)\lib\BalancerService$(OB_LIBVER)$(LIBSUFFIX).dll
SERVLIB_EXP_NAME= $(top_srcdir)\lib\BalancerService$(LIBSUFFIX).exp

ALL_CPPFLAGS    = /I$(top_srcdir)\include /I. $(OB_CPPFLAGS) $(CPPFLAGS)
ALL_IDLFLAGS    = -I$(top_srcdir)/idl -I. $(OB_IDLFLAGS)

!if !defined(BUILD_STAGE)

ALL_CPPFLAGS = /DOB_BALANCER_IMPORT=/**/ $(ALL_CPPFLAGS)

!endif

!if "$(DLL)" == "yes"

ALL_CXXFLAGS	= $(CXXFLAGS) 

ALL_DLLLIBS	= $(OB_LIBS) $(LIBS) $(OB_DLLLIBS) $(DLLLIBS)

$(LIB_NAME): $(LIB_DLL_NAME)

$(SERVLIB_NAME): $(SERVLIB_DLL_NAME)

$(LIB_DEF_NAME):

$(SERVLIB_DEF_NAME):

$(LIB_DLL_NAME): $(LIB_OBJS) $(LIB_DEF_NAME)
	perl $(top_srcdir)\..\config\makedef.pl $(LIB_DEF_NAME) \
	Balancer$(OB_LIBVER)$(LIBSUFFIX) $(LIB_OBJS)
	$(LINK) $(LINKDLLFLAGS) /BASE:0x55000000 /def:$(LIB_DEF_NAME) \
	/IMPLIB:$(LIB_NAME) /out:$@ $(LIB_OBJS) $(ALL_DLLLIBS)

$(SERVLIB_DLL_NAME): $(SERVLIB_OBJS) $(LIB_NAME) $(SERVLIB_DEF_NAME)
	perl $(top_srcdir)\..\config\makedef.pl $(SERVLIB_DEF_NAME) \
	BalancerService$(OB_LIBVER)$(LIBSUFFIX) $(SERVLIB_OBJS)
	$(LINK) $(LINKDLLFLAGS) /BASE:0x55A00000 /def:$(SERVLIB_DEF_NAME) \
	/IMPLIB:$(SERVLIB_NAME) /out:$@ $(SERVLIB_OBJS) $(LIB_NAME) \
	$(ALL_DLLLIBS)

!else

ALL_CXXFLAGS	= $(CXXFLAGS) $(PDB_FLAGS)

$(LIB_NAME): $(LIB_OBJS)
	$(AR) $(ARFLAGS) /out:$@ $(LIB_OBJS)

$(SERVLIB_NAME): $(SERVLIB_OBJS)
	$(AR) $(ARFLAGS) /out:$@ $(SERVLIB_OBJS)

!endif

ALL_LIBS = $(OB_LIBS) $(LIBS)

$(BALANCER_NAME): $(BALANCER_OBJS) $(LIB_NAME) $(SERVLIB_NAME)
	$(LINK) $(LINKFLAGS) /out:$@ $(BALANCER_OBJS) \
	    $(SERVLIB_NAME) $(LIB_NAME) $(ALL_LIBS)

$(NTSERVICE_NAME): $(NTSERVICE_OBJS) $(LIB_NAME) $(SERVLIB_NAME)
	$(LINK) $(LINKFLAGS) /out:$@ $(NTSERVICE_OBJS) \
	    $(SERVLIB_NAME) $(LIB_NAME) $(ALL_LIBS)

NTBalancerMsg.rc : NTBalancerMsg.mc
	mc NTBalancerMsg.mc

NTBalancerMsg.res: NTBalancerMsg.rc
	rc -r -fo NTBalancerMsg.res NTBalancerMsg.rc

NTBalancerMsg.h : NTBalancerMsg.rc

Config.h:
	echo /* dummy file */ > $@

Balancer.cpp $(HDIR)/Balancer.h : $(IDLDIR)/Balancer.idl
	-del Balancer.cpp
	-del $(HDIR)\Balancer.h
	-del $(HDIR)\Balancer_fwd.h
	-del Balancer_skel.cpp
	-del $(HDIR)\Balancer_skel.h
	$(IDL) $(ALL_IDLFLAGS) --fwd --this-header-dir OB \
	    --dll-import OB_BALANCER_IMPORT $(IDLDIR)/Balancer.idl
	move Balancer.h $(HDIR)
	move Balancer_fwd.h $(HDIR)
	move Balancer_skel.h $(HDIR)

Balancer_skel.cpp $(HDIR)/Balancer_skel.h: Balancer.cpp

BalancerTypes.cpp $(HDIR)/BalancerTypes.h : \
    $(IDLDIR)/BalancerTypes.idl
	-del BalancerTypes.cpp
	-del BalancerTypes_skel.cpp
	-del $(HDIR)\BalancerTypes.h
	-del $(HDIR)\BalancerTypes_skel.h
	$(IDL) $(ALL_IDLFLAGS) --this-header-dir OB \
	    --dll-import OB_BALANCER_IMPORT $(IDLDIR)/BalancerTypes.idl
	move BalancerTypes.h $(HDIR)
	move BalancerTypes_skel.h $(HDIR)

BalancerPolicyTypes.cpp $(HDIR)/BalancerPolicyTypes.h : \
    $(IDLDIR)/BalancerPolicyTypes.idl
	-del BalancerPolicyTypes.cpp
	-del $(HDIR)\BalancerPolicyTypes.h
	$(IDL) $(ALL_IDLFLAGS) --no-skeletons --this-header-dir OB \
	    --dll-import OB_BALANCER_IMPORT $(IDLDIR)/BalancerPolicyTypes.idl
	move BalancerPolicyTypes.h $(HDIR)

BalancerProperties.cpp $(HDIR)/BalancerProperties.h : \
    $(IDLDIR)/BalancerProperties.idl
	-del BalancerProperties.cpp
	-del $(HDIR)\BalancerProperties.h
	$(IDL) $(ALL_IDLFLAGS) --no-skeletons --this-header-dir OB \
	    --dll-import OB_BALANCER_IMPORT $(IDLDIR)/BalancerProperties.idl
	move BalancerProperties.h $(HDIR)

BalancerInternal.cpp BalancerInternal.h : BalancerInternal.idl
	-del BalancerInternal.cpp
	-del BalancerInternal.h
	-del BalancerInternal_skel.cpp
	-del BalancerInternal_skel.h
	$(IDL) $(ALL_IDLFLAGS) \
	    --dll-import OB_BALANCER_IMPORT BalancerInternal.idl

BalancerInternal_skel.cpp BalancerInternal_skel.h: BalancerInternal.cpp

GroupORT.cpp GroupORT.h: GroupORT.idl
	-del GroupORT.cpp
	-del GroupORT.h
	$(IDL) $(ALL_IDLFLAGS) --no-skeletons \
	    --dll-import OB_BALANCER_IMPORT GroupORT.idl

BalancerPolicies.cpp BalancerPolicies.h: BalancerPolicies.idl
	-del BalancerPolicies.cpp
	-del BalancerPolicies.h
	$(IDL) $(ALL_IDLFLAGS) --no-skeletons \
	    --dll-import OB_BALANCER_IMPORT BalancerPolicies.idl

BalancerDB.cpp BalancerDB.h: BalancerDB.idl
	-del BalancerDB.cpp
	-del BalancerDB.h
	$(IDL) $(ALL_IDLFLAGS) --no-skeletons BalancerDB.idl

#
# Work around the inference rules problem with nmake.
#
!ifndef BUILD_STAGE

install:: all

all depend install clean test::
    $(MAKE) /f Makefile.mak BUILD_STAGE=1 $@

!elseif $(BUILD_STAGE) == 1

install:: all

all depend install clean test::
    $(MAKE) /f Makefile.mak BUILD_STAGE=2 $@

!elseif $(BUILD_STAGE) == 2

install:: all
	copy $(LIB_NAME) $(libdir)
	if exist $(LIB_PDB_NAME) \
	    copy $(LIB_PDB_NAME) $(libdir)
	copy $(SERVLIB_NAME) $(libdir)
	if exist $(SERVLIB_PDB_NAME) \
	    copy $(SERVLIB_PDB_NAME) $(libdir)
	copy $(BALANCER_NAME) $(bindir)
	copy $(NTSERVICE_NAME) $(bindir)

!if "$(DLL)" == "yes"

LIB_DLL_PDB = $(LIB_DLL_NAME:.dll=.pdb)

SERVLIB_DLL_PDB = $(SERVLIB_DLL_NAME:.dll=.pdb)

install:: all
	copy $(LIB_DLL_NAME) $(libdir)
	copy $(LIB_EXP_NAME) $(libdir)
	if exist $(LIB_DLL_PDB) copy $(LIB_DLL_PDB) $(libdir)
	copy $(SERVLIB_DLL_NAME) $(libdir)
	copy $(SERVLIB_EXP_NAME) $(libdir)
	if exist $(SERVLIB_DLL_PDB) copy $(SERVLIB_DLL_PDB) $(libdir)

!endif

clean::
	-del $(LIB_DLL_NAME) $(LIB_EXP_NAME) $(LIB_DEF_NAME) \
	$(LIB_DLL_NAME:.dll=.pdb) \
	$(SERVLIB_DLL_NAME) $(SERVLIB_EXP_NAME) $(SERVLIB_DEF_NAME) \
	$(SERVLIB_DLL_NAME:.dll=.pdb)

clean::
	-del $(BALANCER_NAME) $(BALANCER_NAME:.exe=.pdb) \
	$(NTSERVICE_NAME) $(NTSERVICE_NAME:.exe=.pdb)

!endif

min:: all

install_min:: install

!include .windep
