// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_ReplicaLocator_h__
#define __OB_ReplicaLocator_h__

#include <OB/Balancer_fwd.h>

namespace LoadBalancing
{

//
// A ServantLocator used to intercept a request, locate the least loaded
// member, and redirect the request to the appropriate replica of the
// member.
//
class ReplicaLocator : virtual public POA_PortableServer::ServantLocator,
		       virtual public PortableServer::RefCountServantBase
{
    ReplicaLocator(const ReplicaLocator&);
    void operator=(const ReplicaLocator&);

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The group factory
    //
    GroupFactory_var factory_;

    //
    // Extract the replica info
    //
    bool extractInfo(const PortableServer::ObjectId&,
		     LoadBalancing::GroupId_out,
		     CORBA::String_out,
		     PortableInterceptor::ObjectId_out);

public:

    ReplicaLocator(PortableServer::POA_ptr, GroupFactory_ptr);
    ~ReplicaLocator();

    virtual PortableServer::POA_ptr _default_POA();

    //
    // IDL to C++ Mapping
    //

    PortableServer::Servant
    preinvoke(const PortableServer::ObjectId&, PortableServer::POA_ptr,
	      const char*, PortableServer::ServantLocator::Cookie&)
	throw(PortableServer::ForwardRequest, CORBA::SystemException);

    void
    postinvoke(const PortableServer::ObjectId&, PortableServer::POA_ptr,
	       const char*, PortableServer::ServantLocator::Cookie,
	       PortableServer::Servant)
	throw(CORBA::SystemException);
};

} // End of namespace LoadBalancing

#endif
