// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_StrategyAdaptiveBase_h__
#define __OB_StrategyAdaptiveBase_h__

#include <OB/Hashtable.h>
#include <OB/list.h>

#include <BalancerDatabase_fwd.h>

#include <StrategyBase.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// The base class for adaptive strategies.
//
class StrategyAdaptiveBase : public StrategyBase
{
public: // For some compilers this must be public

    struct MemberElement
    {
	LoadBalancing::MemberData member;
	CORBA::Float load;
    };
    typedef OB::stl::list< MemberElement* > MemberList;
    typedef OB::Hashtable< CORBA::String_var, MemberElement*,
			   OB::StringHasher > MemberMap;

private:

    StrategyAdaptiveBase(const StrategyAdaptiveBase&);
    void operator=(const StrategyAdaptiveBase&);

    //
    // The member list
    //
    MemberList members_;

    //
    // The member map
    //
    MemberMap memberMap_;

    //
    // The minimum reported load
    //
    CORBA::Float minLoad_;

    //
    // The sum of loads
    //
    CORBA::Float loadSum_;

    //
    // Update the load of the given element.
    //
    void updateLoad(MemberElement* element, CORBA::Float load);

protected:

    //
    // The tolerance value
    //
    CORBA::ULong tolerance_;

    //
    // The load-per-client value
    //
    CORBA::ULong loadPerClient_;

    //
    // The reject-load value
    //
    CORBA::ULong rejectLoad_;

    //
    // The dampening-multiplier value
    //
    CORBA::Float dampeningMultiplier_;

    //
    // Get the minimum reported load.
    //
    CORBA::Float minLoad();

    //
    // Get the sum of loads.
    //
    CORBA::Float loadSum();

    //
    // Get the number of members.
    //
    CORBA::ULong memberCount();

    //
    // Get the base properties.
    //
    LoadBalancing::PropertySeq* getProperties();

    //
    // Set base properties.
    //
    void setProperties(const LoadBalancing::PropertySeq&,
		       LoadBalancing::PropertySeq&,
		       LoadBalancing::PropertyErrorSeq&);

    //
    // Adjust the members.
    //
    void adjustMembers(const LoadBalancing::MemberDataSeq& members)
        throw(CORBA::SystemException);

    //
    // Get the least loaded member.
    //
    LoadBalancing::MemberId getNext()
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // Push the member load.
    //
    void pushLoad(const char* member_id, CORBA::ULong load)
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // Compute the load threshold used to select the next member.
    //
    virtual CORBA::Float loadThreshold() = 0;

    //
    // Is an alert required?
    //
    virtual bool requireAlert(CORBA::Float load) = 0;

    StrategyAdaptiveBase(PortableServer::POA_ptr, Database_ptr);
    virtual ~StrategyAdaptiveBase();
};

} // End of namespace LoadBalancing

#endif
