// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_StrategyLL_impl_h__
#define __OB_StrategyLL_impl_h__

#include <StrategyAdaptiveBase.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// IDL:ooc.com/LoadBalancing/Strategy:1.0
//
// This implements the "least-load" load balancing strategy.
//
class StrategyLL_impl : public StrategyAdaptiveBase,
			public JTCMutex
{
private:

    StrategyLL_impl(const StrategyLL_impl&);
    void operator=(const StrategyLL_impl&);

    //
    // The critical-load value
    //
    CORBA::ULong criticalLoad_;

    //
    // Compute the load threshold used to select the next member.
    //
    virtual CORBA::Float loadThreshold();

    //
    // Is an alert required?
    //
    virtual bool requireAlert(CORBA::Float load);

public:

    StrategyLL_impl(PortableServer::POA_ptr, Database_ptr,
		    const PropertySeq&);
    ~StrategyLL_impl();

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
    //
    virtual char* get_name()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_properties:1.0
    //
    virtual LoadBalancing::PropertySeq* get_properties()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
    //
    virtual void adjust(const LoadBalancing::MemberDataSeq& members)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
    //
    virtual LoadBalancing::MemberId get_next()
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/push_load:1.0
    //
    virtual void push_load(const char* member_id,
			   CORBA::ULong load)
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);
};

} // End of namespace LoadBalancing

#endif
