// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#include <OB/CORBA.h>

#include <BalancerLogger.h>
#include <StrategyProxy_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// StrategyProxy_impl constructor and destructor
// ----------------------------------------------------------------------

//
// IDL:ooc.com/LoadBalancing/StrategyProxy:1.0
//
LoadBalancing::StrategyProxy_impl::StrategyProxy_impl(
    PortableServer::POA_ptr poa,
    Logger_ptr logger,
    Strategy_ptr strategy)
    : poa_(PortableServer::POA::_duplicate(poa)),
      logger_(Logger::_duplicate(logger)),
      strategy_(Strategy::_duplicate(strategy))
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyProxy_impl::StrategyProxy_impl()" << OB_ENDL;
#endif
}

LoadBalancing::StrategyProxy_impl::~StrategyProxy_impl()
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyProxy_impl::~StrategyProxy_impl()" << OB_ENDL;
#endif
}

// ----------------------------------------------------------------------
// StrategyProxy_impl public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
LoadBalancing::StrategyProxy_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

void
LoadBalancing::StrategyProxy_impl::setStrategy(Strategy_ptr strategy)
{
    JTCSynchronized sync(*this);

    strategy_ = Strategy::_duplicate(strategy);
}

//
// IDL:ooc.com/LoadBalancing/StrategyProxy/get_name:1.0
//
char*
LoadBalancing::StrategyProxy_impl::get_name()
    throw(CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    return strategy_ -> get_name();
}

//
// IDL:ooc.com/LoadBalancing/Strategy/get_properties:1.0
//
LoadBalancing::PropertySeq*
LoadBalancing::StrategyProxy_impl::get_properties()
    throw(CORBA::SystemException)
{
    return strategy_ -> get_properties();
}


//
// IDL:ooc.com/LoadBalancing/StrategyProxy/push_load:1.0
//
void
LoadBalancing::StrategyProxy_impl::push_load(const char* member_id,
                                             CORBA::ULong load)
    throw(LoadBalancing::MemberNotFound,
          LoadBalancing::StrategyNotAdaptive,
          CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    strategy_ -> push_load(member_id, load);

    if(logger_ -> traceLoadBalance() >= 3)
    {
	String_var msg = string_dup("Pushed load of ");
	msg += load;
	msg += " to strategy for member `";
	msg += member_id;
	msg += "'";

	logger_ -> trace("balancer", msg);
    }
}

