// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_StrategyRR_impl_h__
#define __OB_StrategyRR_impl_h__

#include <OB/list.h>

#include <BalancerDatabase_fwd.h>

#include <StrategyBase.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// IDL:ooc.com/LoadBalancing/Strategy:1.0
//
// This implements a simple round-robin load balancing strategy.
//
class StrategyRR_impl : public StrategyBase,
			public JTCMutex
{
public: // For some compilers this must be public

    typedef OB::stl::list< MemberData* > MemberList;

private:

    StrategyRR_impl(const StrategyRR_impl&);
    void operator=(const StrategyRR_impl&);

    //
    // The members
    //
    MemberList members_;

public:

    StrategyRR_impl(PortableServer::POA_ptr, Database_ptr);
    ~StrategyRR_impl();

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
    //
    virtual char* get_name()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
    //
    virtual void adjust(const LoadBalancing::MemberDataSeq& members)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
    //
    virtual LoadBalancing::MemberId get_next()
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);
};

} // End of namespace LoadBalancing

#endif
