// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <BalancerDatabase.h>
#include <StrategyRand_impl.h>

#include <time.h>
#ifdef WIN32
#   include <sys/timeb.h>
#endif
#ifdef HAVE_STDLIB_H
#   include <stdlib.h>
#endif

#include <Config.h> // for HAVE_BROKEN_RAND_R

using namespace CORBA;

// ----------------------------------------------------------------------
// StrategyRand_impl private member implementation
// ----------------------------------------------------------------------

ULong
LoadBalancing::StrategyRand_impl::random(ULong n)
{
#if defined(WIN32)
    //
    // NOTE: On windows platforms, srand() must be called to seed the
    //       pseudorandom-number generator before calling rand().
    //       Futhermore, this must be done in each thread that rand() is
    //       called.
    //
    //	     Depending on the concurrency model, this method may be
    //	     called by a different thread each time.  Hence, srand()
    //	     must be called before calling rand().  Note that this will
    //	     not necessarily produce the best set of random numbers, but
    //	     the alternative (having a dedicated thread to create the
    //	     random numbers) is too costly.  Maybe an implementation of
    //	     rand_r() can be added?
    //
    struct _timeb timebuffer;
    _ftime(&timebuffer);
    srand((unsigned)timebuffer.millitm * timebuffer.millitm);

    int r = rand();
#else
    int r = rand_r(&seed_);
#endif

#if defined(__GNUC__) // COMPILERFIX: rand() sometimes returns a
		      //              negative number
    if(r < 0)
	r = -r;
#endif
    assert(0 <= r  && r <= RAND_MAX);

    return (ULong) ((double) n * r / (RAND_MAX + 1.0));
}

// ----------------------------------------------------------------------
// StrategyRand_impl constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::StrategyRand_impl::StrategyRand_impl(
    PortableServer::POA_ptr poa, Database_ptr db)
    : StrategyBase(poa, db),
      seed_((unsigned)time(0))
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyRand_impl::StrategyRand_impl()" << OB_ENDL;
#endif
}

LoadBalancing::StrategyRand_impl::~StrategyRand_impl()
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyRand_impl::~StrategyRand_impl()" << OB_ENDL;
#endif
}

// ----------------------------------------------------------------------
// StrategyRand_impl public member implementation
// ----------------------------------------------------------------------

//
// IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
//
char*
LoadBalancing::StrategyRand_impl::get_name()
    throw(CORBA::SystemException)
{
    return string_dup("random");
}

//
// IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
//
void
LoadBalancing::StrategyRand_impl::adjust(
    const LoadBalancing::MemberDataSeq& members)
    throw(CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    members_ = members;
}

//
// IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
//
LoadBalancing::MemberId
LoadBalancing::StrategyRand_impl::get_next()
    throw(LoadBalancing::MemberNotFound,
	  CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    ULong len = members_.length();
    if(len == 0)
	throw MemberNotFound();

    ULong next = random(len);
    assert(next < len);

    return string_dup(members_[next].member_id);
}

