// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <LoadCalculator_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// LoadCalculator_impl constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::Util::LoadCalculator_impl::LoadCalculator_impl()
    : activeRequests_(0), completedRequests_(0)
{
#ifdef LB_DEBUG
    OB_CERR << "LoadCalculator_impl::LoadCalculator_impl()" << OB_ENDL;
#endif
}

LoadBalancing::Util::LoadCalculator_impl::~LoadCalculator_impl()
{
#ifdef LB_DEBUG
    OB_CERR << "LoadCalculator_impl::~LoadCalculator_impl()" << OB_ENDL;
#endif
}

// ----------------------------------------------------------------------
// LoadCalculator_impl public member implementation
// ----------------------------------------------------------------------

void
LoadBalancing::Util::LoadCalculator_impl::newRequest()
{
    JTCSynchronized sync(*this);

    activeRequests_++;
}

void
LoadBalancing::Util::LoadCalculator_impl::requestCompleted()
{
    JTCSynchronized sync(*this);

    activeRequests_--;
    assert(activeRequests_ >= 0);

    completedRequests_++;
}

ULong
LoadBalancing::Util::LoadCalculator_impl::calculate_load()
    throw(CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    Long load = activeRequests_ + completedRequests_;
    completedRequests_ = 0;

    return load;
}
