// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>

#include <UtilLogger.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(LoadBalancing::Util::Logger_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(LoadBalancing::Util::Logger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// Logger constructor/destructor
// ----------------------------------------------------------------------

LoadBalancing::Util::Logger::Logger(OB::Logger_ptr logger,
				    OB::Properties_ptr properties)
    : logger_(OB::Logger::_duplicate(logger)),
      traceLevel_(0)
{
#ifdef LB_DEBUG
    OB_CERR << "Logger::Logger()" << OB_ENDL;
#endif

    const char* value =
	properties -> getProperty("ooc.balancer.util.trace_level");
    if(value != 0)
	traceLevel_ = atol(value);
}

LoadBalancing::Util::Logger::~Logger()
{
#ifdef LB_DEBUG
    OB_CERR << "Logger::~Logger()" << OB_ENDL;
#endif
}

// ----------------------------------------------------------------------
// Logger trace level accessors
// ----------------------------------------------------------------------

CORBA::ULong
LoadBalancing::Util::Logger::traceLevel()
{
    return traceLevel_;
}

// ----------------------------------------------------------------------
// Logger logging methods
// ----------------------------------------------------------------------

void
LoadBalancing::Util::Logger::info(const char* msg)
{
    logger_ -> info(msg);
}

void
LoadBalancing::Util::Logger::error(const char* msg)
{
    logger_ -> error(msg);
}

void
LoadBalancing::Util::Logger::warning(const char* msg)
{
    logger_ -> warning(msg);
}

void
LoadBalancing::Util::Logger::trace(const char* category, const char* msg)
{
    logger_ -> trace(category, msg);
}
