// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Balancer.h>
#include <OB/Balancer_init.h>

#include <Tester_impl.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;
using namespace LoadBalancing;

int
run(ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve the Root POA
    //
    Object_var obj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Get the Balancer
    //
    try
    {
        obj = orb -> resolve_initial_references("Balancer");
    }
    catch(const ORB::InvalidName&)
    {
        OB_CERR << argv[0] << ": can't resolve `Balancer'" << OB_ENDL;
        return EXIT_FAILURE;
    }

    if(is_nil(obj.in()))
    {
        OB_CERR << argv[0] << ": `Balancer' is a nil object reference" 
		     << OB_ENDL;
        return EXIT_FAILURE;
    }

    GroupFactory_ptr factory = GroupFactory::_narrow(obj);
    if(is_nil(factory))
    {
        OB_CERR << argv[0]
             << ": `Balancer' is not a GroupFactory object "
	     << "reference" << OB_ENDL;
        return EXIT_FAILURE;
    }

    //
    // Create implementation object
    //
    Tester_impl* impl =
	new Tester_impl(orb, rootPOA, factory);
    PortableServer::ServantBase_var servant = impl;
    Tester_var tester = impl -> _this();

    //
    // Save the reference
    //
    String_var s = orb -> object_to_string(tester);
    const char* refFile = "Tester.ref";
    OB_OFSTREAM out(refFile);
    if(out.fail())
    {
	OB_CERR << argv[0] << ": can't open `" << refFile << "': "
		<< strerror(errno) << OB_ENDL;
	return EXIT_FAILURE;
    }
    out << s << OB_ENDL;
    out.close();

    //
    // Run the implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
	LB_init();
	orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
