// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <MemberObject_impl.h>
#include <Tester_impl.h>

using namespace CORBA;
using namespace LoadBalancing;

//
// IDL:Tester/test_RandomStrategy:1.0
//
void
Tester_impl::test_RandomStrategy()
    throw(CORBA::SystemException)
{
    try
    {
	MemberIdSeq ids;

	//
	// Create the oid for the replicas
	//
	PortableServer::ObjectId_var oid =
	    PortableServer::string_to_ObjectId("MemberObject");
	PortableInterceptor::ObjectId piOid(oid -> length(),
					    oid -> length(),
					    oid -> get_buffer());

	//
	// Create a new group
	//
	Group_var group = factory_ -> create("g1");

	//
	// Get the load balancing proxy
	//
	StrategyProxy_var proxy = group -> get_strategy_proxy();

	//
	// Get the ior that when narrowed will get the replica of the least
	// loaded member
	//
	Object_var obj = group -> get_ior("IDL:MemberObject:1.0", piOid);

	//
	// Add the member `1'
	//
	group -> add_member("1");

	//
	// Create the replica POA for member `1'
	//
	PortableServer::POA_var p1 = createReplicaPOA("p1", "g1", "1");

	//
	// Create the servant for the replica of member `1'
	//
	MemberObject_impl* m1Impl = new MemberObject_impl(p1, "1");
	PortableServer::ServantBase_var m1Servant = m1Impl;
	p1 -> activate_object_with_id(oid.in(), m1Impl);

	//
	// Use the random strategy
	//
	PropertySeq props;
	group -> set_strategy("random", props);

	//
	// Verify the strategy
	//
	TEST(checkStrategy(proxy, "random"));

	//
	// Should now get the replica of member `1' when the ior returned
	// by _this() is used
	//
	ids.length(1);
	ids[0] = string_dup("1");
	TEST(checkGroupReplicas(m1Impl, ids));

	//
	// Should now get the replica of member `1' when the client ior is
	// used
	//
	ids.length(1);
	ids[0] = string_dup("1");
	TEST(checkGroupReplicas(obj, ids));

	//
	// Create the replica POA for members `2' and `3' (both
	// members will be implicitely added)
	//
	PortableServer::POA_var p2 = createReplicaPOA("p2", "g1", "2");
	PortableServer::POA_var p3 = createReplicaPOA("p3", "g1", "3");

	//
	// Create the servants for the replicas of members `2' and `3'
	//
	MemberObject_impl* m2Impl = new MemberObject_impl(p2, "2");
	PortableServer::ServantBase_var m2Servant = m2Impl;
	p2 -> activate_object_with_id(oid.in(), m2Impl);

	MemberObject_impl* m3Impl = new MemberObject_impl(p3, "3");
	PortableServer::ServantBase_var m3Servant = m3Impl;
	p3 -> activate_object_with_id(oid.in(), m3Impl);

	//
	// Get the replica 20 times and print the ids to the screen
	//
	OB_COUT << "(Should see random numbers between 1 and 3: " << OB_FLUSH;
	for(int i = 0 ; i < 20 ; i++)
	{
	    MemberObject_var mobj = MemberObject::_narrow(obj);
	    MemberId_var id =  mobj -> get_id();
	    OB_COUT << " " << id << OB_FLUSH;
	}
	OB_COUT << ")" << OB_FLUSH;

	//
	// Destroy the POAs
	//
	p1 -> destroy(true, false);
	p2 -> destroy(true, false);
	p3 -> destroy(true, false);

	//
	// Destroy the group
	//
	group -> destroy();
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	TEST(false);
    }
}
