// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/BootManager.h>

#if !defined(HAVE_SHARED) && !defined(OB_DLL)
#include <OB/OCI_init.h>
#include <OB/OCI_BiDir_init.h>
#endif

#include <Hello_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(poaObj);
    PortableServer::POAManager_var manager = root -> the_POAManager();

    //
    // Create implementation object and activate it on the RootPOA
    //
    Hello_impl* helloImpl = new Hello_impl(orb, root);
    PortableServer::ServantBase_var servant = helloImpl;
    Hello_var hello = helloImpl -> _this();

    //
    // Register the servant with the BootManager to allow the
    // use of corbaloc URLs
    //
    CORBA::Object_var bmObj = orb -> resolve_initial_references("BootManager");
    OB::BootManager_var bm = OB::BootManager::_narrow(bmObj);
    PortableServer::ObjectId_var oid =
        PortableServer::string_to_ObjectId("Hello");
    bm -> add_binding(oid, hello);

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(hello);

    const char* refFile = "Hello.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "': "
             << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
#if !defined(HAVE_SHARED) && !defined(OB_DLL)
        //
        // When linking statically, we need to explicitly register
        // the plug-in prior to ORB initialization
        //
        OCI::register_plugin("bidir", OCI_init_bidir);
#endif

        //
        // Initialize the ORB
        //
        orb = CORBA::ORB_init(argc, argv);

        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
