dnl **********************************************************************
dnl
dnl Copyright (c) 2002
dnl IONA Technologies, Inc.
dnl Waltham, MA, USA
dnl
dnl All Rights Reserved
dnl
dnl **********************************************************************

dnl ----------------------------------------------------------------------
dnl Check whether the user executed runconfig
dnl ----------------------------------------------------------------------

if test "$RUNCONFIG" != "yes"
then
    AC_MSG_ERROR(You must run ../runconfig before running configure!
		  See the file ../INSTALL for more information.)
fi

dnl ----------------------------------------------------------------------
dnl Check for byte order option (required for some platforms when
dnl cross-compiling as these platforms do not provide byte order
dnl information in their header files)
dnl ----------------------------------------------------------------------

if test -n "$enable_big_endian"
then
    ac_cv_c_bigendian="$enable_big_endian"
fi

dnl ----------------------------------------------------------------------
dnl Standard tests
dnl ----------------------------------------------------------------------

AC_CANONICAL_HOST
AC_ISC_POSIX

dnl ----------------------------------------------------------------------
dnl Checks for programs
dnl ----------------------------------------------------------------------

AC_PROG_CC
AC_PROG_CPP
AC_PROG_CXX
AC_PROG_CXXCPP
AC_PROG_MAKE_SET
AC_PROG_INSTALL
AC_PROG_LN_S
AC_PROG_RANLIB
AC_PROG_YACC
AC_PROG_LEX
if test -z "$LEXLIB"
then
    AC_CHECK_LIB(l, yywrap, LEXLIB=-ll)
fi

dnl ----------------------------------------------------------------------
dnl Check for header files
dnl Note: Do not test for string.h, strings.h, unistd.h ...
dnl These files are checked by autoconf and AC_HEADER_STDC anyway and the
dnl order in which they are checked should not be changed.
dnl ----------------------------------------------------------------------

AC_HEADER_STDC
AC_CHECK_HEADERS(stddef.h)
AC_CHECK_HEADERS(bstring.h)
AC_CHECK_HEADERS(termio.h termios.h)
AC_CHECK_HEADERS(sys/types.h sys/select.h sys/param.h)

AC_CHECK_HEADERS(time.h sys/time.h sys/times.h)
AC_HEADER_TIME

dnl ----------------------------------------------------------------------
dnl Currently (OB 4.1.0) only needed for idlcpp on Solaris
dnl ----------------------------------------------------------------------

AC_CHECK_HEADERS(fcntl.h)

dnl ----------------------------------------------------------------------
dnl Check for functions
dnl ----------------------------------------------------------------------

AC_CHECK_FUNCS(strerror)
AC_CHECK_FUNCS(strcasecmp)
AC_CHECK_FUNCS(strncasecmp)
AC_CHECK_FUNCS(vprintf)

dnl These are checked for but not yet used. OB::wcscpy etc. should be
dnl replaced by them as soon as they are available on any platform.
AC_CHECK_FUNCS(wcslen)
AC_CHECK_FUNCS(wcscpy)
AC_CHECK_FUNCS(wcsncpy)
AC_CHECK_FUNCS(wcscmp)
AC_CHECK_FUNCS(wcschr)

dnl ----------------------------------------------------------------------
dnl Check for gethostname prototype
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether there is a prototype for gethostname,
ac_cv_gethostname_has_prototype,
AC_EGREP_HEADER(gethostname, unistd.h,
		ac_cv_gethostname_has_prototype=yes,
		ac_cv_gethostname_has_prototype=no))

if test $ac_cv_gethostname_has_prototype = no
then
    AC_DEFINE(HAVE_NO_GETHOSTNAME_PROTOTYPE)
fi

dnl ----------------------------------------------------------------------
dnl Check for h_errno declaration
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether there is a declaration for h_errno,
ac_cv_h_errno_is_declared,
AC_EGREP_HEADER(h_errno, netdb.h,
		ac_cv_h_errno_is_declared=yes,
		ac_cv_h_errno_is_declared=no))

if test $ac_cv_h_errno_is_declared = no
then
    AC_DEFINE(HAVE_NO_H_ERRNO_DECLARATION)
fi

dnl ----------------------------------------------------------------------
dnl Check for typedefs, structures, and compiler characteristics
dnl ----------------------------------------------------------------------

AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_CHECK_SIZEOF(char)
AC_CHECK_SIZEOF(short)
AC_CHECK_SIZEOF(int)
AC_CHECK_SIZEOF(long)
AC_CHECK_SIZEOF(float)
AC_CHECK_SIZEOF(double)
AC_CHECK_SIZEOF(bool)
AC_CHECK_SIZEOF(wchar_t)
AC_CHECK_SIZEOF(long long)
AC_CHECK_SIZEOF(long double)
AC_C_BIGENDIAN
AC_LANG_RESTORE

dnl ----------------------------------------------------------------------
dnl Check for library suffix
dnl ----------------------------------------------------------------------

AC_MSG_CHECKING(for library suffix)
AC_SUBST(LIBEXT)
if test -z "$LIBEXT"
then
    LIBEXT=".a"
fi
AC_MSG_RESULT($LIBEXT)

dnl ----------------------------------------------------------------------
dnl Find out how to create libraries
dnl ----------------------------------------------------------------------

AC_MSG_CHECKING(how to build libraries)
AC_SUBST(AR)
AC_SUBST(ARFLAGS)
if test -z "$AR"
then
    AR="ar"
fi
if test -z "$ARFLAGS"
then
    ARFLAGS="cr"
fi
AC_MSG_RESULT(with $AR $ARFLAGS)

dnl ----------------------------------------------------------------------
dnl Find out how to install libraries
dnl ----------------------------------------------------------------------

AC_MSG_CHECKING(how to install libraries)
AC_SUBST(INSTALL_LIBRARY)
if test -z "$INSTALL_LIBRARY"
then
    if test "$LIBEXT" = ".sl" # HP needs executable shared libs
    then
        INSTALL_LIBRARY="$INSTALL_PROGRAM"
    else
        INSTALL_LIBRARY="$INSTALL_DATA"
    fi
else
    INSTALL_LIBRARY="$INSTALL_DATA"
fi
AC_MSG_RESULT(with $INSTALL_LIBRARY)

dnl -----------------------------------------------------------------------
dnl Commands for managing symbolic links for shared libraries
dnl -----------------------------------------------------------------------

AC_SUBST(SYMLNK_SHLIB)
AC_SUBST(RMLNK_SHLIB)
SYMLNK_SHLIB=":"
RMLNK_SHLIB=":"

if test "$LIBEXT" != ".a"
then
    if test "$LN_S" = "ln -s"
    then
        SYMLNK_SHLIB="ln -f -s"
    else
        SYMLNK_SHLIB="$LN_S -f"
    fi
    RMLNK_SHLIB="rm -f"
fi

dnl ----------------------------------------------------------------------
dnl Define HAVE_SHARED and WITH_SHARED=yes if shared libraries are used
dnl ----------------------------------------------------------------------

WITH_SHARED="no"
AC_SUBST(WITH_SHARED)

if test "$LIBEXT" != ".a"
then
    WITH_SHARED="yes"
    AC_DEFINE(HAVE_SHARED)
fi
