dnl **********************************************************************
dnl
dnl Copyright (c) 2002
dnl IONA Technologies, Inc.
dnl Waltham, MA, USA
dnl
dnl All Rights Reserved
dnl
dnl **********************************************************************

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports RTTI
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports RTTI,
ac_cv_rtti,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
struct X { virtual ~X() { } };
struct Y : virtual public X { virtual ~Y() { } };
void f() { Y y; X* xp = &y; Y* yp = dynamic_cast<Y*>(xp); }
,
ac_cv_rtti=yes, ac_cv_rtti=no)
AC_LANG_RESTORE)

if test $ac_cv_rtti = no
then
    AC_MSG_ERROR(Compiler does not support RTTI!)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports exception handling
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports exception handling,
ac_cv_exception_handling,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
void f() { throw "abc"; }
void g() { try { f(); } catch(char*) { } }
,
ac_cv_exception_handling=yes, ac_cv_exception_handling=no)
AC_LANG_RESTORE)

if test $ac_cv_exception_handling = no
then
    AC_MSG_ERROR(Compiler does not support exception handling!)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports namespaces
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports namespaces,
ac_cv_namespace,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
namespace N { typedef int I; }; N::I i;
,
ac_cv_namespace=yes, ac_cv_namespace=no)
AC_LANG_RESTORE)

if test $ac_cv_namespace = no
then
    AC_MSG_ERROR(Compiler does not support namespaces!)
fi

dnl ----------------------------------------------------------------------
dnl Check for C++ header files
dnl ----------------------------------------------------------------------

AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_CHECK_HEADERS(exception exception.h terminate.h unexpected.h)
AC_LANG_RESTORE

dnl
dnl COMPILERFIX: This does not work on AIX with VAC++ 5.0 and SGI 7.3.x
dnl so we have to do the tests manually below.
dnl
dnl AC_CHECK_HEADERS(new iostream sstream fstream iomanip)
AC_CACHE_CHECK(for new,
ac_cv_header_new,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <new>
,
ac_cv_header_new=yes, ac_cv_header_new=no)
AC_LANG_RESTORE)

if test $ac_cv_header_new = yes
then
    AC_DEFINE(HAVE_NEW)
fi

AC_CACHE_CHECK(for iostream,
ac_cv_header_iostream,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <iostream>
,
ac_cv_header_iostream=yes, ac_cv_header_iostream=no)
AC_LANG_RESTORE)

if test $ac_cv_header_iostream = yes
then
    AC_DEFINE(HAVE_IOSTREAM)
fi

AC_CACHE_CHECK(for sstream,
ac_cv_header_sstream,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <sstream>
,
ac_cv_header_sstream=yes, ac_cv_header_sstream=no)
AC_LANG_RESTORE)

if test $ac_cv_header_sstream = yes
then
    AC_DEFINE(HAVE_SSTREAM)
fi

AC_CACHE_CHECK(for fstream,
ac_cv_header_fstream,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <fstream>
,
ac_cv_header_fstream=yes, ac_cv_header_fstream=no)
AC_LANG_RESTORE)

if test $ac_cv_header_fstream = yes
then
    AC_DEFINE(HAVE_FSTREAM)
fi

AC_CACHE_CHECK(for iomanip,
ac_cv_header_iomanip,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <iomanip>
,
ac_cv_header_iomanip=yes, ac_cv_header_iomanip=no)
AC_LANG_RESTORE)

if test $ac_cv_header_iomanip = yes
then
    AC_DEFINE(HAVE_IOMANIP)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports iostream in namespace std
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports iostream in namespace std,
ac_cv_std_iostream,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
if test $ac_cv_header_iostream = yes
then
AC_TRY_COMPILE_GLOBAL(
,
#include <iostream>
void f(std::ostream s) { s << "xxx\n"; }
,
ac_cv_std_iostream=yes, ac_cv_std_iostream=no)
else
AC_TRY_COMPILE_GLOBAL(
,
#include <iostream.h>
void f(std::ostream s) { s << "xxx\n"; }
,
ac_cv_std_iostream=yes, ac_cv_std_iostream=no)
fi
AC_LANG_RESTORE)

if test $ac_cv_std_iostream = yes
then
    AC_DEFINE(HAVE_STD_IOSTREAM)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports wide character stream I/O
dnl (in namespace std)
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports wide character stream I/O,
ac_cv_std_wostream,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS

AC_TRY_COMPILE_GLOBAL(
,
#include <iostream>
void f(std::wostream s) { s << L"xxx\n"; std::wcout << std::endl; }
,
ac_cv_std_wostream=yes, ac_cv_std_wostream=no)
AC_LANG_RESTORE)

if test $ac_cv_std_wostream = no
then
    AC_DEFINE(HAVE_NO_WCHAR_T_SUPPORT_IN_STREAMS)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports STL in namespace std
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports STL in namespace std,
ac_cv_std_stl,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <vector>
typedef std::vector<int> IntVector;
,
ac_cv_std_stl=yes, ac_cv_std_stl=no)
AC_LANG_RESTORE)

if test $ac_cv_std_stl = yes
then
    AC_DEFINE(HAVE_STD_STL)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler has set_unexpected and set_terminate in std
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports std::set_*,
ac_cv_std_set,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
#include <exception>
void test() { std::set_unexpected(0); std::set_terminate(0); }
,
ac_cv_std_set=yes, ac_cv_std_set=no)
AC_LANG_RESTORE)

if test $ac_cv_std_set = yes
then
    AC_DEFINE(HAVE_STD_SET_X)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler can explicitly instantiate templates
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler can explicitly instantiate templates,
ac_cv_explicit_templates,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
template<class T> class X { T t; }; template class X<int>; X<int> x;
,
ac_cv_explicit_templates=yes, ac_cv_explicit_templates=no)
AC_LANG_RESTORE)

if test $ac_cv_explicit_templates = no
then
    AC_DEFINE(HAVE_NO_EXPLICIT_TEMPLATES)
fi

dnl ----------------------------------------------------------------------
dnl Check whether c++ compiler supports typename
dnl ----------------------------------------------------------------------

AC_CACHE_CHECK(whether c++ compiler supports typename,
ac_cv_typename,
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE_GLOBAL(
,
template<class T> class X { public: typedef typename T::Y Y; };
class Z { public: typedef int Y; };
X<Z> x;
,
ac_cv_typename=yes, ac_cv_typename=no)
AC_LANG_RESTORE)

if test $ac_cv_typename = no
then
    AC_DEFINE(HAVE_NO_TYPENAME)
fi
