// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <EventLogger.h>
#include <PipeSegment.h>
#include <ProxyBase.h>
#include <Reaper.h>

#include <AdminBase.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// AdminBase protected member implementation
// ----------------------------------------------------------------------

//
// Reap proxies
//
void
AdminBase::reap()
{
    //
    // Find the element, remove it from the sequence.
    //
    ProxyPtrList::iterator iter;
    for(iter = proxies_.begin() ; iter != proxies_.end() ; ++iter)
    {
	if((*iter) -> destroyed())
	{
	    (*iter) -> _remove_ref();
	    proxies_.erase(iter);
	}
    }
}

//
// Add a proxy
//
void
AdminBase::addProxy(ProxyBase* proxy)
{
    reap();
    proxies_.push_back(proxy);
    if(reaper_)
	reaper_ -> add(proxy);
}

//
// Fill constructor info
//
void
AdminBase::fillProxyConstructorInfo(ProxyConstructorInfo& info)
{
    info.orb = ORB::_duplicate(orb_);
    info.poa = PortableServer::POA::_duplicate(poa_);
    info.logger = OB::EventLogger::_duplicate(logger_);
    info.pipe = PipeSegment::_duplicate(myPipe_);
    info.id = nextId_++;
}

// ----------------------------------------------------------------------
// AdminBase constructor/destructor
// ----------------------------------------------------------------------

AdminBase::AdminBase(ORB_ptr orb, PortableServer::POA_ptr poa,
		     OB::EventLogger_ptr logger, Reaper* reaper,
		     PipeSegment_ptr pipe)
    : orb_(ORB::_duplicate(orb)),
      poa_(PortableServer::POA::_duplicate(poa)),
      logger_(OB::EventLogger::_duplicate(logger)),
      myPipe_(pipe),
      nextId_(0),
      destroyed_(false),
      reaper_(reaper)
{
}

AdminBase::~AdminBase()
{
    assert(proxies_.size() == 0);
}

// ----------------------------------------------------------------------
// AdminBase public member implementation
// ----------------------------------------------------------------------

void
AdminBase::disconnect()
{
    JTCSynchronized sync(*this);
    
    assert(!destroyed_);
    destroyed_ = true;

    //
    // disconnect each proxy.
    //
    ProxyPtrList::iterator iter;
    for(iter = proxies_.begin() ; iter != proxies_.end() ; ++iter)
    {
	(*iter) -> disconnect();
	(*iter) -> _remove_ref();
    }
    proxies_.clear();

    //
    // Disconnect the servant from the ORB.
    //
    deactivateObject();
}

void
AdminBase::discardRequests()
{
    JTCSynchronized sync(*this);
    
    if(!destroyed_)
    {
	ProxyPtrList::iterator iter;
	for(iter = proxies_.begin() ; iter != proxies_.end() ; ++iter)
	    (*iter) -> discardRequests();
    }
}

PortableServer::POA_ptr
AdminBase::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}
