// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
#ifndef COS_EVENT_CHANNEL_IMPL_H
#define COS_EVENT_CHANNEL_IMPL_H

//
// Forward declaration
//
class EC_ConsumerAdmin_impl;
class EC_SupplierAdmin_impl;

class PipeSegment;
typedef PipeSegment* PipeSegment_ptr;
typedef OB::ObjVar< PipeSegment > PipeSegment_var;
void OBDuplicate(PipeSegment_ptr);
void OBRelease(PipeSegment_ptr);

class Reaper;

//
// The event channel implementation
//
class EC_EventChannel_impl :
    virtual public PortableServer::RefCountServantBase,
    public POA_CosEventChannelAdmin::EventChannel,
    public JTCRecursiveMutex
{
    //
    // No copy constructor/assignment operator
    //
    EC_EventChannel_impl(const EC_EventChannel_impl&);
    void operator=(const EC_EventChannel_impl&);
    
    PortableServer::POA_var poa_; // The POA

    OB::EventLogger_var logger_; // The message logger
    
    PipeSegment_var myPipe_; // My pipe segment

    EC_ConsumerAdmin_impl* consumerAdmin_; // The consumer admin
    
    EC_SupplierAdmin_impl* supplierAdmin_; // The supplier admin

    CORBA::String_var myId_; // The channel's Id

    bool destroy_; // Has the channel been destroyed?

public:

    EC_EventChannel_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
    			 PortableServer::POA_ptr, OB::EventLogger_ptr,
			 Reaper*, const char*, OB::EventReceiver_ptr);

    ~EC_EventChannel_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // Return ConsumerAdmin object.
    //
    virtual CosEventChannelAdmin::ConsumerAdmin_ptr for_consumers()
	throw(CORBA::SystemException);

    //
    // Return SupplierAdmin object.
    //
    virtual CosEventChannelAdmin::SupplierAdmin_ptr for_suppliers()
	throw(CORBA::SystemException);

    //
    // Destroy this event channel.
    //
    virtual void destroy()
	throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr
    _default_POA();

    //
    // Internal methods
    //

    //
    // Stop accepting new requests.
    //
    void discardRequests();

    //
    // Has the event channel been destroyed?
    //
    bool destroyed();
};

#endif
