// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/Properties.h>
#include <OB/Logger.h>

#include <EventLogger.h>

using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(EventLogger_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(EventLogger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// EventLogger constructor/destructor
// ----------------------------------------------------------------------

EventLogger::EventLogger(OB::Logger_ptr logger,
			 OB::Properties_ptr properties)
    : logger_(OB::Logger::_duplicate(logger)),
      traceEvents_(0),
      traceLifecycle_(0)
{
    OB::Properties::KeySeq keys = properties -> getKeys("ooc.event.trace.");
    for(CORBA::ULong i = 0; i < keys.length(); ++i)
    {
        const char* key = keys[i];
	const char* value = properties -> getProperty(key);
	
	if(strcmp(key, "ooc.event.trace.events") == 0)
	{
	    traceEvents_ = atol(value);
	}
	else if(strcmp(key, "ooc.event.trace.lifecycle") == 0)
	{
	    traceLifecycle_ = atol(value);
	}
	else
	{
	    CORBA::String_var err =
	        CORBA::string_dup("EventService: unknown property: ");
	    err += key;
	    logger_ -> warning(err);
	}
    }
}

EventLogger::~EventLogger()
{
}

// ----------------------------------------------------------------------
// EventLogger trace level accessors
// ----------------------------------------------------------------------

CORBA::ULong
EventLogger::traceEvents()
{
    return traceEvents_;
}

CORBA::ULong
EventLogger::traceLifecycle()
{
    return traceLifecycle_;
}

// ----------------------------------------------------------------------
// EventLogger logging methods
// ----------------------------------------------------------------------

void
EventLogger::info(const char* msg)
{
    logger_ -> info(msg);
}

void
EventLogger::error(const char* msg)
{
    logger_ -> error(msg);
}

void
EventLogger::warning(const char* msg)
{
    logger_ -> warning(msg);
}

void
EventLogger::trace(const char* category, const char* msg)
{
    logger_ -> trace(category, msg);
}
