// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef EVENT_LOGGER_H
#define EVENT_LOGGER_H

#include <OB/Basic.h>

#include <OB/Properties_fwd.h>
#include <OB/Logger_fwd.h>

#include <EventLogger_fwd.h>

namespace OB
{

class EventLogger : public OB::RefCount
{
    //
    // The message logger
    //
    OB::Logger_var logger_; 

    //
    // The trace levels
    //
    CORBA::ULong traceEvents_;
    CORBA::ULong traceLifecycle_;

public:

    EventLogger(OB::Logger_ptr, OB::Properties_ptr);
    ~EventLogger();

    static inline EventLogger_ptr _duplicate(EventLogger_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline EventLogger_ptr _nil()
    { return 0; }

    //
    // Methods to get trace levels
    //
    CORBA::ULong traceEvents();
    CORBA::ULong traceLifecycle();

    //
    // Methods to perform logging
    //
    void info(const char*);
    void error(const char*);
    void warning(const char*);
    void trace(const char*, const char*);
};

}; // End of namespace OB

namespace CORBA
{

inline void
release(OB::EventLogger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(OB::EventLogger_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
