// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>

#include <OB/CosTypedEventChannelAdmin_skel.h>

#include <EventLogger.h>
#include <PropagatorBaseThread.h> // SGI 7.0/7.1 needs this, and PusherBase...
#include <PusherBaseThread.h>
#include <PipeSegment.h>
#include <ProxyBase.h>
#include <TEC_ProxyPushSupplier_impl.h>
#include <AdminBase.h>
#include <ChannelType.h>

#include <TEC_TypedConsumerAdmin_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// TEC_TypedConsumerAdmin_impl private member implementation
// ----------------------------------------------------------------------

void
TEC_TypedConsumerAdmin_impl::deactivateObject()
{
    PortableServer::POA_var poa = _default_POA();
    PortableServer::ObjectId_var oid = poa -> servant_to_id(this);
    poa -> deactivate_object(oid.in());
}

// ----------------------------------------------------------------------
// TEC_TypedConsumerAdmin_impl constructor and destructor
// ----------------------------------------------------------------------

TEC_TypedConsumerAdmin_impl::TEC_TypedConsumerAdmin_impl(
    ORB_ptr orb,
    PortableServer::POA_ptr poa,
    OB::EventLogger_ptr logger,
    Reaper* reaper,
    PipeSegment_ptr pipe,
    ChannelType* type)
    : AdminBase(orb, poa, logger, reaper, pipe),
      type_(type)
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "creating TypedConsumerAdmin");
}

TEC_TypedConsumerAdmin_impl::~TEC_TypedConsumerAdmin_impl()
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "destroying TypedConsumerAdmin");
}

// ----------------------------------------------------------------------
// Standard IDL to C++ mapping
// ----------------------------------------------------------------------

CosEventChannelAdmin::ProxyPushSupplier_ptr
TEC_TypedConsumerAdmin_impl::obtain_push_supplier()
    throw(SystemException)
{
    throw NO_IMPLEMENT();
    return CosEventChannelAdmin::ProxyPushSupplier::_nil();
}

CosEventChannelAdmin::ProxyPullSupplier_ptr
TEC_TypedConsumerAdmin_impl::obtain_pull_supplier()
    throw(SystemException)
{
    throw NO_IMPLEMENT();
    return CosEventChannelAdmin::ProxyPullSupplier::_nil();
}

CosTypedEventChannelAdmin::TypedProxyPullSupplier_ptr
TEC_TypedConsumerAdmin_impl::obtain_typed_pull_supplier(const char*)
    throw(CosTypedEventChannelAdmin::InterfaceNotSupported,
	  SystemException)
{
    throw NO_IMPLEMENT();
    return CosTypedEventChannelAdmin::TypedProxyPullSupplier::_nil();
}

CosEventChannelAdmin::ProxyPushSupplier_ptr
TEC_TypedConsumerAdmin_impl::obtain_typed_push_supplier(const char* typeId)
    throw(CosTypedEventChannelAdmin::NoSuchImplementation,
	  SystemException)
{
    JTCSynchronized sync(*this);

    if(destroyed_)
        throw OBJECT_NOT_EXIST();

    //
    // Attempt to assign this type id to the channel.
    //
    if(!type_ -> assignTypeId(typeId))
	throw CosTypedEventChannelAdmin::NoSuchImplementation();

    ProxyConstructorInfo info;
    fillProxyConstructorInfo(info);

    TEC_ProxyPushSupplier_impl* p = new TEC_ProxyPushSupplier_impl(info);
    addProxy(p);

    return p -> _this();
}
