// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <OB/CosEventChannelAdmin.h>
#include <OB/CosTypedEventChannelAdmin.h>
#include <OB/OBEventChannelFactory.h>
#include <OB/OBTypedEventChannelFactory.h>

#include <TestSuiteUtilities.h>

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_STRINGS_H
#   include <strings.h>
#endif

#ifndef HAVE_STRCASECMP
#   define strcasecmp stricmp
#endif

//
// Global variable that tells the current trace level.
//
int TraceLevel;

static int 
GetNextID()
{
    static int id = 1;
    return id++;
}

static void
Usage() 
{
    OB_CERR << "Usage: admin\n\n";
    OB_CERR << "\tShutdown\n";
    OB_CERR << "\tCreateChannel\n";
    OB_CERR << "\tCreateConsumerAdmin <channel-ior>\n";
    OB_CERR << "\tCreateSupplierAdmin <channel-ior>\n";
    OB_CERR << "\tCreatePushConsumer <admin-ior>\n";
    OB_CERR << "\tCreatePushSupplier <admin-ior>\n";
    OB_CERR << "\tCreatePullConsumer <admin-ior>\n";
    OB_CERR << "\tCreatePullSupplier <admin-ior>\n";
    OB_CERR << "\tDestroyChannel <channel-ior>\n";
    OB_CERR << "\tDestroyProxyConsumer <proxy-ior>\n"; 
    OB_CERR << "\tDestroyProxySupplier <proxy-ior>\n";
    OB_CERR << "\tTypedShutdown\n";
    OB_CERR << "\tCreateTypedChannel\n";
    OB_CERR << "\tCreateTypedConsumerAdmin <channel-ior>\n";
    OB_CERR << "\tCreateTypedSupplierAdmin <channel-ior>\n";
    OB_CERR << "\tCreateTypedPushConsumer <admin-ior>\n";
    OB_CERR << "\tCreateTypedPushSupplier <admin-ior>\n";
    OB_CERR << "\tCreateTypedPullConsumer <admin-ior>\n";
    OB_CERR << "\tCreateTypedPullSupplier <admin-ior>\n";
    OB_CERR << "\tDestroyTypedChannel <channel-ior>\n";
    OB_CERR << "\tDestroyTypedProxyConsumer <proxy-ior>\n"; 
    OB_CERR << "\tDestroyTypedProxySupplier <proxy-ior>\n";
    OB_CERR << OB_FLUSH;
}

static OBTypedEventChannelFactory::TypedEventChannelFactory_ptr
GetTypedFactory(CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj;
    try
    {
	obj = orb -> 
		resolve_initial_references("TypedEventChannelFactory");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
	OB_COUT << "ERROR" << OB_ENDL;
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`TypedEventChannelFactory' causes InvalidName");
	orb -> destroy();
	exit(1);
    }

    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`TypedEventChannelFactory' reference not found");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    OBTypedEventChannelFactory::TypedEventChannelFactory_var factory = 
	OBTypedEventChannelFactory::TypedEventChannelFactory::_narrow(obj);
    
    if(CORBA::is_nil(factory))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`TypedEventChannelFactory' Initial "
		       "reference is not a TypedEventChannelFactory");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return factory._retn();
}

static OBEventChannelFactory::EventChannelFactory_ptr
GetFactory(CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj;
    try
    {
	obj = orb -> resolve_initial_references("EventChannelFactory");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`EventChannelFactory' causes InvalidName");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`EventChannelFactory' reference not found");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    OBEventChannelFactory::EventChannelFactory_var factory  = 
	OBEventChannelFactory::EventChannelFactory::_narrow(obj);
    
    if(CORBA::is_nil(factory))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "`EventChannelFactory' initial reference "
		       "is not an EventChannelFactory");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return factory._retn();
}

static CosEventChannelAdmin::EventChannel_ptr
GetChannel(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetChannel");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosEventChannelAdmin::EventChannel_var channel =
	CosEventChannelAdmin::EventChannel::_narrow(obj);
    if(CORBA::is_nil(channel))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetChannel");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return channel._retn();
}

static CosTypedEventChannelAdmin::TypedEventChannel_ptr
GetTypedChannel(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetTypedChannel");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosTypedEventChannelAdmin::TypedEventChannel_var channel =
	CosTypedEventChannelAdmin::TypedEventChannel::_narrow(obj);
    if(CORBA::is_nil(channel))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetTypedChannel");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return channel._retn();
}

static CosEventChannelAdmin::ConsumerAdmin_ptr
GetConsumerAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetConsumerAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosEventChannelAdmin::ConsumerAdmin_var consumer =
	CosEventChannelAdmin::ConsumerAdmin::_narrow(obj);
    if(CORBA::is_nil(consumer))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetConsumerAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return consumer._retn();
}

static CosTypedEventChannelAdmin::TypedConsumerAdmin_ptr
GetTypedConsumerAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetTypedConsumerAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosTypedEventChannelAdmin::TypedConsumerAdmin_var consumer =
	CosTypedEventChannelAdmin::TypedConsumerAdmin::_narrow(obj);
    if(CORBA::is_nil(consumer))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetTypedConsumerAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return consumer._retn();
}

static CosEventChannelAdmin::SupplierAdmin_ptr
GetSupplierAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetSupplierAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosEventChannelAdmin::SupplierAdmin_var supplier =
	CosEventChannelAdmin::SupplierAdmin::_narrow(obj);
    if(CORBA::is_nil(supplier))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetSupplierAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return supplier._retn();
}

static CosTypedEventChannelAdmin::TypedSupplierAdmin_ptr
GetTypedSupplierAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetTypedSupplierAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    CosTypedEventChannelAdmin::TypedSupplierAdmin_var supplier =
	CosTypedEventChannelAdmin::TypedSupplierAdmin::_narrow(obj);
    if(CORBA::is_nil(supplier))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Cannot narrow object in GetTypedSupplierAdmin");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    return supplier._retn();
}

static CosEventChannelAdmin::ProxyPullSupplier_ptr
GetProxyPullSupplier(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetProxyPullSupplier");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    return CosEventChannelAdmin::ProxyPullSupplier::_narrow(obj);
}

static CosEventChannelAdmin::ProxyPushSupplier_ptr
GetProxyPushSupplier(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetProxyPushSupplier");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    return CosEventChannelAdmin::ProxyPushSupplier::_narrow(obj);
}

static CosEventChannelAdmin::ProxyPullConsumer_ptr
GetProxyPullConsumer(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetProxyPullConsumer");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    return CosEventChannelAdmin::ProxyPullConsumer::_narrow(obj);
}

static CosEventChannelAdmin::ProxyPushConsumer_ptr
GetProxyPushConsumer(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetProxyPushConsumer");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    return CosEventChannelAdmin::ProxyPushConsumer::_narrow(obj);
}

static CosTypedEventChannelAdmin::TypedProxyPushConsumer_ptr
GetTypedProxyPushConsumer(const char* ior, CORBA::ORB_ptr orb)
{
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Nil object in GetTypedProxyPushConsumer");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    return CosTypedEventChannelAdmin::TypedProxyPushConsumer::_narrow(obj);
}

static void
Shutdown(CORBA::ORB_ptr orb)
{
    OBEventChannelFactory::EventChannelFactory_var factory = GetFactory(orb);
    try
    {
	factory -> shutdown();
	PrintStatusMsg(TraceLevel, "Admin", 1, "Shutdown complete!");
    }
    catch(const CORBA::TRANSIENT&)
    {
        //
        // This call will deactivate the server and thus
        // throw an exception.
        //
    }
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << "System exception in Admin Shutdown" << OB_ENDL;
	OB_CERR << ex << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
}
    
static void 
CreateChannel(CORBA::ORB_ptr orb)
{
    OBEventChannelFactory::EventChannelFactory_var factory = GetFactory(orb);
    try
    {
	CORBA::String_var id = CORBA::string_dup("");
	id += GetNextID();
 	CosEventChannelAdmin::EventChannel_var channel =	
	    factory -> create_channel(id);

	PrintStatusMsg(TraceLevel, "Admin", 1, "Created Channel");

	CORBA::String_var ior = orb -> object_to_string(channel);
	OB_COUT << ior << OB_ENDL;
    }
    catch(const OBEventChannelFactory::ChannelAlreadyExists&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, "Channel already exists");
    }
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << "System exception in Admin CreateChannel" << OB_ENDL;
	OB_CERR << ex << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
}	

static void
DestroyChannel(const char* ch_ior, CORBA::ORB_ptr orb)
{
    CosEventChannelAdmin::EventChannel_var channel = GetChannel(ch_ior, orb);
    channel -> destroy();
    PrintStatusMsg(TraceLevel, "Admin", 1, "Destroyed Channel");
}   

static void
CreateConsumerAdmin(const char* ch_ior, CORBA::ORB_ptr orb)
{
    CosEventChannelAdmin::EventChannel_var channel = GetChannel(ch_ior, orb);
    CosEventChannelAdmin::ConsumerAdmin_var consumer = 
	channel -> for_consumers();
    CORBA::String_var ior = orb -> object_to_string(consumer);
    OB_COUT << ior << OB_ENDL;
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Consumer Admin");
}

static void
CreateSupplierAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CosEventChannelAdmin::EventChannel_var channel = GetChannel(ior, orb);
    CosEventChannelAdmin::SupplierAdmin_var supplier = 
	channel -> for_suppliers();
    CORBA::String_var supplier_ior = orb -> object_to_string(supplier);
    OB_COUT << supplier_ior << OB_ENDL;
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Supplier Admin");
}

static void
CreateConsumer(const char* ior, CORBA::ORB_ptr orb, const char* model) 
{
    CosEventChannelAdmin::SupplierAdmin_var supplier = 
	GetSupplierAdmin(ior, orb);

    CORBA::String_var consumer_ior;
    if(strcasecmp(model,"CreatePushConsumer") == 0) 
    {
	CosEventChannelAdmin::ProxyPushConsumer_var consumer = 
	    supplier -> obtain_push_consumer();
	consumer_ior = orb -> object_to_string(consumer);
    }
    else
    {
	CosEventChannelAdmin::ProxyPullConsumer_var consumer = 
	    supplier -> obtain_pull_consumer();
	consumer_ior = orb -> object_to_string(consumer);
    }
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Proxy Consumer");
    OB_COUT << consumer_ior << OB_ENDL;
}

static void
CreateSupplier(const char* ior, CORBA::ORB_ptr orb, const char* model) 
{
    CosEventChannelAdmin::ConsumerAdmin_var consumer = 
	GetConsumerAdmin(ior, orb);

    CORBA::String_var supplier_ior;
    if(strcasecmp(model,"CreatePushSupplier") == 0) 
    {
	CosEventChannelAdmin::ProxyPushSupplier_var supplier = 
	    consumer -> obtain_push_supplier();
	supplier_ior = orb -> object_to_string(supplier);
    }
    else
    {
	CosEventChannelAdmin::ProxyPullSupplier_var supplier = 
	    consumer -> obtain_pull_supplier();
	supplier_ior = orb -> object_to_string(supplier);
    }
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Proxy Supplier");
    OB_COUT << supplier_ior << OB_ENDL;
}

static void
DestroyProxySupplier(const char* ior, CORBA::ORB_ptr orb) 
{
    //
    // Push Supplier
    //
    CosEventChannelAdmin::ProxyPushSupplier_var pushSupplier =
	GetProxyPushSupplier(ior, orb);
    if(!CORBA::is_nil(pushSupplier))
    {
	pushSupplier -> disconnect_push_supplier();
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Destroyed Proxy Push Supplier");
	return;
    }
    
    //
    // Pull Supplier
    //
    CosEventChannelAdmin::ProxyPullSupplier_var pullSupplier =
	GetProxyPullSupplier(ior, orb);
    if(!CORBA::is_nil(pullSupplier))
    {
	pullSupplier -> disconnect_pull_supplier();
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Destroyed Proxy Pull Supplier");
    }
}

static void
DestroyProxyConsumer(const char* ior, CORBA::ORB_ptr orb) 
{
   
    //
    // Push Consumer 
    //
    CosEventChannelAdmin::ProxyPushConsumer_var pushConsumer =
	GetProxyPushConsumer(ior, orb);
    if(!CORBA::is_nil(pushConsumer))
    {
	pushConsumer -> disconnect_push_consumer();
	PrintStatusMsg(TraceLevel, "Admin", 1, "Destroyed Proxy Consumer");
	return;
    }
    
    //
    // PullConsumer 
    //
    CosEventChannelAdmin::ProxyPullConsumer_var pullConsumer =
	GetProxyPullConsumer(ior, orb);
    if(!CORBA::is_nil(pullConsumer))
    {
	pullConsumer -> disconnect_pull_consumer();
	PrintStatusMsg(TraceLevel, "Admin", 1, "Destroyed Proxy Consumer");
    }
}

static void
TypedShutdown(CORBA::ORB_ptr orb)
{
    OBTypedEventChannelFactory::TypedEventChannelFactory_var factory = 
        GetTypedFactory(orb); 
    try
    {
	factory -> shutdown();
    }
    catch(const CORBA::TRANSIENT&)
    {
        //
        // This call will deactivate the server and thus
        // throw an exception.
        //
    }
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << "System exception in Admin TypedShutdown" << OB_ENDL;
	OB_CERR << ex << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
}
    
static void 
CreateTypedChannel(CORBA::ORB_ptr orb)
{

    try
    {
	CORBA::Object_var obj =
	    orb -> resolve_initial_references("TypedEventService");
	if(CORBA::is_nil(obj))
	{
	    PrintStatusMsg(TraceLevel, "Admin", 1, 
			   "No `TypedEventService' initial reference");
	    OB_COUT << "ERROR" << OB_ENDL;
	    orb -> destroy();
	    exit(1);
	}

	CosTypedEventChannelAdmin::TypedEventChannel_var e = 
	    CosTypedEventChannelAdmin::TypedEventChannel::_narrow(obj);

	if(CORBA::is_nil(e))
	{
	    PrintStatusMsg(TraceLevel, "Admin", 1, 
			   "`TypedEventService' narrows to nil");
	    OB_COUT << "ERROR" << OB_ENDL;
	    orb -> destroy();
	    exit(1);
	}

	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Created default event channel");
	CORBA::String_var ior = orb -> object_to_string(e);
	OB_COUT << ior << OB_ENDL;
    }
    catch(const CORBA::ORB::InvalidName&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Can't resolve `TypedEventService'");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << "System exception in CreateTypedChannel" << OB_ENDL;
	OB_CERR << ex << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
}	

static void
DestroyTypedChannel(const char* ch_ior, CORBA::ORB_ptr orb)
{
    CosTypedEventChannelAdmin::TypedEventChannel_var channel = 
	GetTypedChannel(ch_ior, orb);
    channel -> destroy();
    PrintStatusMsg(TraceLevel, "Admin", 1, "Destroyed Typed Channel");
}   

static void
CreateTypedConsumerAdmin(const char* ch_ior, CORBA::ORB_ptr orb)
{
    CosTypedEventChannelAdmin::TypedEventChannel_var channel = 
	GetTypedChannel(ch_ior, orb);
    CosTypedEventChannelAdmin::TypedConsumerAdmin_var consumer = 
	channel -> for_consumers();
    CORBA::String_var ior = orb -> object_to_string(consumer);
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Typed Consumer Admin");
    OB_COUT << ior << OB_ENDL;
}

static void
CreateTypedSupplierAdmin(const char* ior, CORBA::ORB_ptr orb)
{
    CosTypedEventChannelAdmin::TypedEventChannel_var channel = 
	GetTypedChannel(ior, orb);
    CosTypedEventChannelAdmin::TypedSupplierAdmin_var supplier = 
	channel -> for_suppliers();
    CORBA::String_var supplier_ior = orb -> object_to_string(supplier);
    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Typed Supplier Admin");
    OB_COUT << supplier_ior << OB_ENDL;
}

static void
CreateTypedConsumer(const char* ior, const char* ref, CORBA::ORB_ptr orb,
		    const char* type)
{
    CosTypedEventChannelAdmin::TypedSupplierAdmin_var supplier = 
	GetTypedSupplierAdmin(ior, orb);

    CORBA::String_var consumer_ior;
    try
    {
	if(strcmp(type, "CreateTypedPushConsumer") == 0)
	{
	    CosTypedEventChannelAdmin::TypedProxyPushConsumer_var consumer = 
		supplier -> obtain_typed_push_consumer(ref);
	    consumer_ior = orb -> object_to_string(consumer);
	}
	else
	{
	    CosEventChannelAdmin::ProxyPullConsumer_var consumer = 
		supplier -> obtain_typed_pull_consumer(ref);
	    consumer_ior = orb -> object_to_string(consumer);
	}
    }
    catch(const CosTypedEventChannelAdmin::InterfaceNotSupported&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "InterfaceNotSupported in CreateTypedConsumer");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    catch(const CosTypedEventChannelAdmin::NoSuchImplementation&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "NoSuchImplementation in CreateTypedConsumer");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Typed Proxy Consumer");
    OB_COUT << consumer_ior << OB_ENDL;
}

static void
CreateTypedSupplier(const char* ior, const char* ref, CORBA::ORB_ptr orb)
{
    CosTypedEventChannelAdmin::TypedConsumerAdmin_var consumer = 
	GetTypedConsumerAdmin(ior, orb);

    CORBA::String_var supplier_ior;
    try
    {
	CosEventChannelAdmin::ProxyPushSupplier_var supplier = 
	    consumer -> obtain_typed_push_supplier(ref);
	supplier_ior = orb -> object_to_string(supplier);
    }
    catch(const CosTypedEventChannelAdmin::InterfaceNotSupported&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "InterfaceNotSupported in CreateTypedSupplier");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }
    catch(const CosTypedEventChannelAdmin::NoSuchImplementation&)
    {
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "NoSuchImplementation in CreateTypedSupplier");
	OB_COUT << "ERROR" << OB_ENDL;
	orb -> destroy();
	exit(1);
    }

    PrintStatusMsg(TraceLevel, "Admin", 1, "Created Typed Proxy Supplier");
    OB_COUT << supplier_ior << OB_ENDL;
}

static void
DestroyTypedProxyConsumer(const char* ior, CORBA::ORB_ptr orb) 
{
    CosTypedEventChannelAdmin::TypedProxyPushConsumer_var pushConsumer =
	GetTypedProxyPushConsumer(ior, orb);

    if(!CORBA::is_nil(pushConsumer))
    {
	pushConsumer -> disconnect_push_consumer();
	PrintStatusMsg(TraceLevel, "Admin", 1, 
		       "Destroyed Typed Proxy Consumer");
    }
}


int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    if(argc < 2)
    {
	Usage();
	return EXIT_FAILURE;
    }
    
    //
    // See what the command is and evaluate the proper function.
    //
    CORBA::String_var msg = CORBA::string_dup("Received instruction: ");
    msg += argv[1];
    PrintStatusMsg(TraceLevel, "Admin", 3, msg);

    if(strcasecmp(argv[1], "Shutdown") == 0)
    {
	if(argc != 2)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	Shutdown(orb);
    }
    else if(strcasecmp(argv[1], "CreateChannel") == 0)
    {
	if(argc != 2)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateChannel(orb);
    }
    else if(strcasecmp(argv[1], "DestroyChannel") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	DestroyChannel(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "CreateConsumerAdmin") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateConsumerAdmin(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "CreateSupplierAdmin") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateSupplierAdmin(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "CreatePullConsumer") == 0 ||
       strcasecmp(argv[1], "CreatePushConsumer") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateConsumer(argv[2], orb, argv[1]);
    }
    else if(strcasecmp(argv[1], "CreatePushSupplier") == 0 ||
       strcasecmp(argv[1], "CreatePullSupplier") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateSupplier(argv[2], orb, argv[1]);
    }
    else if((strcasecmp(argv[1], "DestroyProxySupplier") == 0) ||
       (strcasecmp(argv[1], "DestroyTypedProxySupplier") == 0))
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	DestroyProxySupplier(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "DestroyProxyConsumer") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	DestroyProxyConsumer(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "TypedShutdown") == 0)
    {
	if(argc != 2)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	TypedShutdown(orb);
    }
    else if(strcasecmp(argv[1], "CreateTypedChannel") == 0)
    {
	if(argc != 2)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateTypedChannel(orb);
    }
    else if(strcasecmp(argv[1], "DestroyTypedChannel") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	DestroyTypedChannel(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "CreateTypedConsumerAdmin") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateTypedConsumerAdmin(argv[2], orb);
    }
    else if(strcasecmp(argv[1], "CreateTypedSupplierAdmin") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateTypedSupplierAdmin(argv[2], orb);
    }
    else if((strcasecmp(argv[1], "CreateTypedPushConsumer") == 0) ||
       (strcasecmp(argv[1], "CreateTypedPullConsumer") == 0))
    {
	if(argc != 4)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateTypedConsumer(argv[2], argv[3], orb, argv[1]);
    }
    else if((strcasecmp(argv[1], "CreateTypedPullSupplier") == 0) ||
       (strcasecmp(argv[1], "CreateTypedPushSupplier") == 0))
    {
	if(argc != 4)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	CreateTypedSupplier(argv[2], argv[3], orb);
    }
    else if(strcasecmp(argv[1], "DestroyTypedProxyConsumer") == 0)
    {
	if(argc != 3)
	{
	    Usage();
	    return EXIT_FAILURE;
	}
	DestroyTypedProxyConsumer(argv[2], orb);
    }
    else
    {
	Usage();
    }
 
    return EXIT_SUCCESS;
}

int
main(int argc, char** argv)
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    { 
        OB::Properties_var properties = OB::Properties::getDefaultProperties();
        OB::ParseArgs(argc, argv, properties);
        OB::Properties::KeySeq keys = 
	    properties -> getKeys("ooc.testsuite.event.");

        for(CORBA::ULong i = 0 ; i < keys.length() ; ++i)
        {
            const char* key = keys[i];
            if(strcmp(key, "ooc.testsuite.event.trace") == 0)
            {
                TraceLevel = atoi(properties -> getProperty(key));
		break;
	    }
	    else 
	    {
		OB_CERR << "Warning: Unknown property - " << key << OB_ENDL;
	    }
        }
	
	//
        // Create the ORB.
        //
        orb = OBCORBA::ORB_init(argc, argv, properties);
	status = run(orb, argc, argv);
    } 
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << ex << OB_ENDL;
        OB_COUT << "ERROR" << OB_ENDL;
	status = EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }
    return status;
}


