// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CoreTraceLevels.h>

#if !defined(WIN32)
#    include <unistd.h>
#endif

#include <TestSuiteUtilities.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#include <stdlib.h>
#include <ctype.h>

void
PrintStatusMsg(int current, const char* source, int level, const char *msg)
{
    CORBA::String_var txt = CORBA::string_dup(source);
    txt += ": ";
    txt += msg;
    if(current >= level)
        OB_ERROR(txt);
}

void
PrintAny(OB_STD(ostream&) out, const CORBA::Any& any)
{
    CORBA::TypeCode_var type = any.type();

    switch(type -> kind())
    {
    case CORBA::tk_short:
    {
	CORBA::Short value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_long:
    {
	CORBA::Long value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_ushort:
    {
	CORBA::UShort value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_ulong:
    {
	CORBA::ULong value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_float:
    {
	CORBA::Float value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_double:
    {
	CORBA::Double value;
	if(any >>= value)
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_boolean:
    {
	CORBA::Boolean value;
	if(any >>= CORBA::Any::to_boolean(value))
	{
	    if(value)
		out << "true";
	    else
		out << "false";
	    return;
	}
	break;
    }
    case CORBA::tk_char:
    {
	CORBA::Boolean value;
	if(any >>= CORBA::Any::to_boolean(value))
	{
	    out << value;
	    return;
	}
	break;
    }
    case CORBA::tk_octet:
    {
	CORBA::Octet value;
	if(any >>= CORBA::Any::to_octet(value))
	{
	    out << value;
	    return;
	}
	break;
    }
    default:	
        out << "Undecipherable type";
    }
    return;
}
