// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef NAMING_DATABASE_H
#define NAMING_DATABASE_H

namespace CosNaming
{

//
// This class reads or writes the contents of a NamingContext to or from
// a persistent store.
//
class NamingDatabase : public JTCRecursiveMutex
{
    PortableServer::POA_var userPOA_; // The USER_ID POA.
    PortableServer::POA_var systemPOA_; // The SYSTEM_ID POA.
    CORBA::ORB_var orb_; // The ORB.

    CORBA::String_var host_; /// The host.
    CORBA::UShort port_; // The port.

    CORBA::String_var logFile_; // The name of the logfile.
    CORBA::String_var logFileNew_; // The name of the new logfile.
    CORBA::String_var logFileBack_; // The name of the backup logfile.

    NamingContextSet* ncs_; // The current set of naming contexts.
    FILE* os_; // A FILE* to the database.

    CompactorHandle compactor_; // The database compactor thread

    const CORBA::PolicyList& timeoutPolicy_;  // Timeout for callbacks

    bool noUpdates_; // No automatic updates.
    bool initializing_; // Are we restoring the naming contexts?
    bool update_; // Do we need an journal update?
    bool noflush_; // Do we want to flush on each update?

    IOP::Codec_var cdrCodec_; // The CDR Codec

    OB::Logger_var logger_; // Tracing information
    CORBA::ULong maxBindings_;
    CORBA::ULong traceLevel_;

    bool usingIMR_; // Are we using IMR

    void dumpContext(const PortableServer::ObjectId&,
    		     CosNaming::NamingContext_ptr);

    void writeAny(const CORBA::Any&);
    
    bool doRename(const char*, const char*);
    bool exists(const char*);
    void openLog(bool);
    PortableServer::ObjectId* key_to_ObjectId(const ContextKey& key);
    PortableServer::Servant key_to_servant(const ContextKey& key);

public:

    NamingDatabase(PortableServer::POA_ptr, PortableServer::POA_ptr,
    		   CORBA::ORB_ptr, const char*, NamingContextSet*,
		   const CORBA::PolicyList&, bool, bool&, long,
		   CORBA::ULong, CORBA::ULong, bool = false);
    ~NamingDatabase();
    
    //
    // Dump the full journal
    //
    void updateJournal();

    //
    // Create journal entries
    //
    void create(const PortableServer::ObjectId&, long);
    void destroy(const PortableServer::ObjectId&);
    void bind(const PortableServer::ObjectId&, const CosNaming::Name&,
              const CosNaming::OBNamingContext::BindingUnion&, long, bool);
    void unbind(const PortableServer::ObjectId&, const CosNaming::Name&);
    void info();
};

} // End of namespace CosNaming

#endif
