// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#ifdef HAVE_IOSTREAM
#   include <iostream>
#else
#   include <iostream.h>
#endif

void TestBind(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestRebind(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestBindContext(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestRebindContext(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestResolve(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestUnbind(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestDestroy(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestList(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestListExtended(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestRename(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestCallbacks(CORBA::ORB_ptr, CosNaming::NamingContext_ptr);
void TestToString(CORBA::ORB_ptr, CosNaming::NamingContextExt_ptr);
void TestToName(CORBA::ORB_ptr, CosNaming::NamingContextExt_ptr);
void TestToURL(CORBA::ORB_ptr, CosNaming::NamingContextExt_ptr);
void TestResolveStr(CORBA::ORB_ptr, CosNaming::NamingContextExt_ptr);

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Resolve the Root POA
    //
    CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var poaManager = root -> the_POAManager();

    //
    // Initialize the server so that incoming requests are
    // handled from this point on
    //
    poaManager -> activate();

    //
    // Get naming service
    //
    try
    {
        obj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        OB_ERROR(argv[0] << ": can't resolve `NameService'");
        return EXIT_FAILURE;
    }

    if(CORBA::is_nil(obj.in()))
    {
        OB_ERROR(argv[0] << ": `NameService' is a nil object reference");
        return EXIT_FAILURE;
    }

    CosNaming::NamingContextExt_var nc =
        CosNaming::NamingContextExt::_narrow(obj.in());

    if(CORBA::is_nil(nc.in()))
    {
        OB_ERROR(argv[0]
		 << ": `NameService' is not a NamingContextExt object "
		 << "reference");
        return EXIT_FAILURE;
    }

    CosNaming::NamingContext_var ctx, tmp;
    CosNaming::Name name;

    if(argc > 1)
    {
        OB_INFO_LN("using: " << argv[1] << " as root context.");
        try
        {
            CosNaming::Name_var name = nc -> to_name(argv[1]);
            obj = nc -> resolve(name);
        }
        catch(const CORBA::UserException&)
        {
            OB_ERROR("Error resolving context.");
            return EXIT_FAILURE;
        }
        ctx = CosNaming::NamingContextExt::_narrow(obj);
        assert(!CORBA::is_nil(ctx));
    }
    else
    {
        //
        // Perform tests in a temporary context
        //
        name.length(1);
        name[0].id = CORBA::string_dup("__OB_nsTest");
        name[0].kind = CORBA::string_dup("__OB_nsTest");
        try
        {
            //
            // Unbind it if it already exists, and attempt to destroy it
            //
            obj = nc -> resolve(name);
            tmp = CosNaming::NamingContextExt::_narrow(obj.in());
            assert(!CORBA::is_nil(tmp.in()));
            nc -> unbind(name);
            tmp -> destroy();
        }
        catch(const CosNaming::NamingContext::NotFound&)
        {
            // Ignore
        }
        catch(const CosNaming::NamingContext::NotEmpty&)
        {
            // Ignore
        }

        tmp = nc -> bind_new_context(name);
        ctx = CosNaming::NamingContext::_duplicate(tmp);
    }

    try
    {
        OB_INFO("Testing bind()... ");
        TestBind(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing rebind()... ");
        TestRebind(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing bind_context()... ");
        TestBindContext(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing rebind_context()... ");
        TestRebindContext(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing resolve()... ");
        TestResolve(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing unbind()... ");
        TestUnbind(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing destroy()... ");
        TestDestroy(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing list()... ");
        TestList(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing list_extended()... ");
        TestListExtended(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing rename()... ");
        TestRename(orb, tmp.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing callbacks... ");
        TestCallbacks(orb, tmp.in());
        OB_INFO_LN("Done!");

        CosNaming::NamingContextExt_var ext =
	    CosNaming::NamingContextExt::_narrow(tmp.in());

        OB_INFO("Testing to_string()... ");
        TestToString(orb, ext.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing to_name()... ");
        TestToName(orb, ext.in());
        OB_INFO_LN("Done!");

        OB_INFO("Testing to_url()... ");
        TestToURL(orb, ext.in());
        OB_INFO_LN("Done!");

	OB_INFO("Testing resolve_str()... ");
        TestResolveStr(orb, ext.in());
        OB_INFO_LN("Done!");

        if(!CORBA::is_nil(tmp))
        {
            //
            // Each test should clean up after itself, so we should be
            // able to destroy the temporary context without a NotEmpty
            // exception being raised
            //
            nc -> unbind(name);
            tmp -> destroy();
        }
    }
    catch(const CORBA::UserException&)
    {
        OB_ERROR(argv[0] << ": Unexpected user exception was thrown");
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
