// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <string.h>
#include "TestListener_impl.h"

TestListener_impl::TestListener_impl() :
    op_(TestListener_impl::OP_NONE), ctx_(CosNaming::NamingContext::_nil()),
    obj_(CORBA::Object::_nil()), valid_(false)
{
}

void
TestListener_impl::expect(TestListener_impl::Operation op,
                          CosNaming::NamingContext_ptr ctx,
			  CORBA::Object_ptr obj,
                          const CosNaming::NameComponent& name)

{
    op_ = op;
    ctx_ = CosNaming::NamingContext::_duplicate(ctx);
    obj_ = CORBA::Object::_duplicate(obj);
    name_ = name;
}

bool
TestListener_impl::isValid()
{
    if(valid_)
    {
	reset();
	return true;
    }
    else
	return false;
}

//
// CosNaming::BindingListener operations
//

void
TestListener_impl::context_added(CosNaming::NamingContext_ptr parent,
                                 CosNaming::NamingContext_ptr child,
                                 const CosNaming::NameComponent& name)
    throw(CORBA::SystemException)
{
    validate(CONTEXT_ADDED, parent, child, name);
}

void
TestListener_impl::context_removed(CosNaming::NamingContext_ptr parent,
                                   const CosNaming::NameComponent& name)
    throw(CORBA::SystemException)
{
    validate(CONTEXT_REMOVED, parent, CORBA::Object::_nil(), name);
}

void
TestListener_impl::context_changed(CosNaming::NamingContext_ptr parent,
                                   CosNaming::NamingContext_ptr child,
                                   const CosNaming::NameComponent&
                                   /* old_name */,
				   const CosNaming::NameComponent& new_name)
    throw(CORBA::SystemException)
{
    validate(CONTEXT_CHANGED, parent, child, new_name);
}

void
TestListener_impl::object_added(CosNaming::NamingContext_ptr parent,
				CORBA::Object_ptr child,
                                const CosNaming::NameComponent& name)
    throw(CORBA::SystemException)
{
    validate(OBJECT_ADDED, parent, child, name);
}

void
TestListener_impl::object_removed(CosNaming::NamingContext_ptr parent,
                                  const CosNaming::NameComponent& name)
    throw(CORBA::SystemException)
{
    validate(OBJECT_REMOVED, parent, CORBA::Object::_nil(), name);
}

void
TestListener_impl::object_changed(CosNaming::NamingContext_ptr parent,
                                  CORBA::Object_ptr child,
                                  const CosNaming::NameComponent&
                                  /* old_name */,
				  const CosNaming::NameComponent& new_name)
    throw(CORBA::SystemException)
{
    validate(OBJECT_CHANGED, parent, child, new_name);
}

//
// Private functions
//

void
TestListener_impl::validate(TestListener_impl::Operation op,
                            CosNaming::NamingContext_ptr /* ctx */,
			    CORBA::Object_ptr obj,
                            const CosNaming::NameComponent& name)
{
    assert(op_ == op);
    // The following assert is wrong, as in some cases ctx_ and ctx are
    // equivalent even though _is_equivalent() returns false.
    // assert(ctx_ -> _is_equivalent(ctx));
    if(!CORBA::is_nil(obj_))
	assert(obj_ -> _is_equivalent(obj));
    assert(equal(name_, name));
    valid_ = true;
}

bool
TestListener_impl::equal(const CosNaming::NameComponent& n1,
                         const CosNaming::NameComponent& n2)
{
    return(strcmp(n1.id, n2.id) == 0 && strcmp(n1.kind, n2.kind) == 0);
}

void
TestListener_impl::reset()
{
    op_ = OP_NONE;
    valid_ = false;
}
