// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __TestListener_impl_h__
#define __TestListener_impl_h__

#include <OB/CORBA.h>
#include <OB/OBNaming_skel.h>

class TestListener_impl : virtual public POA_CosNaming::BindingListener
{
public:
    enum Operation
    {
	OP_NONE,
        CONTEXT_ADDED,
        CONTEXT_REMOVED,
        CONTEXT_CHANGED,
        OBJECT_ADDED,
        OBJECT_REMOVED,
        OBJECT_CHANGED
    };

    TestListener_impl();

    void expect(Operation op, CosNaming::NamingContext_ptr ctx,
                CORBA::Object_ptr obj, const CosNaming::NameComponent& name);

    bool isValid();

    //
    // CosNaming::BindingListener operations
    //

    virtual void context_added(CosNaming::NamingContext_ptr parent,
                               CosNaming::NamingContext_ptr child,
			       const CosNaming::NameComponent& name)
	throw(CORBA::SystemException);

    virtual void context_removed(CosNaming::NamingContext_ptr parent,
				 const CosNaming::NameComponent& name)
	throw(CORBA::SystemException);

    virtual void context_changed(CosNaming::NamingContext_ptr parent,
                                 CosNaming::NamingContext_ptr child,
                                 const CosNaming::NameComponent& old_name,
				 const CosNaming::NameComponent& new_name)
	throw(CORBA::SystemException);

    virtual void object_added(CosNaming::NamingContext_ptr parent,
                              CORBA::Object_ptr child,
                              const CosNaming::NameComponent& name)
	throw(CORBA::SystemException);

    virtual void object_removed(CosNaming::NamingContext_ptr parent,
                                const CosNaming::NameComponent& name)
	throw(CORBA::SystemException);

    virtual void object_changed(CosNaming::NamingContext_ptr parent,
                                CORBA::Object_ptr child,
                                const CosNaming::NameComponent& old_name,
				const CosNaming::NameComponent& new_name)
	throw(CORBA::SystemException);

private:
    void validate(Operation op,
                  CosNaming::NamingContext_ptr ctx,
		  CORBA::Object_ptr obj,
                  const CosNaming::NameComponent& name);

    bool equal(const CosNaming::NameComponent& n1,
               const CosNaming::NameComponent& n2);

    void reset();

    Operation op_;
    CosNaming::NamingContext_var ctx_;
    CORBA::Object_ptr obj_;
    CosNaming::NameComponent name_;
    bool valid_;
};

#endif
