// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/NTService.h>
#include <OB/Logger.h>

#include <Hello_impl.h>
#include <HelloWorldService.h>

#include <HelloMsg.h>

#ifdef HAVE_FSTREAM
#include <fstream>
#else
#include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace OB;

// ----------------------------------------------------------------------
// HelloWorldService constructor
// ----------------------------------------------------------------------

HelloWorldService::HelloWorldService()
    : NTService("HelloWorld", "Hello World Service", HELLO_MSG),
      stopped_(false)
{
}

// ----------------------------------------------------------------------
// HelloWorldService public member implementation
// ----------------------------------------------------------------------

void
HelloWorldService::orb_run(Logger_ptr logger)
{
    CORBA::Object_var poaObj = 
        orb_ -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPoa = PortableServer::POA::_narrow(poaObj);
    PortableServer::POAManager_var manager = rootPoa -> the_POAManager();

    //
    // Create implementation object
    //
    Hello_impl* helloImpl = new Hello_impl(orb_, rootPoa);
    PortableServer::ServantBase_var servant = helloImpl;
    Hello_var hello = helloImpl -> _this();

    CORBA::String_var ior = orb_ -> object_to_string(hello);

    const char* refFile = "c:\\Hello.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        CORBA::String_var msg = CORBA::string_dup("HelloWorld: can't open '");
        msg += refFile;
        msg += "': ";
        msg += strerror(errno);
        logger -> error(msg);
        return;
    }

    out << ior << endl;
    out.close();

    //
    // Inform the status manager that the service is running.
    //
    statusUpdate(SERVICE_RUNNING);
        
    //
    // Run implementation
    //
    if(!stopped_)
    {
        manager -> activate();
        orb_ -> run();
    }
}

void
HelloWorldService::start(int argc, char** argv, Logger_ptr logger)
{
    try
    {
        OB::Properties_var properties = OB::Properties::getDefaultProperties();
	OB::ParseArgs(argc, argv, properties);

	properties -> setProperty("ooc.iiop.port", "12345");
	properties -> setProperty("ooc.orb.conc_model", "threaded");
	properties -> setProperty("ooc.orb.oa.conc_model", "threaded");
	properties -> setProperty("ooc.orb.id", "HelloWorldService");

	orb_ = OBCORBA::ORB_init(argc, argv, OB::Properties::_nil(), logger);
	orb_run(logger);
    }
    catch(const CORBA::Exception& ex)
    {
        CORBA::String_var err = ex._to_string();
	logger -> error(err);
    }

    if(!CORBA::is_nil(orb_))
    {
	try
	{
	    orb_ -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
            CORBA::String_var err = ex._to_string();
	    logger -> error(err);
	}
    }
}

void
HelloWorldService::stop()
{
    //
    // The service manager calls this in a seperate thread. So first
    // we have inform JTC to adopt this thread.
    //
    JTCAdoptCurrentThread adopt;
    stopped_ = true;
    if(!CORBA::is_nil(orb_))
	orb_ -> shutdown(false);
}

