// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef _TIMER_H_
#define _TIMER_H_


//
// High performance timer class. Uses the NT built-in high performance timer
// The resolution on this is sub-microsecond, but varies by machine.
//

#ifdef WIN32

class Timer 
{
public:

    Timer ()
    {
	QueryPerformanceFrequency(&frequency);
    }

    ~Timer () {}

    void start () 
    {
        QueryPerformanceCounter(&start_time);
    }

    void stop ()
    {
        QueryPerformanceCounter(&stop_time);
    }

    double timeElapsed () 
    {
        return ( ( static_cast<long double>(stop_time.QuadPart - 
				            start_time.QuadPart) ) /
                 ( static_cast<long double>(frequency.QuadPart) ) );
    }

private:
    LARGE_INTEGER start_time, stop_time;
	LARGE_INTEGER frequency;
};

#else

class Timer 
{
public:

    Timer ()
    {
    }

    ~Timer () {}

    void start () 
    {
        gettimeofday(&start_, 0);
    }

    void stop ()
    {
        gettimeofday(&stop_, 0);
    }

    double timeElapsed () 
    {
        timeval tv;
	tv.tv_sec = stop_.tv_sec - start_.tv_sec;
	tv.tv_usec = stop_.tv_usec - start_.tv_usec;

	tv.tv_sec += tv.tv_usec / 1000000;
	tv.tv_usec = tv.tv_usec % 1000000;

	if(tv.tv_usec < 0)
	{
	    tv.tv_usec += 1000000;
            tv.tv_sec -= 1;
	}

	return (double)tv.tv_sec + (double)tv.tv_usec / 1000000.0;
    }

private:

    timeval start_;
    timeval stop_;
};

#endif

#endif /*!_TIMER_H_*/
