#include <OB/CORBA.h>

#if defined(HAVE_STD_IOSTREAM)
using namespace std;
#endif

#include <time.h>
#include "server.hh"

//-----------------------------------------------------------------------------

TimeOfDay
Time_impl::
get_gmt() throw(CORBA::SystemException)
{
    time_t time_now = time(0);
    struct tm * time_p = gmtime(&time_now);

    TimeOfDay tod;
    tod.hour = time_p->tm_hour;
    tod.minute = time_p->tm_min;
    tod.second = time_p->tm_sec;

    return tod;
}

//-----------------------------------------------------------------------------

int
run(CORBA::ORB_ptr orb, int /* argc */, char*[])
{
    // Get reference to Root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

    // Activate POA manager
    PortableServer::POAManager_var mgr = poa->the_POAManager();
    mgr->activate();

    // Create an object
    Time_impl time_servant;

    // Write its stringified reference to stdout
    Time_var tm = time_servant._this();
    CORBA::String_var str = orb->object_to_string(tm);
    cout << str << endl;

    // Accept requests
    orb->run();

    return 0;
}

int
main(int argc, char* argv[], char*[])
{
    int status = 0;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << "Uncaught CORBA exception" << endl;
	status = 1;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = 1;
	}
    }
    
    return status;
}
