// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Timer.h>

#include <Chain_impl.h>

#include <stdlib.h>

#ifdef HAVE_IOMANIP
#    include <iomanip>
#else
#    include <iomanip.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

// ----------------------------------------------------------------------
// Link_impl constructor and desctructor
// ----------------------------------------------------------------------

Link_impl::Link_impl(
    CORBA::ORB_ptr orb,
    PortableServer::POA_ptr poa,
    const char* file
)
    : orb_(CORBA::ORB::_duplicate(orb)),
      poa_(PortableServer::POA::_duplicate(poa)),
      file_(file),
      master_(false),
      nextId_(0)
{
}

PortableServer::POA_ptr
Link_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

// ----------------------------------------------------------------------
// Link_impl public members
// ----------------------------------------------------------------------

void
Link_impl::forward(const Link::Message& msg)
    throw(CORBA::SystemException)
{
    cout << "Received message " << msg.id << endl;

    //
    // If this isn't the master, then forward the message.
    //
    if(!master_)
    {
	try
	{
	    cout << "Forwarding to next link." << endl;
	    next_ -> forward(msg);
	}
	catch(const CORBA::COMM_FAILURE&)
	{
	    cout << "COMM_FAILURE" << endl;
	}
    }
    else
    {
        timeval tp = OB::TimeSub(OB::TimeNow(), startTime_);
        cout << tp.tv_sec << "." << setw(6) << setfill('0')
             << tp.tv_usec << " sec roundtrip" << endl;
    }
}

void
Link_impl::send(CORBA::Long nPings, CORBA::Long msgSize)
    throw(CORBA::SystemException)
{
    cout << "start: initializing message." << endl;
    Link::Message msg;
    msg.data.length(msgSize);
    memset(msg.data.get_buffer(), 0, msgSize);
    cout << "start: done initialization." << endl;

    master_ = true;
    CORBA::Long i;
    try
    {
	for(i = 0; i < nPings; ++i)
	{
	    //
	    // Set the message ID
	    //
	    msg.id = nextId_++;

	    //
	    // Calculate the start time
	    //
            startTime_ = OB::TimeNow();

	    //
	    // Forward to the next link in the chain
	    //
	    next_ -> forward(msg);
	}
    }
    catch(const CORBA::COMM_FAILURE&)
    {
	cout << "COMM_FAILURE::start on forward " << i << endl;
    }
    master_ = false;
}

void
Link_impl::close_loop()
    throw(CORBA::SystemException)
{
    //
    // we're done if we already have the next element in the chain
    //
    if(!CORBA::is_nil(next_))
	return;

    CORBA::String_var ior = CORBA::string_dup("relfile:/");
    ior += file_;
    CORBA::Object_var obj = orb_ -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
	cerr << "Cannot read IOR from " << file_ << endl;
	exit(EXIT_FAILURE);
    }
    
    next_ = Link::_narrow(obj);
    assert(!CORBA::is_nil(next_));

    next_ -> close_loop();
}
