// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <Chain_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif


int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Create POA
    //
    CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
    PortableServer::POAManager_var manager = poa -> the_POAManager();

    const char* myRef = argv[1];
    const char* next = argv[2];

    //
    // Create implementation object
    //
    Link_impl* impl = new Link_impl(orb, poa, next);
    PortableServer::ServantBase_var servant = impl;
    Link_var r = impl -> _this();

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(r);
	
    ofstream out(myRef);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << myRef << "': "
    	     << strerror(errno) << endl;
        return EXIT_FAILURE;
    }
	
    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    if(argc < 3)
    {
        cerr << "\n Usage: " << argv[0] << ": mine next" << endl;
        return EXIT_FAILURE;
    }

    try
    {
    	OB::Properties_var properties = OB::Properties::getDefaultProperties();
	OB::ParseArgs(argc, argv, properties);

        //
        // Check to see if the concurrency model is usable for nested
        // method invocations. Either reactive/reactive or threaded/
        // thread_per_request/thread_pool is needed.
        //
        const char* orbModel =
	    properties -> getProperty("ooc.orb.conc_model");
        const char* oaModel =
	    properties -> getProperty("ooc.orb.oa.conc_model");
        bool valid;
        if(oaModel != 0 && orbModel != 0)
        {
            valid = ((strcmp(orbModel, "reactive") == 0 &&
                     strcmp(oaModel, "reactive") == 0) ||
                     (strcmp(orbModel, "threaded") == 0 &&
                     (strcmp(oaModel, "thread_per_request") == 0 ||
                      strcmp(oaModel, "thread_pool") == 0)));
        }
        else
            valid = false;

	//
	// Set the concurrency model to reactive/reactive if a valid
        // concurrency model wasn't selected
	//
        if(!valid)
        {
	    properties -> setProperty("ooc.orb.conc_model", "reactive");
	    properties -> setProperty("ooc.orb.oa.conc_model", "reactive");
        }

	//
	// Use own link name as ORB ID
	//
	orb = CORBA::ORB_init(argc, argv, argv[1]);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
