// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef BROADCASTERADMIN_IMPL_H
#define BROADCASTERADMIN_IMPL_H

#include <Broadcaster_skel.h>
#include <list>

namespace chat 
{

class BroadcasterAdmin_impl : public POA_chat::BroadcasterAdmin,
			      public PortableServer::RefCountServantBase
{
    CORBA::ORB_var          orb_;
    PortableServer::POA_var adminPoa_;
    PortableServer::POA_var broadcasterPoa_;
    
    //
    // Definitions for Receiver administration
    //
    struct ReceiverListItem
    {
	ReceiverDesc receiverDesc;
	CORBA::ULong broadcasterId;
    };

    typedef OB_STL::list<ReceiverListItem> ReceiverList;

    //
    // Internal administrative list with Receiver information
    //
    ReceiverList receiverList_;

    //
    // Internal Broadcaster identifier value
    //
    CORBA::ULong nextBroadcasterId_;

    //
    // Remove receiver from internal list
    //
    void remove_receiver(CORBA::ULong);

public:

    //
    // Internal operations - also used by Broadcaster_impl servants
    //
    BroadcasterAdmin_impl(
	CORBA::ORB_ptr orb,
	PortableServer::POA_ptr adminPoa,
	PortableServer::POA_ptr broadcasterPoa
    );

    PortableServer::POA_ptr _default_POA();
 
    void unregister(CORBA::ULong);

    void setNickName(CORBA::ULong, const char*)
        throw(NickExists);

    void say(const char*)
        throw(CORBA::SystemException);

    virtual void getReceiverByNick(const char*, ReceiverDesc_out)
        throw(UnknownNick);

    virtual StringSeq* getReceiverNames();

    //
    // Operations from the .idl file
    //
    virtual Broadcaster_ptr subscribe(const ReceiverDesc&)
	throw(NickExists, CORBA::SystemException);

    virtual Broadcaster_ptr subscribeWithCheck(const ReceiverDesc&)
        throw(ReceiverUnreachable, NickExists, CORBA::SystemException);

}; // class BroadcasterAdmin_impl

} // namespace chat

#endif
