// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Reactor.h>

#include <Receiver_impl.h>

#include <ctype.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace chat; 

// 
// Receiver constructor and desctructor
//

Receiver_impl::Receiver_impl(CORBA::ORB_ptr orb, 
                             PortableServer::POA_ptr poa,
			     BroadcasterAdmin_ptr server) : 
    ReceiverBase_impl(orb, poa, server), 
    input_(CORBA::string_dup(""))
{
    //
    // Register with the server
    //
    subscribe();

    //
    // Create event handler
    //
    handler_ = new EventHandler(this);
    
    //
    // Register with the reactor
    //
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb_);
    OB::Reactor_var reactor = oborb -> reactor();
    reactor -> register_handler(handler_, OB::EventRead, 0, 0);
}

Receiver_impl::~Receiver_impl()
{
    //
    // Unregister with the reactor
    //
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb_);
    OB::Reactor_var reactor = oborb -> reactor();
    reactor -> unregister_handler(handler_);
    
    //
    // Destroy our event handler
    //
    dynamic_cast<EventHandler*>(handler_.in()) -> destroy();    
}

//
// Print message
//

void
Receiver_impl::print(const char* text)
    throw(CORBA::SystemException)
{
    //
    // Remove active input line temporarily
    //
    for(CORBA::ULong i = 0 ; i < strlen(input_) ; i++)
    {
	cout << "\b \b";
    }
    
    //
    // Print the text
    //
    ReceiverBase_impl::print(text);

    //
    // Restore active input line
    //
    cout << input_ << flush;
}

//
// ORBacus-specific event handling
//
 
OB::Mask
Receiver_impl::handleEvent(OB::Mask mask)
{
    assert(mask == OB::EventRead);

    char c;
    cin.read(&c, 1);

    switch(c)
    {
    case '\b':
    {
    	if(strlen(input_) > 0)
	{
	    input_[strlen(input_) - 1] = '\0';
    	    cout << "\b \b" << flush;
	}
    	break;
    }
    
    case '\n':
    {
	cout << c << flush;
	CORBA::String_var s = input_;
	input_[0] = '\0';
	parse(s);
	break;
    }
    
    default:
    {
	if(isprint(c))
	{
	    cout << c << flush;
	    input_ += c;
	}
	break;
    }
    } // switch

    return OB::EventRead;
}
