// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OCI.h>
#include <OB/OCI_IIOP.h>
#include <OB/BootManager.h>

#include <BroadcasterAdmin_impl.h>

#include <stdlib.h>
#include <time.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif


int
run(CORBA::ORB_ptr orb, int /* argc */, char*[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj =
        orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPoa = PortableServer::POA::_narrow(poaObj);
 
    //
    // Get POA Manager of the RootPOA
    //
    PortableServer::POAManager_var manager = rootPoa -> the_POAManager();

    //
    // Create persistent POA
    //
    CORBA::PolicyList policies;
    policies.length(2);
    policies[0] =
        rootPoa -> create_id_assignment_policy(PortableServer::USER_ID);
    policies[1] =
        rootPoa -> create_lifespan_policy(PortableServer::PERSISTENT);

    PortableServer::POA_var adminPoa;
    try
    {
        adminPoa = rootPoa -> create_POA("chatPOA", manager, policies);
    }
    catch(const PortableServer::POA::AdapterAlreadyExists&)
    {
    }
    assert(!CORBA::is_nil(adminPoa));

    //
    // Resolve the Boot Manager - necessary for connection via corbaloc URLs
    //
    CORBA::Object_var bmgrObj =
        orb -> resolve_initial_references("BootManager");
    OB::BootManager_var bootManager = OB::BootManager::_narrow(bmgrObj);

    //
    // Create implementation object
    //
    chat::BroadcasterAdmin_impl* broadcasterAdminImpl =
	new chat::BroadcasterAdmin_impl(orb, adminPoa, rootPoa);
    PortableServer::ServantBase_var servant = broadcasterAdminImpl;
    PortableServer::ObjectId_var id =
        PortableServer::string_to_ObjectId("broadcasterAdmin");
    adminPoa -> activate_object_with_id(id, servant);
 
    // 
    // Create binding for administrative object in the BootManager
    // - necessary for client connections via corbaloc URLs
    //
    chat::BroadcasterAdmin_var p = broadcasterAdminImpl -> _this();
    bootManager -> add_binding(id, p);

    //
    // Print "server started" message
    //
    time_t t;
    char* d;
    t = time(0);
    d = asctime(localtime(&t));
    assert(d && strlen(d) > 0);
    d[strlen(d) - 1] = '\0';
    cout << "*** Server started at " << d << endl;

    //
    // This special class of POAManager is needed for access to the OCI
    //
    OBPortableServer::POAManager_var obManager =
        OBPortableServer::POAManager::_narrow(manager);

    //
    // Display the host and port
    //
    OCI::AcceptorSeq_var acceptors = obManager -> get_acceptors();

    for(CORBA::ULong i = 0 ; i < acceptors -> length() ; i++)
    {
	OCI::AcceptorInfo_var info = acceptors[i] -> get_info();
	OCI::IIOP::AcceptorInfo_var iiopInfo =
	    OCI::IIOP::AcceptorInfo::_narrow(info);
	    
        if(!CORBA::is_nil(iiopInfo))
        {
    	    CORBA::StringSeq_var hosts = iiopInfo -> hosts();
    	    CORBA::UShort port = iiopInfo -> port();
    	
    	    cout << "*** Running on host `" << hosts[0]
    	         << "', port number " << port << endl;
        }
    }

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv, "Chat-Server");
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}






