// WinClient.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"

#include "WinClient.h"

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>
#include <OB/Reactor.h>
#include <OB/OBWindows.h>

#include <Broadcaster.h>
#include <ReceiverBase_impl.h>

#include "WinClientDlg.h"
#include "WinClientServerDlg.h"

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


using namespace chat;


/////////////////////////////////////////////////////////////////////////////
// CWinClientApp

BEGIN_MESSAGE_MAP(CWinClientApp, CWinApp)
    //{{AFX_MSG_MAP(CWinClientApp)
    // NOTE - the ClassWizard will add and remove mapping macros here.
    //    DO NOT EDIT what you see in these blocks of generated code!
    //}}AFX_MSG
    ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWinClientApp construction

CWinClientApp::CWinClientApp()
{
    // TODO: add construction code here,
    // Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CWinClientApp object

CWinClientApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CWinClientApp initialization

BOOL CWinClientApp::InitInstance()
{
    // CG: The following block was added by the Windows Sockets component.
    {
#ifndef _MAC
	if (!AfxSocketInit())
	{
	    AfxMessageBox(CG_IDS_SOCKETS_INIT_FAILED);
	    return false;
	}
#endif  // _MAC
    }
    
    // Standard initialization
    // If you are not using these features and wish to reduce the size
    //  of your final executable, you should remove from the following
    //  the specific initialization routines you do not need.
    
#ifdef _AFXDLL
    Enable3dControls();		// Call this when using MFC in a shared DLL
#else
    Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif
    
    CWinClientServerDlg serverDlg;
    int nResponse = serverDlg.DoModal();
    
    if (nResponse == IDOK)
    {
	const char* host = serverDlg.m_EditHost;
	const char* port = serverDlg.m_EditPort;
	CORBA::UShort portNum = atoi(serverDlg.m_EditPort);
	OBCORBA::ORB_var orb;
	
	//
	// Get Windows logger
	//
	OB::Logger_var logger = OB::GetWindowsLogger();
	
	try
	{
	    OB::Properties_var properties =
		OB::Properties::getDefaultProperties();
	    int argcDummy = 0;
	    char* argvDummy[1];
	    argvDummy[0] = 0;
	    OB::ParseArgs(argcDummy, argvDummy, properties);
	    
	    //
	    // Set concurrency models
	    //
	    properties -> setProperty("ooc.orb.conc_model", "reactive");
	    properties -> setProperty("ooc.orb.oa.conc_model", "reactive");
	    properties -> setProperty("ooc.orb.id", "Chat-WinClient");
	    
	    //
	    // Get Windows reactor
	    //
	    OB::Reactor_var reactor = OB::GetWindowsReactor(m_hInstance);
	    
	    //
	    // Create ORB
	    //
	    orb = OBCORBA::ORB_init(argcDummy, argvDummy,
	    			    OB::Properties::_nil(),
				    logger, reactor);

	    //
	    // Resolve Root POA
	    //
	    CORBA::Object_var poaObj =
		orb -> resolve_initial_references("RootPOA");
	    PortableServer::POA_var rootPoa =
		PortableServer::POA::_narrow(poaObj);
	    
	    PortableServer::POAManager_var poa_manager =
		rootPoa -> the_POAManager();
	    
	    //
	    // Get the broadcaster object
	    //
	    CORBA::String_var str = CORBA::string_dup("corbaloc::");
	    str += host;
	    str += ":";
	    str += port;
	    str += "/broadcasterAdmin";
	    CORBA::Object_var obj = orb -> string_to_object(str);
	    assert(!CORBA::is_nil(obj));
	    chat::BroadcasterAdmin_var server = 
		chat::BroadcasterAdmin::_narrow(obj);
	    assert(!CORBA::is_nil(server));
	    
	    //
	    // Activate and initialize
	    //
	    poa_manager -> activate();
	    
	    //
	    // Create and activate the receiver object
	    //
	    CWinClientDlg* dlg = 
	        new CWinClientDlg(orb, rootPoa, server, m_pMainWnd);
	    PortableServer::ServantBase_var servant = dlg;
	    Receiver_var receiver = dlg -> _this();
	    m_pMainWnd = dlg;
		
	    //
	    // Run implementation
	    //
	    dlg -> DoModal();
	}
	catch(const CORBA::Exception& ex)
	{
	    CORBA::String_var msg = ex._to_string();
	    logger -> error(msg);
	}
	
	if(!CORBA::is_nil(orb))
	{
	    try
	    {
		orb -> destroy();
	    }
	    catch(const CORBA::Exception& ex)
	    {
		CORBA::String_var msg = ex._to_string();
		logger -> error(msg);
	    }
	}
    }
    
    // Since the dialog has been closed, return FALSE so that we exit the
    //  application, rather than start the application's message pump.
    return false;
}


