// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Pizzeria.h>
#include <Customer_impl.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Obtain the server's object reference from a file
    //
    CORBA::Object_var obj =
        orb -> string_to_object("relfile:/Pizza.ref");
    if(CORBA::is_nil(obj))
    {      
        cerr << argv[0] << ": cannot read IOR from Pizza.ref"
             << endl;
        return EXIT_FAILURE;
    }        

    Pizzeria_var pizzeria = Pizzeria::_narrow(obj);
    assert(!CORBA::is_nil(pizzeria));

    //
    // Place an order
    //
    Pizzeria::Toppings toppings;
    toppings.length(3);
    toppings[0] = Pizzeria::CHEESE;
    toppings[1] = Pizzeria::SALAMI;
    toppings[2] = Pizzeria::PEPPERONI;
    Customer_var customer = new Customer_impl;
    customer -> name((const char*)"Bruno Strizzi");
    customer -> credit_card((const char*)"1234 5678 9012 3456");
    customer -> expires((const char*)"09/2001");
    pizzeria -> order_pizza(Pizzeria::LARGE, toppings, customer);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
