// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Echo.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>

#if !defined(HAVE_STD_IOSTREAM) && defined(WIN32)
#define HAVE_NO_WCHAR_T_SUPPORT_IN_STREAMS
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get "echo" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Echo.ref");
    if(CORBA::is_nil(obj))
    {
	fprintf(stderr, "%s: cannot read IOR from Echo.ref\n", argv[0]);
	return EXIT_FAILURE;
    }
    
    Echo_var echo = Echo::_narrow(obj);
    assert(!CORBA::is_nil(echo));
    
    //
    // Main loop
    //
    puts("Enter 'h' for 'Hello' message, or 'x' for exit:");
    int c = 0;
    CORBA::WString_var reply = CORBA::wstring_dup(L"");
    do
    {
	if(!isspace(c))
            printf("> ");
	c = fgetc(stdin);
        if(c == 'h')
	{
	    reply = echo -> echo_message(L"Hello!");

#ifdef HAVE_NO_WCHAR_T_SUPPORT_IN_STREAMS
	    //
	    // Work-around for missing wcout.
	    //
	    for(CORBA::ULong i = 0 ; i < OB::wcslen(reply) ; i++)
		printf("%c", (char)reply[i]);
	    printf("\n");
#else
	    std::wcout << reply << std::endl;
#endif

	}
    }
    while(c != EOF && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
