// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <DefaultFactory_impl.h>
#include <LocatorFactory_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Check for debug option
    //
    bool debug = false;
    if(argc > 1 && strcmp(argv[1], "-d") == 0)
        debug = true;
    
    //
    // Get the root POA
    //
    CORBA::Object_var rootObj =
        orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(rootObj);

    //
    // Create persistent POA
    //
    PortableServer::POAManager_var poaManager = root -> the_POAManager();
    CORBA::PolicyList policies;
    policies.length(2);
    policies[0] =
        root -> create_id_assignment_policy(PortableServer::USER_ID);
    policies[1] =
        root -> create_lifespan_policy(PortableServer::PERSISTENT);
    PortableServer::POA_var poa =
        root -> create_POA("persistent", poaManager, policies);

    //
    // Create default servant factory implementation object
    //
    DefaultFactory_impl* defFactory = new DefaultFactory_impl(orb, poa);
    PortableServer::ServantBase_var defFactoryServant = defFactory;
    PortableServer::ObjectId_var oid =
        PortableServer::string_to_ObjectId("def_factory");
    poa -> activate_object_with_id(oid, defFactoryServant);
    CORBA::Object_var obj = defFactory -> _this();

    //
    // Save reference in file
    //
    {
        const char* myRef = "dfactory.ref";
        ofstream out(myRef);
        if(out.fail())
        {
            cerr << argv[0] << ": can't open `" << myRef << "': "
                 << strerror(errno) << endl;
            return EXIT_FAILURE;
        }

	CORBA::String_var ref = orb -> object_to_string(obj);
        out << ref;
        out.close();
    }

    //
    // Create servant locator factory implementation object
    //
    LocatorFactory_impl* locFactory = new LocatorFactory_impl(orb, poa, debug);
    PortableServer::ServantBase_var locFactoryServant = locFactory;
    oid = PortableServer::string_to_ObjectId("loc_factory");
    poa -> activate_object_with_id(oid, locFactoryServant);
    obj = locFactory -> _this();

    //
    // Save reference in file
    //
    {
        const char* myRef = "lfactory.ref";
        ofstream out(myRef);
        if(out.fail())
        {
            cerr << argv[0] << ": can't open `" << myRef << "': "
                 << strerror(errno) << endl;
            return EXIT_FAILURE;
        }

	CORBA::String_var ref = orb -> object_to_string(obj);
        out << ref;
        out.close();
    }
	
    //
    // Run implementation
    //
    poaManager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
