// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Timer.h>

#include <Simple.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace CORBA;

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Get number of repetitions
    //
    if(argc < 2)
    {
        cerr << "Usage: " << argv[0] << " repetitions" << endl;
        return EXIT_FAILURE;
    }

    int reps = atoi(argv[1]);
    if(reps <= 0)
    {
        cerr << argv[0] << ": number of repetitions must be larger than 0"
    	 << endl;
        return EXIT_FAILURE;
    }
	
    //
    // Get "I" object
    //
    Object_var obj = orb -> string_to_object("relfile:/I.ref");
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from I.ref"
             << endl;
        return EXIT_FAILURE;
    }

	
    I_var i = I::_narrow(obj);
    assert(!is_nil(i));
    
    //
    // If the server is using a thread-pool force the pool to be
    // initialized.
    //
    timeval tv = OB::TimeNow();
    Boolean exists = !(i -> _non_existent());
    assert(exists);
    tv = OB::TimeSub(OB::TimeNow(), tv);
    cout << "Time needed for initialization: "
         << ((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0) * 1000.0
         << " ms" << endl;

    //
    // Main loop
    //
    tv = OB::TimeNow();
    for(int j = 0; j < reps; ++j)
        i -> f();
    tv = OB::TimeSub(OB::TimeNow(), tv);
    cout << "Total time needed: "
         << ((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0) * 1000.0
         << " ms" << endl;
    cout << "Time per method call: "
         << (((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0)
    	 / (double)reps) * 1000.0 << " ms" << endl;
    
    return EXIT_SUCCESS;
}


int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
