// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Random.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char*[])
{
    //
    // We are connecting to an Orbix server. Note that we can
    // either use the generated IOR, or the corbaloc form. The
    // object key for corbaloc was extracted using iordump.
    //
    const char* randomIOR = "IOR:000000000000000f49444c3a52616e646f6d3a312e3000000000000100000000000000500001000000000016706c616e7874792e6473672e63732e7463642e69650006220000002c3a5c706c616e7874792e6473672e63732e7463642e69653a52616e646f6d3a303a3a49523a52616e646f6d00";

    //
    // Note that this doesn't work since this type of IOR doesn't
    // have a RepositoryID. This means that ORBacus calls _is_a on
    // the call to _narrow below -- but unfortunately ORBIX has
    // some bugs related to _is_a which causes the _narrow to
    // fail.
    //
    // const char* randomIOR = "corbaloc::planxty.dsg.cs.tcd.ie:1570/"
    // ":\\planxty.dsg.cs.tcd.ie:Random:0::IR:Random";
    //
    CORBA::Object_var obj = orb -> string_to_object(randomIOR);
    assert(!CORBA::is_nil(obj));
	
    Random_var random = Random::_narrow(obj);
    assert(!CORBA::is_nil(random));

    //
    // Main loop
    //
    cout << "Enter 'r' for a random number, or 'x' for exit:\n";
    char c;
    do
    {
        cout << "> ";
        cin >> c;
        if(c == 'r')
        {
    	    CORBA::ULong l = random -> lrand48();
    	    cout << l << endl;
        }
    }
    while(cin.good() && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
