// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>

#include <Demo.h>
#include <Value_impl.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Install value factory
    //
    CORBA::ValueFactoryBase_var factory = new Demo::Value_init;
    orb -> register_value_factory("IDL:Demo/Value:1.0", factory);

    //
    // Get Demo::I object reference
    //
    CORBA::Object_var obj =
        orb -> string_to_object("relfile:/Demo.ref");
    if(CORBA::is_nil(obj))
    {      
        cerr << argv[0] << ": cannot read IOR from Demo.ref"
             << endl;
        return EXIT_FAILURE;
    }        
        
    Demo::I_var interf = Demo::I::_narrow(obj);
    assert(!CORBA::is_nil(interf));

    //
    // Print interface repository content for Demo::I
    //
    CORBA::InterfaceDef_var idef = interf -> _get_interface();
    if(CORBA::is_nil(idef))
    {
        cerr << argv[0] << ": no interface repository available" << endl;
	return EXIT_FAILURE;
    }

    CORBA::InterfaceDef::FullInterfaceDescription_var desc =
        idef -> describe_interface();

    CORBA::ULong i;

    cout << "name = " << desc -> name << endl;
    cout << "id = " << desc -> id << endl;
    cout << "defined_in = " << desc -> defined_in << endl;
    cout << "version = " << desc -> version << endl;
    cout << "operations:" << endl;
    for(i = 0 ; i < desc -> operations.length() ; i++)
    {
        cout << i << ": " << desc -> operations[i].name << endl;
    }
    cout << "attributes:" << endl;
    for(i = 0 ; i < desc -> attributes.length() ; i++)
    {
        cout << i << ": " << desc -> attributes[i].name << endl;
    }
    cout << "base_interfaces:" << endl;
    for(i = 0 ; i < desc -> base_interfaces.length() ; i++)
    {
        cout << i << ": " << desc -> base_interfaces[i] << endl;
    }

    //
    // Get valuetype
    //
    Demo::BaseValue_var value = interf -> get_value();

    //
    // Print interface repository content for Demo::Value
    //
    CORBA::ValueDef_var vdef = value -> _get_value_def();
    if(CORBA::is_nil(vdef))
    {
        cerr << argv[0] << ": no interface repository available" << endl;
        return 1;
    }

    CORBA::ValueDef::FullValueDescription_var vdesc = vdef -> describe_value();

    cout << endl;
    cout << "name = " << vdesc -> name << endl;
    cout << "id = " << vdesc -> id << endl;
    cout << "defined_in = " << vdesc -> defined_in << endl;
    cout << "version = " << vdesc -> version << endl;
    cout << "operations:" << endl;
    for(i = 0 ; i < vdesc -> operations.length() ; i++)
    {
        cout << i << ": " << vdesc -> operations[i].name << endl;
    }
    cout << "attributes:" << endl;
    for(i = 0 ; i < vdesc -> attributes.length() ; i++)
    {
        cout << i << ": " << vdesc -> attributes[i].name << endl;
    }
    cout << "members:" << endl;
    for(i = 0 ; i < vdesc -> members.length() ; i++)
    {
        cout << i << ": " << vdesc -> members[i].name << endl;
    }
    cout << "base_value: " << vdesc -> base_value << endl;
    cout << "abstract_base_values:" << endl;
    for(i = 0 ; i < vdesc -> abstract_base_values.length() ; i++)
    {
        cout << i << ": " << vdesc -> abstract_base_values[i] << endl;
    }
    
    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
