// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/BootManager.h>

#include <Simple_skel_tie.h>
#include <Simple_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace Simple;

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

class POA_Simple_refcount_tie :
    public POA_Simple::SimpleObject_tie< SimpleTie_impl >,
    public PortableServer::RefCountServantBase
{
public:

    POA_Simple_refcount_tie(SimpleTie_impl* impl,
			    PortableServer::POA_ptr poa)
        : POA_Simple::SimpleObject_tie< SimpleTie_impl >(impl, poa, true)
    {
    }

    virtual
    ~POA_Simple_refcount_tie()
    {
    }
};

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
    
    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();
    
    //
    // Create implementation objects
    //
    SimpleInherit_impl* simpleInheritImpl = new SimpleInherit_impl(rootPOA);
    PortableServer::ServantBase_var servant1 = simpleInheritImpl;
    SimpleObject_var simpleInherit = simpleInheritImpl -> _this();

    SimpleTie_impl* simpleTieImpl = new SimpleTie_impl();
    POA_Simple_refcount_tie servant2(simpleTieImpl, rootPOA);
    SimpleObject_var simpleTie = servant2._this();

    //
    // Resolve the BootManager and register the servers
    //
    CORBA::Object_var obj = orb -> resolve_initial_references("BootManager");
    OB::BootManager_var bootManager = OB::BootManager::_narrow(obj);

    PortableServer::ObjectId_var oid1 =
        PortableServer::string_to_ObjectId("SimpleInherit");
    bootManager -> add_binding(oid1, simpleInherit);
    PortableServer::ObjectId_var oid2 =
        PortableServer::string_to_ObjectId("SimpleTie");
    bootManager -> add_binding(oid2, simpleTie);
    
    //
    // Save references
    //
    CORBA::String_var s1 = orb -> object_to_string(simpleInherit);
    
    const char* refFile1 = "simple1.ior";
    ofstream out1(refFile1);
    if(out1.fail())
    {
	cerr << argv[0] << ": can't open `" << refFile1 << "': "
	     << strerror(errno) << endl;
	return EXIT_FAILURE;
    }

    out1 << s1 << endl;
    out1.close();
    
    CORBA::String_var s2 = orb -> object_to_string(simpleTie);

    const char* refFile2 = "simple2.ior";
    ofstream out2(refFile2);
    if(out2.fail())
    {
	cerr << argv[0] << ": can't open `" << refFile2 << "': "
	     << strerror(errno) << endl;
	return EXIT_FAILURE;
    }

    out2 << s2 << endl;
    out2.close();
    
    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv, "Simple-Server");
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
