// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Finance_skel.h>
#include <EmployeeFactory_impl.h>

#include <stdlib.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

//
// Implementation of Payroll interface
//
class Payroll_impl : public POA_Payroll,
		     public PortableServer::RefCountServantBase
{

public:

    virtual void pay_employee(Employee* emp)
        throw(CORBA::SystemException)
    {
        if(emp != 0)
            emp -> print_check();
    }
};


int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Create & install value factory - if we forget to do this,
    // the client will receive a MARSHAL exception when attempting
    // to transmit a Employee value.
    //
    CORBA::ValueFactoryBase_var factory = new EmployeeFactory_impl();
    CORBA::ValueFactoryBase_var oldFactory =
        orb -> register_value_factory("IDL:Employee:1.0", factory);

    //
    // Resolve RootPOA
    //
    CORBA::Object_var obj;
    obj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(obj);
    PortableServer::POAManager_var manager = root -> the_POAManager();

    //
    // Instantiate and activate the Payroll servant
    //
    Payroll_impl* impl = new Payroll_impl();
    PortableServer::ServantBase_var servant = impl;
    obj = impl -> _this();

    //
    // Export the object reference to a file
    //
    const char* refFile = "Payroll.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "'" << endl;
        return EXIT_FAILURE;
    }

    CORBA::String_var ref = orb -> object_to_string(obj);
    out << ref << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
