// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/Properties.h>
#include <OB/Logger.h>

#include <IMRLogger.h>

using namespace IMROAD;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(Logger_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(Logger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// IMRLogger constructor/destructor
// ----------------------------------------------------------------------

Logger::Logger(OB::Logger_ptr logger, OB::Properties_ptr properties)
    : logger_(OB::Logger::_duplicate(logger)),
      tracePeerStatus_(0),
      traceProcessControl_(0),
      traceServerStatus_(0)
{
    OB::Properties::KeySeq keys = properties -> getKeys("ooc.imr.trace.");
    for(CORBA::ULong i = 0; i < keys.length(); ++i)
    {
        const char* key = keys[i];
	const char* value = properties -> getProperty(key);
	
	if(strcmp(key, "ooc.imr.trace.peer_status") == 0)
	{
	    tracePeerStatus_ = atol(value);
	}
	else if(strcmp(key, "ooc.imr.trace.process_control") == 0)
	{
	    traceProcessControl_ = atol(value);
	}
	else if(strcmp(key, "ooc.imr.trace.server_status") == 0)
	{
	    traceServerStatus_ = atol(value);
	}
	else
	{
	    CORBA::String_var err =
	        CORBA::string_dup("IMR: unknown property: ");
	    err += key;
	    logger -> warning(err);
	}
    }
}

Logger::~Logger()
{
}

// ----------------------------------------------------------------------
// IMRLogger trace level accessors
// ----------------------------------------------------------------------

CORBA::ULong
Logger::tracePeerStatus()
{
    return tracePeerStatus_;
}

CORBA::ULong
Logger::traceProcessControl()
{
    return traceProcessControl_;
}

CORBA::ULong
Logger::traceServerStatus()
{
    return traceServerStatus_;
}

// ----------------------------------------------------------------------
// EventLogger logging methods
// ----------------------------------------------------------------------

void
Logger::info(const char* msg)
{
    logger_ -> info(msg);
}

void
Logger::error(const char* msg)
{
    logger_ -> error(msg);
}

void
Logger::warning(const char* msg)
{
    logger_ -> warning(msg);
}

void
Logger::trace(const char* category, const char* msg)
{
    logger_ -> trace(category, msg);
}
