// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef IMR_LOGGER_H
#define IMR_LOGGER_H

#include <OB/Basic.h>

#include <OB/Properties_fwd.h>
#include <OB/Logger_fwd.h>

#include <IMRLogger_fwd.h>

namespace IMROAD
{

class Logger : public OB::RefCount
{
    //
    // The message logger
    //
    OB::Logger_var logger_;

    //
    // The trace levels
    //
    CORBA::ULong tracePeerStatus_;
    CORBA::ULong traceProcessControl_;
    CORBA::ULong traceServerStatus_;

public:

    Logger(OB::Logger_ptr, OB::Properties_ptr);
    ~Logger();

    static inline Logger_ptr _duplicate(Logger_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Logger_ptr _nil()
    { return 0; }

    //
    // Methods to get trace levels
    //
    CORBA::ULong tracePeerStatus();
    CORBA::ULong traceProcessControl();
    CORBA::ULong traceServerStatus();

    //
    // Methods to perform logging
    //
    void info(const char*);
    void error(const char*);
    void warning(const char*);
    void trace(const char*, const char*);
};

} // End of namespace IMROAD

namespace CORBA
{

inline void
release(IMROAD::Logger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(IMROAD::Logger_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
