// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef INTERNAL_SERVER_IMPL_H
#define INTERNAL_SERVER_IMPL_H

#include <IMRLogger_fwd.h>
#include <ServerRecord_fwd.h>
#include <IMRInternal_skel.h>
#include <IMRDB.h>

namespace IMR
{

//
// Forward declarations
//
class OADDatabase;
class ServerDatabase;
class ServerStarter;

class InternalServer_impl : public POA_IMR::InternalServer
{
    ServerDatabase* db_;
    OADDatabase* oaddb_;

    PortableServer::Current_var current_;

    PortableInterceptor::ObjectReferenceTemplate* fwdORT_;

    IMROAD::Logger_var logger_;

    CORBA::ULong count_;

    ServerStarter* serverStarter_;

    ServerRecord_ptr getRecord();

    void forkFailed();

public:

    InternalServer_impl(ServerDatabase*, OADDatabase*, CORBA::ORB_ptr,
			PortableServer::POA_ptr, IMROAD::Logger_ptr, 
			ServerStarter*);
    ~InternalServer_impl();

    //
    // Standard IDL to C++ Mapping
    //
    
    //
    // IDL:ooc.com/IMR/ActiveState/set_status:1.0
    //
    virtual void set_status(const char*, IMR::ServerStatus status)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/poa_create:1.0
    //
    virtual PortableInterceptor::ObjectReferenceTemplate*
    poa_create(IMR::POAStatus state,
    	       PortableInterceptor::ObjectReferenceTemplate* ort)
        throw(IMR::NoSuchPOA,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/poa_status_update:1.0
    //
    virtual void poa_status_update(const IMR::POANameSeq& poas, 
    			   	   IMR::POAStatus state)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/startup:1.0
    //
    virtual void startup(const char* instance,
    			 PortableInterceptor::ObjectReferenceTemplate* rootORT,
			 OAD::ProcessEndpointManager_out endpoint)
        throw(IMR::NoSuchOAD,
	      IMR::OADNotRunning,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/kill:1.0
    //
    virtual void kill()
	throw(NoSuchOAD, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/fork:1.0
    //
    virtual void fork()
	throw(NoSuchOAD, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/make_object:1.0
    //
    virtual CORBA::Object* make_object(const IMR::POAName& poa,
    				       const PortableServer::ObjectId& oid)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/wait:1.0
    //
    virtual void wait(IMR::ServerStatus, const IMR::POAName&, IMR::POAStatus)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/timeout:1.0
    //
    virtual CORBA::Boolean timeout()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/status:1.0
    //
    virtual void status(const IMR::POAName&, IMR::ServerStatus&,
    			IMR::POAStatus&)
        throw(CORBA::SystemException);
};

}; // End namespace IMR

#endif
