// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#ifndef OSLAYER_H
#define OSLAYER_H

#ifdef WIN32
#include <io.h>
#include <direct.h>
#else
#include <unistd.h>
#include <dirent.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <stdlib.h>
#include <stdio.h> // rename is in here for some reason under linux libc 5

#ifdef WIN32
#   include <sys/timeb.h>
#endif

#include <time.h>

#ifdef WIN32
//
// Provide dirent stuff
//
struct DIR
{
    long handle;
};

struct dirent
{
    char d_name[MAX_PATH +1];
};
#endif

class OSLayer
{
public:
    static bool rename(const char* from, const char* to)
    {
#ifdef WIN32
        int result = MoveFile(from, to);
#else
        int result = (::rename(from, to) == 0);
#endif
        return result ? true : false;
    }
    
    static bool unlink(const char* path)
    {
#ifdef WIN32
        if(::_unlink(path) != 0)
            return false;
#else
        if(::unlink(path) != 0)
            return false;
#endif
        return true;
    }

    static bool stat(const char* path, struct stat& sbuf)
    {
#if 0
        if(::_stat(path, &sbuf) < 0)
            return false;
#else
        if(::stat(path, &sbuf) < 0)
            return false;
#endif
        return true;
    }

    static bool mkdir(const char* path, int perms = 0755)
    {
#ifdef WIN32
	if(::mkdir(path) < 0 )
	    return false;
#else
        if(::mkdir(path, perms) < 0)
            return false;
#endif
        return true;
    }

    static bool isDir(const struct stat& sbuf)
    {
        return (sbuf.st_mode & S_IFDIR) ? true : false;
    }

    static unsigned long fileSize(const struct stat& sbuf)
    {
        return sbuf.st_size;
    }

    static int truncate(int fd, size_t size)
    {
#ifdef WIN32
        return _chsize(fd, size);
#else
        return ftruncate(fd, size);
#endif
    }

    static DIR* opendir(const char* d)
    {
#ifdef WIN32
        char buf[MAX_PATH  +1];
        const char* sep = "/";
	int len = strlen(d);
        if(len > 0 && d[len-1] == '/')
            sep = "";
        sprintf(buf, "%s%s*", d, sep);
        struct _finddata_t data;
        long handle = _findfirst(buf, &data);
        if(handle < 0)
            return 0;
        DIR* dir = new DIR();
        dir -> handle = handle;
        return dir;
#else
        return ::opendir(d);
#endif
    }

    static struct dirent* readdir(DIR* d)
    {
#ifdef WIN32
        static struct dirent de;
        struct _finddata_t data;
        if(_findnext(d -> handle, &data) == 0)
        {
            strcpy(de.d_name, data.name);
            return &de;
        }
        return 0;
#else
        return ::readdir(d);
#endif
    }

    static void closedir(DIR* d)
    {
#ifdef WIN32
        if(d != 0)
        {
            _findclose(d -> handle);
            delete d;
        }
#else
        ::closedir(d);
#endif
    }
    
    static long time()
    {
	//
	// Get the current time.
	//
#ifdef WIN32
	struct _timeb timebuffer;
	::_ftime(&timebuffer);
	long t = timebuffer.time;
#else
	long t = ::time(0);
#endif
	return t;
    }
};

#endif
