// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PROCESS_CONTROLLER_H
#define PROCESS_CONTROLLER_H

#include <OB/IMRTypes.h>
#include <IMRLogger_fwd.h>
#include <OAD.h> // For ExecFailed

namespace OAD
{

class ProcObserver;

//
// A process controller. This class is used to create and destroy
// processes. The ProcObserver is notified when a spawned process
// terminates if the spawn occurred within this process lifecycle.
//
class ProcController : public JTCMonitor
{
protected:

    //
    // The Process Spectator
    //
    ProcObserver* observer_;

    //
    // The Trace Levels
    //
    IMROAD::Logger_var logger_;

public:

    ProcController(ProcObserver*, IMROAD::Logger_ptr);
    ~ProcController();

    //
    // Spawn a new process
    //
    virtual IMR::ProcessID
    spawn(const char*, const char*, const IMR::ArgList&, const char*, 
          const char*)
	throw(OAD::ExecFailed) = 0;

    //
    // Kill a process
    //
    virtual void
    kill(IMR::ProcessID)
	throw(OAD::KillFailed) = 0;
};

//
// A process controller factory. This object is used to create
// ProcControllers
//
class ProcControllerFactory
{
public:
    static ProcController*
    createProcController(ProcObserver*, IMROAD::Logger_ptr);
};

} // End of namespace OAD

#endif
