// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Logger.h>

#include <IMRLogger.h>
#include <ProcObserver.h>
#include <ProcessDatabase.h>
#include <IMROAD.h>

using namespace OAD;

// ----------------------------------------------------------------------
// ProcObserver constructor/destructor
// ----------------------------------------------------------------------

ProcObserver::ProcObserver(ProcessDatabase* db,
			   IMROAD::Logger_ptr logger)
    : db_(db),
      logger_(IMROAD::Logger::_duplicate(logger))
{
}

// ----------------------------------------------------------------------
// ProcObserver public member implementation
// ----------------------------------------------------------------------

void
ProcObserver::processCreate(const char* server, const char* id,
			    IMR::ProcessID pid)
{
    JTCSynchronized sync(*db_);

    OADDB_4_0_2::ServerRecord rec;

    //
    // Add a record for this child process
    //
    db_ -> createRecord(server, id, pid, rec);
}

void
ProcObserver::processDestroy(const char* server, const char* id)
{
    OADDB_4_0_2::ServerRecord rec;

    {
        JTCSynchronized sync(*db_);

        if(!db_ -> queryServerById(server, id, rec))
	    return;

        if(logger_ -> traceProcessControl() >= 2)
        {
	    CORBA::String_var msg =
	    CORBA::string_dup(rec.server);
	    msg += '(';
	    if(rec.pid == 0xFFFFFFFF)
	        msg += "UNKNOWN";
	    else
	        msg += rec.pid;
	    msg += "): dead";
	    logger_ -> trace("OAD", msg);
	}

	//
	// Remove the database record
	//
	db_ -> removeRecord(rec.server);
    }

    IMR::ServerObserver_var observer = getObserver();
    if(!CORBA::is_nil(observer_))
    {
        try
        {
	    observer_ -> server_died(rec.server, rec.instance);
        }
        catch(const CORBA::SystemException&)
        {
	    setObserver(IMR::ServerObserver::_nil());
        }
    }
}

void
ProcObserver::processDestroy(IMR::ProcessID pid)
{
    OADDB_4_0_2::ServerRecord rec;

    {
        JTCSynchronized sync(*db_);

        if(!db_ -> queryServerByPid(pid, rec))
        return;

        if(logger_ -> traceProcessControl() >= 2)
        {
	    CORBA::String_var msg = CORBA::string_dup(rec.server);
	    msg += '(';
	    msg += rec.pid;
	    msg += "): dead";
	    logger_ -> trace("OAD", msg);
	}

	//
	// Remove the database record
	//
	db_ -> removeRecord(rec.server);
    }

    IMR::ServerObserver_var observer = getObserver();
    if(!CORBA::is_nil(observer_))
    {
        try
        {
	    observer_ -> server_died(rec.server, rec.instance);
        }
        catch(const CORBA::SystemException&)
        {
	    setObserver(IMR::ServerObserver::_nil());
        }
    }
}

void
ProcObserver::setObserver(IMR::ServerObserver_ptr observer)
{
    JTCSynchronized sync(observerMutex_);

    observer_ = IMR::ServerObserver::_duplicate(observer);
}

IMR::ServerObserver_ptr
ProcObserver::getObserver()
{
    JTCSynchronized sync(observerMutex_);

    return IMR::ServerObserver::_duplicate(observer_);
}
