// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PROCESS_CONTROLLER_IMPL_H
#define PROCESS_CONTROLLER_IMPL_H


#include <IMROAD_fwd.h>
#include <IMRLogger_fwd.h>
#include <OAD_skel.h>

namespace OAD
{

//
// Forward declarations
//
class ProcessDatabase;
class ProcController;
class ProcObserver;
class ProcessEndpointManager_impl;

class ProcessController_impl : virtual public POA_OAD::ProcessController
{
    //
    // Hide copy constructor and assignment operator
    //
    ProcessController_impl(const ProcessController_impl&);
    void operator=(const ProcessController_impl&);

    CORBA::String_var host_;

    ProcessDatabase* db_;

    PortableServer::POA_var poa_;

    IMROAD::Logger_var logger_;

    ProcessEndpointManager_impl* endpointManager_;
    IMR::OADEndpointManager_var oadEndpointManager_;

    //
    // The process observer
    //
    ProcObserver* observer_;

    //
    // The process controller
    //
    ProcController* controller_;

    void reconnect();

public:

    ProcessController_impl(ProcessDatabase*,
			   CORBA::ORB_ptr,
			   PortableServer::POA_ptr,
			   IMR::OADEndpointManager_ptr,
			   const char* host,
			   IMROAD::Logger_ptr);
    virtual ~ProcessController_impl();

    void shutdown();

    //
    // Standard IDL to C++ Mapping
    //

    virtual char*
    host()
        throw(CORBA::SystemException);

    virtual void
    exec_process(const char*, const char*, const IMR::ArgList&, const char*,
                 const char*)
        throw(OAD::ExecFailed, CORBA::SystemException);

    virtual void
    stop_process(const char*)
        throw(CORBA::SystemException);

    virtual void
    kill_process(const char*)
        throw(OAD::KillFailed, CORBA::SystemException);

    virtual void
    reset_process(const char*)
        throw(CORBA::SystemException);

    virtual void
    list_processes(OAD::ProcessList_out)
        throw(CORBA::SystemException);

    virtual void
    connect_imr(IMR::OADEndpointManager_ptr registrar)
        throw(IMR::NoSuchOAD, CORBA::SystemException);

    virtual PortableServer::POA_ptr
    _default_POA();

    //
    // Internal methods
    //
    void
    _OB_setObserver(IMR::ServerObserver_ptr);

    ProcessEndpointManager_ptr
    _OB_getEndpointManager();
};

}

#endif
