// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef SERVER_DATABASE_H
#define SERVER_DATABASE_H

#include <OB/Hashtable.h>

#include <IMRDB.h>
#include <ServerRecord.h>
#include <Database.h>

namespace IMR
{

class ServerDatabase : public JTCMonitor
{
    //
    // The Logger
    //
    OB::Logger_var logger_;

    //
    // The database
    //
    OB::Database* db_;

    //
    // Server Records
    //
    typedef OB::Hashtable<CORBA::String_var, IMR::ServerRecord_var,
    			  OB::StringHasher> ServerNameTable;

    typedef OB::Hashtable<CORBA::Long, IMR::ServerRecord_var,
    			  OB::LongHasher> ServerIdTable;

    ServerNameTable* serversByName_;
    ServerIdTable* serversById_;

    void load();

public:

    ServerDatabase(CORBA::ORB_ptr);
    ~ServerDatabase();

    ServerRecord_ptr createRecord(const char*);

    bool destroyRecord(const char*);

    ServerRecord_ptr queryRecordByServer(const char*);

    ServerRecord_ptr queryRecordById(IMR::ServerID);

    IMRDB_4_1_0::ServerIdSeq* getAllServers();

    IMRDB_4_1_0::ServerIdSeq* queryAllByHost(const char*);

    void updateRecord(ServerRecord_ptr);
};

} // End namespace IMR

#endif
