// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Object.h>
#include <OB/AbstractBase.h>
#include <OB/POAServantBase.h>
#include <OB/ValueBase.h>
#include <OB/TypeCode.h>
#include <OB/Stream.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// AbstractBaseInfo public member functions
// ----------------------------------------------------------------------

void
AbstractBaseInfo::free(void* v) const
{
    CORBA::release((AbstractBase_ptr)v);
}

void*
AbstractBaseInfo::dup(const void* v) const
{
    AbstractBase_ptr ab = (AbstractBase_ptr)v;
    return AbstractBase::_duplicate(ab);
}

void
AbstractBaseInfo::marshal(const void* v, OutputStreamImpl* out) const
{
    out -> write_abstract_interface((AbstractBase_ptr)v);
}

void
AbstractBaseInfo::unmarshal(void* v, InputStreamImpl* in) const
{
    AbstractBase_ptr ab = (AbstractBase_ptr)v;
    if(!is_nil(ab))
    {
        ValueBase_var vb = ab -> _to_value();
        if(vb.in())
            vb -> _OB_unmarshal(in);
    }
}

// ----------------------------------------------------------------------
// AbstractBase static members
// ----------------------------------------------------------------------

AbstractBaseInfo AbstractBase::_OB_abstractBaseInfo;

// ----------------------------------------------------------------------
// AbstractBase constructor and destructor
// ----------------------------------------------------------------------

AbstractBase::AbstractBase()
{
}

AbstractBase::AbstractBase(const AbstractBase&)
{
}
    
AbstractBase::~AbstractBase()
{
}

// ----------------------------------------------------------------------
// AbstractBase public member functions
// ----------------------------------------------------------------------

AbstractBase_ptr
AbstractBase::_duplicate(AbstractBase_ptr p)
{
    if(p)
    {
        Object_ptr obj = p -> _to_object(); // duplicates p
        if(is_nil(obj))
        {
            ValueBase* vb = p -> _to_value(); // duplicates p
            assert(vb != 0);
        }
    }

    return p;
}

AbstractBase_ptr
AbstractBase::_narrow(AbstractBase_ptr p)
{
    return _duplicate(p);
}

AbstractBase_ptr
AbstractBase::_nil()
{
    return 0;
}

Object_ptr
AbstractBase::_to_object()
{
    return Object::_nil();
}

ValueBase*
AbstractBase::_to_value()
{
    return 0;
}

Boolean
CORBA::is_nil(AbstractBase_ptr p)
{
    return (p == 0);
}

void
CORBA::release(AbstractBase_ptr p)
{
    if(p)
    {
        Object_var obj = p -> _to_object();
        if(!is_nil(obj))
        {
            release(obj);
        }
        else
        {
            ValueBase_var vb = p -> _to_value();
            assert(vb.in() != 0);
            remove_ref(vb);
        }
    }
}

void
OBRelease(CORBA::AbstractBase_ptr p)
{
    CORBA::release(p);
}

void
OBDuplicate(CORBA::AbstractBase_ptr p)
{
    CORBA::AbstractBase::_duplicate(p);
}

void
operator<<=(CORBA::Any& any, CORBA::AbstractBase_ptr* v)
{
    static AbstractBaseInfo info;
    any.replace(CORBA::_tc_AbstractBase, *v, true, &info);
}

void
operator<<=(CORBA::Any& any, CORBA::AbstractBase_ptr v)
{
    CORBA::AbstractBase_ptr val = CORBA::AbstractBase::_duplicate(v);
    any <<= &val;
}
