// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Object.h>
#include <OB/DirectStubImpl.h>
#include <OB/POAServantBase.h>
#include <OB/Any.h> // Needed for gcc
#include <OB/OCI.h>
#include <OB/ORBInstance.h>
#include <OB/RefCounted.h>
#include <OB/Stream.h>

#include <POA_impl.h>
#include <POAManagerFactory_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

/*
void
OBDuplicate(DirectStubImpl_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(DirectStubImpl_ptr p)
{
    if(p)
	p -> _OB_decRef();
}
*/

void
OBDuplicate(OB::DirectStubImplFactory_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::DirectStubImplFactory_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// DirectStubImpl private and protected member implementation
// ----------------------------------------------------------------------

OutputStream_ptr
DirectStubImpl::_OB_createOutputStream()
{
    OCI::Buffer_var buf = new OCI::Buffer;
    return new OutputStream(buf);
}

InputStream_ptr
DirectStubImpl::_OB_createInputStream(OutputStream_ptr out)
{
    OCI::Buffer_var buf = new OCI::Buffer;
    buf -> consume(out -> _OB_buffer_nodup());
    InputStream_ptr in = new InputStream(buf, 0, false);
    OB::ORBInstance_var orbInstance = _OB_ORBInstance();
    in -> _OB_ORBInstance(orbInstance);
    return in;
}

// ----------------------------------------------------------------------
// DirectStubImpl constructor and destructor
// ----------------------------------------------------------------------

#ifdef HAVE_VCPLUSPLUS_BUGS

void
DirectStubImpl::_ob_initialize(PortableServer::POA_ptr poa,
                               const PortableServer::ObjectId& oid,
                               PortableServer::ServantBase* servant)
{
    poa_ = dynamic_cast<OBPortableServer::POA_impl*>(poa);
    oid_ = oid;
    deactivated_ = false;
    _ob_servant_ = servant;

    assert(poa_);
    assert(_ob_servant_);
}

DirectStubImpl::DirectStubImpl(PortableServer::POA_ptr poa,
			       const PortableServer::ObjectId& oid,
			       PortableServer::ServantBase* servant)
{
    _ob_initialize(poa, oid, servant);
}

#else

DirectStubImpl::DirectStubImpl(PortableServer::POA_ptr poa,
			       const PortableServer::ObjectId& oid,
			       PortableServer::ServantBase* servant)
    : poa_(dynamic_cast<OBPortableServer::POA_impl*>(poa)),
      oid_(oid), deactivated_(false), _ob_servant_(servant)
{
    assert(poa_);
    assert(_ob_servant_);
}

#endif

DirectStubImpl::~DirectStubImpl()
{
    //
    // If we're not deactivated then inform the POA that we're going
    // away
    //
    if(!deactivated_)
        poa_ -> _OB_removeDirectStubImpl(oid_, this);
}

// ----------------------------------------------------------------------
// DirectStubImpl public member implementation
// ----------------------------------------------------------------------

InterfaceDef_ptr
DirectStubImpl::_get_interface()
{
    InvocationHandler _ob_handler(this, "_get_interface");
    return _ob_servant_ -> _get_interface();
}

Boolean
DirectStubImpl::_is_a(const char* id)
{
    InvocationHandler _ob_handler(this, "_is_a");
    return _ob_servant_ -> _is_a(id);
}

Boolean
DirectStubImpl::_non_existent()
{
    InvocationHandler _ob_handler(this, "_non_existent");
    return _ob_servant_ -> _non_existent();
}

Boolean
DirectStubImpl::_locate_request()
{
    try
    {
	InvocationHandler _ob_handler(this, "_locate");
	return true;
    }
    catch(const OBJECT_NOT_EXIST&)
    {
	return false;
    }
}

OCI::ConnectorInfo_ptr
DirectStubImpl::_get_oci_connector_info()
{
    return OCI::ConnectorInfo::_nil();
}

OCI::TransportInfo_ptr
DirectStubImpl::_get_oci_transport_info()
{
    return OCI::TransportInfo::_nil();
}

void
DirectStubImpl::_OB_deactivate()
{
    JTCSynchronized sync(deactivatedMutex_);
    deactivated_ = true;
}

PortableServer::ServantBase*
DirectStubImpl::_OB_setupServant()
{
    JTCSynchronized sync(deactivatedMutex_);

    if(deactivated_)
        throw StubImplChanged();

    _ob_servant_ -> _add_ref();
    return _ob_servant_;
}

void
DirectStubImpl::_OB_preinvoke(const char* op)
{
    //
    // Validate POA manager state
    //
    poa_ -> _OB_validateManagerState();

    //
    // Increment the POAs request count
    //
    if(!poa_ -> _OB_incrementRequestCount())
        throw StubImplChanged();

    //
    // Preinvoke
    //
    poa_ -> _OB_preinvoke(op, oid_, _ob_servant_, 0,
			  OCI::TransportInfo::_nil());
}

void
DirectStubImpl::_OB_postinvoke()
{
    //
    // Postinvoke
    //
    poa_ -> _OB_postinvoke();

    //
    // Decrement the outstanding request count
    //
    poa_ -> _OB_decrementRequestCount();
}

ORBInstance_ptr
DirectStubImpl::_OB_ORBInstance()
{
    return poa_ -> _OB_ORBInstance();
}

// ----------------------------------------------------------------------
// DirectStubImplFactory constructor and destructor
// ----------------------------------------------------------------------

DirectStubImplFactory::DirectStubImplFactory()
    : destroy_(false)
{
}

DirectStubImplFactory::~DirectStubImplFactory()
{
    assert(destroy_);
}

// ----------------------------------------------------------------------
// DirectStubImplFactory public member implementation
// ----------------------------------------------------------------------

void
DirectStubImplFactory::destroy()
{
    assert(!destroy_); // May only be destroyed once
    destroy_ = true;

    orbInstance_ = ORBInstance::_nil();
}

void
DirectStubImplFactory::setORBInstance(ORBInstance_ptr orbInstance)
{
    orbInstance_ = ORBInstance::_duplicate(orbInstance);
}

DirectStubImpl_ptr
DirectStubImplFactory::createStubImpl(RefCountIOR_ptr IOR,
				      RefCountIOR_ptr,
				      RefCountPolicyList_ptr policies)
    throw(LocationForward,
	  SystemException)
{
    //
    // The object factory destroys this object, so it's an
    // initialization error if the this operation is called after ORB
    // destruction
    //
    if(destroy_)
	throw INITIALIZE(MinorORBDestroyed, COMPLETED_NO);

    //
    // Get the POAManagerFactory
    //
    OBPortableServer::POAManagerFactory_var pmFactory =
	orbInstance_ -> getPOAManagerFactory();

    //
    // TODO: Do we want to pass the impl* instead of the _ptr type?
    //
    OBPortableServer::POAManagerFactory_impl* pmImpl =
        dynamic_cast<OBPortableServer::POAManagerFactory_impl*>
        (pmFactory.in());
    return pmImpl -> _OB_createDirectStubImpl(IOR, policies);
}
