// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/TypeCode.h>
#include <OB/MarshalStubImpl.h>
#include <OB/ValueBase.h>
#include <OB/ORBInstance.h>
#include <OB/Stream.h>
#include <OB/IFR1.h>
#include <OB/NativeTypes.h>
#include <OB/AbstractBase.h>

#include <DynamicAny_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// DynAny_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynAny_impl::DynAny_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : factory_(DynamicAny::DynAnyFactory::_duplicate(factory)),
      orbInstance_(ORBInstance::_duplicate(orbInstance)),
      type_(TypeCode::_duplicate(tc)), parent_(DynamicAny::DynAny::_nil()),
      destroyed_(false)
{
    origType_ = GetOrigType_nodup(type_);
}

DynamicAny::DynAny_impl::~DynAny_impl()
{
}

// ----------------------------------------------------------------------
// DynAny_impl protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynAny_impl::checkValue(Any* any, TypeCode_ptr tc)
{
    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr type = any -> _OB_type_nodup();

    if(!type -> equivalent(tc))
        throw TypeMismatch();
}

DynamicAny::DynAny_ptr
DynamicAny::DynAny_impl::create(const Any& any, bool adopt)
{
    DynAny_ptr result = factory_ -> create_dyn_any(any);
    if(adopt)
        adoptChild(result);
    return result;
}

DynamicAny::DynAny_ptr
DynamicAny::DynAny_impl::create(TypeCode_ptr tc, bool adopt)
{
    DynAny_ptr result = factory_ -> create_dyn_any_from_type_code(tc);
    if(adopt)
        adoptChild(result);
    return result;
}

void
DynamicAny::DynAny_impl::adoptChild(DynamicAny::DynAny_ptr d)
{
    DynAny_impl* impl = dynamic_cast<DynAny_impl*>(d);
    assert(impl != 0);
    impl -> parent_ = this;
}

void
DynamicAny::DynAny_impl::notifyParent()
{
    if(!is_nil(parent_))
    {
        DynAny_impl* p = dynamic_cast<DynAny_impl*>(parent_);
        assert(p != 0);
        p -> childModified(this);
    }
}

void
DynamicAny::DynAny_impl::childModified(DynamicAny::DynAny_ptr)
{
    // do nothing
}

// ----------------------------------------------------------------------
// DynAny_impl public member functions
// ----------------------------------------------------------------------

TypeCode_ptr
DynamicAny::DynAny_impl::type()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return TypeCode::_duplicate(type_);
}

void
DynamicAny::DynAny_impl::destroy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

//
// TODO: component objects must not be destroyed
//
//    destroyed_ = true;
}

void
DynamicAny::DynAny_impl::insert_boolean(Boolean value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_boolean);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Boolean(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_octet(Octet value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_octet);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Octet(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_char(Char value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_char);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Char(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_short(Short value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_short);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Short(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ushort(UShort value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ushort);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new UShort(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_long(Long value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_long);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Long(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ulong(ULong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ulong);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new ULong(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_float(Float value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_float);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Float(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_double(Double value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_double);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new Double(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_string(const char* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(value == 0)
        throw InvalidValue();

    Any* any = _OB_currentAny();

    // Don't use checkValue() - we must accomodate bounded and
    // unbounded strings

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_string)
        throw TypeMismatch();

    //
    // Check for bounded string
    //
    ULong len = origTC -> length();
    if(len > 0 && strlen(value) > len)
        throw InvalidValue();

    any -> replace(tc, string_dup(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_reference(Object_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    // Don't use checkValue()

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_objref &&
       origTC -> kind() != tk_local_interface)
        throw TypeMismatch();

    any -> replace(tc, Object::_duplicate(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_typecode(TypeCode_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_TypeCode);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, TypeCode::_duplicate(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_longlong(LongLong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_longlong);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new LongLong(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ulonglong(ULongLong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ulonglong);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new ULongLong(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_longdouble(LongDouble value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
#if SIZEOF_LONG_DOUBLE >= 12

    Any* any = _OB_currentAny();

    checkValue(any, _tc_longdouble);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new LongDouble(value), true);

    notifyParent();

#else

    throw InvalidValue();

#endif
}

void
DynamicAny::DynAny_impl::insert_wchar(WChar value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_wchar);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new WChar(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_wstring(const wchar_t* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(value == 0)
        throw InvalidValue();

    Any* any = _OB_currentAny();

    // Don't use checkValue() - we must accomodate bounded and
    // unbounded wstrings

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_wstring)
        throw TypeMismatch();

    //
    // Check for bounded wstring
    //
    ULong len = origTC -> length();
    if(len > 0 && OB::wcslen(value) > len)
        throw InvalidValue();

    any -> replace(tc, wstring_dup(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_any);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    Any* val = new Any(value);
    val -> _OB_ORBInstance(orbInstance_);
    any -> replace(tc, val, true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_dyn_any(DynamicAny::DynAny_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(is_nil(value))
        throw TypeMismatch();

    if(!_OB_insertDynAny(value))
    {
	//
	// may throw TypeMismatch
	//
        DynAny_var comp = current_component();
        if(is_nil(comp))
            throw InvalidValue();
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(comp.in());
        assert(impl != 0);
        if(!impl -> _OB_insertDynAny(value))
            throw TypeMismatch();
    }

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_val(ValueBase* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    //
    // TODO: Custom valuetypes are not currently supported
    //
    if(value -> _OB_isCustom())
        throw BAD_PARAM();

    //
    // may throw TypeMismatch
    //
    DynAny_var comp = current_component();
    if(is_nil(comp))
        throw InvalidValue();

    //
    // Ensure the given value has the proper type
    //
    TypeCode_var tc = comp -> type();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_value && origTC -> kind() != tk_value_box)
        throw TypeMismatch();

    if(value != 0)
    {
        const char* id = origTC -> id();
        const char** ids = value -> _OB_truncatableIds();
        ULong i;
        for(i = 0 ; ids[i] != 0 ; i++)
            if(strcmp(id, ids[i]) == 0)
                break;

        if(ids[i] == 0)
            throw TypeMismatch();
    }

    DynValue_var dv = DynValue::_narrow(comp);
    assert(!is_nil(dv));

    //
    // Create an any and invoke from_any
    //
    static ValueBaseInfo info;
    add_ref(value);
    Any any;
    any.replace(tc, value, true, &info);
    dv -> from_any(any);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_abstract(AbstractBase_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_abstract_interface)
        throw TypeMismatch();

    if(value == 0)
    {
        static ValueBaseInfo info;
        any -> replace(tc, 0, true, &info);
    }
    else
    {
        ValueBase_var vb = value -> _to_value();
        Object_var obj = value -> _to_object();

        if(vb.in() != 0)
        {
            static ValueBaseInfo info;
            any -> replace(tc, vb._retn(), true, &info);
        }
        else if(!is_nil(obj))
        {
            static AbstractBaseInfo info;
            any -> replace(tc, obj._retn(), true, &info);
        }
        else
            throw TypeMismatch();
    }

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_boolean_seq(const BooleanSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_BooleanSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new BooleanSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_octet_seq(const OctetSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_OctetSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new OctetSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_char_seq(const CharSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_CharSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new CharSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_wchar_seq(const WCharSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_WCharSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new WCharSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_short_seq(const ShortSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ShortSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new ShortSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ushort_seq(const UShortSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_UShortSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new UShortSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_long_seq(const LongSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new LongSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ulong_seq(const ULongSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ULongSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new ULongSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_longlong_seq(const LongLongSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongLongSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new LongLongSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_ulonglong_seq(const ULongLongSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ULongLongSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new ULongLongSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_float_seq(const FloatSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_FloatSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new FloatSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_double_seq(const DoubleSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_DoubleSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new DoubleSeq(value), true);

    notifyParent();
}

void
DynamicAny::DynAny_impl::insert_longdouble_seq(const LongDoubleSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
#if SIZEOF_LONG_DOUBLE >= 12

    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongDoubleSeq);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    any -> replace(tc, new LongDoubleSeq(value), true);

    notifyParent();

#else

    throw InvalidValue();

#endif
}

Boolean
DynamicAny::DynAny_impl::get_boolean()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_boolean);
    assert(any -> value() != 0);

    Boolean* result = (Boolean*)any -> value();

    return *result;
}

Octet
DynamicAny::DynAny_impl::get_octet()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_octet);
    assert(any -> value() != 0);

    Octet* result = (Octet*)any -> value();

    return *result;
}

Char
DynamicAny::DynAny_impl::get_char()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_char);
    assert(any -> value() != 0);

    Char* result = (Char*)any -> value();

    return *result;
}

Short
DynamicAny::DynAny_impl::get_short()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_short);
    assert(any -> value() != 0);

    Short* result = (Short*)any -> value();

    return *result;
}

UShort
DynamicAny::DynAny_impl::get_ushort()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ushort);
    assert(any -> value() != 0);

    UShort* result = (UShort*)any -> value();

    return *result;
}

Long
DynamicAny::DynAny_impl::get_long()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_long);
    assert(any -> value() != 0);

    Long* result = (Long*)any -> value();

    return *result;
}

ULong
DynamicAny::DynAny_impl::get_ulong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ulong);
    assert(any -> value() != 0);

    ULong* result = (ULong*)any -> value();

    return *result;
}

Float
DynamicAny::DynAny_impl::get_float()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_float);
    assert(any -> value() != 0);

    Float* result = (Float*)any -> value();

    return *result;
}

Double
DynamicAny::DynAny_impl::get_double()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_double);
    assert(any -> value() != 0);

    Double* result = (Double*)any -> value();

    return *result;
}

char*
DynamicAny::DynAny_impl::get_string()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    // Don't use checkValue()

    if(any == 0)
        throw InvalidValue();

    assert(any -> value() != 0);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_string)
        throw TypeMismatch();

    const char* result = (const char*)any -> value();

    return string_dup(result);
}

Object_ptr
DynamicAny::DynAny_impl::get_reference()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    // Don't use checkValue()

    if(any == 0)
        throw InvalidValue();

    Object_ptr result = (Object_ptr)any -> value();

    return Object::_duplicate(result);
}

TypeCode_ptr
DynamicAny::DynAny_impl::get_typecode()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    // Don't use checkValue()

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr result = (TypeCode_ptr)any -> value();

    return TypeCode::_duplicate(result);
}

LongLong
DynamicAny::DynAny_impl::get_longlong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_longlong);
    assert(any -> value() != 0);

    LongLong* result = (LongLong*)any -> value();

    return *result;
}

ULongLong
DynamicAny::DynAny_impl::get_ulonglong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ulonglong);
    assert(any -> value() != 0);

    ULongLong* result = (ULongLong*)any -> value();

    return *result;
}

LongDouble
DynamicAny::DynAny_impl::get_longdouble()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
#if SIZEOF_LONG_DOUBLE >= 12

    Any* any = _OB_currentAny();

    checkValue(any, _tc_longdouble);
    assert(any -> value() != 0);

    LongDouble* result = (LongDouble*)any -> value();

    return *result;

#else

    throw InvalidValue();

#endif
}

WChar
DynamicAny::DynAny_impl::get_wchar()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_wchar);
    assert(any -> value() != 0);

    WChar* result = (WChar*)any -> value();

    return *result;
}

wchar_t*
DynamicAny::DynAny_impl::get_wstring()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    // Don't use checkValue()

    if(any == 0)
        throw InvalidValue();

    assert(any -> value() != 0);

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_wstring)
        throw TypeMismatch();

    const wchar_t* result = (const wchar_t*)any -> value();

    return wstring_dup(result);
}

Any*
DynamicAny::DynAny_impl::get_any()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_any);
    assert(any -> value() != 0);

    const Any* a = (const Any*)any -> value();

    return new Any(*a);
}

DynamicAny::DynAny_ptr
DynamicAny::DynAny_impl::get_dyn_any()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynAny_var da = _OB_getDynAny();
    if(is_nil(da))
    {
	//
	// may throw TypeMismatch
	//
        DynAny_var comp = current_component();
        if(is_nil(comp))
            throw InvalidValue();
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(comp.in());
        assert(impl != 0);
        da = impl -> _OB_getDynAny();
    }

    if(is_nil(da))
        throw TypeMismatch();

    return da._retn();
}

ValueBase*
DynamicAny::DynAny_impl::get_val()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    //
    // may throw TypeMismatch
    //
    DynAny_var comp = current_component();
    if(is_nil(comp))
        throw InvalidValue();

    //
    // Ensure the given value has the proper type
    //
    TypeCode_var tc = comp -> type();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_value && origTC -> kind() != tk_value_box)
        throw TypeMismatch();

    DynAny_impl* impl = dynamic_cast<DynAny_impl*>(comp.in());
    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    impl -> _OB_marshal(&out);
    InputStream_var in = out.create_input_stream();
    in -> _OB_ORBInstance(orbInstance_);
    ValueBase_var vb = in -> read_value();

    return vb._retn();
}

AbstractBase_ptr
DynamicAny::DynAny_impl::get_abstract()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    if(any == 0)
        throw InvalidValue();

    TypeCode_ptr tc = any -> _OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    if(origTC -> kind() != tk_abstract_interface)
        throw TypeMismatch();

    Object_var obj;
    ValueBase_var vb;
    if(*any >>= Any::to_object(obj.out()))
    {
        if(!is_nil(obj))
        {
            AbstractBase_ptr ab = obj -> _OB_toAbstractBase();
            assert(!is_nil(ab));
            return AbstractBase::_duplicate(ab);
        }
    }
    else if(*any >>= Any::to_value(vb.out()))
    {
        if(vb.in() != 0)
        {
            AbstractBase_ptr ab = vb -> _OB_toAbstractBase();
            assert(!is_nil(ab));
            return AbstractBase::_duplicate(ab);
        }
    }

    return AbstractBase::_nil();
}

BooleanSeq*
DynamicAny::DynAny_impl::get_boolean_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_BooleanSeq);
    assert(any -> value() != 0);

    return (BooleanSeq*)any -> value();
}

OctetSeq*
DynamicAny::DynAny_impl::get_octet_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_OctetSeq);
    assert(any -> value() != 0);

    return (OctetSeq*)any -> value();
}

CharSeq*
DynamicAny::DynAny_impl::get_char_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_CharSeq);
    assert(any -> value() != 0);

    return (CharSeq*)any -> value();
}

WCharSeq*
DynamicAny::DynAny_impl::get_wchar_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_WCharSeq);
    assert(any -> value() != 0);

    return (WCharSeq*)any -> value();
}

ShortSeq*
DynamicAny::DynAny_impl::get_short_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ShortSeq);
    assert(any -> value() != 0);

    return (ShortSeq*)any -> value();
}

UShortSeq*
DynamicAny::DynAny_impl::get_ushort_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_UShortSeq);
    assert(any -> value() != 0);

    return (UShortSeq*)any -> value();
}

LongSeq*
DynamicAny::DynAny_impl::get_long_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongSeq);
    assert(any -> value() != 0);

    return (LongSeq*)any -> value();
}

ULongSeq*
DynamicAny::DynAny_impl::get_ulong_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ULongSeq);
    assert(any -> value() != 0);

    return (ULongSeq*)any -> value();
}

LongLongSeq*
DynamicAny::DynAny_impl::get_longlong_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongLongSeq);
    assert(any -> value() != 0);

    return (LongLongSeq*)any -> value();
}

ULongLongSeq*
DynamicAny::DynAny_impl::get_ulonglong_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_ULongLongSeq);
    assert(any -> value() != 0);

    return (ULongLongSeq*)any -> value();
}

FloatSeq*
DynamicAny::DynAny_impl::get_float_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_FloatSeq);
    assert(any -> value() != 0);

    return (FloatSeq*)any -> value();
}

DoubleSeq*
DynamicAny::DynAny_impl::get_double_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_DoubleSeq);
    assert(any -> value() != 0);

    return (DoubleSeq*)any -> value();
}

LongDoubleSeq*
DynamicAny::DynAny_impl::get_longdouble_seq()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    Any* any = _OB_currentAny();

    checkValue(any, _tc_LongDoubleSeq);
    assert(any -> value() != 0);

    return (LongDoubleSeq*)any -> value();
}

// ----------------------------------------------------------------------
// DynAny_impl public internal functions
// ----------------------------------------------------------------------

DynamicAny::DynAny_ptr
DynamicAny::DynAny_impl::_OB_getDynAny()
{
    return DynAny::_nil();
}

bool
DynamicAny::DynAny_impl::_OB_insertDynAny(DynamicAny::DynAny_ptr)
{
    return false;
}

// ----------------------------------------------------------------------
// DynBasic_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynBasic_impl::DynBasic_impl(DynamicAny::DynAnyFactory_ptr factory,
					 ORBInstance_ptr orbInstance,
					 TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc)
{
    TypeCode_ptr origTC = GetOrigType_nodup(tc);
    switch(origTC -> kind())
    {
    case tk_null:
    case tk_void:
        any_ = new Any(tc, 0, true);
        break;

    case tk_short:
        any_ = new Any(tc, new Short(0), true);
        break;

    case tk_long:
        any_ = new Any(tc, new Long(0), true);
        break;

    case tk_ushort:
        any_ = new Any(tc, new UShort(0), true);
        break;

    case tk_ulong:
        any_ = new Any(tc, new ULong(0), true);
        break;

    case tk_float:
        any_ = new Any(tc, new Float(0), true);
        break;

    case tk_double:
        any_ = new Any(tc, new Double(0), true);
        break;

    case tk_boolean:
        any_ = new Any(tc, new Boolean(false), true);
        break;

    case tk_char:
        any_ = new Any(tc, new Char(0), true);
        break;

    case tk_octet:
        any_ = new Any(tc, new Octet(0), true);
        break;

    case tk_any:
        any_ = new Any(tc, new Any, true);
        break;

    case tk_TypeCode:
        any_ = new Any(tc, TypeCode::_duplicate(_tc_null), true);
        break;

    case tk_objref:
    case tk_local_interface:
        any_ = new Any(tc, Object::_nil(), true);
        break;

    case tk_string:
        any_ = new Any(tc, string_dup(""), true);
        break;

    case tk_longlong:
        any_ = new Any(tc, new LongLong(0), true);
        break;

    case tk_ulonglong:
        any_ = new Any(tc, new ULongLong(0), true);
        break;

    case tk_longdouble:
#if SIZEOF_LONG_DOUBLE >= 12
        any_ = new Any(tc, new LongDouble(0), true);
#else
        assert(false);
#endif
        break;

    case tk_wchar:
        any_ = new Any(tc, new WChar(0), true);
        break;

    case tk_wstring:
        any_ = new Any(tc, wstring_dup(L""), true);
        break;

    case tk_abstract_interface:
        any_ = new Any(tc, AbstractBase::_nil(), true);
        break;

    case tk_value:
        //
        // Only custom valuetypes are supported by DynBasic_impl
        //
        assert(origTC -> type_modifier() == VM_CUSTOM);
        any_ = new Any(tc, 0, true);
        break;

    case tk_Principal:
    default:
        assert(false);
    }

    any_ -> _OB_ORBInstance(orbInstance_);
}

DynamicAny::DynBasic_impl::~DynBasic_impl()
{
}

// ----------------------------------------------------------------------
// DynAny_impl protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynBasic_impl::notifyParent()
{
    //
    // We have intercepted a change notification. If our type is tk_any,
    // then we release our component. It will be created again during
    // get_dyn_any().
    //
    if(origType_ -> kind() == tk_any)
    {
        if(!is_nil(comp_))
            comp_ = DynAny::_nil();
    }

    DynAny_impl::notifyParent();
}

// ----------------------------------------------------------------------
// DynBasic_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynBasic_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    any_ = dyn_any -> to_any();

    notifyParent();
}

void
DynamicAny::DynBasic_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    TypeCode_ptr tc = value._OB_type_nodup();
    TypeCode_ptr origTC = GetOrigType_nodup(tc);

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    //
    // Check for an invalid value
    //
    // NOTE: Some types can legally have a null value
    //
    if(value.value() == 0)
    {
        switch(origTC -> kind())
        {
        case tk_null:
        case tk_void:
        case tk_TypeCode:
        case tk_objref:
        case tk_value:
        case tk_abstract_interface:
        case tk_local_interface:
            //
            // Some types can legally have a null value
            //
            break;

        default:
            throw InvalidValue();
        }
    }

    any_ = new Any(value);
    any_ -> type(type_);
    any_ -> _OB_ORBInstance(orbInstance_);

    notifyParent();
}

Any*
DynamicAny::DynBasic_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return new Any(any_);
}

Boolean
DynamicAny::DynBasic_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var type = dyn_any -> type();
    if(!type -> equivalent(type_))
        return false;

    TypeCode_ptr origTC = GetOrigType_nodup(type);
    Any_var any = dyn_any -> to_any();
    const void* v1 = any_ -> value();
    const void* v2 = any -> value();

    switch(origTC -> kind())
    {
    case tk_null:
    case tk_void:
        return true;

    case tk_short:
        if(v1 != 0 && v2 != 0)
            return(*(Short*)v1 == *(Short*)v2);
        break;

    case tk_long:
        if(v1 != 0 && v2 != 0)
            return(*(Long*)v1 == *(Long*)v2);
        break;

    case tk_ushort:
        if(v1 != 0 && v2 != 0)
            return(*(UShort*)v1 == *(UShort*)v2);
        break;

    case tk_ulong:
        if(v1 != 0 && v2 != 0)
            return(*(ULong*)v1 == *(ULong*)v2);
        break;

    case tk_float:
        if(v1 != 0 && v2 != 0)
            return(*(Float*)v1 == *(Float*)v2);
        break;

    case tk_double:
        if(v1 != 0 && v2 != 0)
            return(*(Double*)v1 == *(Double*)v2);
        break;

    case tk_boolean:
        if(v1 != 0 && v2 != 0)
            return(*(Boolean*)v1 == *(Boolean*)v2);
        break;

    case tk_char:
        if(v1 != 0 && v2 != 0)
            return(*(Char*)v1 == *(Char*)v2);
        break;

    case tk_octet:
        if(v1 != 0 && v2 != 0)
            return(*(Octet*)v1 == *(Octet*)v2);
        break;

    case tk_any:
        if(v1 != 0 && v2 != 0)
        {
            DynAny_var d1, d2;
            try
            {
                d1 = create(*(Any*)v1, false);
                d2 = create(*(Any*)v2, false);
                bool eq = d1 -> equal(d2);
                d1 -> destroy();
                d2 -> destroy();
                return eq;
            }
            catch(const UserException&)
            {
                // ignore
            }
            catch(const SystemException&)
            {
                // ignore
            }

            if(!is_nil(d1))
                d1 -> destroy();
            if(!is_nil(d2))
                d2 -> destroy();
        }
        break;

    case tk_TypeCode:
        if(v1 == 0 && v2 == 0)
            return true;
        else if(v1 == 0 || v2 == 0)
            return false;
        else
            return ((TypeCode_ptr)v1) -> equal((TypeCode_ptr)v2);

    case tk_objref:
    case tk_local_interface:
        if(v1 == 0 && v2 == 0)
            return true;
        else if(v1 == 0 || v2 == 0)
            return false;
        else
            return ((Object_ptr)v1) -> _is_equivalent((Object_ptr)v2);

    case tk_string:
        if(v1 != 0 && v2 != 0)
            return(strcmp((const char*)v1, (const char*)v2) == 0);
        break;

    case tk_longlong:
        if(v1 != 0 && v2 != 0)
            return(*(LongLong*)v1 == *(LongLong*)v2);
        break;

    case tk_ulonglong:
        if(v1 != 0 && v2 != 0)
            return(*(ULongLong*)v1 == *(ULongLong*)v2);
        break;

    case tk_longdouble:
#if SIZEOF_LONG_DOUBLE >= 12
        if(v1 != 0 && v2 != 0)
            return(*(LongDouble*)v1 == *(LongDouble*)v2);
#else
        assert(false);
#endif
        break;

    case tk_wchar:
        if(v1 != 0 && v2 != 0)
            return(*(WChar*)v1 == *(WChar*)v2);
        break;

    case tk_wstring:
        if(v1 != 0 && v2 != 0)
            return(OB::wcscmp((const wchar_t*)v1, (const wchar_t*)v2) == 0);
        break;

    case tk_value:
    {
        if(v1 == v2)
            return true;
        if(v1 == 0 || v2 == 0)
            return false;
        else
        {
            //
            // Currently, it's not possible for a custom valuetype
            // to be represented as an InputStream in an any
            //
            const ConstructedInfo* info1 = any_ -> info();
            const ConstructedInfo* info2 = any -> info();
            assert(info1 != 0 && info2 != 0);
            return false;
        }
    }

    case tk_abstract_interface:
    {
        if(v1 == v2)
            return true;
        else if(v1 == 0 || v2 == 0)
            return false;
        else
        {
            //
            // Currently, it's not possible for an abstract interface
            // to be represented as an InputStream in an any
            //
            const ConstructedInfo* info1 = any_ -> info();
            const ConstructedInfo* info2 = any -> info();
            assert(info1 != 0 && info2 != 0);

            Object_var o1 = ((AbstractBase_ptr)v1) -> _to_object();
            Object_var o2 = ((AbstractBase_ptr)v2) -> _to_object();
            if(!is_nil(o1) && !is_nil(o2))
                return o1 -> _is_equivalent(o2);
            ValueBase_var vb1 = ((AbstractBase_ptr)v1) -> _to_value();
            ValueBase_var vb2 = ((AbstractBase_ptr)v2) -> _to_value();
            return vb1.in() == vb2.in();
        }
        break;
    }

    default:
        assert(false);
    }

    return false;
}

DynamicAny::DynAny_ptr
DynamicAny::DynBasic_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynBasic_impl* result = new DynBasic_impl(factory_, orbInstance_, type_);
    result -> any_ = new Any(any_);
    return result;
}

Boolean
DynamicAny::DynBasic_impl::seek(Long)
    throw(SystemException)
{
    return false;
}

void
DynamicAny::DynBasic_impl::rewind()
    throw(SystemException)
{
    // do nothing
}

Boolean
DynamicAny::DynBasic_impl::next()
    throw(SystemException)
{
    return false;
}

ULong
DynamicAny::DynBasic_impl::component_count()
    throw(SystemException)
{
    return 0;
}

DynamicAny::DynAny_ptr
DynamicAny::DynBasic_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    throw TypeMismatch();

    return DynAny::_nil(); // some compilers need this
}

// ----------------------------------------------------------------------
// DynamicAny::DynBasic_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynBasic_impl::_OB_marshal(OutputStreamImpl* out)
{
    Any* any = _OB_currentAny();
    out -> write_any_NoTypeCode(*any);
}

void
DynamicAny::DynBasic_impl::_OB_unmarshal(InputStreamImpl* in)
{
    in -> read_any_NoTypeCode(any_.inout(), type_);

    if(!is_nil(comp_))
        comp_ = DynAny::_nil();

    notifyParent();
}

Any*
DynamicAny::DynBasic_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(!is_nil(comp_))
    {
        Any* p = comp_ -> to_any();
        any_ -> replace(type_, p, true);
    }

    return any_.ptr();
}

Any*
DynamicAny::DynBasic_impl::_OB_currentAnyValue()
{
    return _OB_currentAny();
}

DynamicAny::DynAny_ptr
DynamicAny::DynBasic_impl::_OB_getDynAny()
{
    if(origType_ -> kind() != tk_any)
        return DynAny::_nil();

    if(is_nil(comp_))
    {
        const Any* cap = (const Any*)any_ -> value();
        comp_ = create(*cap, true);
    }

    return DynAny::_duplicate(comp_);
}

bool
DynamicAny::DynBasic_impl::_OB_insertDynAny(DynamicAny::DynAny_ptr p)
{
    //
    // Do nothing if caller is passing our component
    //
    if(p == comp_.in())
        return true;

    checkValue(any_.ptr(), _tc_any);

    DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
    Any* implAny = impl -> _OB_currentAny();
    checkValue(implAny, _tc_any);
    const Any* cap = (const Any*)implAny -> value();

    TypeCode_ptr tc = any_ -> _OB_type_nodup();
    any_ -> replace(tc, new Any(*cap), true);

    if(!is_nil(comp_))
        comp_ = DynAny::_nil();

    return true;
}

// ----------------------------------------------------------------------
// DynFixed_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynFixed_impl::DynFixed_impl(DynamicAny::DynAnyFactory_ptr factory,
					 ORBInstance_ptr orbInstance,
					 TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc), value_((Long)0)
{
}

DynamicAny::DynFixed_impl::~DynFixed_impl()
{
}

// ----------------------------------------------------------------------
// DynamicAny::DynFixed_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynFixed_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynFixed_impl* impl = dynamic_cast<DynFixed_impl*>(dyn_any);
    value_ = impl -> value_;

    notifyParent();
}

void
DynamicAny::DynFixed_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(value.value() == 0)
        throw InvalidValue();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    Fixed* f = (Fixed*)value.value();
    if(f -> fixed_scale() > origType_ -> fixed_scale())
        throw InvalidValue();
    value_ = *f;

    notifyParent();
}

Any*
DynamicAny::DynFixed_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return new Any(type_, new Fixed(value_), true);
}

Boolean
DynamicAny::DynFixed_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    DynFixed_impl* impl = dynamic_cast<DynFixed_impl*>(dyn_any);
    return (value_ == impl -> value_);
}

DynamicAny::DynAny_ptr
DynamicAny::DynFixed_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynFixed_impl* result = new DynFixed_impl(factory_, orbInstance_, type_);
    result -> value_ = value_;
    return result;
}

Boolean
DynamicAny::DynFixed_impl::seek(Long)
    throw(SystemException)
{
    return false;
}

void
DynamicAny::DynFixed_impl::rewind()
    throw(SystemException)
{
    // do nothing
}

Boolean
DynamicAny::DynFixed_impl::next()
    throw(SystemException)
{
    return false;
}

ULong
DynamicAny::DynFixed_impl::component_count()
    throw(SystemException)
{
    return 0;
}

DynamicAny::DynAny_ptr
DynamicAny::DynFixed_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    throw TypeMismatch();

    return DynAny::_nil(); // some compilers need this
}

char*
DynamicAny::DynFixed_impl::get_value()
    throw(SystemException)
{
    return value_.to_string();
}

Boolean
DynamicAny::DynFixed_impl::set_value(const char* val)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(strlen(val) == 0)
        throw InvalidValue();

    Fixed f;

    try
    {
        f = Fixed(val);
    }
    catch(DATA_CONVERSION&)
    {
        throw TypeMismatch();
    }

    //
    // Raise InvalidValue if this DynamicAny::DynFixed is incapable of
    // representing the value (even with a loss of precision)
    //
    if((f.fixed_digits() - f.fixed_scale()) >
       (origType_ -> fixed_digits() - origType_ -> fixed_scale()))
        throw InvalidValue();

    //
    // Return true if there was no loss of precision, otherwise
    // truncate and return false
    //
    bool result = true;
    if(f.fixed_scale() > origType_ -> fixed_scale())
    {
        value_ = f.truncate(origType_ -> fixed_scale());
        result = false;
    }
    else
    {
        value_ = f;
        value_.setScale(origType_ -> fixed_scale());
    }

    notifyParent();

    return result;
}

// ----------------------------------------------------------------------
// DynFixed_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynFixed_impl::_OB_marshal(OutputStreamImpl* out)
{
    out -> write_fixed(value_.movePointRight(origType_ -> fixed_scale()));
}

void
DynamicAny::DynFixed_impl::_OB_unmarshal(InputStreamImpl* in)
{
    value_ = in -> read_fixed().movePointLeft(origType_ -> fixed_scale());

    notifyParent();
}

Any*
DynamicAny::DynFixed_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return 0;
}

Any*
DynamicAny::DynFixed_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynamicAny::DynEnum_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynEnum_impl::DynEnum_impl(DynamicAny::DynAnyFactory_ptr factory,
				       ORBInstance_ptr orbInstance,
				       TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc), value_(0)
{
}

DynamicAny::DynEnum_impl::~DynEnum_impl()
{
}

// ----------------------------------------------------------------------
// DynamicAny::DynEnum_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynEnum_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynamicAny::DynEnum* dyn_enum =
	dynamic_cast<DynamicAny::DynEnum*>(dyn_any);
    assert(dyn_enum != 0);
    value_ = dyn_enum -> get_as_ulong();


    notifyParent();
}

void
DynamicAny::DynEnum_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(value.value() == 0)
        throw InvalidValue();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    ULong* v = (ULong*)value.value();
    value_ = *v;

    notifyParent();
}

Any*
DynamicAny::DynEnum_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return new Any(type_, new ULong(value_), true);
}

Boolean
DynamicAny::DynEnum_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    DynamicAny::DynEnum* dyn_enum =
	dynamic_cast<DynamicAny::DynEnum*>(dyn_any);
    assert(dyn_enum != 0);
    return (value_ == dyn_enum -> get_as_ulong());
}

DynamicAny::DynAny_ptr
DynamicAny::DynEnum_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynEnum_impl* result = new DynEnum_impl(factory_, orbInstance_, type_);
    result -> value_ = value_;
    return result;
}

Boolean
DynamicAny::DynEnum_impl::seek(Long)
    throw(SystemException)
{
    return false;
}

void
DynamicAny::DynEnum_impl::rewind()
    throw(SystemException)
{
    // do nothing
}

Boolean
DynamicAny::DynEnum_impl::next()
    throw(SystemException)
{
    return false;
}

ULong
DynamicAny::DynEnum_impl::component_count()
    throw(SystemException)
{
    return 0;
}

DynamicAny::DynAny_ptr
DynamicAny::DynEnum_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    throw TypeMismatch();

    return DynAny::_nil(); // some compilers need this
}

char*
DynamicAny::DynEnum_impl::get_as_string()
    throw(SystemException)
{
    return string_dup(origType_ -> member_name(value_));
}

void
DynamicAny::DynEnum_impl::set_as_string(const char* value)
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
#if defined(__GNUC__) && defined(__OPTIMIZE__) // COMPILERFIX
    volatile int dummy = 1;
    dummy++;
#endif

    ULong count = origType_ -> member_count();
    for(ULong i = 0 ; i < count ; i++)
    {
        if(strcmp(value, origType_ -> member_name(i)) == 0)
        {
            value_ = i;
            notifyParent();
            return;
        }
    }

    throw InvalidValue();
}

ULong
DynamicAny::DynEnum_impl::get_as_ulong()
    throw(SystemException)
{
    return value_;
}

void
DynamicAny::DynEnum_impl::set_as_ulong(ULong value)
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(value >= origType_ -> member_count())
        throw InvalidValue();

    value_ = value;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynEnum_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynEnum_impl::_OB_marshal(OutputStreamImpl* out)
{
    out -> write_ulong(value_);
}

void
DynamicAny::DynEnum_impl::_OB_unmarshal(InputStreamImpl* in)
{
    value_ = in -> read_ulong();

    notifyParent();
}

Any*
DynamicAny::DynEnum_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    return 0;
}

Any*
DynamicAny::DynEnum_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynStruct_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynStruct_impl::DynStruct_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc)
{
    ULong count = origType_ -> member_count();
    components_.length(count);

    for(ULong i = 0 ; i < count ; i++)
    {
        TypeCode_ptr memberType = origType_ -> _OB_memberType_nodup(i);
        components_[i] = create(memberType, true);
    }

    if(count == 0) // empty exception
        index_ = -1;
    else
        index_ = 0;
}

DynamicAny::DynStruct_impl::~DynStruct_impl()
{
}

// ----------------------------------------------------------------------
// DynStruct_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynStruct_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    assert(components_.length() == dyn_any -> component_count());

    dyn_any -> rewind();
    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        DynAny_var comp = dyn_any -> current_component();
        components_[i] -> assign(comp);
        dyn_any -> next();
    }

    if(components_.length() == 0) // empty exception
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

void
DynamicAny::DynStruct_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(value.value() == 0)
        throw InvalidValue();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    InputStream_var in;

    if(origType_ -> _OB_isSystemException())
    {
        SystemException* ex = (SystemException*)value.value();
        OCI::BufferImpl buf;
	OutputStreamImpl out(&buf);
        ex -> _OB_marshal(&out);
        in = out.create_input_stream();
    }
    else if(value.info())
    {
        const ConstructedInfo* info = value.info();
        OCI::BufferImpl buf;
	OutputStreamImpl out(&buf);
        info -> marshal(value.value(), &out);
        in = out.create_input_stream();
    }
    else
    {
        in = InputStream::_duplicate((InputStream_ptr)value.value());
        in -> _OB_reset();
    }

    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    if(components_.length() == 0) // empty exception
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

Any*
DynamicAny::DynStruct_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    Any* result;
    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    _OB_marshal(&out);
    InputStream_var in = out.create_input_stream();
    if(origType_ -> _OB_isSystemException())
    {
        SystemException* ex = SystemException::_OB_unmarshal(in);
        result = new Any(type_, ex, true);
    }
    else
    {
        result = new Any(type_, in._retn(), true);
    }
    result -> _OB_ORBInstance(orbInstance_);
    return result;
}

Boolean
DynamicAny::DynStruct_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    dyn_any -> rewind();
    try
    {
	for(ULong i = 0 ; i < components_.length() ; i++)
	{
	    DynAny_var comp = dyn_any -> current_component();
	    if(!components_[i] -> equal(comp))
		return false;
	    dyn_any -> next();
	}
    }
    catch(DynAny::TypeMismatch&)
    {
	return false;
    }

    return true;
}

DynamicAny::DynAny_ptr
DynamicAny::DynStruct_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynStruct_impl* result = new DynStruct_impl(factory_, orbInstance_, type_);

    for(ULong i = 0 ; i < components_.length() ; i++)
        result -> components_[i] -> assign(components_[i]);

    return result;
}

Boolean
DynamicAny::DynStruct_impl::seek(Long index)
    throw(SystemException)
{
    if(index < 0 || index >= (Long)components_.length())
    {
        index_ = -1;
        return false;
    }

    index_ = index;
    return true;
}

void
DynamicAny::DynStruct_impl::rewind()
    throw(SystemException)
{
    seek(0);
}

Boolean
DynamicAny::DynStruct_impl::next()
    throw(SystemException)
{
    if(index_ + 1 >= (Long)components_.length())
    {
        index_ = -1;
        return false;
    }

    index_++;
    return true;
}

ULong
DynamicAny::DynStruct_impl::component_count()
    throw(SystemException)
{
    return components_.length();
}

DynamicAny::DynAny_ptr
DynamicAny::DynStruct_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(components_.length() == 0) // empty exception
        throw TypeMismatch();

    if(index_ == -1)
        return DynAny::_nil();

    return DynAny::_duplicate(components_[index_]);
}

DynamicAny::FieldName
DynamicAny::DynStruct_impl::current_member_name()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(components_.length() == 0) // empty exception
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    return string_dup(origType_ -> member_name(index_));
}

TCKind
DynamicAny::DynStruct_impl::current_member_kind()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(components_.length() == 0) // empty exception
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    TypeCode_ptr memberTC = origType_ -> _OB_memberType_nodup(index_);
    TypeCode_ptr origMemberTC = GetOrigType_nodup(memberTC);
    return origMemberTC -> kind();
}

DynamicAny::NameValuePairSeq*
DynamicAny::DynStruct_impl::get_members()
    throw(SystemException)
{
    NameValuePairSeq* result = new NameValuePairSeq;
    result -> length(components_.length());

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        (*result)[i].id = string_dup(origType_ -> member_name(i));
        Any_var any = components_[i] -> to_any();
        (*result)[i].value = any;
    }

    return result;
}

void
DynamicAny::DynStruct_impl::set_members(
    const DynamicAny::NameValuePairSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;

    if(value.length() != components_.length())
        throw InvalidValue();

    //
    // Prior to modifying our components, validate the supplied
    // name-value pairs to check for matching member names and
    // compatible TypeCodes
    //
    for(i = 0 ; i < components_.length() ; i++)
    {
        const char* name = origType_ -> member_name(i);

        if(strlen(value[i].id) > 0 && strlen(name) > 0 &&
           strcmp(value[i].id, name) != 0)
            throw TypeMismatch();

        TypeCode_var valueType = value[i].value.type();
        TypeCode_var memberType = components_[i] -> type();
        if(!valueType -> equivalent(memberType))
            throw TypeMismatch();
    }

    for(i = 0 ; i < components_.length() ; i++)
        components_[i] -> from_any(value[i].value);

    if(components_.length() == 0) // empty exception
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

DynamicAny::NameDynAnyPairSeq*
DynamicAny::DynStruct_impl::get_members_as_dyn_any()
    throw(SystemException)
{
    NameDynAnyPairSeq* result = new NameDynAnyPairSeq;
    result -> length(components_.length());

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        (*result)[i].id = string_dup(origType_ -> member_name(i));
        (*result)[i].value = DynAny::_duplicate(components_[i]);
    }

    return result;
}

void
DynamicAny::DynStruct_impl::set_members_as_dyn_any(
    const DynamicAny::NameDynAnyPairSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;

    if(value.length() != components_.length())
        throw InvalidValue();

    //
    // Prior to modifying our components, validate the supplied
    // name-value pairs to check for matching member names and
    // compatible TypeCodes
    //
    for(i = 0 ; i < components_.length() ; i++)
    {
        const char* name = origType_ -> member_name(i);

        if(strlen(value[i].id) > 0 && strlen(name) > 0 &&
           strcmp(value[i].id, name) != 0)
            throw TypeMismatch();

        DynAny_ptr p = value[i].value;
        TypeCode_var valueType = p -> type();
        TypeCode_var memberType = components_[i] -> type();
        if(!valueType -> equivalent(memberType))
            throw TypeMismatch();
    }

    for(i = 0 ; i < components_.length() ; i++)
        components_[i] -> assign(value[i].value);

    if(components_.length() == 0) // empty exception
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynStruct_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynStruct_impl::_OB_marshal(OutputStreamImpl* out)
{
    if(origType_ -> kind() == tk_except)
        out -> write_string(origType_ -> id());

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        DynAny_ptr p = components_[i];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_marshal(out);
    }
}

void
DynamicAny::DynStruct_impl::_OB_unmarshal(InputStreamImpl* in)
{
    if(origType_ -> kind() == tk_except)
    {
        String_var id = in -> read_string();
    }

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        DynAny_ptr p = components_[i];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_unmarshal(in);
    }

    notifyParent();
}

Any*
DynamicAny::DynStruct_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(index_ >= 0 && index_ < (Long)components_.length())
    {
        DynAny_ptr p = components_[index_];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        return impl -> _OB_currentAnyValue();
    }
    else
        return 0;
}

Any*
DynamicAny::DynStruct_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynamicAny::DynUnion_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynUnion_impl::DynUnion_impl(DynamicAny::DynAnyFactory_ptr factory,
					 ORBInstance_ptr orbInstance,
					 TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc)
{
    TypeCode_ptr discTC = origType_ -> _OB_discriminatorType_nodup();

    index_ = 0;
    defaultMember_ = origType_ -> default_index();
    selectedMember_ = 0;
    origDiscTC_ = GetOrigType_nodup(discTC);

    ULong count = origType_ -> member_count();
    memberLabels_.length(count);
    for(ULong i = 0 ; i < count ; i++)
    {
        if((Long)i != defaultMember_)
        {
            Any_var any = origType_ -> member_label(i);
            memberLabels_[i] = getDiscriminatorValue(any);
        }
    }

    if(defaultMember_ != -1)
        memberLabels_[defaultMember_] = findUnusedDiscriminator();

    disc_ = create(discTC, true);
    ignoreDiscChange_ = true;
    resetDiscriminator(memberLabels_[selectedMember_]);

    TypeCode_ptr memberType = 
	origType_ -> _OB_memberType_nodup(selectedMember_);
    member_ = create(memberType, true);
}

DynamicAny::DynUnion_impl::~DynUnion_impl()
{
}

// ----------------------------------------------------------------------
// DynUnion_impl protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynUnion_impl::childModified(DynamicAny::DynAny_ptr p)
{
    //
    // We need to monitor the discriminator component. If it is
    // modified, then we need to initialize the active member
    // appropriately. If ignoreDiscChange_ is true, then we ignore
    // this change to the discriminator.
    //
    if(p == disc_.in())
    {
        if(!ignoreDiscChange_)
        {
            //
            // Act as if set_discriminator had been called
            //

            initMember();

            if(is_nil(member_))
                index_ = 0;
            else
                index_ = 1;

            notifyParent();
        }

        ignoreDiscChange_ = false;
    }
}

void
DynamicAny::DynUnion_impl::initMember()
{
    LongLong discValue;

    if(discriminator_kind() == tk_enum)
    {
        DynEnum_var de = DynEnum::_narrow(disc_);
        assert(!is_nil(de));
        discValue = (LongLong)de -> get_as_ulong();
    }
    else
    {
        DynAny_impl* discImpl = dynamic_cast<DynAny_impl*>(disc_.in());
        Any* discAny = discImpl -> _OB_currentAny();
        assert(discAny != 0);
        discValue = getDiscriminatorValue(*discAny);
    }

    //
    // Search the labels for one matching the discriminator
    //
    ULong i;
    for(i = 0 ; i < memberLabels_.length() ; i++)
        if(discValue == memberLabels_[i])
            break;

    //
    // If no matching label was found, check for a default member.
    // If there's no default member, then we have no member value.
    //
    if(i == memberLabels_.length())
    {
        if(defaultMember_ != -1)
            i = defaultMember_;
        else
        {
            if(!is_nil(member_))
                member_ = DynAny::_nil();
            selectedMember_ = origType_ -> member_count();
            return;
        }
    }

    //
    // If the member names are different, then we deactivate the current
    // member and initialize the new one
    //
    if(selectedMember_ == origType_ -> member_count() ||
       strcmp(origType_ -> member_name(selectedMember_),
              origType_ -> member_name(i)) != 0)
    {
        TypeCode_ptr memberType = origType_ -> _OB_memberType_nodup(i);
        member_ = create(memberType, true);
    }
    selectedMember_ = i;
}

void
DynamicAny::DynUnion_impl::resetDiscriminator(LongLong val)
{
    TCKind kind = discriminator_kind();

    //
    // Insert the default discriminator value
    //
    switch(kind)
    {
    case tk_boolean:
        disc_ -> insert_boolean(val == 1);
        break;

    case tk_char:
        disc_ -> insert_char((Char)val);
        break;

    case tk_short:
        disc_ -> insert_short((Short)val);
        break;

    case tk_ushort:
        disc_ -> insert_ushort((UShort)val);
        break;

    case tk_long:
        disc_ -> insert_long((Long)val);
        break;

    case tk_ulong:
        disc_ -> insert_ulong((ULong)val);
        break;

    case tk_longlong:
        disc_ -> insert_longlong(val);
        break;

    case tk_ulonglong:
        disc_ -> insert_ulonglong((ULongLong)val);
        break;

    case tk_enum:
    {
        DynEnum_var e = DynEnum::_narrow(disc_);
        assert(!is_nil(e));
        e -> set_as_ulong((ULong)val);
        break;
    }

    default:
        assert(false);
    }
}

LongLong
DynamicAny::DynUnion_impl::getDiscriminatorValue(const Any& any)
{
    TCKind kind = discriminator_kind();
    LongLong result;

    switch(kind)
    {
    case tk_boolean:
    {
        Boolean* b = (Boolean*)any.value();
        result = *b ? 1 : 0;
        break;
    }

    case tk_char:
    {
        Char* c = (Char*)any.value();
        result = (LongLong)*c;
        break;
    }

    case tk_short:
    {
        Short* s = (Short*)any.value();
        result = (LongLong)*s;
        break;
    }

    case tk_ushort:
    {
        UShort* s = (UShort*)any.value();
        result = (LongLong)*s;
        break;
    }

    case tk_long:
    {
        Long* l = (Long*)any.value();
        result = (LongLong)*l;
        break;
    }

    case tk_ulong:
    {
        ULong* l = (ULong*)any.value();
        result = (LongLong)*l;
        break;
    }

    case tk_longlong:
    {
        LongLong* l = (LongLong*)any.value();
        result = *l;
        break;
    }

    case tk_ulonglong:
    {
        ULongLong* l = (ULongLong*)any.value();
        result = (LongLong)*l;
        break;
    }

    case tk_enum:
    {
        ULong* p = (ULong*)any.value();
        result = (LongLong)*p;
        break;
    }

    default:
        assert(false);
    }

    return result;
}

LongLong
DynamicAny::DynUnion_impl::findUnusedDiscriminator()
{
    //
    // Find an unused value among the member labels
    //

    TCKind kind = discriminator_kind();
    LongLong min, max;

    switch(kind)
    {
    case tk_boolean:
        min = 0;
        max = 1;
        break;

    case tk_char:
        min = 0;
        max = 255;
        break;

    case tk_short:
        min = -32768;
        max = 32767;
        break;

    case tk_ushort:
        min = 0;
        max = 65535;
        break;

    case tk_long:
#ifndef HAVE_VCPLUSPLUS_BUGS
        min = -2147483648LL;
        max = 2147483647LL;
#else
        min = -2147483647 - 1;
        max = 2147483647;
#endif
        break;

    case tk_ulong:
        min = 0;
#ifndef HAVE_VCPLUSPLUS_BUGS
        max = 4294967294LL;
#else
        max = 2147483647;
#endif
        break;

    case tk_longlong:
#ifndef HAVE_VCPLUSPLUS_BUGS
        //min = -9223372036854775808LL; // egcs complains about this
        min = -9223372036854775807LL;
        max = 9223372036854775807LL;
#else
        min = -2147483647 - 1;
        max = 2147483647;
#endif
        break;

    case tk_ulonglong:
        min = 0;
#ifndef HAVE_VCPLUSPLUS_BUGS
        max = 9223372036854775807LL;
#else
        max = 2147483647;
#endif
        break;

    case tk_enum:
    {
        min = 0;
        max = (LongLong)(origDiscTC_ -> member_count() - 1);
        break;
    }

    default:
        assert(false);
    }

    for(LongLong i = max ; i >= min ; i--)
    {
        bool found = false;
        for(ULong j = 0 ; !found && j < memberLabels_.length() ; j++)
            if(Long(j) != defaultMember_ && memberLabels_[j] == i)
                found = true;

        if(!found)
            return i;
    }

    //
    // No value found
    //
    throw TypeMismatch();

    return 0; // some compilers need this
}

// ----------------------------------------------------------------------
// DynUnion_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynUnion_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynUnion_var du = DynUnion::_narrow(dyn_any);
    assert(!is_nil(du));
    DynUnion_impl* impl = dynamic_cast<DynUnion_impl*>(du.in());

    selectedMember_ = impl -> selectedMember_;
    ignoreDiscChange_ = true;
    disc_ -> assign(impl -> disc_);

    if(!is_nil(member_))
        member_ = DynAny::_nil();

    if(!is_nil(impl -> member_))
    {
        member_ = impl -> member_ -> copy();
        adoptChild(member_);
    }

    index_ = 0;

    notifyParent();
}

void
DynamicAny::DynUnion_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(value.value() == 0)
        throw InvalidValue();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    InputStream_var in;

    if(value.info())
    {
        const ConstructedInfo* info = value.info();
        OCI::BufferImpl buf;
	OutputStreamImpl out(&buf);
        info -> marshal(value.value(), &out);
        in = out.create_input_stream();
    }
    else
    {
        in = InputStream::_duplicate((InputStream_ptr)value.value());
        in -> _OB_reset();
    }

    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    index_ = 0;

    notifyParent();
}

Any*
DynamicAny::DynUnion_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    _OB_marshal(&out);
    InputStream_ptr in = out.create_input_stream();
    Any* result = new Any(type_, in, true);
    result -> _OB_ORBInstance(orbInstance_);
    return result;
}

Boolean
DynamicAny::DynUnion_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    if(component_count() != dyn_any -> component_count())
        return false;

    DynUnion_var du = DynUnion::_narrow(dyn_any);
    assert(!is_nil(du));

    DynAny_var d = du -> get_discriminator();
    if(!disc_ -> equal(d))
        return false;

    if(!is_nil(member_))
    {
        d = du -> member();
        if(!member_ -> equal(d))
            return false;
    }

    return true;
}

DynamicAny::DynAny_ptr
DynamicAny::DynUnion_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynamicAny::DynUnion_impl* result =
	new DynamicAny::DynUnion_impl(factory_, orbInstance_, type_);
    result -> set_discriminator(disc_);

    if(!is_nil(member_))
    {
        DynAny_var comp = result -> member();
        comp -> assign(member_);
    }

    return result;
}

Boolean
DynamicAny::DynUnion_impl::seek(Long index)
    throw(SystemException)
{
    Long max = is_nil(member_) ? 1 : 2;

    if(index < 0 || index >= max)
    {
        index_ = -1;
        return false;
    }

    index_ = index;
    return true;
}

void
DynamicAny::DynUnion_impl::rewind()
    throw(SystemException)
{
    seek(0);
}

Boolean
DynamicAny::DynUnion_impl::next()
    throw(SystemException)
{
    Long max = is_nil(member_) ? 1 : 2;

    if(index_ + 1 >= max)
    {
        index_ = -1;
        return false;
    }

    index_++;
    return true;
}

ULong
DynamicAny::DynUnion_impl::component_count()
    throw(SystemException)
{
    if(is_nil(member_))
        return 1;
    else
        return 2;
}

DynamicAny::DynAny_ptr
DynamicAny::DynUnion_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(index_ < 0)
        return DynAny::_nil();

    assert(index_ >= 0 && index_ < 2);

    if(index_ == 0)
        return DynAny::_duplicate(disc_);
    else
    {
        assert(!is_nil(member_));
        return DynAny::_duplicate(member_);
    }
}

DynamicAny::DynAny_ptr
DynamicAny::DynUnion_impl::get_discriminator()
    throw(SystemException)
{
    return DynAny::_duplicate(disc_);
}

void
DynamicAny::DynUnion_impl::set_discriminator(DynamicAny::DynAny_ptr d)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    TypeCode_var type = d -> type();
    if(!type -> equivalent(origDiscTC_))
        throw TypeMismatch();

    //
    // Check if the new discriminator is the same as the current one
    //
    if(!d -> equal(disc_))
    {
        //
        // Change the discriminator - the member will be initialized
        // by childModified()
        //
        disc_ -> assign(d);

        notifyParent();
    }
}

void
DynamicAny::DynUnion_impl::set_to_default_member()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(defaultMember_ == -1)
        throw TypeMismatch();

    //
    // Reset the discriminator to select the default member. The
    // member will be initialized by childModified().
    //
    resetDiscriminator(memberLabels_[defaultMember_]);
    index_ = 0;

    notifyParent();
}

void
DynamicAny::DynUnion_impl::set_to_no_active_member()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(defaultMember_ != -1)
        throw TypeMismatch();

    //
    // Reset the discriminator to an unused value. The current
    // member will be deactivated by childModified().
    //
    LongLong val = findUnusedDiscriminator();
    resetDiscriminator(val);
    index_ = 0;

    notifyParent();
}

Boolean
DynamicAny::DynUnion_impl::has_no_active_member()
    throw(SystemException)
{
    return is_nil(member_);
}

TCKind
DynamicAny::DynUnion_impl::discriminator_kind()
    throw(SystemException)
{
    return origDiscTC_ -> kind();
}

DynamicAny::DynAny_ptr
DynamicAny::DynUnion_impl::member()
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_nil(member_))
        throw InvalidValue();

    return DynAny::_duplicate(member_);
}

DynamicAny::FieldName
DynamicAny::DynUnion_impl::member_name()
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_nil(member_))
        throw InvalidValue();

    return string_dup(origType_ -> member_name(selectedMember_));
}

TCKind
DynamicAny::DynUnion_impl::member_kind()
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_nil(member_))
        throw InvalidValue();

    TypeCode_ptr memberTC = origType_ -> _OB_memberType_nodup(selectedMember_);

    return memberTC -> kind();
}

Boolean
DynamicAny::DynUnion_impl::is_set_to_default_member()
    throw(SystemException)
{
    return(defaultMember_ != -1 && (Long)selectedMember_ == defaultMember_);
}

// ----------------------------------------------------------------------
// DynUnion_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynUnion_impl::_OB_marshal(OutputStreamImpl* out)
{
    DynAny_ptr p;
    DynAny_impl* impl;

    p = disc_.in();
    impl = dynamic_cast<DynAny_impl*>(p);
    impl -> _OB_marshal(out);

    if(!is_nil(member_))
    {
        p = member_.in();
        impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_marshal(out);
    }
}

void
DynamicAny::DynUnion_impl::_OB_unmarshal(InputStreamImpl* in)
{
    DynAny_ptr p;
    DynAny_impl* impl;

    p = disc_.in();
    impl = dynamic_cast<DynAny_impl*>(p);
    impl -> _OB_unmarshal(in);

    //
    // The member is initialized by childModified()
    //

    if(!is_nil(member_))
    {
        p = member_.in();
        impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_unmarshal(in);
    }

    notifyParent();
}

Any*
DynamicAny::DynUnion_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynAny_ptr p = DynAny::_nil();

    if(index_ == 0) // discriminator
    {
        p = disc_.in();
    }
    else if(index_ == 1) // member
    {
        assert(!is_nil(member_));
        p = member_.in();
    }

    Any* result = 0;

    if(!is_nil(p))
    {
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        result = impl -> _OB_currentAnyValue();
    }

    return result;
}

Any*
DynamicAny::DynUnion_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynSeqBase_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynSeqBase_impl::DynSeqBase_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc)
{
    contentType_ = origType_ -> _OB_contentType_nodup();
    TypeCode_ptr origContent = GetOrigType_nodup(contentType_);
    contentKind_ = origContent -> kind();
    length_ = 0;
    index_ = -1;
    buf_ = 0;
    ignoreChild_ = false;

    //
    // Primitive types are optimized using a primitive array in addition
    // to the components_ sequence. For primitive types, the components
    // are only created as needed.
    //
    switch(contentKind_)
    {
    case tk_short:
    case tk_long:
    case tk_ushort:
    case tk_ulong:
    case tk_float:
    case tk_double:
    case tk_boolean:
    case tk_char:
    case tk_octet:
    case tk_string:
    case tk_longlong:
    case tk_ulonglong:
    case tk_longdouble:
    case tk_wchar:
    case tk_wstring:
        primitive_ = true;
        break;

    default:
        primitive_ = false;
        break;
    }

    //
    // An array must be initialized to its proper length
    //
    max_ = origType_ -> length();
    if(origType_ -> kind() == tk_array)
    {
        resize(max_, true);
        index_ = 0;
    }
}

DynamicAny::DynSeqBase_impl::~DynSeqBase_impl()
{
    //
    // Destroy primitive array
    //
    switch(contentKind_)
    {
    case tk_short:
    {
        Short* buf = (Short*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_long:
    {
        Long* buf = (Long*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_ushort:
    {
        UShort* buf = (UShort*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_ulong:
    {
        ULong* buf = (ULong*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_float:
    {
        Float* buf = (Float*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_double:
    {
        Double* buf = (Double*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_boolean:
    {
        Boolean* buf = (Boolean*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_char:
    {
        Char* buf = (Char*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_octet:
    {
        Octet* buf = (Octet*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_string:
    {
        OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
        if(seq)
            delete seq;
        break;
    }

    case tk_longlong:
    {
        LongLong* buf = (LongLong*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_ulonglong:
    {
        ULongLong* buf = (ULongLong*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_longdouble:
    {
        LongDouble* buf = (LongDouble*)buf_;
        if(buf)
            delete []buf;
        break;
    }

    case tk_wchar:
    {
        OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
        if(seq)
            delete seq;
        break;
    }

    case tk_wstring:
    {
        OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
        if(seq)
            delete seq;
        break;
    }

    default:
        // nothing to do
        break;
    }
}

// ----------------------------------------------------------------------
// DynSeqBase_impl protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynSeqBase_impl::childModified(DynamicAny::DynAny_ptr p)
{
    if(ignoreChild_)
    {
        ignoreChild_ = false;
        return;
    }

    //
    // If this object holds a sequence of primitive types, we need
    // to keep the optimized array and the components in sync
    //
    if(primitive_)
    {
        Long i;
        for(i = 0 ; i < length_ ; i++)
            if(components_[i].in() == p)
                break;
        assert(i < length_);

        setValue(i, p);

        notifyParent();
    }
}

void
DynamicAny::DynSeqBase_impl::validate(TCKind kind)
{
    if(kind != contentKind_)
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    assert(length_ > 0);
}

void
DynamicAny::DynSeqBase_impl::getValue(ULong index, Any& any)
{
    //
    // Get the value from the primitive array at the given index
    // and insert it into the any
    //

    assert(Long(index) < length_);
    assert(primitive_);

    switch(contentKind_)
    {
    case tk_short:
    {
        Short* buf = (Short*)buf_;
        any.replace(contentType_, new Short(buf[index]), true);
        break;
    }

    case tk_long:
    {
        Long* buf = (Long*)buf_;
        any.replace(contentType_, new Long(buf[index]), true);
        break;
    }

    case tk_ushort:
    {
        UShort* buf = (UShort*)buf_;
        any.replace(contentType_, new UShort(buf[index]), true);
        break;
    }

    case tk_ulong:
    {
        ULong* buf = (ULong*)buf_;
        any.replace(contentType_, new ULong(buf[index]), true);
        break;
    }

    case tk_float:
    {
        Float* buf = (Float*)buf_;
        any.replace(contentType_, new Float(buf[index]), true);
        break;
    }

    case tk_double:
    {
        Double* buf = (Double*)buf_;
        any.replace(contentType_, new Double(buf[index]), true);
        break;
    }

    case tk_boolean:
    {
        Boolean* buf = (Boolean*)buf_;
        any.replace(contentType_, new Boolean(buf[index]), true);
        break;
    }

    case tk_char:
    {
        Char* buf = (Char*)buf_;
        any.replace(contentType_, new Char(buf[index]), true);
        break;
    }

    case tk_octet:
    {
        Octet* buf = (Octet*)buf_;
        any.replace(contentType_, new Octet(buf[index]), true);
        break;
    }

    case tk_string:
    {
        OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
        any.replace(contentType_, string_dup((*seq)[index]), true);
        break;
    }

    case tk_longlong:
    {
        LongLong* buf = (LongLong*)buf_;
        any.replace(contentType_, new LongLong(buf[index]), true);
        break;
    }

    case tk_ulonglong:
    {
        ULongLong* buf = (ULongLong*)buf_;
        any.replace(contentType_, new ULongLong(buf[index]), true);
        break;
    }

    case tk_longdouble:
    {
        LongDouble* buf = (LongDouble*)buf_;
        any.replace(contentType_, new LongDouble(buf[index]), true);
        break;
    }

    case tk_wchar:
    {
        OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
        any.replace(contentType_, new WChar((*seq)[index]), true);
        break;
    }

    case tk_wstring:
    {
        OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
        any.replace(contentType_, wstring_dup((*seq)[index]), true);
        break;
    }

    default:
        assert(false);
    }
}

void
DynamicAny::DynSeqBase_impl::setValue(ULong index, const Any& any)
{
    //
    // Set the value of the primitive array and/or component from
    // the contents of the any
    //

    assert(Long(index) < length_);

    if(!is_nil(components_[index]))
        components_[index] -> from_any(any);
    else
    {
        switch(contentKind_)
        {
        case tk_short:
        {
            Short* buf = (Short*)buf_;
            buf[index] = *((Short*)any.value());
            break;
        }

        case tk_long:
        {
            Long* buf = (Long*)buf_;
            buf[index] = *((Long*)any.value());
            break;
        }

        case tk_ushort:
        {
            UShort* buf = (UShort*)buf_;
            buf[index] = *((UShort*)any.value());
            break;
        }

        case tk_ulong:
        {
            ULong* buf = (ULong*)buf_;
            buf[index] = *((ULong*)any.value());
            break;
        }

        case tk_float:
        {
            Float* buf = (Float*)buf_;
            buf[index] = *((Float*)any.value());
            break;
        }

        case tk_double:
        {
            Double* buf = (Double*)buf_;
            buf[index] = *((Double*)any.value());
            break;
        }

        case tk_boolean:
        {
            Boolean* buf = (Boolean*)buf_;
            buf[index] = *((Boolean*)any.value());
            break;
        }

        case tk_char:
        {
            Char* buf = (Char*)buf_;
            buf[index] = *((Char*)any.value());
            break;
        }

        case tk_octet:
        {
            Octet* buf = (Octet*)buf_;
            buf[index] = *((Octet*)any.value());
            break;
        }

        case tk_string:
        {
            OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
            (*seq)[index] = (const char*)any.value();
            break;
        }

        case tk_longlong:
        {
            LongLong* buf = (LongLong*)buf_;
            buf[index] = *((LongLong*)any.value());
            break;
        }

        case tk_ulonglong:
        {
            ULongLong* buf = (ULongLong*)buf_;
            buf[index] = *((ULongLong*)any.value());
            break;
        }

        case tk_longdouble:
        {
            LongDouble* buf = (LongDouble*)buf_;
            buf[index] = *((LongDouble*)any.value());
            break;
        }

        case tk_wchar:
        {
            OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
            (*seq)[index] = *((WChar*)any.value());
            break;
        }

        case tk_wstring:
        {
            OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
            (*seq)[index] = (const WChar*)any.value();
            break;
        }

        default:
            assert(false);
        }
    }
}

void
DynamicAny::DynSeqBase_impl::setValue(ULong index, DynamicAny::DynAny_ptr p)
{
    //
    // Set value from a DynAny object. We update our primitive
    // array and our component.
    //

    if(primitive_)
    {
        switch(contentKind_)
        {
        case tk_short:
        {
            Short* buf = (Short*)buf_;
            buf[index] = p -> get_short();
            break;
        }

        case tk_long:
        {
            Long* buf = (Long*)buf_;
            buf[index] = p -> get_long();
            break;
        }

        case tk_ushort:
        {
            UShort* buf = (UShort*)buf_;
            buf[index] = p -> get_ushort();
            break;
        }

        case tk_ulong:
        {
            ULong* buf = (ULong*)buf_;
            buf[index] = p -> get_ulong();
            break;
        }

        case tk_float:
        {
            Float* buf = (Float*)buf_;
            buf[index] = p -> get_float();
            break;
        }

        case tk_double:
        {
            Double* buf = (Double*)buf_;
            buf[index] = p -> get_double();
            break;
        }

        case tk_boolean:
        {
            Boolean* buf = (Boolean*)buf_;
            buf[index] = p -> get_boolean();
            break;
        }

        case tk_char:
        {
            Char* buf = (Char*)buf_;
            buf[index] = p -> get_char();
            break;
        }

        case tk_octet:
        {
            Octet* buf = (Octet*)buf_;
            buf[index] = p -> get_octet();
            break;
        }

        case tk_string:
        {
            OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
            (*seq)[index] = p -> get_string();
            break;
        }

        case tk_longlong:
        {
            LongLong* buf = (LongLong*)buf_;
            buf[index] = p -> get_longlong();
            break;
        }

        case tk_ulonglong:
        {
            ULongLong* buf = (ULongLong*)buf_;
            buf[index] = p -> get_ulonglong();
            break;
        }

        case tk_longdouble:
        {
            LongDouble* buf = (LongDouble*)buf_;
            buf[index] = p -> get_longdouble();
            break;
        }

        case tk_wchar:
        {
            OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
            (*seq)[index] = p -> get_wchar();
            break;
        }

        case tk_wstring:
        {
            OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
            (*seq)[index] = p -> get_wstring();
            break;
        }

        default:
            assert(false);
        }
    }

    if(!is_nil(components_[index]) && components_[index].in() != p)
    {
        ignoreChild_ = true;
        components_[index] -> assign(p);
    }
}

void
DynamicAny::DynSeqBase_impl::updateComponent(ULong index, bool createComp)
{
    //
    // Update the component with the value from the primitive array.
    // If desired, the component will be created first.
    //

    if(is_nil(components_[index]))
    {
        if(createComp)
            components_[index] = create(contentType_, true);
        else
            return;
    }

    Any any;
    getValue(index, any);
    ignoreChild_ = true;
    components_[index] -> from_any(any);
}

void
DynamicAny::DynSeqBase_impl::resize(ULong len, bool init)
{
    if(primitive_ && Long(len) > length_)
    {
        switch(contentKind_)
        {
        case tk_short:
        {
            Short* oldBuf = (Short*)buf_;
            if(len)
            {
                Short* buf = new Short[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Short));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Short));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_long:
        {
            Long* oldBuf = (Long*)buf_;
            if(len)
            {
                Long* buf = new Long[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Long));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Long));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_ushort:
        {
            UShort* oldBuf = (UShort*)buf_;
            if(len)
            {
                UShort* buf = new UShort[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(UShort));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(UShort));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_ulong:
        {
            ULong* oldBuf = (ULong*)buf_;
            if(len)
            {
                ULong* buf = new ULong[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(ULong));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(ULong));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_float:
        {
            Float* oldBuf = (Float*)buf_;
            if(len)
            {
                Float* buf = new Float[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Float));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Float));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_double:
        {
            Double* oldBuf = (Double*)buf_;
            if(len)
            {
                Double* buf = new Double[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Double));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Double));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_boolean:
        {
            Boolean* oldBuf = (Boolean*)buf_;
            if(len)
            {
                Boolean* buf = new Boolean[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Boolean));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Boolean));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_char:
        {
            Char* oldBuf = (Char*)buf_;
            if(len)
            {
                Char* buf = new Char[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Char));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Char));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_octet:
        {
            Octet* oldBuf = (Octet*)buf_;
            if(len)
            {
                Octet* buf = new Octet[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(Octet));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(Octet));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_string:
        {
            OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
            if(seq == 0)
            {
                seq = new OB::StrSeq< int >;
                buf_ = seq;
            }
            seq -> length(len);
            if(init)
            {
                for(ULong i = length_ ; i < len ; i++)
                    (*seq)[i] = string_dup("");
            }
            break;
        }

        case tk_longlong:
        {
            LongLong* oldBuf = (LongLong*)buf_;
            if(len)
            {
                LongLong* buf = new LongLong[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(LongLong));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(LongLong));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_ulonglong:
        {
            ULongLong* oldBuf = (ULongLong*)buf_;
            if(len)
            {
                ULongLong* buf = new ULongLong[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(ULongLong));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(ULongLong));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_longdouble:
        {
            LongDouble* oldBuf = (LongDouble*)buf_;
            if(len)
            {
                LongDouble* buf = new LongDouble[len];
                if(init)
                {
                    memset(buf, 0, len * sizeof(LongDouble));
                    if(oldBuf)
                        memcpy(buf, oldBuf, length_ * sizeof(LongDouble));
                }
                buf_ = buf;
            }
            else
                buf_ = 0;
            delete []oldBuf;
            break;
        }

        case tk_wchar:
        {
            OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
            if(seq == 0)
            {
                seq = new OB::FixSeq< WChar, int >;
                buf_ = seq;
            }
            seq -> length(len);
            break;
        }

        case tk_wstring:
        {
            OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
            if(seq == 0)
            {
                seq = new OB::WStrSeq< int >;
                buf_ = seq;
            }
            seq -> length(len);
            if(init)
            {
                for(ULong i = length_ ; i < len ; i++)
                    (*seq)[i] = wstring_dup(L"");
            }
            break;
        }

        default:
            assert(false);
            break;
        }
    }

    //
    // Update the components_ sequence. Although we use a separate
    // array for primitive types, we still keep the length of the
    // components_ sequence consistent. New elements are initialized
    // to nil for primitive types.
    //

    if(Long(len) > length_) // increase length
    {
        components_.length(len);
        for(ULong i = length_ ; i < len ; i++)
        {
            if(!primitive_)
                components_[i] = create(contentType_, true);
            else
                components_[i] = DynAny::_nil();
        }
    }
    else if(Long(len) < length_) // decrease length
    {
        components_.length(len);
    }

    //
    // Update the current position, if necessary
    //

    if(Long(len) > length_) // length was increased
    {
        if(index_ == -1)
            index_ = length_; // first newly-added element
    }
    else if(Long(len) < length_) // length was decreased
    {
        if(len == 0 || index_ >= Long(len))
            index_ = -1;
    }

    length_ = len;
}

DynamicAny::AnySeq*
DynamicAny::DynSeqBase_impl::getElements()
{
    AnySeq* result = new AnySeq;
    result -> length(length_);
    for(ULong i = 0 ; Long(i) < length_ ; i++)
    {
        if(primitive_)
            getValue(i, (*result)[i]);
        else
        {
            Any_var av = components_[i] -> to_any();
            (*result)[i] = av;
        }
    }

    return result;
}

DynamicAny::DynAnySeq*
DynamicAny::DynSeqBase_impl::getElementsAsDynAny()
{
    if(primitive_)
    {
        for(Long i = 0 ; i < length_ ; i++)
        {
            if(is_nil(components_[i]))
                updateComponent(i, true);
        }
    }

    return new DynAnySeq(components_);
}

// ----------------------------------------------------------------------
// DynSeqBase_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynSeqBase_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynAny_impl* impl =	dynamic_cast<DynAny_impl*>(dyn_any);
    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    impl -> _OB_marshal(&out);
    InputStream_var in = out.create_input_stream();
    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(value.value() == 0)
        throw InvalidValue();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    InputStream_var in;

    if(value.info())
    {
        const ConstructedInfo* info = value.info();
	OCI::BufferImpl buf;
	OutputStreamImpl out(&buf);
        info -> marshal(value.value(), &out);
        in = out.create_input_stream();
    }
    else
    {
        in = InputStream::_duplicate((InputStream_ptr)value.value());
        in -> _OB_reset();
    }

    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    index_ = 0;

    notifyParent();
}

Any*
DynamicAny::DynSeqBase_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    _OB_marshal(&out);
    InputStream_ptr in = out.create_input_stream();
    Any* result = new Any(type_, in, true);
    result -> _OB_ORBInstance(orbInstance_);
    return result;
}

Boolean
DynamicAny::DynSeqBase_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    if(length_ != (Long)dyn_any -> component_count())
        return false;

    if(length_ == 0)
        return true;

    DynSeqBase_impl* seq =
	dynamic_cast<DynSeqBase_impl*>(dyn_any);
    assert(seq != 0);

    if(primitive_)
    {
        switch(contentKind_)
        {
        case tk_short:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Short)) != 0)
                return false;
            break;

        case tk_long:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Long)) != 0)
                return false;
            break;

        case tk_ushort:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(UShort)) != 0)
                return false;
            break;

        case tk_ulong:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(ULong)) != 0)
                return false;
            break;

        case tk_float:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Float)) != 0)
                return false;
            break;

        case tk_double:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Double)) != 0)
                return false;
            break;

        case tk_boolean:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Boolean)) != 0)
                return false;
            break;

        case tk_char:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Char)) != 0)
                return false;
            break;

        case tk_octet:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(Octet)) != 0)
                return false;
            break;

        case tk_string:
        {
            OB::StrSeq< int >* seq1 = (OB::StrSeq< int >*)buf_;
            OB::StrSeq< int >* seq2 = (OB::StrSeq< int >*)seq -> buf_;
            for(ULong i = 0 ; Long(i) < length_ ; i++)
            {
                if(strcmp((*seq1)[i], (*seq2)[i]) != 0)
                    return false;
            }
            break;
        }

        case tk_longlong:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(LongLong)) != 0)
                return false;
            break;

        case tk_ulonglong:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(ULongLong)) != 0)
                return false;
            break;

        case tk_longdouble:
            if(memcmp(buf_, seq -> buf_, length_ * sizeof(LongDouble)) != 0)
                return false;
            break;

        case tk_wchar:
        {
            OB::FixSeq< WChar, int >* seq1 = (OB::FixSeq< WChar, int >*)buf_;
            OB::FixSeq< WChar, int >* seq2 = (OB::FixSeq< WChar, int >*)seq -> buf_;
            for(ULong i = 0 ; Long(i) < length_ ; i++)
            {
                if((*seq1)[i] != (*seq2)[i])
                    return false;
            }
            break;
        }

        case tk_wstring:
        {
            OB::WStrSeq< int >* seq1 = (OB::WStrSeq< int >*)buf_;
            OB::WStrSeq< int >* seq2 = (OB::WStrSeq< int >*)seq -> buf_;
            for(ULong i = 0 ; Long(i) < length_ ; i++)
            {
                if(OB::wcscmp((*seq1)[i], (*seq2)[i]) != 0)
                    return false;
            }
            break;
        }

        default:
            assert(false);
            break;
        }
    }
    else
    {
        for(ULong i = 0 ; Long(i) < length_ ; i++)
            if(!components_[i] -> equal(seq -> components_[i]))
                return false;
    }

    return true;
}

DynamicAny::DynAny_ptr
DynamicAny::DynSeqBase_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynAny_var result = create(type_, false);

    DynSeqBase_impl* seq = dynamic_cast<DynSeqBase_impl*>(result.in());
    assert(seq != 0);

    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    _OB_marshal(&out);
    InputStream_var in = out.create_input_stream();
    in -> _OB_ORBInstance(orbInstance_);
    seq -> _OB_unmarshal(in);

    return result._retn();
}

Boolean
DynamicAny::DynSeqBase_impl::seek(Long index)
    throw(SystemException)
{
    if(index >= length_ || index < 0)
    {
        index_ = -1;
        return false;
    }

    index_ = index;
    return true;
}

void
DynamicAny::DynSeqBase_impl::rewind()
    throw(SystemException)
{
    seek(0);
}

Boolean
DynamicAny::DynSeqBase_impl::next()
    throw(SystemException)
{
    index_++;

    if(index_ == length_)
    {
        index_ = -1;
        return false;
    }

    return true;
}

ULong
DynamicAny::DynSeqBase_impl::component_count()
    throw(SystemException)
{
    return length_;
}

DynamicAny::DynAny_ptr
DynamicAny::DynSeqBase_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(index_ < 0)
        return DynAny::_nil();

    //
    // Create component if necessary
    //
    if(is_nil(components_[index_]))
        updateComponent(index_, true);

    return DynAny::_duplicate(components_[index_]);
}

void
DynamicAny::DynSeqBase_impl::insert_boolean(Boolean value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_boolean);

    Boolean* buf = (Boolean*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_octet(Octet value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_octet);

    Octet* buf = (Octet*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_char(Char value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_char);

    Char* buf = (Char*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_short(Short value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_short);

    Short* buf = (Short*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_ushort(UShort value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ushort);

    UShort* buf = (UShort*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_long(Long value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_long);

    Long* buf = (Long*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_ulong(ULong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ulong);

    ULong* buf = (ULong*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_float(Float value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_float);

    Float* buf = (Float*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_double(Double value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_double);

    Double* buf = (Double*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_string(const char* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(value == 0)
        throw InvalidValue();

    validate(tk_string);

    //
    // Check for bounded string
    //
    TypeCode_ptr origContent = GetOrigType_nodup(contentType_);
    ULong len = origContent -> length();
    if(len > 0 && strlen(value) > len)
        throw InvalidValue();

    OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
    (*seq)[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_reference(Object_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_objref);

    //
    // Delegate to component
    //
    components_[index_] -> insert_reference(value);

    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_typecode(TypeCode_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_TypeCode);

    //
    // Delegate to component
    //
    components_[index_] -> insert_typecode(value);

    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_longlong(LongLong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_longlong);

    LongLong* buf = (LongLong*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_ulonglong(ULongLong value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ulonglong);

    ULongLong* buf = (ULongLong*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_longdouble(LongDouble value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
#if SIZEOF_LONG_DOUBLE >= 12

    validate(tk_longdouble);

    LongDouble* buf = (LongDouble*)buf_;
    buf[index_] = value;
    updateComponent(index_, false);
    notifyParent();

#else

    throw InvalidValue();

#endif
}

void
DynamicAny::DynSeqBase_impl::insert_wchar(WChar value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_wchar);

    OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
    (*seq)[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_wstring(const wchar_t* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(value == 0)
        throw InvalidValue();

    validate(tk_wstring);

    TypeCode_ptr origContent = GetOrigType_nodup(contentType_);

    //
    // Check for bounded wstring
    //
    ULong len = origContent -> length();
    if(len > 0 && OB::wcslen(value) > len)
        throw InvalidValue();

    OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
    (*seq)[index_] = value;
    updateComponent(index_, false);
    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_any);

    //
    // Delegate to component
    //
    components_[index_] -> insert_any(value);

    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_dyn_any(DynamicAny::DynAny_ptr value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_nil(value))
        throw TypeMismatch();

    validate(tk_any);

    //
    // Delegate to component
    //
    components_[index_] -> insert_dyn_any(value);

    notifyParent();
}

void
DynamicAny::DynSeqBase_impl::insert_val(ValueBase* value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(contentKind_ != tk_value && contentKind_ != tk_value_box)
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    //
    // Delegate to component
    //
    components_[index_] -> insert_val(value);

    notifyParent();
}

Boolean
DynamicAny::DynSeqBase_impl::get_boolean()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_boolean);

    Boolean* buf = (Boolean*)buf_;
    return buf[index_];
}

Octet
DynamicAny::DynSeqBase_impl::get_octet()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_octet);

    Octet* buf = (Octet*)buf_;
    return buf[index_];
}

Char
DynamicAny::DynSeqBase_impl::get_char()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_char);

    Char* buf = (Char*)buf_;
    return buf[index_];
}

Short
DynamicAny::DynSeqBase_impl::get_short()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_short);

    Short* buf = (Short*)buf_;
    return buf[index_];
}

UShort
DynamicAny::DynSeqBase_impl::get_ushort()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ushort);

    UShort* buf = (UShort*)buf_;
    return buf[index_];
}

Long
DynamicAny::DynSeqBase_impl::get_long()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_long);

    Long* buf = (Long*)buf_;
    return buf[index_];
}

ULong
DynamicAny::DynSeqBase_impl::get_ulong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ulong);

    ULong* buf = (ULong*)buf_;
    return buf[index_];
}

Float
DynamicAny::DynSeqBase_impl::get_float()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_float);

    Float* buf = (Float*)buf_;
    return buf[index_];
}

Double
DynamicAny::DynSeqBase_impl::get_double()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_double);

    Double* buf = (Double*)buf_;
    return buf[index_];
}

char*
DynamicAny::DynSeqBase_impl::get_string()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_string);

    OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
    return string_dup((*seq)[index_]);
}

Object_ptr
DynamicAny::DynSeqBase_impl::get_reference()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_objref);

    //
    // Delegate to component
    //
    return components_[index_] -> get_reference();
}

TypeCode_ptr
DynamicAny::DynSeqBase_impl::get_typecode()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_TypeCode);

    //
    // Delegate to component
    //
    return components_[index_] -> get_typecode();
}

LongLong
DynamicAny::DynSeqBase_impl::get_longlong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_longlong);

    LongLong* buf = (LongLong*)buf_;
    return buf[index_];
}

ULongLong
DynamicAny::DynSeqBase_impl::get_ulonglong()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_ulonglong);

    ULongLong* buf = (ULongLong*)buf_;
    return buf[index_];
}

LongDouble
DynamicAny::DynSeqBase_impl::get_longdouble()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_longdouble);

    LongDouble* buf = (LongDouble*)buf_;
    return buf[index_];
}

WChar
DynamicAny::DynSeqBase_impl::get_wchar()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_wchar);

    OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
    return (*seq)[index_];
}

wchar_t*
DynamicAny::DynSeqBase_impl::get_wstring()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_wstring);

    OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
    return wstring_dup((*seq)[index_]);
}

Any*
DynamicAny::DynSeqBase_impl::get_any()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_any);

    //
    // Delegate to component
    //
    return components_[index_] -> get_any();
}

DynamicAny::DynAny_ptr
DynamicAny::DynSeqBase_impl::get_dyn_any()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_any);

    //
    // Delegate to component
    //
    return components_[index_] -> get_dyn_any();
}

ValueBase*
DynamicAny::DynSeqBase_impl::get_val()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    validate(tk_value);

    //
    // Delegate to component
    //
    return components_[index_] -> get_val();
}

// ----------------------------------------------------------------------
// DynamicAny::DynSeqBase_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynSeqBase_impl::_OB_marshal(OutputStreamImpl* out)
{
    if(origType_ -> kind() == tk_sequence)
    {
        out -> write_ulong(length_);

        if(length_ == 0)
            return;
    }

    switch(contentKind_)
    {
    case tk_short:
        out -> write_short_array((Short*)buf_, length_);
        break;

    case tk_long:
        out -> write_long_array((Long*)buf_, length_);
        break;

    case tk_ushort:
        out -> write_ushort_array((UShort*)buf_, length_);
        break;

    case tk_ulong:
        out -> write_ulong_array((ULong*)buf_, length_);
        break;

    case tk_float:
        out -> write_float_array((Float*)buf_, length_);
        break;

    case tk_double:
        out -> write_double_array((Double*)buf_, length_);
        break;

    case tk_boolean:
        out -> write_boolean_array((Boolean*)buf_, length_);
        break;

    case tk_char:
        out -> write_char_array((Char*)buf_, length_);
        break;

    case tk_octet:
        out -> write_octet_array((Octet*)buf_, length_);
        break;

    case tk_string:
    {
        OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
        for(Long i = 0 ; i < length_ ; i++)
            out -> write_string((*seq)[i]);
        break;
    }

    case tk_longlong:
        out -> write_longlong_array((LongLong*)buf_, length_);
        break;

    case tk_ulonglong:
        out -> write_ulonglong_array((ULongLong*)buf_, length_);
        break;

    case tk_longdouble:
        out -> write_longdouble_array((LongDouble*)buf_, length_);
        break;

    case tk_wchar:
    {
        OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
        for(Long i = 0 ; i < length_ ; i++)
            out -> write_wchar((*seq)[i]);
        break;
    }

    case tk_wstring:
    {
        OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
        for(Long i = 0 ; i < length_ ; i++)
            out -> write_wstring((*seq)[i]);
        break;
    }

    default:
    {
        for(Long i = 0 ; i < length_ ; i++)
        {
            DynAny_ptr p = components_[i];
            DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
            impl -> _OB_marshal(out);
        }
    }
    }
}

void
DynamicAny::DynSeqBase_impl::_OB_unmarshal(InputStreamImpl* in)
{
    ULong len;
    if(origType_ -> kind() == tk_array)
        len = length_;
    else
    {
        len = in -> read_ulong();
        resize(len, false);

        if(len == 0)
            return;
    }

    switch(contentKind_)
    {
    case tk_short:
    {
        Short* buf = (Short*)buf_;
        in -> read_short_array(buf, len);
        break;
    }

    case tk_long:
    {
        Long* buf = (Long*)buf_;
        in -> read_long_array(buf, len);
        break;
    }

    case tk_ushort:
    {
        UShort* buf = (UShort*)buf_;
        in -> read_ushort_array(buf, len);
        break;
    }

    case tk_ulong:
    {
        ULong* buf = (ULong*)buf_;
        in -> read_ulong_array(buf, len);
        break;
    }

    case tk_float:
    {
        Float* buf = (Float*)buf_;
        in -> read_float_array(buf, len);
        break;
    }

    case tk_double:
    {
        Double* buf = (Double*)buf_;
        in -> read_double_array(buf, len);
        break;
    }

    case tk_boolean:
    {
        Boolean* buf = (Boolean*)buf_;
        in -> read_boolean_array(buf, len);
        break;
    }

    case tk_char:
    {
        Char* buf = (Char*)buf_;
        in -> read_char_array(buf, len);
        break;
    }

    case tk_octet:
    {
        Octet* buf = (Octet*)buf_;
        in -> read_octet_array(buf, len);
        break;
    }

    case tk_string:
    {
        OB::StrSeq< int >* seq = (OB::StrSeq< int >*)buf_;
        for(ULong i = 0 ; i < len ; i++)
            (*seq)[i] = in -> read_string();
        break;
    }

    case tk_longlong:
    {
        LongLong* buf = (LongLong*)buf_;
        in -> read_longlong_array(buf, len);
        break;
    }

    case tk_ulonglong:
    {
        ULongLong* buf = (ULongLong*)buf_;
        in -> read_ulonglong_array(buf, len);
        break;
    }

    case tk_longdouble:
    {
        LongDouble* buf = (LongDouble*)buf_;
        in -> read_longdouble_array(buf, len);
        break;
    }

    case tk_wchar:
    {
        OB::FixSeq< WChar, int >* seq = (OB::FixSeq< WChar, int >*)buf_;
        for(ULong i = 0 ; i < len ; i++)
            (*seq)[i] = in -> read_wchar();
        break;
    }

    case tk_wstring:
    {
        OB::WStrSeq< int >* seq = (OB::WStrSeq< int >*)buf_;
        for(ULong i = 0 ; i < len ; i++)
            (*seq)[i] = in -> read_wstring();
        break;
    }

    default:
    {
        for(ULong i = 0 ; i < len ; i++)
        {
            DynAny_ptr p = components_[i];
            DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
            impl -> _OB_unmarshal(in);
        }
    }
    }

    notifyParent();
}

Any*
DynamicAny::DynSeqBase_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(index_ >= 0 && index_ < length_ && !primitive_)
    {
        DynAny_ptr p = components_[index_];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        return impl -> _OB_currentAnyValue();
    }
    else
        return 0;
}

Any*
DynamicAny::DynSeqBase_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynSequence_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynSequence_impl::DynSequence_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : DynSeqBase_impl(factory, orbInstance, tc)
{
}

DynamicAny::DynSequence_impl::~DynSequence_impl()
{
}

// ----------------------------------------------------------------------
// DynSequence_impl public member functions
// ----------------------------------------------------------------------

ULong
DynamicAny::DynSequence_impl::get_length()
    throw(SystemException)
{
    return length_;
}

void
DynamicAny::DynSequence_impl::set_length(ULong len)
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    //
    // If bounded sequence, ensure new length is not greater
    // than the bounds
    //
    if(max_ > 0 && Long(len) > max_)
        throw InvalidValue();

    resize(len, true);

    notifyParent();
}

DynamicAny::AnySeq*
DynamicAny::DynSequence_impl::get_elements()
    throw(SystemException)
{
    return getElements();
}

void
DynamicAny::DynSequence_impl::set_elements(const DynamicAny::AnySeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;
    for(i = 0 ; i < value.length() ; i++)
    {
        TypeCode_var tc = value[i].type();
        TypeCode_ptr origTC = GetOrigType_nodup(tc);
        if(origTC -> kind() != contentKind_)
            throw TypeMismatch();
    }

    resize(value.length(), false);

    for(i = 0 ; i < value.length() ; i++)
        setValue(i, value[i]);

    if(value.length() == 0)
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

DynamicAny::DynAnySeq*
DynamicAny::DynSequence_impl::get_elements_as_dyn_any()
    throw(SystemException)
{
    return getElementsAsDynAny();
}

void
DynamicAny::DynSequence_impl::set_elements_as_dyn_any(
    const DynamicAny::DynAnySeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;
    for(i = 0 ; i < value.length() ; i++)
    {
        TypeCode_var tc = value[i] -> type();
        TypeCode_ptr origTC = GetOrigType_nodup(tc);
        if(origTC -> kind() != contentKind_)
            throw TypeMismatch();
    }

    resize(value.length(), false);

    for(i = 0 ; i < value.length() ; i++)
        setValue(i, value[i]);

    if(value.length() == 0)
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynArray_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynArray_impl::DynArray_impl(DynamicAny::DynAnyFactory_ptr factory,
					 ORBInstance_ptr orbInstance,
					 TypeCode_ptr tc)
    : DynSeqBase_impl(factory, orbInstance, tc)
{
}

DynamicAny::DynArray_impl::~DynArray_impl()
{
}

// ----------------------------------------------------------------------
// DynArray_impl public member functions
// ----------------------------------------------------------------------

DynamicAny::AnySeq*
DynamicAny::DynArray_impl::get_elements()
    throw(SystemException)
{
    return getElements();
}

void
DynamicAny::DynArray_impl::set_elements(const DynamicAny::AnySeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;
    for(i = 0 ; i < value.length() ; i++)
    {
        TypeCode_var tc = value[i].type();
        TypeCode_ptr origTC = GetOrigType_nodup(tc);
        if(origTC -> kind() != contentKind_)
            throw TypeMismatch();
    }

    if(Long(value.length()) != length_)
        throw InvalidValue();

    for(i = 0 ; i < value.length() ; i++)
        setValue(i, value[i]);

    index_ = 0;

    notifyParent();
}

DynamicAny::DynAnySeq*
DynamicAny::DynArray_impl::get_elements_as_dyn_any()
    throw(SystemException)
{
    return getElementsAsDynAny();
}

void
DynamicAny::DynArray_impl::set_elements_as_dyn_any(
    const DynamicAny::DynAnySeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;
    for(i = 0 ; i < value.length() ; i++)
    {
        TypeCode_var tc = value[i] -> type();
        TypeCode_ptr origTC = GetOrigType_nodup(tc);
        if(origTC -> kind() != contentKind_)
            throw TypeMismatch();
    }

    if(Long(value.length()) != length_)
        throw InvalidValue();

    for(i = 0 ; i < value.length() ; i++)
        setValue(i, value[i]);

    index_ = 0;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynValueCommon_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynValueCommon_impl::DynValueCommon_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : DynAny_impl(factory, orbInstance, tc), isNull_(true)
{
}

DynamicAny::DynValueCommon_impl::~DynValueCommon_impl()
{
}

// ----------------------------------------------------------------------
// DynValueCommon_impl public member functions
// ----------------------------------------------------------------------

Boolean
DynamicAny::DynValueCommon_impl::is_null()
    throw(SystemException)
{
    return isNull_;
}

void
DynamicAny::DynValueCommon_impl::set_to_null()
    throw(SystemException)
{
    if(!isNull_)
    {
        isNull_ = true;
        destroyComponents();
    }
}

void
DynamicAny::DynValueCommon_impl::set_to_value()
    throw(SystemException)
{
    if(isNull_)
    {
        isNull_ = false;
        createComponents();
    }
}

// ----------------------------------------------------------------------
// DynValue_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynValue_impl::DynValue_impl(DynamicAny::DynAnyFactory_ptr factory,
					 ORBInstance_ptr orbInstance,
					 TypeCode_ptr tc)
    : DynValueCommon_impl(factory, orbInstance, tc),
      DynAny_impl(factory, orbInstance, tc)
{
    //
    // Custom valuetypes are not supported by DynValue_impl
    //
    assert(origType_ -> type_modifier() != VM_CUSTOM);

    getMembers(type_, names_, types_);

    //
    // Collect our repository IDs for marshalling purposes. If the
    // type is truncatable, include IDs of all base types.
    //
    CORBA::ULong idsLength = ids_.length();
    ids_.length(idsLength + 1);
    ids_[idsLength] = origType_ -> id();

    ValueModifier mod = origType_ -> type_modifier();
    if(mod == VM_TRUNCATABLE)
    {
        TypeCode_ptr baseType = origType_ -> _OB_concreteBaseType_nodup();
        while(!is_nil(baseType))
        {
            TypeCode_ptr origBaseType = GetOrigType_nodup(baseType);

            CORBA::ULong idsLength = ids_.length();
            ids_.length(idsLength + 1);
            ids_[idsLength] = origBaseType -> id();

            baseType = origBaseType -> _OB_concreteBaseType_nodup();
        }
    }

    //
    // Initial value is null
    //
    index_ = -1;
}

DynamicAny::DynValue_impl::~DynValue_impl()
{
}

// ----------------------------------------------------------------------
// DynValue_impl private and protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValue_impl::getMembers(
    TypeCode_ptr tc,
    OB::StrSeq< int >& names,
    OB::ObjSeq< TypeCode, int >& types)
{
    TypeCode_ptr origTC = GetOrigType_nodup(tc);

    TypeCode_ptr base = origTC -> _OB_concreteBaseType_nodup();
    if(!is_nil(base))
        getMembers(base, names, types);

    for(ULong i = 0 ; i < origTC -> member_count() ; i++)
    {
        CORBA::ULong namesLength = names.length();
        names.length(namesLength + 1);
        names[namesLength] = origTC -> member_name(i);

        CORBA::ULong typesLength = types.length();
        types.length(typesLength + 1);
        types[typesLength] = origTC -> member_type(i);
    }
}

void
DynamicAny::DynValue_impl::createComponents()
{
    if(components_.length() == 0 && types_.length() > 0)
    {
        components_.length(types_.length());
        for(ULong i = 0 ; i < types_.length() ; i++)
            components_[i] = create(types_[i], true);
        index_ = 0;
    }
}

void
DynamicAny::DynValue_impl::destroyComponents()
{
    if(components_.length() > 0)
    {
        for(ULong i = 0 ; i < types_.length() ; i++)
            components_[i] -> destroy();
        components_.length(0);
    }

    index_ = -1;
}

// ----------------------------------------------------------------------
// DynValue_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValue_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynValue_var dv = DynValue::_narrow(dyn_any);
    assert(!is_nil(dv));

    if(dv -> is_null())
        set_to_null();
    else
    {
        set_to_value();

        assert(components_.length() == dv -> component_count());

        dv -> rewind();
        for(ULong i = 0 ; i < components_.length() ; i++)
        {
            DynAny_var comp = dv -> current_component();
            components_[i] -> assign(comp);
            dv -> next();
        }

        if(components_.length() == 0)
            index_ = -1;
        else
            index_ = 0;
    }

    notifyParent();
}

void
DynamicAny::DynValue_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    InputStream_var in;

    if(value.info())
    {
        const ConstructedInfo* info = value.info();
	OCI::BufferImpl buf;
	OutputStreamImpl out(&buf);
        info -> marshal(value.value(), &out);
        in = out.create_input_stream();
    }
    else
    {
        in = InputStream::_duplicate((InputStream_ptr)value.value());
        in -> _OB_reset();
    }

    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    if(is_null() || components_.length() == 0)
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

Any*
DynamicAny::DynValue_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    Any* result;
    if(is_null())
    {
        static ValueBaseInfo info;
        result = new Any();
        result -> replace(type_, 0, true, &info);
    }
    else
    {
        OCI::BufferImpl buf;
        OutputStreamImpl out(&buf);
        _OB_marshal(&out);
        InputStream_ptr in = out.create_input_stream();
        in -> _OB_ORBInstance(orbInstance_);
        result = new Any(type_, in, true);
    }
    result -> _OB_ORBInstance(orbInstance_);
    return result;
}

Boolean
DynamicAny::DynValue_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    DynValue_var dv = DynValue::_narrow(dyn_any);
    assert(!is_nil(dv));

    if(is_null() || dv -> is_null())
        return(is_null() && dv -> is_null());

    assert(components_.length() == dv -> component_count());

    DynValue_impl* impl = dynamic_cast<DynValue_impl*>(dv.in());
    for(ULong i = 0 ; i < components_.length() ; i++)
        if(!components_[i] -> equal(impl -> components_[i]))
            return false;

    return true;
}

DynamicAny::DynAny_ptr
DynamicAny::DynValue_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynValue_impl* result = new DynValue_impl(factory_, orbInstance_, type_);

    if(!is_null())
    {
        result -> set_to_value();
        for(ULong i = 0 ; i < components_.length() ; i++)
        {
            DynAny_var comp = result -> current_component();
            comp -> assign(components_[i]);
            result -> next();
        }
    }

    return result;
}

Boolean
DynamicAny::DynValue_impl::seek(Long index)
    throw(SystemException)
{
    if(index < 0 || index >= (Long)names_.length())
    {
        index_ = -1;
        return false;
    }

    index_ = index;
    return true;
}

void
DynamicAny::DynValue_impl::rewind()
    throw(SystemException)
{
    seek(0);
}

Boolean
DynamicAny::DynValue_impl::next()
    throw(SystemException)
{
    if(is_null() || index_ + 1 >= (Long)names_.length())
    {
        index_ = -1;
        return false;
    }

    index_++;
    return true;
}

ULong
DynamicAny::DynValue_impl::component_count()
    throw(SystemException)
{
    return components_.length();
}

DynamicAny::DynAny_ptr
DynamicAny::DynValue_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(names_.length() == 0)
        throw TypeMismatch();

    if(index_ == -1)
        return DynAny::_nil();

    return DynAny::_duplicate(components_[index_]);
}

DynamicAny::FieldName
DynamicAny::DynValue_impl::current_member_name()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(names_.length() == 0)
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    return string_dup(names_[index_]);
}

TCKind
DynamicAny::DynValue_impl::current_member_kind()
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(types_.length() == 0)
        throw TypeMismatch();

    if(index_ < 0)
        throw InvalidValue();

    TypeCode_ptr origTC = GetOrigType_nodup(types_[index_]);
    return origTC -> kind();
}

DynamicAny::NameValuePairSeq*
DynamicAny::DynValue_impl::get_members()
    throw(SystemException)
{
    if(is_null())
        throw InvalidValue();

    NameValuePairSeq_var result = new NameValuePairSeq;
    result -> length(components_.length());

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        result[i].id = string_dup(names_[i]);
        Any_var any = components_[i] -> to_any();
        result[i].value = any;
    }

    return result._retn();
}

void
DynamicAny::DynValue_impl::set_members(
    const DynamicAny::NameValuePairSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;

    if(value.length() != names_.length())
        throw InvalidValue();

    //
    // Prior to modifying our components, validate the supplied
    // name-value pairs to check for matching member names and
    // compatible TypeCodes
    //
    for(i = 0 ; i < components_.length() ; i++)
    {
        if(strlen(value[i].id) > 0 && strlen(names_[i]) > 0 &&
           strcmp(value[i].id, names_[i]) != 0)
            throw TypeMismatch();

        TypeCode_var valueType = value[i].value.type();
        if(!valueType -> equivalent(types_[i]))
            throw TypeMismatch();
    }

    set_to_value();

    for(i = 0 ; i < components_.length() ; i++)
        components_[i] -> from_any(value[i].value);

    if(components_.length() == 0)
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

DynamicAny::NameDynAnyPairSeq*
DynamicAny::DynValue_impl::get_members_as_dyn_any()
    throw(SystemException)
{
    if(is_null())
        throw InvalidValue();

    NameDynAnyPairSeq_var result = new NameDynAnyPairSeq;
    result -> length(components_.length());

    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        result[i].id = string_dup(names_[i]);
        result[i].value = DynAny::_duplicate(components_[i]);
    }

    return result._retn();
}

void
DynamicAny::DynValue_impl::set_members_as_dyn_any(
    const DynamicAny::NameDynAnyPairSeq& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    ULong i;

    if(value.length() != names_.length())
        throw InvalidValue();

    //
    // Prior to modifying our components, validate the supplied
    // name-value pairs to check for matching member names and
    // compatible TypeCodes
    //
    for(i = 0 ; i < components_.length() ; i++)
    {
        if(strlen(value[i].id) > 0 && strlen(names_[i]) > 0 &&
           strcmp(value[i].id, names_[i]) != 0)
            throw TypeMismatch();

        DynAny_ptr p = value[i].value;
        TypeCode_var valueType = p -> type();
        if(!valueType -> equivalent(types_[i]))
            throw TypeMismatch();
    }

    set_to_value();

    for(i = 0 ; i < components_.length() ; i++)
        components_[i] -> assign(value[i].value);

    if(components_.length() == 0)
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynValue_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValue_impl::_OB_marshal(OutputStreamImpl* out)
{
    if(is_null())
        out -> write_ulong(0);
    else
    {
        Long tag;
        bool chunk = false;

        ValueModifier mod = origType_ -> type_modifier();
        if(mod == VM_TRUNCATABLE)
        {
            chunk = true;
            tag = 0x7fffff06;
        }
        else
            tag = 0x7fffff02;

        out -> _OB_beginValue(tag, ids_, chunk);

        for(ULong i = 0 ; i < components_.length() ; i++)
        {
            DynAny_ptr p = components_[i];
            DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
            impl -> _OB_marshal(out);
        }

        out -> _OB_endValue();
    }
}

void
DynamicAny::DynValue_impl::_OB_unmarshal(InputStreamImpl* in)
{
    //
    // Peek at value tag
    //
    ULong save = in -> _OB_pos();
    ULong ind = 0;
    Long tag = in -> read_long();

    if(tag == 0) // null value
    {
        set_to_null();
        return;
    }
    else if(tag == (Long)0xffffffff)
    {
        //
        // Indirection - rewind to offset
        //
        Long offs = in -> read_long();
        ind = in -> _OB_pos(); // save position after offset
        in -> _OB_pos(in -> _OB_pos() - 4 + offs);
    }
    else
        in -> _OB_pos(save); // restore tag position

    set_to_value();

    in -> _OB_beginValue();

    //
    // Unmarshal component state
    //
    for(ULong i = 0 ; i < components_.length() ; i++)
    {
        DynAny_ptr p = components_[i];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_unmarshal(in);
    }

    in -> _OB_endValue();

    //
    // Restore position after indirection
    //
    if(ind != 0)
        in -> _OB_pos(ind);

    notifyParent();
}

Any*
DynamicAny::DynValue_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(index_ >= 0 && index_ < (Long)components_.length())
    {
        DynAny_ptr p = components_[index_];
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        return impl -> _OB_currentAnyValue();
    }
    else
        return 0;
}

Any*
DynamicAny::DynValue_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynValueBox_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynValueBox_impl::DynValueBox_impl(
    DynamicAny::DynAnyFactory_ptr factory,
    ORBInstance_ptr orbInstance,
    TypeCode_ptr tc)
    : DynValueCommon_impl(factory, orbInstance, tc),
      DynAny_impl(factory, orbInstance, tc)
{
    boxedType_ = origType_ -> content_type();

    //
    // Initial value is null
    //
    index_ = -1;
}

DynamicAny::DynValueBox_impl::~DynValueBox_impl()
{
}

// ----------------------------------------------------------------------
// DynValueBox_impl private and protected member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValueBox_impl::createComponents()
{
    if(is_nil(component_))
    {
        component_ = create(boxedType_, true);
        index_ = 0;
    }
}

void
DynamicAny::DynValueBox_impl::destroyComponents()
{
    if(!is_nil(component_))
    {
        component_ -> destroy();
        component_ = DynamicAny::DynAny::_nil();
    }

    index_ = -1;
}

// ----------------------------------------------------------------------
// DynValueBox_impl public member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValueBox_impl::assign(DynamicAny::DynAny_ptr dyn_any)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return;

    TypeCode_var tc = dyn_any -> type();

    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    DynValueBox_var dv = DynValueBox::_narrow(dyn_any);
    assert(!is_nil(dv));

    if(dv -> is_null())
        set_to_null();
    else
    {
        set_to_value();
        dv -> rewind();
        component_ -> assign(dv -> current_component());
        index_ = 0;
    }

    notifyParent();
}

void
DynamicAny::DynValueBox_impl::from_any(const Any& value)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    TypeCode_ptr tc = value._OB_type_nodup();
    if(!tc -> equivalent(type_))
        throw TypeMismatch();

    InputStream_var in;

    if(value.info())
    {
        const ConstructedInfo* info = value.info();
        OCI::BufferImpl buf;
        OutputStreamImpl out(&buf);
        info -> marshal(value.value(), &out);
        in = out.create_input_stream();
    }
    else
    {
        in = InputStream::_duplicate((InputStream_ptr)value.value());
        in -> _OB_reset();
    }

    in -> _OB_ORBInstance(orbInstance_);
    _OB_unmarshal(in);

    if(is_null())
        index_ = -1;
    else
        index_ = 0;

    notifyParent();
}

Any*
DynamicAny::DynValueBox_impl::to_any()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    Any* result;
    if(is_null())
    {
        static ValueBaseInfo info;
        result = new Any();
        result -> replace(type_, 0, true, &info);
    }
    else
    {
        OCI::BufferImpl buf;
        OutputStreamImpl out(&buf);
        _OB_marshal(&out);
        InputStream_ptr in = out.create_input_stream();
        in -> _OB_ORBInstance(orbInstance_);
        result = new Any(type_, in, true);
    }
    result -> _OB_ORBInstance(orbInstance_);
    return result;
}

Boolean
DynamicAny::DynValueBox_impl::equal(DynamicAny::DynAny_ptr dyn_any)
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(this == dyn_any)
        return true;

    TypeCode_var tc = dyn_any -> type();
    if(!tc -> equivalent(type_))
        return false;

    DynValueBox_var dv = DynValueBox::_narrow(dyn_any);
    assert(!is_nil(dv));

    if(is_null() || dv -> is_null())
        return(is_null() && dv -> is_null());

    DynValueBox_impl* impl = dynamic_cast<DynValueBox_impl*>(dv.in());
    return component_ -> equal(impl -> component_);
}

DynamicAny::DynAny_ptr
DynamicAny::DynValueBox_impl::copy()
    throw(SystemException)
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    DynValueBox_impl* result =
        new DynValueBox_impl(factory_, orbInstance_, type_);

    if(!is_null())
        result -> set_boxed_value_as_dyn_any(component_);

    return result;
}

Boolean
DynamicAny::DynValueBox_impl::seek(Long index)
    throw(SystemException)
{
    if(is_null() || index != 0)
    {
        index_ = -1;
        return false;
    }

    index_ = index;
    return true;
}

void
DynamicAny::DynValueBox_impl::rewind()
    throw(SystemException)
{
    seek(0);
}

Boolean
DynamicAny::DynValueBox_impl::next()
    throw(SystemException)
{
    if(index_ == 0)
    {
        index_ = -1;
        return false;
    }

    assert(index_ == -1);
    index_++;
    return true;
}

ULong
DynamicAny::DynValueBox_impl::component_count()
    throw(SystemException)
{
    return(is_null() ? 0 : 1);
}

DynamicAny::DynAny_ptr
DynamicAny::DynValueBox_impl::current_component()
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    if(index_ == -1)
        return DynAny::_nil();

    assert(index_ == 0);
    return DynAny::_duplicate(component_);
}

Any*
DynamicAny::DynValueBox_impl::get_boxed_value()
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_null())
        throw InvalidValue();

    return component_ -> to_any();
}

void
DynamicAny::DynValueBox_impl::set_boxed_value(const Any& boxed)
    throw(DynamicAny::DynAny::TypeMismatch,
          DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    TypeCode_ptr tc = boxed._OB_type_nodup();
    if(!tc -> equivalent(boxedType_))
        throw TypeMismatch();

    bool isNull = is_null();

    try
    {
        set_to_value();
        component_ -> from_any(boxed);
    }
    catch(...)
    {
        //
        // Restore previous state if necessary
        //
        if(isNull)
            set_to_null();
        throw;
    }

    index_ = 0;

    notifyParent();
}

DynamicAny::DynAny_ptr
DynamicAny::DynValueBox_impl::get_boxed_value_as_dyn_any()
    throw(DynamicAny::DynAny::InvalidValue,
          SystemException)
{
    if(is_null())
        throw InvalidValue();

    return DynamicAny::DynAny::_duplicate(component_);
}

void
DynamicAny::DynValueBox_impl::set_boxed_value_as_dyn_any(DynamicAny::DynAny_ptr
                                                         boxed)
    throw(DynamicAny::DynAny::TypeMismatch,
          SystemException)
{
    TypeCode_var tc = boxed -> type();
    if(!tc -> equivalent(boxedType_))
        throw TypeMismatch();

    bool isNull = is_null();

    try
    {
        set_to_value();
        component_ -> assign(boxed);
    }
    catch(...)
    {
        //
        // Restore previous state if necessary
        //
        if(isNull)
            set_to_null();
        throw;
    }

    index_ = 0;

    notifyParent();
}

// ----------------------------------------------------------------------
// DynValueBox_impl internal member functions
// ----------------------------------------------------------------------

void
DynamicAny::DynValueBox_impl::_OB_marshal(OutputStreamImpl* out)
{
    if(is_null())
        out -> write_ulong(0);
    else
    {
        Long tag = 0x7fffff02;
        bool chunk = false;
        StrSeq< int > ids;
        ids.length(1);
        ids[0] = string_dup(type_ -> id());

        out -> _OB_beginValue(tag, ids, chunk);

        DynAny_ptr p = component_;
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        impl -> _OB_marshal(out);

        out -> _OB_endValue();
    }
}

void
DynamicAny::DynValueBox_impl::_OB_unmarshal(InputStreamImpl* in)
{
    //
    // Peek at value tag
    //
    ULong save = in -> _OB_pos();
    ULong ind = 0;
    Long tag = in -> read_long();

    if(tag == 0) // null value
    {
        set_to_null();
        return;
    }
    else if(tag == (Long)0xffffffff)
    {
        //
        // Indirection - rewind to offset
        //
        Long offs = in -> read_long();
        ind = in -> _OB_pos(); // save position after offset
        in -> _OB_pos(in -> _OB_pos() - 4 + offs);
    }
    else
        in -> _OB_pos(save); // restore tag position

    set_to_value();

    in -> _OB_beginValue();

    //
    // Unmarshal component state
    //
    DynAny_ptr p = component_;
    DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
    impl -> _OB_unmarshal(in);

    in -> _OB_endValue();

    //
    // Restore position after indirection
    //
    if(ind != 0)
        in -> _OB_pos(ind);

    notifyParent();
}

Any*
DynamicAny::DynValueBox_impl::_OB_currentAny()
{
    if(destroyed_)
	throw OBJECT_NOT_EXIST();

    if(index_ == 0)
    {
        DynAny_ptr p = component_;
        DynAny_impl* impl = dynamic_cast<DynAny_impl*>(p);
        return impl -> _OB_currentAnyValue();
    }
    else
        return 0;
}

Any*
DynamicAny::DynValueBox_impl::_OB_currentAnyValue()
{
    return 0;
}

// ----------------------------------------------------------------------
// DynAnyFactory_impl constructor and destructor
// ----------------------------------------------------------------------

DynamicAny::DynAnyFactory_impl::DynAnyFactory_impl(ORBInstance_ptr factory)
    : orbInstance_(ORBInstance::_duplicate(factory))
{
}

DynamicAny::DynAnyFactory_impl::~DynAnyFactory_impl()
{
}

// ----------------------------------------------------------------------
// DynamicAny::DynAnyFactory_impl public member functions
// ----------------------------------------------------------------------

DynamicAny::DynAny_ptr
DynamicAny::DynAnyFactory_impl::create_dyn_any(const Any& value)
    throw(DynamicAny::DynAnyFactory::InconsistentTypeCode,
          SystemException)
{
    try
    {
        TypeCode_ptr type = value._OB_type_nodup();
        DynAny_var p = create_dyn_any_from_type_code(type);
        p -> from_any(value);
        return p._retn();
    }
    catch(DynAny::TypeMismatch&)
    {
    }
    catch(DynAny::InvalidValue&)
    {
    }

    throw DynAnyFactory::InconsistentTypeCode();
    return DynAny::_nil(); // some compilers need this
}

DynamicAny::DynAny_ptr
DynamicAny::DynAnyFactory_impl::create_dyn_any_from_type_code(
    TypeCode_ptr type)
    throw(DynamicAny::DynAnyFactory::InconsistentTypeCode,
          SystemException)
{
    DynAny_ptr result;

    TypeCode_ptr origTC = GetOrigType_nodup(type);
    switch(origTC -> kind())
    {
    case tk_null:
    case tk_void:
    case tk_short:
    case tk_long:
    case tk_ushort:
    case tk_ulong:
    case tk_float:
    case tk_double:
    case tk_boolean:
    case tk_char:
    case tk_octet:
    case tk_any:
    case tk_TypeCode:
    case tk_objref:
    case tk_string:
    case tk_longlong:
    case tk_ulonglong:
    case tk_longdouble:
    case tk_wchar:
    case tk_wstring:
    case tk_abstract_interface:
    case tk_local_interface:
        result = new DynBasic_impl(this, orbInstance_, type);
        break;

    case tk_fixed:
        result = new DynFixed_impl(this, orbInstance_, type);
        break;

    case tk_enum:
        result = new DynEnum_impl(this, orbInstance_, type);
        break;

    case tk_struct:
    case tk_except:
        result = new DynStruct_impl(this, orbInstance_, type);
        break;

    case tk_union:
        result = new DynUnion_impl(this, orbInstance_, type);
        break;

    case tk_sequence:
        result = new DynSequence_impl(this, orbInstance_, type);
        break;

    case tk_array:
        result = new DynArray_impl(this, orbInstance_, type);
        break;

    case tk_value:
        if(origTC -> type_modifier() == VM_CUSTOM)
            result = new DynBasic_impl(this, orbInstance_, type);
        else
            result = new DynValue_impl(this, orbInstance_, type);
        break;

    case tk_value_box:
        result = new DynValueBox_impl(this, orbInstance_, type);
        break;

    case tk_Principal:
    case tk_native:
        throw InconsistentTypeCode();

    case tk_alias:
        assert(false);
        break;
    }

    return result;
}
