// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_GIOP_CLIENT_H
#define OB_GIOP_CLIENT_H

#include <OB/ORBInstance_fwd.h>

#include <GIOPClient_fwd.h>
#include <GIOPClientWorker_fwd.h>

#include <OB/Client.h>

namespace OB
{

//
// The GIOPClient class
//
class GIOPClient : public Client, public JTCMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    GIOPClient(const GIOPClient&);
    void operator=(const GIOPClient&);

protected:

    ORBInstance_var orbInstance_; // The ORB instance
    CORBA::ULong nextRequestId_; // The next request ID
    JTCMutex nextRequestIdMutex_; // The next request ID mutex

    OCI::Connector_var connector_; // The connector
    GIOPClientWorker_var worker_; // The worker
    IOP::ServiceContext_var codeSetSC_; // The code set service context
    bool destroy_; // True if destroy() was called

    //
    // Get the worker. If the bool flag is true, and no worker exists,
    // a new worker is created, with the timeout specified as second
    // parameter.
    //
    GIOPClientWorker_ptr getWorker(bool, CORBA::Long);
    
public:

    GIOPClient(ORBInstance_ptr, OCI::Connector_ptr, Client::ConcModel,
	       const CodeConverters&);
    virtual ~GIOPClient();

    static inline GIOPClient_ptr _duplicate(GIOPClient_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline GIOPClient_ptr _nil()
    { return 0; }

    //
    // Destroy the client
    //
    virtual void destroy();

    //
    // Remove the worker. This operation is called by GIOPClientWorker
    // and derived classes only.
    //
    void removeWorker(GIOPClientWorker_ptr);

    //
    // Get a new request ID
    //
    virtual CORBA::ULong requestId();

    //
    // Get all profiles that are usable with this client
    //
    virtual OCI::ProfileInfoSeq* getUsableProfiles(const OCI::IOR&,
						   const CORBA::PolicyList&);

    //
    // Get the OCI Connector info
    //
    virtual OCI::ConnectorInfo_ptr connectorInfo();

    //
    // Get the OCI Transport info
    //
    virtual OCI::TransportInfo_ptr transportInfo();
    
    //
    // Start a downcall, returning a downcall emitter and an
    // OutputStream for marshalling a request
    //
    virtual DowncallEmitter_ptr startDowncall(Downcall_ptr, OutputStream_out)
	throw();

    //
    // Checks whether this client is equal to another client
    //
    virtual bool equal(Client_ptr);

    //
    // Force connection establishment
    //
    virtual void bind(CORBA::Long);

    //
    // Determines whether this client supports twoway invocations
    //
    virtual bool twoway();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::GIOPClient_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::GIOPClient_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
