// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/OAInterface.h>
#include <OB/Logger.h>
#include <OB/Except.h>
#include <OB/OCI.h>
#include <OB/ORBInstance.h>

#include <GIOPServer.h>
#include <GIOPServerStarter.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// GIOPServer constructor and destructor
// ----------------------------------------------------------------------

GIOPServer::GIOPServer(ORBInstance_ptr orbInstance,
		       OCI::Acceptor_ptr acceptor,
                       OAInterface_ptr oaInterface,
		       Server::ConcModel concModel)
    : Server(concModel),
      orbInstance_(ORBInstance::_duplicate(orbInstance)),
      destroy_(false),
      acceptor_(OCI::Acceptor::_duplicate(acceptor)),
      oaInterface_(OAInterface::_duplicate(oaInterface))
{
    try
    {
	switch(concModel_)
	{
	case Reactive:
	    starter_ = new GIOPServerStarterReactive(orbInstance_, acceptor_,
						     oaInterface_);
	    break;
	    
	case Threaded:
	    starter_ = new GIOPServerStarterThreaded(orbInstance_, acceptor_,
						     oaInterface_);
	    break;
	}
    }
    catch(...)
    {
	destroy_ = true;
	_OB_setRef(0);
	throw;
    }
}

GIOPServer::~GIOPServer()
{
    assert(destroy_);
    assert(is_nil(starter_));
}

// ----------------------------------------------------------------------
// GIOPServer public member implementation
// ----------------------------------------------------------------------

void
GIOPServer::destroy()
{
    //
    // Don't destroy twice
    //
    if(destroy_)
	return;

    //
    // Set the destroy flag
    //
    destroy_ = true;

    //
    // Close and remove the starter
    //
    assert(!is_nil(starter_));
    starter_ -> setState(GIOPServerStarter::StateClosed);
    starter_ = GIOPServerStarter::_nil();
}

void
GIOPServer::hold()
{
    assert(!destroy_);
    assert(!is_nil(starter_));
    starter_ -> setState(GIOPServerStarter::StateHolding);
}

void
GIOPServer::activate()
{
    assert(!destroy_);
    assert(!is_nil(starter_));
    starter_ -> setState(GIOPServerStarter::StateActive);
}
