// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_GIOP_SERVER_H
#define OB_GIOP_SERVER_H

#include <OB/OAInterface_fwd.h>
#include <OB/ORBInstance_fwd.h>

#include <GIOPServer_fwd.h>
#include <GIOPServerStarter_fwd.h>

#include <Server.h>

namespace OB
{

//
// The GIOPServer class
//
class GIOPServer : public Server
{
    //
    // Hide copy-constructor and assignment operator
    //
    GIOPServer(const GIOPServer&);
    void operator=(const GIOPServer&);

protected:

    ORBInstance_var orbInstance_; // The ORB Instance
    bool destroy_; // True if destroy() was called
    OCI::Acceptor_var acceptor_; // The acceptor
    OAInterface_var oaInterface_; // The object adapter interface
    GIOPServerStarter_var starter_; // The server starter

public:
    
    GIOPServer(ORBInstance_ptr, OCI::Acceptor_ptr, OAInterface_ptr,
	       Server::ConcModel);
    virtual ~GIOPServer();

    static inline GIOPServer_ptr _duplicate(GIOPServer_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline GIOPServer_ptr _nil()
    { return 0; }

    //
    // Destroy the server
    //
    virtual void destroy();

    //
    // Hold any new requests that arrive for the Server
    //
    virtual void hold();

    //
    // Dispatch any requests that arrive for the Server
    //
    virtual void activate();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::GIOPServer_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::GIOPServer_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
