// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/Object.h>
#include <OB/RefCounted.h>
#include <OB/ObjectKey.h>
#include <OB/IMRORT_impl.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// OBPortableInterceptor::IMRORT_impl
// constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::
IMRORT_impl::IMRORT_impl()
{
}

OBPortableInterceptor::
IMRORT_impl::IMRORT_impl(
    const char* serverId,
    const PortableInterceptor::AdapterName& adapterName,
    PortableInterceptor::ObjectReferenceTemplate* realTemplate)
{
    the_server_id(serverId);
    the_adapter_name(adapterName);
    the_real_template(realTemplate);
}


OBPortableInterceptor::
IMRORT_impl::~IMRORT_impl()
{
}

// ----------------------------------------------------------------------
// IMRORT_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::IMRORT_impl::_copy_value()
    throw(SystemException)
{
    IMRORT_impl* r = new IMRORT_impl(the_server_id(), 
    				     the_adapter_name(),
				     the_real_template());
    return r;
}

PortableInterceptor::ServerId
OBPortableInterceptor::IMRORT_impl::server_id()
    throw(SystemException)
{
    return the_real_template() -> server_id();
}

PortableInterceptor::ORBId
OBPortableInterceptor::IMRORT_impl::orb_id()
    throw(SystemException)
{
    return the_real_template() -> orb_id();
}

PortableInterceptor::AdapterName*
OBPortableInterceptor::IMRORT_impl::adapter_name()
    throw(SystemException)
{
    return the_real_template() -> adapter_name();
}

Object_ptr
OBPortableInterceptor::IMRORT_impl::make_object(
    const char* repoid,
    const PortableInterceptor::ObjectId& id)
    throw(SystemException)
{
    //
    // Create the ORBacus IMR ObjectKey
    //

    OB::ObjectKeyData obkey;
    obkey.serverId = the_server_id();
    obkey.poaId = the_adapter_name();
    obkey.oid.length(id.length());
    memcpy(obkey.oid.get_buffer(), id.get_buffer(), id.length());
    obkey.persistent = true;
    obkey.createTime = 0;

    //
    // CreatePersistentObjectKey/CreateTransientObjectKey instead of
    // populating this ObjectKey data to avoid the copy?
    //
    OCI::ObjectKey_var key = CreateObjectKey(obkey);
    PortableInterceptor::ObjectId newId(key);

    //
    // Create and return reference using the real template
    //
    return the_real_template() -> make_object(repoid, newId);
}

// ----------------------------------------------------------------------
// IMRORTFactory_impl constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::IMRORTFactory_impl::IMRORTFactory_impl()
{
}

OBPortableInterceptor::IMRORTFactory_impl::~IMRORTFactory_impl()
{
}

// ----------------------------------------------------------------------
// IMRORTFactory_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::
IMRORTFactory_impl::create_for_unmarshal()
    throw(SystemException)
{
    return new IMRORT_impl();
}
