// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/INS.h>
#include <OB/OBObject.h>
#include <OB/ORBInstance.h>
#include <OB/URL.h>

using namespace CORBA;
using namespace OB;

namespace OB
{

// ----------------------------------------------------------------------
// CORBANameParser private member implementation
// ----------------------------------------------------------------------

char*
CORBANameParser::next()
{
    String_var field = string_dup("");
    while(curr_ < path_.length() && valid_)
    {
	char ch = path_[curr_];
	if(ch == '.' || ch == '/')
	{
	    ++curr_;
	    terminator_ = ch;
	    return field._retn();
	}
	
	//
	// Escape character? '.', '/' and '\' are permitted to follow
	//
	if(ch == '\\')
	{
	    ++curr_;
	    if(curr_ >= path_.length())
	    {
		valid_ = false;
		continue;
	    }
	    
	    ch = path_[curr_];
	    if(ch != '.' && ch != '/' && ch != '\\')
	    {
		valid_ = false;
		continue;
	    }
	}
	field += ch;
	++curr_;
    }
    
    terminator_ = 0;
    return field._retn();
}

bool
CORBANameParser::atEnd() const
{
    return curr_ >= path_.length();
}
    
char
CORBANameParser::terminator() const
{
    return terminator_;
}

void
CORBANameParser::parse()
{
    while(!atEnd() && valid_)
    {
	String_var id = next();
	String_var kind;
	    
	//
	// If the terminator is a '.' then the next field is the kind
	//
	if(terminator() == '.')
	{
	    kind = next();
	    //
	    // The kind field is not permitted to end with a '.'
	    //
	    if(terminator() == '.')
		valid_ = false;
	}
	else
	    kind = string_dup("");
	if(valid_)
	{
	    CORBA::ULong contentsLength = contents_.length();
	    contents_.length(contentsLength + 1);
	    contents_[contentsLength] = id;

	    contentsLength = contents_.length();
	    contents_.length(contentsLength + 1);
	    contents_[contentsLength] = kind;
	}
    }

    if(!valid_)
	contents_.length(0);
}
    
// ----------------------------------------------------------------------
// CORBANameParser constructor/destructor
// ----------------------------------------------------------------------

CORBANameParser::CORBANameParser(const OCI::ObjectKey& path)
    : path_(path),
      curr_(0),
      terminator_(0),
      valid_(true)
{
    parse();
}
    
CORBANameParser::CORBANameParser(const char* path)
    : curr_(0),
      terminator_(0),
      valid_(true)
{
    ULong len = strlen(path);
    path_.length(len);
    ::memcpy((char*)path_.get_buffer(), path, len);
    
    parse();
}

// ----------------------------------------------------------------------
// CORBANameParser public member implementation
// ----------------------------------------------------------------------

bool
CORBANameParser::isValid() const
{
    return valid_;
}

const OB::StrSeq< int >&
CORBANameParser::getContents() const
{
    return contents_;
}

} // End namespace OB

// ----------------------------------------------------------------------
// ObjectFactory public member functions related to INS
// ----------------------------------------------------------------------

Object_ptr
ObjectFactory::stringToObject(const char* s)
{
    URLRegistry_var registry = orbInstance_ -> getURLRegistry();
    return registry -> parse_url(s);
}
