// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/PIManager.h>
#include <OB/InitialServiceManager.h>
#include <OB/Codec.h>

#include <OBORBInitInfo_impl.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// ORBInitInfo_impl constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::ORBInitInfo_impl::ORBInitInfo_impl(
    ORB_ptr orb,
    int ac, char** av, const char* id,
    PIManager_ptr manager,
    InitialServiceManager_ptr initServiceManager,
    IOP::CodecFactory_ptr codecFactory)
    : orb_(ORB::_duplicate(orb)), ac_(ac), av_(av), id_(id),
      manager_(PIManager::_duplicate(manager)),
      initServiceManager_(InitialServiceManager::_duplicate(
	  initServiceManager)),
      codecFactory_(IOP::CodecFactory::_duplicate(codecFactory)),
      destroy_(false)
{
}

OBPortableInterceptor::ORBInitInfo_impl::~ORBInitInfo_impl()
{
}

// ----------------------------------------------------------------------
// ORBInitInfo_impl public member implementation
// ----------------------------------------------------------------------

//
// IDL to C++ Mapping
//
StringSeq*
OBPortableInterceptor::ORBInitInfo_impl::arguments()
    throw(SystemException)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    StringSeq_var seq = new StringSeq(ac_);
    seq -> length(ac_);
    for(int i = 0 ; i < ac_ ; i++)
	seq[i] = string_dup(av_[i]);
    
    return seq._retn();
}

char*
OBPortableInterceptor::ORBInitInfo_impl::orb_id()
    throw(SystemException)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return string_dup(id_);
}

IOP::CodecFactory_ptr
OBPortableInterceptor::ORBInitInfo_impl::codec_factory()
    throw(SystemException)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return IOP::CodecFactory::_duplicate(codecFactory_);
}

void
OBPortableInterceptor::ORBInitInfo_impl::register_initial_reference(
    const char* name, Object_ptr obj)
    throw(SystemException,
	  PortableInterceptor::ORBInitInfo::InvalidName)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    try
    {
	initServiceManager_ -> addInitialReference(name, obj);
    }
    catch(const ORB::InvalidName&)
    {
	throw PortableInterceptor::ORBInitInfo::InvalidName();
    }
}

Object_ptr
OBPortableInterceptor::ORBInitInfo_impl::resolve_initial_references(
    const char* name)
    throw(SystemException,
	  PortableInterceptor::ORBInitInfo::InvalidName)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    //TODO: check state
    try
    {
	return initServiceManager_ -> resolveInitialReferences(name);
    }
    catch(const ORB::InvalidName&)
    {
	throw PortableInterceptor::ORBInitInfo::InvalidName();
    }
    return Object::_nil(); // Some compilers need this
}

void
OBPortableInterceptor::ORBInitInfo_impl::add_client_request_interceptor(
    PortableInterceptor::ClientRequestInterceptor_ptr i)
    throw(SystemException,
	  PortableInterceptor::ORBInitInfo::DuplicateName)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    
    manager_ -> addClientRequestInterceptor(i);
}

void
OBPortableInterceptor::ORBInitInfo_impl::add_server_request_interceptor(
    PortableInterceptor::ServerRequestInterceptor_ptr i)
    throw(SystemException,
	  PortableInterceptor::ORBInitInfo::DuplicateName)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    
    manager_ -> addServerRequestInterceptor(i);
}

void
OBPortableInterceptor::ORBInitInfo_impl::add_ior_interceptor(
    PortableInterceptor::IORInterceptor_ptr i)
    throw(SystemException,
	  PortableInterceptor::ORBInitInfo::DuplicateName)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    
    manager_ -> addIORInterceptor(i);
}

PortableInterceptor::SlotId
OBPortableInterceptor::ORBInitInfo_impl::allocate_slot_id()
    throw(SystemException)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return manager_ -> allocateSlotId();
}

void
OBPortableInterceptor::ORBInitInfo_impl::register_policy_factory(
    PolicyType type,
    PortableInterceptor::PolicyFactory_ptr factory)
    throw(SystemException)
{
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    manager_ -> registerPolicyFactory(type, factory);
}

ORB_ptr
OBPortableInterceptor::ORBInitInfo_impl::orb()
    throw(SystemException)
{
    return ORB::_duplicate(orb_);
}

void
OBPortableInterceptor::ORBInitInfo_impl::_OB_destroy()
{
    destroy_ = true;
    orb_ = ORB::_nil();
    manager_ = PIManager::_nil();
    initServiceManager_ = InitialServiceManager::_nil();
    codecFactory_ = IOP::CodecFactory::_nil();
}
