// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBPolicies_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ProtocolPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ProtocolPolicy_impl::ProtocolPolicy_impl(const OCI::PluginIdSeq& value)
    : value_(value)
{
}

ProtocolPolicy_impl::~ProtocolPolicy_impl()
{
}

// ----------------------------------------------------------------------
// ProtocolPolicy_impl public member implementation
// ----------------------------------------------------------------------

OCI::PluginIdSeq*
ProtocolPolicy_impl::value()
    throw(SystemException)
{
    return new OCI::PluginIdSeq(value_);
}

Boolean
ProtocolPolicy_impl::contains(const char* id)
    throw(SystemException)
{
    for(ULong i = 0 ; i < value_.length() ; i++)
        if(strcmp(id, value_[i]) == 0)
            return true;

    return false;
}

PolicyType
ProtocolPolicy_impl::policy_type()
    throw(SystemException)
{
    return PROTOCOL_POLICY_ID;
}

Policy_ptr
ProtocolPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ProtocolPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// ConnectionReusePolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ConnectionReusePolicy_impl::ConnectionReusePolicy_impl(Boolean r)
    : value_(r)
{
}

ConnectionReusePolicy_impl::~ConnectionReusePolicy_impl()
{
}

// ----------------------------------------------------------------------
// ConnectionReusePolicy_impl public member implementation
// ----------------------------------------------------------------------

Boolean
ConnectionReusePolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ConnectionReusePolicy_impl::policy_type()
    throw(SystemException)
{
    return CONNECTION_REUSE_POLICY_ID;
}

Policy_ptr
ConnectionReusePolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ConnectionReusePolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// RetryPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

RetryPolicy_impl::RetryPolicy_impl(CORBA::Short mode, CORBA::ULong interval,
				   CORBA::ULong max, bool remote)
    : mode_(mode),
      interval_(interval),
      max_(max),
      remote_(remote)
{
}

RetryPolicy_impl::~RetryPolicy_impl()
{
}

// ----------------------------------------------------------------------
// RetryPolicy_impl public member implementation
// ----------------------------------------------------------------------

CORBA::Short
RetryPolicy_impl::retry_mode()
    throw(SystemException)
{
    return mode_;
}

CORBA::ULong
RetryPolicy_impl::retry_interval()
    throw(SystemException)
{
    return interval_;
}

CORBA::ULong
RetryPolicy_impl::retry_max()
    throw(SystemException)
{
    return max_;
}

CORBA::Boolean
RetryPolicy_impl::retry_remote()
    throw(SystemException)
{
    return remote_;
}

PolicyType
RetryPolicy_impl::policy_type()
    throw(SystemException)
{
    return RETRY_POLICY_ID;
}

Policy_ptr
RetryPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
RetryPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

TimeoutPolicy_impl::TimeoutPolicy_impl(ULong t)
{
    try
    {
	if(t > 0x7fffffff)
	    throw IMP_LIMIT();
	value_ = t;
    }
    catch(...)
    {
	_OB_setRef(0);
	throw;
    }
}

TimeoutPolicy_impl::~TimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
TimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
TimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return TIMEOUT_POLICY_ID;
}

Policy_ptr
TimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
TimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// ConnectTimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ConnectTimeoutPolicy_impl::ConnectTimeoutPolicy_impl(ULong t)
{
    if(t > 0x7fffffff)
        throw IMP_LIMIT();
    value_ = t;
}

ConnectTimeoutPolicy_impl::~ConnectTimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
ConnectTimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ConnectTimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return CONNECT_TIMEOUT_POLICY_ID;
}

Policy_ptr
ConnectTimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ConnectTimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// RequestTimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

RequestTimeoutPolicy_impl::RequestTimeoutPolicy_impl(ULong t)
{
    try
    {
	if(t > 0x7fffffff)
	    throw IMP_LIMIT();
	value_ = t;
    }
    catch(...)
    {
	_OB_setRef(0);
	throw;
    }
}

RequestTimeoutPolicy_impl::~RequestTimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
RequestTimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
RequestTimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return REQUEST_TIMEOUT_POLICY_ID;
}

Policy_ptr
RequestTimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
RequestTimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// LocationTransparencyPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

LocationTransparencyPolicy_impl::LocationTransparencyPolicy_impl(Short value)
    : value_(value)
{
}

LocationTransparencyPolicy_impl::~LocationTransparencyPolicy_impl()
{
}

// ----------------------------------------------------------------------
// LocationTransparencyPolicy_impl public member implementation
// ----------------------------------------------------------------------

Short
LocationTransparencyPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
LocationTransparencyPolicy_impl::policy_type()
    throw(SystemException)
{
    return LOCATION_TRANSPARENCY_POLICY_ID;
}

Policy_ptr
LocationTransparencyPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
LocationTransparencyPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// InterceptorPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

InterceptorPolicy_impl::InterceptorPolicy_impl(bool value)
    : value_(value)
{
}

InterceptorPolicy_impl::~InterceptorPolicy_impl()
{
}

// ----------------------------------------------------------------------
// InterceptorPolicy_impl public member implementation
// ----------------------------------------------------------------------

CORBA::Boolean
InterceptorPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
InterceptorPolicy_impl::policy_type()
    throw(SystemException)
{
    return INTERCEPTOR_POLICY_ID;
}

Policy_ptr
InterceptorPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
InterceptorPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// LocateRequestPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

LocateRequestPolicy_impl::LocateRequestPolicy_impl(bool value)
    : value_(value)
{
}

LocateRequestPolicy_impl::~LocateRequestPolicy_impl()
{
}

// ----------------------------------------------------------------------
// LocateRequestPolicy_impl public member implementation
// ----------------------------------------------------------------------

CORBA::Boolean
LocateRequestPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
LocateRequestPolicy_impl::policy_type()
    throw(SystemException)
{
    return LOCATE_REQUEST_POLICY_ID;
}

Policy_ptr
LocateRequestPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
LocateRequestPolicy_impl::destroy()
    throw(SystemException)
{
}
