// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Object.h>
#include <OB/Except.h>
#include <OB/OCIBuffer.h>

#include <stdlib.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OCI::Buffer_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OCI::Buffer_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// OCI::BufferImpl private member implementation
// ----------------------------------------------------------------------

void
OCI::BufferImpl::doRealloc(ULong len)
{
    if(real_ == 0)
    {
        alloc(len);
    }
    else
    {
	assert(len > 1200 - 7);
        assert(len > max_);

        ULong newMax = len > 2 * max_ ? len : 2 * max_;
        Octet* newReal;
	try
	{
	    newReal = new Octet[newMax + 7];
	}
	catch(...)
	{
	    newReal = 0;
	}
	if(newReal == 0)
	    throw NO_MEMORY("", MinorAllocationFailure, COMPLETED_MAYBE);
        Octet* newData = newReal + 7;
        newData -= (unsigned long)newData % 8;
        memcpy(newData, data_, len_);
#ifdef OB_CLEAR_MEM
        memset(newData + len_, 0, len - len_);
#endif

	if(rel_ && real_ && real_ != defBuf_)
	    delete [] real_;
	
	rel_ = true;
	max_ = newReal + newMax + 7 - newData;
	len_ = len;
        cur_ = cur_ - data_ + newData;
        last_ = newData + len_;
        real_ = newReal;
        data_ = newData;
    }
}

// ----------------------------------------------------------------------
// OCI::BufferImpl constructor and destructor
// ----------------------------------------------------------------------

OCI::BufferImpl::BufferImpl(ULong n)
    : rel_(false)
{
    alloc(n);
}

OCI::BufferImpl::~BufferImpl()
{
    if(rel_ && real_ && real_ != defBuf_)
	delete [] real_;
}

// ----------------------------------------------------------------------
// OCI::BufferImpl public member implementation
// ----------------------------------------------------------------------

void
OCI::BufferImpl::alloc(ULong len)
{
    if(rel_ && real_ && real_ != defBuf_)
	delete [] real_;
    
    if(len <= 1200 - 7)
    {
	real_ = defBuf_;
	data_ = real_ + 7;
	data_ -= (unsigned long)data_ % 8;
	max_ = real_ + 1200 - data_;
    }
    else
    {
	try
	{
	    real_ = new Octet[len + 7];
	}
	catch(...)
	{
	    real_ = 0;
	}
	if(real_ == 0)
	    throw NO_MEMORY("", MinorAllocationFailure, COMPLETED_MAYBE);
	data_ = real_ + 7;
	data_ -= (unsigned long)data_ % 8;
	max_ = real_ + len + 7 - data_;
    }
    
    rel_ = true;
    len_ = len;
    cur_ = data_;
    last_ = data_ + len_;
#ifdef OB_CLEAR_MEM
    memset(data_, 0, len_);
#endif
}

void
OCI::BufferImpl::consume(BufferImpl* buf)
{
    if(rel_ && real_ && real_ != defBuf_)
	delete [] real_;

    if(buf -> real_ == buf -> defBuf_)
    {
	rel_ = true;
	real_ = defBuf_;
	data_ = real_ + 7;
	data_ -= (unsigned long)data_ % 8;
	memcpy(data_, buf -> data_, buf -> len_);
	max_ = real_ + 1200 - data_;
	len_ = buf -> len_;
	cur_ = buf -> cur_ - buf -> data_ + data_;
	last_ = data_ + len_;
    }
    else
    {
	rel_ = buf -> rel_;
	real_ = buf -> real_;
	data_ = buf -> data_;
	max_ = buf -> max_;
	len_ = buf -> len_;
	cur_ = buf -> cur_;
	last_ = buf -> last_;
    }

    buf -> rel_ = false;
    buf -> real_ = 0;
    buf -> data_ = 0;
    buf -> max_ = 0;
    buf -> len_ = 0;
    buf -> cur_ = 0;
    buf -> last_ = 0;
}
