// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Object.h>
#include <OB/Stream.h>
#include <OB/IIOP.h>
#include <OB/OBPolicies.h>
#include <OB/NativeTypes.h>
#include <OB/MarshalStubImpl.h>
#include <OB/OBORB.h>
#include <OB/OCI_IIOP_init.h>
#include <OB/Properties.h>

#include <OCI_IIOP_impl.h>
#include <IIOPUtil.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Global functions
// ----------------------------------------------------------------------

extern "C"
void
OCI_version_iiop(CORBA::ORB_ptr, const char*)
{
    // Nothing to do
}

extern "C"
OCI::Plugin_ptr
OCI_init_iiop(CORBA::ORB_ptr orb, int& ac, char** av)
{
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    Properties_var props = oborb -> properties();

    //
    // Parse arguments
    //
    OCI_parse_args_iiop(ac, av, props);

    return new OCI::IIOP::Plugin_impl(orb);
}

void
OCI_parse_args_iiop(int& ac, char** av, Properties_ptr props)
{
    const char* backlog = 0;
    const char* bind = 0;
    const char* host = 0;
    bool numeric = false;
    const char* port = 0;
    bool haveArgs = false;

    int i = 0;
    while(i < ac)
    {
        if(strcmp(av[i], "-IIOPbacklog") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("iiop: argument expected for -IIOPbacklog");

            backlog = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strcmp(av[i], "-IIOPbind") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("iiop: argument expected for -IIOPbind");

            bind = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strcmp(av[i], "-IIOPhost") == 0 ||
                strcmp(av[i], "-OAhost") == 0)
        {
            if(i + 1 >= ac)
            {
                String_var msg = string_dup("iiop: argument expected for ");
                msg += av[i];
                throw INITIALIZE(msg);
            }

            host = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strcmp(av[i], "-IIOPnumeric") == 0 ||
                strcmp(av[i], "-OAnumeric") == 0)
        {
            numeric = true;
            haveArgs = true;

            for(int j = i ; j + 1 < ac ; j++)
                av[j] = av[j + 1];

            ac -= 1;
        }
        else if(strcmp(av[i], "-IIOPport") == 0 ||
                strcmp(av[i], "-OAport") == 0)
        {
            if(i + 1 >= ac)
            {
                String_var msg = string_dup("iiop: argument expected for ");
                msg += av[i];
                throw INITIALIZE(msg);
            }

            port = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strncmp(av[i], "-IIOP", 4) == 0)
        {   
            String_var err = string_dup("iiop: unknown option `");
            err += av[i];
            err += "'";
            throw INITIALIZE(err);
        }
        else
            i++;
    }

    if(haveArgs)
    {
        String_var propName =
            string_dup("ooc.orb.poamanager.RootPOAManager.endpoint");
        const char* value = props -> getProperty(propName);
        if(value == 0)
        {
            propName = string_dup("ooc.orb.oa.endpoint");
            value = props -> getProperty(propName);
        }

        String_var str = string_dup("iiop");
        if(backlog != 0)
        {
            str += " --backlog ";
            str += backlog;
        }
        if(bind != 0)
        {
            str += " --bind ";
            str += bind;
        }
        if(host != 0)
        {
            str += " --host ";
            //
            // If host contains a comma, then we must put the value
            // in quotes
            //
            if(strchr(host, ',') != 0)
            {
                str += '"';
                str += host;
                str += '"';
            }
            else
                str += host;
        }
        if(numeric)
        {
            str += " --numeric";
        }
        if(port != 0)
        {
            str += " --port ";
            str += port;
        }

        if(value == 0)
            props -> setProperty(propName, str);
        else
        {
            //
            // Append to existing property value
            //
            String_var newValue = value;
            newValue += ',';
            newValue += str;
            props -> setProperty(propName, newValue);
        }
    }
}
