// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <ORBPolicyManager_impl.h>

#include <OB/seq_converter.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ORBPolicyManager_impl constructor and destructor
// ----------------------------------------------------------------------

ORBPolicyManager_impl::ORBPolicyManager_impl(PolicyList& policies)
    : policies_(policies)
{
}

ORBPolicyManager_impl::~ORBPolicyManager_impl()
{
}

// ----------------------------------------------------------------------
// ORBPolicyManager_impl public member implementation
// ----------------------------------------------------------------------

Policy_ptr
ORBPolicyManager_impl::get_policy_override(PolicyType policy_type)
    throw(SystemException)
{
    JTCSynchronized synchronized(*this);

    ULong policiesLen = policies_.length();
    for(ULong i = 0 ; i < policiesLen ; i++)
	if(policies_[i] -> policy_type() == policy_type)
	    return Policy::_duplicate(policies_[i]);

    return Policy::_nil();
}

PolicyList*
ORBPolicyManager_impl::get_specific_policy_overrides(
    const PolicyTypeSeq& types)
    throw(SystemException)
{
    JTCSynchronized synchronized(*this);

    ULong len = policies_.length();
    ULong typesLen = types.length();

    PolicyList_var policies = new PolicyList(typesLen);

    for(ULong i = 0 ; i < typesLen ; i++)
    {
        PolicyType currType = types[i];
        for(ULong j = 0 ; j < len ; j++)
            if(policies_[j] -> policy_type() == currType) {
                CORBA::ULong policiesLength  = policies -> length();
                policies -> length(policiesLength + 1);
                policies[policiesLength] = policies_[j];
            }
    }

    return policies._retn();
}

PolicyList*
ORBPolicyManager_impl::get_all_policy_overrides()
    throw(SystemException)
{
    JTCSynchronized synchronized(*this);

    return new PolicyList(policies_);
}

void
ORBPolicyManager_impl::add_policy_overrides(const PolicyList& np)
    throw(PolicyManager::InvalidPolicies, SystemException)
{
    JTCSynchronized synchronized(*this);

    PolicyList newPolicies = policies_;
    
    ULong npLen  = np.length();
    for(ULong i = 0 ; i < npLen ; i++)
    {
        ULong len = newPolicies.length();
        ULong j;
        
        for(j = 0 ; j < len ; j++)
            if(newPolicies[j] -> policy_type() == np[i] -> policy_type())
                break;
        
        if(j < len) {
            newPolicies[j] = np[i];
        } else {
            CORBA::ULong newPoliciesLength = newPolicies.length();
            newPolicies.length(newPoliciesLength + 1);
            newPolicies[newPoliciesLength] = np[i];
        }
    }

    policies_ = newPolicies;
}

void
ORBPolicyManager_impl::remove_policy_overrides(const PolicyTypeSeq& ts)
    throw(SystemException)
{
    JTCSynchronized synchronized(*this);

    ULong typesLen = ts.length();

    for(ULong i = 0 ; i < typesLen ; i++)
    {
        PolicyType currType = ts[i];
        ULong len = policies_.length();
        for(ULong j = 0 ; j < len ; j++)
        {
            if(policies_[j] -> policy_type() == currType)
            {
		remove_seq_item(policies_, j);
                break;
            }
        }
    }
}

void
ORBPolicyManager_impl::replace_all_policy_overrides(
    const PolicyList& policies)
    throw(PolicyManager::InvalidPolicies, SystemException) 
{
    JTCSynchronized synchronized(*this);

    policies_ = policies;
}
