// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_ARG_STRATEGY_H
#define PI_ARG_STRATEGY_H

#include <OB/Object.h>
#include <OB/Except.h>

#include <OB/Dynamic.h>
#include <OB/PIArgs.h>
#include <OB/Any.h>

namespace PortableInterceptor
{

class ArgumentStrategy
{
protected:

    //
    // Are arguments available?
    //
    bool argsAvail_;

    //
    // Is the result available?
    //
    bool resultAvail_;

    //
    // Are exceptions available?
    //
    bool exceptAvail_;

    //
    // Are exceptions never available?
    //
    bool exceptNeverAvail_;

public:

    ArgumentStrategy()
	throw();
    virtual ~ArgumentStrategy();

    //
    // Get the arguments
    //
    virtual Dynamic::ParameterList*
    arguments()
	throw(CORBA::SystemException) = 0;
    
    //
    // Get the exceptions
    //
    virtual Dynamic::ExceptionList*
    exceptions()
	throw(CORBA::SystemException) = 0;
    
    //
    // Get the result
    //
    virtual CORBA::Any*
    result()
	throw(CORBA::SystemException) = 0;

    //
    // Set the result (server side only)
    //
    virtual void
    setResult(const CORBA::Any&) = 0;

    //
    // Are the args available?
    //
    void setArgsAvail(bool);

    //
    // Are the exceptions available?
    //
    void setExceptAvail(bool);

    //
    // Exceptions are never available
    //
    void setExceptNeverAvail();

    //
    // Is the result available?
    //
    void setResultAvail(bool);
};

class ArgumentStrategyNull : public ArgumentStrategy
{
public:

    virtual Dynamic::ParameterList*
    arguments()
	throw(CORBA::SystemException);
    
    virtual Dynamic::ExceptionList*
    exceptions()
	throw(CORBA::SystemException);
    
    virtual CORBA::Any*
    result()
	throw(CORBA::SystemException);

    virtual void
    setResult(const CORBA::Any&);
};

class ArgumentStrategySII : public ArgumentStrategy
{
protected:

    OB::ParameterDesc* argDesc_;
    CORBA::ULong nargDesc_;
    OB::ParameterDesc* retDesc_;
    CORBA::TypeCode_ptr* exceptionTC_;
    CORBA::ULong nexceptionTC_;

    //
    // Fill the contents of the Any using a ParameterDesc descriptor
    //
    void fillAny(CORBA::Any&, const OB::ParameterDesc&);

public:

    ArgumentStrategySII(OB::ParameterDesc*,
			CORBA::ULong,
			OB::ParameterDesc*,
			CORBA::TypeCode_ptr*,
			CORBA::ULong)
	throw();
    virtual ~ArgumentStrategySII();

    virtual Dynamic::ParameterList*
    arguments()
	throw(CORBA::SystemException);
    
    virtual Dynamic::ExceptionList*
    exceptions()
	throw(CORBA::SystemException);
    
    virtual CORBA::Any*
    result()
	throw(CORBA::SystemException);

    virtual void
    setResult(const CORBA::Any&);
};

class ArgumentStrategyDII : public ArgumentStrategy
{
protected:

    CORBA::NVList_var args_;
    CORBA::NamedValue_var result_;
    CORBA::ExceptionList_var exceptions_;

    CORBA::Any_var resultAny_;

public:

    ArgumentStrategyDII(
	CORBA::NVList_ptr,
	CORBA::NamedValue_ptr,
	CORBA::ExceptionList_ptr)
	throw();

    ArgumentStrategyDII(CORBA::NVList_ptr);

    virtual Dynamic::ParameterList*
    arguments()
	throw(CORBA::SystemException);
    
    virtual Dynamic::ExceptionList*
    exceptions()
	throw(CORBA::SystemException);
    
    virtual CORBA::Any*
    result()
	throw(CORBA::SystemException);

    virtual void
    setResult(const CORBA::Any&);
};

} // End namespace PortableInterceptor

#endif
