// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/ORBInstance.h>
#include <OB/RefCounted.h>
//#include <OB/StubImpl.h>
#include <OB/OBObject.h>

#include <PIArgStrategy.h>
#include <PIClient_impl.h>
#include <PICurrent_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ClientRequestInfo_impl constructor and destructor
// ----------------------------------------------------------------------

//
// No arguments
//
PortableInterceptor::ClientRequestInfo_impl::ClientRequestInfo_impl(
    ULong id,
    const char* op,
    bool responseExpected,
    RefCountIOR_ptr IOR,
    RefCountIOR_ptr origIOR,
    RefCountProfileInfo_ptr profileInfo,
    RefCountPolicyList_ptr policies,
    IOP::ServiceContextList& request,
    IOP::ServiceContextList& reply,
    ORBInstance_ptr orbInstance,
    Current_impl* current)
    : RequestInfo_impl(id, op, responseExpected,
                       request, reply, orbInstance, policies, current),
    IOR_(RefCountIOR::_duplicate(IOR)),
    origIOR_(RefCountIOR::_duplicate(origIOR)),
    profileInfo_(RefCountProfileInfo::_duplicate(profileInfo))
{
    argStrategy_ = new ArgumentStrategyNull;
    status_ = NO_REPLY;
    currentSlots_ = current_ -> _OB_newSlotTable();
}

//
// DII style arguments
//
PortableInterceptor::ClientRequestInfo_impl::ClientRequestInfo_impl(
    ULong id,
    const char* op,
    bool responseExpected,
    RefCountIOR_ptr IOR,
    RefCountIOR_ptr origIOR,
    RefCountProfileInfo_ptr profileInfo,
    RefCountPolicyList_ptr policies,
    IOP::ServiceContextList& request,
    IOP::ServiceContextList& reply,
    ORBInstance_ptr orbInstance,
    Current_impl* current,
    NVList_ptr args,
    NamedValue_ptr result,
    ExceptionList_ptr exceptions)
    : RequestInfo_impl(id, op, responseExpected,
                       request, reply, orbInstance, policies, current),
    IOR_(RefCountIOR::_duplicate(IOR)),
    origIOR_(RefCountIOR::_duplicate(origIOR)),
    profileInfo_(RefCountProfileInfo::_duplicate(profileInfo))
{
    argStrategy_ = new ArgumentStrategyDII(args, result, exceptions);
    status_ = NO_REPLY;
    currentSlots_ = current_ -> _OB_newSlotTable();
}

//
// SII style arguments
//
PortableInterceptor::ClientRequestInfo_impl::ClientRequestInfo_impl(
    ULong id,
    const char* op,
    bool responseExpected,
    RefCountIOR_ptr IOR,
    RefCountIOR_ptr origIOR,
    RefCountProfileInfo_ptr profileInfo,
    RefCountPolicyList_ptr policies,
    IOP::ServiceContextList& request,
    IOP::ServiceContextList& reply,
    ORBInstance_ptr orbInstance,
    Current_impl* current,
    ParameterDesc* argDesc,
    ULong nargDesc,
    ParameterDesc* retDesc,
    TypeCode_ptr* exceptionTC,
    ULong nexceptionTC)
    : RequestInfo_impl(id, op, responseExpected,
                       request, reply, orbInstance, policies, current),
    IOR_(RefCountIOR::_duplicate(IOR)),
    origIOR_(RefCountIOR::_duplicate(origIOR)),
    profileInfo_(RefCountProfileInfo::_duplicate(profileInfo))
{
    argStrategy_ = new ArgumentStrategySII(
	argDesc, nargDesc, retDesc, exceptionTC, nexceptionTC);
    status_ = NO_REPLY;
    currentSlots_ = current_ -> _OB_newSlotTable();
}

PortableInterceptor::ClientRequestInfo_impl::~ClientRequestInfo_impl()
{
}

// ----------------------------------------------------------------------
// ClientRequestInfo_impl public member implementation
// ----------------------------------------------------------------------

//
// Returns the target object on which the current request was invoked.
//
// Client side:
//
// send_request: yes send_poll: yes receive_reply: yes
// receive_exception: yes receive_other: yes
//
Object_ptr
PortableInterceptor::ClientRequestInfo_impl::target()
    throw(SystemException)
{
    ObjectFactory_var factory = orbInstance_ -> getObjectFactory();
    return factory -> createObject(origIOR_);
}

//
// Returns the actual target object on which the current request was
// invoked.
//
// Client side:
//
// send_request: yes send_poll: yes receive_reply: yes
// receive_exception: yes receive_other: yes
//
Object_ptr
PortableInterceptor::ClientRequestInfo_impl::effective_target()
    throw(SystemException)
{
    ObjectFactory_var factory = orbInstance_ -> getObjectFactory();
    return factory -> createObject(IOR_);
}

//
// Returns the profile that will be used to send this request.
//
// Client side:
//
// send_request: yes send_poll: yes receive_reply: yes
// receive_exception: yes receive_other: yes
//
IOP::TaggedProfile*
PortableInterceptor::ClientRequestInfo_impl::effective_profile()
    throw(SystemException)
{
    for(ULong i = 0 ; i < IOR_ -> value.profiles.length() ; i++)
        if(IOR_ -> value.profiles[i].tag == profileInfo_ -> value.id)
            return new IOP::TaggedProfile(IOR_ -> value.profiles[i]);

    //
    // This shouldn't happen
    //
    assert(false);
    throw INTERNAL();
    return 0; // Some compilers need this
}

//
// If the result of the invocation is an exception this is the result.
//
// Client side:
//
// send_request: no send_poll: no receive_reply: no
// receive_exception: yes receive_other: no
//
Any*
PortableInterceptor::ClientRequestInfo_impl::received_exception()
    throw(SystemException)
{
    //
    // If status is not SYSTEM_EXCEPTION or USER_EXCEPTION then this
    // is a BAD_INV_ORDER exception
    //
    if(status_ != SYSTEM_EXCEPTION && status_ != USER_EXCEPTION)
        throw BAD_INV_ORDER(MinorInvalidPICall, COMPLETED_NO);

    //
    // UnknownUserException? Extract the contained UserException.
    //
    UnknownUserException* unk =
        UnknownUserException::_downcast(receivedException_);
    if(unk != 0)
        return new Any(unk -> exception());

    Any* any = new Any;
    *any <<= receivedException_ -> _OB_clone();
    
    return any;
}

//
// If the result of the invocation is an exception this is the
// respository id of the exception
//
// Client side:
//
// send_request: no send_poll: no receive_reply: no
// receive_exception: yes receive_other: no
//
char*
PortableInterceptor::ClientRequestInfo_impl::received_exception_id()
    throw(SystemException)
{
    //
    // If status is not SYSTEM_EXCEPTION or USER_EXCEPTION then this
    // is a BAD_INV_ORDER exception
    //
    if(status_ != SYSTEM_EXCEPTION && status_ != USER_EXCEPTION)
        throw BAD_INV_ORDER(MinorInvalidPICall, COMPLETED_NO);

    if(receivedId_.in() == 0)
        receivedId_ = receivedException_ -> _rep_id();

    return string_dup(receivedId_);
}

//
// Return the TaggedComponent with the given ID in the effective
// profile.
//
// Client side:
//
// send_request: yes send_poll: no receive_reply: yes
// receive_exception: yes receive_other: yes
//
IOP::TaggedComponent*
PortableInterceptor::ClientRequestInfo_impl::get_effective_component(
    IOP::ComponentId id)
    throw(SystemException)
{
    for(ULong i = 0 ; i < profileInfo_ -> value.components.length() ; i++)
        if(profileInfo_ -> value.components[i].tag == id)
	    return new IOP::TaggedComponent(profileInfo_ ->
					    value.components[i]);

    throw BAD_PARAM(MinorInvalidComponentId, COMPLETED_NO);
    return 0; // Some compilers need this
}

//
// Return all TaggedComponents with the given ID in all profiles.
//
// Client side:
//
// send_request: yes send_poll: no receive_reply: yes
// receive_exception: yes receive_other: yes
//
IOP::TaggedComponentSeq*
PortableInterceptor::ClientRequestInfo_impl::get_effective_components(
    IOP::ComponentId)
    throw(SystemException)
{
    throw NO_IMPLEMENT(); //TODO: implement this
    return 0; // Some compilers need this
}

//
// Returns the policy of the given type in effect for this request.
//
// Client side:
//
// send_request: yes send_poll: no receive_reply: yes
// receive_exception: yes receive_other: yes
//
Policy_ptr 
PortableInterceptor::ClientRequestInfo_impl::get_request_policy(
    PolicyType type)
    throw(SystemException)
{
    for(ULong i = 0 ; i < policies_ -> value.length() ; i++)
        if(policies_ -> value[i] -> policy_type() == type)
            return Policy::_duplicate(policies_ -> value[i]);

    throw INV_POLICY(MinorInvalidPolicyType, COMPLETED_NO);
    return Policy::_nil(); // Some compilers need this
}

//
// Add a service context for this request.
//
// Client side:
//
// send_request: yes send_poll: no receive_reply: no
// receive_exception: no receive_other: no
//
void
PortableInterceptor::ClientRequestInfo_impl::add_request_service_context(
    const IOP::ServiceContext& sc,
    Boolean addReplace)
    throw(SystemException)
{
    //
    // This isn't valid in any call other than send_request (note that
    // send_poll isn't currently implemented)
    //
    if(status_ >= 0)
        throw BAD_INV_ORDER(MinorInvalidPICall, COMPLETED_NO);

    addServiceContext(requestSCL_, sc, addReplace);
}

void
PortableInterceptor::ClientRequestInfo_impl::_OB_request(
    const ClientRequestInterceptorSeq& interceptors)
    throw(SystemException, LocationForward)
{
    //
    // The PICurrent needs a new set of slot data
    //
    slots_ = current_ -> _OB_currentSlotData();
    popCurrent_ = true;
    current_ -> _OB_pushSlotData(currentSlots_.inout());

    //
    // The result not available, arguments and exceptions are
    // available
    //
    argStrategy_ -> setResultAvail(false);
    argStrategy_ -> setArgsAvail(true);
    argStrategy_ -> setExceptAvail(true);

    for(ULong i = 0 ; i < interceptors.length() ; i++)
    {
        try
        {
            interceptors[i] -> send_request(this);

            CORBA::ULong interceptorsLength = interceptors_.length();
            interceptors_.length(interceptorsLength + 1);
            interceptors_[interceptorsLength] = interceptors[i];
        }
        catch(const SystemException& ex)
        {
            status_ = SYSTEM_EXCEPTION;
            delete receivedException_;
            receivedException_ = ex._OB_clone();
            _OB_reply();
        }
        catch(const ForwardRequest& ex)
        {
            status_ = LOCATION_FORWARD;
            forwardReference_ = ex.forward -> _OB_IOR();
            _OB_reply();
        }
    }

    if(popCurrent_)
    {
	popCurrent_ = false;
	current_ -> _OB_popSlotData();
    }
}

void
PortableInterceptor::ClientRequestInfo_impl::_OB_reply()
    throw(SystemException, LocationForward)
{
    if(!popCurrent_)
    {
	popCurrent_ = true;
	current_ -> _OB_pushSlotData(currentSlots_.inout());
    }

    ULong curr = interceptors_.length() -1;
    while(interceptors_.length() > 0)
    {
        try
        {
            if(status_ == SUCCESSFUL)
            {
		//
		// The result, arguments are available
		//
		argStrategy_ -> setResultAvail(true);

                interceptors_[curr] -> receive_reply(this);
            }
            else if(status_ == SYSTEM_EXCEPTION || status_ == USER_EXCEPTION)
            {
		//
		// The result, arguments not available
		//
		argStrategy_ -> setResultAvail(false);
		argStrategy_ -> setArgsAvail(false);

                interceptors_[curr] -> receive_exception(this);
            }
            else
            {
		//
		// The result, arguments not available
		//
		argStrategy_ -> setResultAvail(false);
		argStrategy_ -> setArgsAvail(false);

                interceptors_[curr] -> receive_other(this);
            }
        }
        catch(const SystemException& ex)
        {
            status_ = SYSTEM_EXCEPTION;
            delete receivedException_;
            receivedException_ = ex._OB_clone();
            receivedId_ = ex._rep_id();
        }
        catch(const ForwardRequest& ex)
        {
            status_ = LOCATION_FORWARD;
            forwardReference_ = ex.forward -> _OB_IOR();
        }
        interceptors_.length(interceptors_.length() - 1);
        --curr;
    }

    //
    // If a set of slots was provided to the Current implementation
    // then the slots have to be popped
    //
    popCurrent_ = false;
    current_ -> _OB_popSlotData();

    //
    // Raise the appropriate exception, if necessary.  Can't use a
    // switch statement -- the values are not integer constants
    //
    if(status_ == SYSTEM_EXCEPTION)
    {
	SystemException* se = SystemException::_downcast(receivedException_);
	assert(se != 0);
	se -> _raise();
    }
    if(status_ == LOCATION_FORWARD)
        throw LocationForward(forwardReference_, false);
}
