// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/RefCounted.h>

#include <PIIORInfo_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// IORInfo_impl constructor/destructor
// ----------------------------------------------------------------------

PortableInterceptor::IORInfo_impl::IORInfo_impl(
    const OCI::AcceptorSeq& acceptors,
    RefCountPolicyList_ptr policies,
    PortableInterceptor::AdapterManagerId id,
    PortableInterceptor::AdapterState state)
    : acceptors_(acceptors),
      policies_(RefCountPolicyList::_duplicate(policies)),
      adapterTemplate_(0),
      currentFactory_(0),
      id_(id),
      state_(state)
{
    //
    // Add an entry to the component table for each acceptor
    //
    for(ULong i = 0 ; i < acceptors.length() ; i++)
        table_.put(acceptors[i] -> tag(), new IOP::TaggedComponentSeq);
}

PortableInterceptor::IORInfo_impl::~IORInfo_impl()
{
    ComponentTable::Enumerator keys = table_.keys();
    for(ULong i = 0 ; i < keys.length() ; i++)
    {
        IOP::TaggedComponentSeq* components;
        table_.get(keys[i], components);
        delete components;
    }
}


// ----------------------------------------------------------------------
// IORInfo_impl public member implementation
// ----------------------------------------------------------------------

Policy_ptr
PortableInterceptor::IORInfo_impl::get_effective_policy(PolicyType type)
    throw(SystemException)
{
    for(ULong i = 0 ; i < policies_ -> value.length() ; i++)
	if(policies_ -> value[i] -> policy_type() == type)
	    return Policy::_duplicate(policies_ -> value[i]);
    
    throw INV_POLICY(MinorNoPolicyFactory, COMPLETED_NO);
    return Policy::_nil(); // Some compilers need this
}

void
PortableInterceptor::IORInfo_impl::add_ior_component(
    const IOP::TaggedComponent& data)
    throw(SystemException)
{
    CORBA::ULong allLength = all_.length();
    all_.length(allLength + 1);
    all_[allLength] = data;
}

void
PortableInterceptor::IORInfo_impl::add_ior_component_to_profile(
    const IOP::TaggedComponent& data,
    IOP::ProfileId id)
    throw(SystemException)
{
    IOP::TaggedComponentSeq* profile;
    if(!table_.get(id, profile))
	throw BAD_PARAM(MinorInvalidProfileId, COMPLETED_NO);
    
    CORBA::ULong profileLength  = profile -> length();
    profile -> length(profileLength + 1);
        (*profile)[profileLength] = data;
}

PortableInterceptor::ObjectReferenceTemplate*
PortableInterceptor::IORInfo_impl::adapter_template()
    throw(SystemException)
{
    if(adapterTemplate_.in() == 0)
	throw CORBA::BAD_INV_ORDER(); // TODO: XXX: Minor code
    add_ref(adapterTemplate_);
    return adapterTemplate_;
}

PortableInterceptor::ObjectReferenceFactory*
PortableInterceptor::IORInfo_impl::current_factory()
    throw(SystemException)
{
    add_ref(currentFactory_);
    return currentFactory_;
}

void
PortableInterceptor::IORInfo_impl::current_factory(
    PortableInterceptor::ObjectReferenceFactory* current)
    throw(SystemException)
{
    currentFactory_ = current;
    add_ref(currentFactory_);
}

PortableInterceptor::AdapterManagerId
PortableInterceptor::IORInfo_impl::manager_id()
    throw(CORBA::SystemException)
{
    return id_;
}

PortableInterceptor::AdapterState
PortableInterceptor::IORInfo_impl::state()
    throw(CORBA::SystemException)
{
    return state_;
}

void
PortableInterceptor::IORInfo_impl::_OB_adapterTemplate(
    PortableInterceptor::ObjectReferenceTemplate* adapterTemplate)
{
    assert(adapterTemplate_.in() == 0);
    adapterTemplate_ = adapterTemplate;
    add_ref(adapterTemplate_);
}

void
PortableInterceptor::IORInfo_impl::_OB_state(
    PortableInterceptor::AdapterState state)
{
    state_ = state;
}

void
PortableInterceptor::IORInfo_impl::_OB_addComponents(
    IOP::IOR& ior,
    const GIOP::Version& version)
{
    OCI::ProfileInfo profileInfo;
    profileInfo.major = version.major;
    profileInfo.minor = version.minor;

    //
    // Not necessary since the ObjectKey is empty
    //
    //profileInfo.key = key; 

    for(ULong i = 0 ; i < acceptors_.length() ; i++)
    {
        //
        // Collect the components to be supplied to add_profiles().
        // We start with the generic components (allComponents_), and
        // may have to append components specific to this acceptor's
        // profile tag.
        //
        IOP::TaggedComponentSeq* components;
        table_.get(acceptors_[i] -> tag(), components);
        if(components -> length() > 0)
        {
            profileInfo.components.length(all_.length() +
                                          components -> length());
            ULong j;
            for(j = 0 ; j < all_.length() ; j++)
                profileInfo.components[j] = all_[j];
            for(j = 0 ; j < components -> length() ; j++)
                profileInfo.components[all_.length() + j] =
                    (*components)[j];
        }
        else
            profileInfo.components = all_;

        acceptors_[i] -> add_profiles(profileInfo, ior);
    }
}
