// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POA_CONTROL_H
#define OB_POA_CONTROL_H

namespace OBPortableServer
{

//
// This class is used to control the POA
//
class POAControl : public JTCMonitorT<JTCMutex>
{
    enum State
    {
        DestroyNotCalled,
        DestroyPending,
	DestroyInProgress,
        DestroyCompleted
    };

    //
    // The POA state
    //
    State state_;

    //
    // Should the servants be etherealized?
    //
    bool etherealize_;

    //
    // How many pending requests are there?
    //
    int requests_;

public:

    POAControl();

    //
    // Mark the state as destroy pending.  Return true if the destroy
    // should complete, false otherwise.
    //
    bool markDestroyPending(bool, bool);

    //
    // Increment the outstanding request count. Return true if the POA
    // is not destroyed, false otherwise.
    //
    bool incrementRequestCount();

    //
    // Decrement the outstanding request count. If the state is
    // DestroyPending then wake any waiting threads and return true.
    //
    bool decrementRequestCount();

    //
    // Wait for any pending requests to terminate. Return true if
    // the destroy should complete, false otherwise.
    //
    bool waitPendingRequests();

    //
    // Return true if there are outstanding pending requests
    //
    bool hasPendingRequests();

    //
    // Has the POA been destroyed?
    //
    bool getDestroyed();

    //
    // Mark the state as DestroyCompleted, wake any waiting threads
    //
    void markDestroyCompleted();

    //
    // Should etherealize be called?
    //
    bool etherealize();
};

} // End namespace OBPortableServer

#endif
