// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/ObjectKey.h>
#include <OB/Object.h>
#include <OB/StubImpl.h>
#include <OB/MarshalStubImpl.h>
#include <OB/POAInterface.h>
#include <OB/seq_converter.h>

#include <POALocator.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// POAManager_impl::POANameHasher public member implementations
// ----------------------------------------------------------------------

ULong
OBPortableServer::POANameHasher::hash(const StringSeq& k)
{
    ULong result = 0;

    for(ULong i = 0 ; i < k.length() ; i++)
        result ^= StringHasher::hash(k[i]);

    return result;
}

bool
OBPortableServer::POANameHasher::comp(const StringSeq& k1,
				      const StringSeq& k2)
{
    if(k1.length() == k2.length())
    {
        for(ULong i = 0 ; i < k1.length() ; i++)
        {
            if(strcmp(k1[i], k2[i]) != 0)
                return false;
        }

        return true;
    }

    return false;
}

// ----------------------------------------------------------------------
// POALocator constructor/destructor
// ----------------------------------------------------------------------

OBPortableServer::POALocator::POALocator()
{
}

OBPortableServer::POALocator::~POALocator()
{
}

// ----------------------------------------------------------------------
// POALocator public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
OBPortableServer::POALocator::locate(const ObjectKeyData& data)
    throw(LocationForward, SystemException)
{
    JTCSynchronized synchronized(*this);

    //
    // If length of poa name sequence is zero we are looking
    // for a root POA and are doomed to fail.
    //
    if(data.poaId.length() == 0)
        return PortableServer::POA::_nil();

    PortableServer::POA_var poa;
    if(!poas_.get(data.poaId, poa))
    {
        //
        // Arrange for POA activation. The algorithm for this is to
        // find the first POA in the `path' that exists. We then call
        // find_POA on each remaining POA in the path to cause the
        // activation.
        //
        StringSeq poaId = data.poaId;
        StringSeq remaining;
        do
        {
            CORBA::ULong remainingLength = remaining.length();
            remaining.length(remainingLength + 1);
            remaining[remainingLength] = poaId[poaId.length()-1];

            poaId.length(poaId.length() - 1);
            if(poas_.get(poaId, poa))
                break;
        }
        while(poaId.length() > 0);
        
        //
        // Now find each POA in the path until we fail to find one of
        // the POA's or we're done.
        //
	for(CORBA::ULong i = remaining.length(); i > 0 && !is_nil(poa); i--)
        {
            try
            {
                poa = poa -> find_POA(remaining[i-1], true);
            }
            catch(PortableServer::POA::AdapterNonExistent&)
            {
                poa = PortableServer::POA::_nil();
            }
        }
    }

    return poa._retn();
}

void
OBPortableServer::POALocator::add(PortableServer::POA_ptr poa,
				  const StringSeq& id)
{
    JTCSynchronized synchronized(*this);

    assert(!poas_.containsKey(id));
    poas_.put(id, PortableServer::POA::_duplicate(poa));
}

void
OBPortableServer::POALocator::remove(const StringSeq& id)
{
    JTCSynchronized synchronized(*this);

    assert(poas_.containsKey(id));
    poas_.remove(id);
}

