// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBAServer.h>
#include <OB/POAServantActivator_skel.h>
#include <OB/OCI.h>

using namespace CORBA;
using namespace OB;

namespace POA_PortableServer
{

//
// The ServantActivatorDelegate. This class is used as a weak reference
// to the actual servant locator implementation. This is necessary so
// that in the case that reference counting is not used on the servant
// manager implementation that user code doesn't get a direct pointer
// to the servant itself.
//
class ServantActivatorDelegate : public PortableServer::ServantActivator,
				 public OBCORBA::RefCountLocalObject
{
    //
    // This cannot be a _var type otherwise in the reference counted
    // case there will be a circular reference
    //
    PortableServer::ServantActivator_ptr delegate_;

public:

    ServantActivatorDelegate(PortableServer::ServantActivator_ptr delegate)
	: delegate_(delegate)
    {
    }

    ~ServantActivatorDelegate()
    {
    }

    static inline ServantActivatorDelegate_ptr
    _duplicate(ServantActivatorDelegate_ptr p)
    { if(p) p -> _add_ref(); return p; }

    static inline ServantActivatorDelegate_ptr _nil()
    { return 0; }

    virtual PortableServer::Servant
    incarnate(const PortableServer::ObjectId& oid,
	      PortableServer::POA_ptr adapter)
    {
	assert(delegate_);
	return delegate_ -> incarnate(oid, adapter);
    }
    
    virtual void
    etherealize(const PortableServer::ObjectId& oid,
		PortableServer::POA_ptr adapter,
		PortableServer::Servant serv,
		CORBA::Boolean cleanup_in_progress,
		CORBA::Boolean remaining_activations)
    {
	assert(delegate_);
	delegate_ -> etherealize(oid, adapter, serv, cleanup_in_progress,
				 remaining_activations);
    }

    void
    clear()
    {
	delegate_ = PortableServer::ServantActivator::_nil();
    }
};

} // End namespace POA_PortableServer

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBRelease(POA_PortableServer::ServantActivatorDelegate_ptr o)
{
    if(o)
	o -> _remove_ref();
}

void
OBDuplicate(POA_PortableServer::ServantActivatorDelegate_ptr o)
{
    if(o)
	o -> _add_ref();
}

// ----------------------------------------------------------------------
// ServantActivator constructor/destructor
// ----------------------------------------------------------------------

POA_PortableServer::ServantActivator::ServantActivator()
    : delegate_(new ServantActivatorDelegate(this))
{
}

POA_PortableServer::ServantActivator::~ServantActivator()
{
    delegate_ -> clear();
}

// ----------------------------------------------------------------------
// ServantActivator public member implementation
// ----------------------------------------------------------------------

PortableServer::ServantActivator_ptr
POA_PortableServer::ServantActivator::_this()
{
    return ServantActivatorDelegate::_duplicate(delegate_);
}

CORBA::RepositoryId
POA_PortableServer::ServantActivator::_primary_interface(
    const PortableServer::ObjectId&, PortableServer::POA_ptr)
{
    throw CORBA::NO_IMPLEMENT();
    return 0; // Some compilers need this
}

void
POA_PortableServer::ServantActivator::_OB_dispatch(OB::Upcall_ptr)
{
    throw CORBA::NO_IMPLEMENT();
}
