// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/ValueBase.h>
#include <OB/NativeTypes.h>
#include <OB/MarshalStubImpl.h>
#include <OB/DirectStubImpl.h>
#include <OB/OBORB.h>
#include <OB/ORB_init.h>
#include <OB/POAServantBase.h>
#include <OB/ORBInstance.h>
#include <OB/InitialServiceManager.h>
#include <OB/Upcall.h>
#include <OB/IFR3.h>
#include <OB/Stream.h>
#include <OB/OBPOAInterface.h>

#include <POACurrent_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBRelease(PortableServer::ServantBase* servant)
{
    if(servant)
        servant -> _remove_ref();
}

void
OBDuplicate(PortableServer::ServantBase* servant)
{
    if(servant)
        servant -> _add_ref();
}

// ----------------------------------------------------------------------
// ServantBase private and protected member implementation
// ----------------------------------------------------------------------

//
// TODO: Do we need PIUpcall, etc versions of this?
//
void
PortableServer::ServantBase::_OB_dispatchBase(Upcall_ptr _ob_up)
{
    static const char* names[] =
    {
        "_interface",
        "_is_a",
        "_non_existent"
    };
    static const ULong num = 3;

    switch(_OB_findOperation(_ob_up, names, num))
    {
    case 0: // _interface
    {
	_OB_preUnmarshal(_ob_up);
	_OB_postUnmarshal(_ob_up);
        InterfaceDef_var _ob_r = _get_interface();
	OutputStream_ptr out = _OB_preMarshal(_ob_up);
	try
	{
	    out -> write_Object(_ob_r);
	}
	catch(const CORBA::SystemException& ex)
	{
	    _OB_marshalEx(_ob_up,ex);
	}
	_OB_postMarshal(_ob_up);
	return;
    }

    case 1: // _is_a
    {
	OB::StrForStruct _ob_a0;
	OB::InputStream_ptr _ob_in = _OB_preUnmarshal(_ob_up);
	try
	{
	    _ob_a0 = _ob_in -> read_string();
	}
	catch(const CORBA::SystemException& _ob_ex)
	{
	    _OB_unmarshalEx(_ob_up, _ob_ex);
	}
	_OB_postUnmarshal(_ob_up);
        Boolean _ob_r = _is_a(_ob_a0);
	OutputStream_ptr out = _OB_preMarshal(_ob_up);
	try
	{
	    out -> write_boolean(_ob_r);
	}
	catch(const CORBA::SystemException& ex)
	{
	    _OB_marshalEx(_ob_up,ex);
	}
	_OB_postMarshal(_ob_up);
	return;
    }
    
    case 2: // _non_existent
    {
	_OB_preUnmarshal(_ob_up);
	_OB_postUnmarshal(_ob_up);
        Boolean _ob_r = _non_existent();
	OutputStream_ptr out = _OB_preMarshal(_ob_up);
	try
	{
	    out -> write_boolean(_ob_r);
	}
	catch(const CORBA::SystemException& ex)
	{
	    _OB_marshalEx(_ob_up,ex);
	}
	_OB_postMarshal(_ob_up);
	return;
    }
    }

    throw BAD_OPERATION(0, COMPLETED_NO);
}

Long
PortableServer::ServantBase::_OB_findOperation(Upcall_ptr up,
					       const char* names[],
					       ULong num)
{
    const char* op = up -> operation();

    Long l = 0;
    Long r = (Long)num;

    while(l < r)
    {
        Long m = (l + r) / 2;
        int res = strcmp(names[m], op);
        if(res == 0)
            return m;
        else if(res > 0)
            r = m;
        else
            l = m + 1;
    }

    return -1;
}

InputStream_ptr
PortableServer::ServantBase::_OB_preUnmarshal(Upcall_ptr up)
{
    return up -> preUnmarshal();
}

void
PortableServer::ServantBase::_OB_unmarshalEx(Upcall_ptr up,
					     const SystemException& ex)
{
    up -> unmarshalEx(ex);
}

void
PortableServer::ServantBase::_OB_postUnmarshal(Upcall_ptr up)
{
    up -> postUnmarshal();
}

OutputStream_ptr
PortableServer::ServantBase::_OB_preMarshal(Upcall_ptr up)
{
    return up -> preMarshal();
}

void
PortableServer::ServantBase::_OB_marshalEx(Upcall_ptr up,
					   const SystemException& ex)
{
    up -> marshalEx(ex);
}

void
PortableServer::ServantBase::_OB_postMarshal(Upcall_ptr up)
{
    up -> postMarshal();
}

//
// Set the argument information
//
void
PortableServer::ServantBase::_OB_setArgDesc(
    Upcall_ptr up,
    ParameterDesc* argDesc, ULong nargDesc,
    ParameterDesc* retDesc,
    TypeCode_ptr* exceptionTC, ULong nexceptionTC)
{
    PIUpcall_ptr piup = dynamic_cast<PIUpcall_ptr>(up);
    if(piup)
    {
	piup -> setArgDesc(argDesc, nargDesc,
			   retDesc,
			   exceptionTC, nexceptionTC);
    }
}

// ----------------------------------------------------------------------
// ServantBase public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
PortableServer::ServantBase::_default_POA()
{
    //
    // IMPORTANT!!!! Don't use ORB_init() anywhere else. It is only
    // used here, because ServantBase doesn't have an ORB associated
    // with it, so we have to use the "default" ORB.
    //
    int argcDummy = 0;
    char* argvDummy[1];
    argvDummy[0] = 0;
    ORB_var orb = CORBA::ORB_init(argcDummy, argvDummy, "");
    Object_var obj = orb -> resolve_initial_references("RootPOA");
    return POA::_narrow(obj);
}

InterfaceDef_ptr
PortableServer::ServantBase::_get_interface()
    throw(SystemException)
{
    OBPortableServer::POA_var thePOA =
        OBPortableServer::POA::_narrow(
            PortableServer::POA_var(_default_POA()));

    ORB_var theORB= thePOA -> the_ORB();

    Object_var obj;
    
    try
    {
	obj = theORB -> resolve_initial_references("InterfaceRepository");
    }
    catch(const ORB::InvalidName&)
    {
	//
	// The narrow will fail reporting the INTF_REPOS exception
	//
    }

    Repository_var repository = Repository::_narrow(obj);
    if(is_nil(repository))
	throw INTF_REPOS(MinorNoIntfRepos, COMPLETED_NO);

    PortableServer::Current_var current =
        PortableServer::Current::_narrow(
            Object_var(theORB -> resolve_initial_references("POACurrent")));

    ObjectId_var oid = current -> get_object_id();
    String_var intf = _primary_interface(oid, thePOA);

    Contained_var contained = repository -> lookup_id(intf);
    return InterfaceDef::_narrow(contained);
}
    
Boolean
PortableServer::ServantBase::_is_a(const char*)
    throw(SystemException)
{
    // TODO: Shouldn't this return true if logical_type_id is
    // "IDL:omg.org/CORBA/Object:1.0"?
    return false;
}
    
Boolean
PortableServer::ServantBase::_non_existent()
    throw(SystemException)
{
    return false;
}

void
PortableServer::ServantBase::_OB_dispatch(Upcall_ptr upcall)
{
    _OB_dispatchBase(upcall);
}

Object_ptr
PortableServer::ServantBase::_OB_createReference()
{
    Object_var obj;
    
    OBPortableServer::POA_var thePOA =
        OBPortableServer::POA::_narrow(
            PortableServer::POA_var(_default_POA()));

    ORB_var theORB= thePOA -> the_ORB();

    PortableServer::Current_var current =
        PortableServer::Current::_narrow(
            Object_var(theORB -> resolve_initial_references("POACurrent")));

    PortableServer::Current_impl* currentImpl =
        dynamic_cast<PortableServer::Current_impl*>(current.in());

    //
    // If we're in an upcall to this servant, then obtain the object
    // reference for the target CORBA Object it is incarnating for
    // this request.
    //
    if(currentImpl -> _OB_inUpcall() &&
       currentImpl -> _OB_getServant() == this)
    {
        ObjectId_var oid = current -> get_object_id();
        String_var intf = _primary_interface(oid, thePOA);
	obj = thePOA -> create_reference_with_id(oid, intf);
    }
    else
    {
	try
	{
	    obj = thePOA -> servant_to_reference(this);
	}
	catch(const PortableServer::POA::ServantNotActive&)
	{
	    //
	    // Expected if the servant is not active and the POA
	    // doesn't have the IMPLICIT_ACTIVATION policy. The
	    // specification states that a WrongPolicy exception
	    // should be raised in this case.
	    //
	    throw PortableServer::POA::WrongPolicy();
	}
    }
    return obj._retn();
}

DirectStubImpl_ptr
PortableServer::ServantBase::_OB_createDirectStubImpl(
    PortableServer::POA_ptr poa,
    const PortableServer::ObjectId& oid)
{
    return new DirectStubImpl(poa, oid, this);
}

ULong
PortableServer::ServantBase::_OB_getRefCount()
{
    return 0;
}

// ----------------------------------------------------------------------
// RefCountServantBase public member implementation
// ----------------------------------------------------------------------

void
PortableServer::RefCountServantBase::_add_ref()
{
    _OB_incRef();
}

void
PortableServer::RefCountServantBase::_remove_ref()
{
    _OB_decRef();
}

CORBA::ULong
PortableServer::RefCountServantBase::_OB_getRefCount()
{
    return _OB_getRef();
}
