// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/Except.h>
#include <OB/POAServantBase.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// PortableServer namespace functions
// ----------------------------------------------------------------------

char*
PortableServer::ObjectId_to_string(const ObjectId& id)
{
    //
    // IF we have any '\0' characters in the id then throw a BAD_PARAM
    // exception
    //
    if(::memchr(id.get_buffer(), 0, id.length()) != 0)
        throw BAD_PARAM();
    
    String_var str = string_alloc(id.length());
    memcpy(str.inout(), id.get_buffer(), id.length());
    str[id.length()] = '\0';
    return str._retn();
}

WChar*
PortableServer::ObjectId_to_wstring(const ObjectId& id)
{
    //
    // If the length of the ObjectId isn't divisible by the size of a
    // wide-character then it isn't possible to construct a valid
    // WString for this object-id.
    //
    if(id.length() % sizeof(WChar) != 0)
	throw BAD_PARAM();

    //
    // If we have any L'\0' characters in the id then throw a
    // BAD_PARAM exception.
    //
    const Octet* end = id.get_buffer() + id.length();
    for(const Octet* buf = id.get_buffer(); buf < end; buf += sizeof(WChar))
	if(*((WChar*)buf) == L'\0')
	    throw BAD_PARAM();

    //
    // The length of the WString is the length of the octet sequence
    // divided by the sizeof(WChar)
    //
    CORBA::ULong len = id.length() / sizeof(WChar);

    //
    // Allocate memory
    //
    WString_var str = wstring_alloc(len);
    memcpy(str.inout(), id.get_buffer(), id.length());

    //
    // null-terminate
    //
    str[len] = L'\0';

    return str._retn();
}

PortableServer::ObjectId*
PortableServer::string_to_ObjectId(const char* str)
{
    size_t len = ::strlen(str);
    ObjectId_var oid = new ObjectId(len);
    oid -> length(len);
    memcpy(oid -> get_buffer(), str, len);
    return oid._retn();
}

PortableServer::ObjectId*
PortableServer::wstring_to_ObjectId(const CORBA::WChar* str)
{
    CORBA::ULong len = OB::wcslen(str) * sizeof(WChar);
    ObjectId_var oid = new ObjectId(len);
    oid -> length(len);
    memcpy(oid -> get_buffer(), str, len);
    return oid._retn();
}
