// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Object.h>
#include <OB/Except.h>
#include <OB/Any.h> // Needed for gcc
#include <OB/OCI.h>
#include <OB/ORBInstance.h>
#include <OB/OAInterface.h>

#include <CollocatedServer.h>
#include <GIOPServer.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::Server_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::Server_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(OB::ServerManager_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::ServerManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// Server constructor and destructor
// ----------------------------------------------------------------------

Server::Server(ConcModel concModel)
    : concModel_(concModel)
{
}

Server::~Server()
{
}

// ----------------------------------------------------------------------
// ServerManager constructor/destructor
// ----------------------------------------------------------------------

ServerManager::ServerManager(ORBInstance_ptr orbInstance,
			     const OCI::AcceptorSeq& acceptors,
                             OAInterface_ptr oaInterface,
			     Server::ConcModel concModel)
    : destroy_(false)
{
    //
    // Create a server for each acceptor, and the collocated server
    //
    allServers_.length(acceptors.length() + 1);
    ULong i;
    for(i = 0 ; i < acceptors.length() ; i++)
    {
        GIOPServer* server = new GIOPServer(orbInstance, acceptors[i],
					    oaInterface, concModel);
        allServers_[i] = server;
    }
    collocatedServer_ = new CollocatedServer(oaInterface, concModel);
    allServers_[i] = Server::_duplicate(collocatedServer_);
}

ServerManager::~ServerManager()
{
    assert(destroy_);
    assert(allServers_.length() == 0);
    assert(is_nil(collocatedServer_));
}

// ----------------------------------------------------------------------
// ServerManager public member implementation
// ----------------------------------------------------------------------

void
ServerManager::destroy()
{
    JTCSynchronized synchronized(*this);;

    //
    // Don't destroy twice
    //
    if(destroy_)
	return;

    //
    // Set the destroy flag
    //
    destroy_ = true;

    //
    // Destroy all servers
    //
    for(ULong i = 0; i < allServers_.length(); i++)
	allServers_[i] -> destroy();

    allServers_.length(0);
    collocatedServer_ = CollocatedServer::_nil();
}

void
ServerManager::hold()
{
    JTCSynchronized synchronized(*this);

    for(ULong i = 0; i < allServers_.length(); i++)
        allServers_[i] -> hold();
}


void
ServerManager::activate()
{
    JTCSynchronized synchronized(*this);

    for(ULong i = 0; i < allServers_.length(); i++)
        allServers_[i] -> activate();
}

CollocatedServer_ptr
ServerManager::getCollocatedServer()
{
    JTCSynchronized synchronized(*this);

    return CollocatedServer::_duplicate(collocatedServer_);
}
