// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <SharedLibrary.h>

#ifndef WIN32
#   ifndef HAVE_SHL_LOAD
#       include <dlfcn.h>
#   endif
#endif

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// SharedLibrary implementation (Win32)
// ----------------------------------------------------------------------

#ifdef WIN32

SharedLibrary::SharedLibrary()
    : handle_(0)
{
}

SharedLibrary::~SharedLibrary()
{
    if(handle_ != 0)
        FreeLibrary(handle_);
}

bool
SharedLibrary::open(const char* name)
{
    HINSTANCE handle = LoadLibrary(name);

    if(handle != 0)
    {
        handle_ = handle;
        return true;
    }
    else
        return false;
}

SharedLibrary::proc_type
SharedLibrary::findsym(const char* sym)
{
    assert(handle_ != 0);
    return GetProcAddress(handle_, sym);
}

char*
SharedLibrary::error() const
{
    return string_dup("");
}

#else

SharedLibrary::SharedLibrary()
    : handle_(0)
{
}

SharedLibrary::~SharedLibrary()
{
#ifdef HAVE_SHL_LOAD
    //
    // On HP-UX 10, only call shl_unload if the library's reference
    // count is 1
    //
    if(handle_ != 0)
    {
        struct shl_descriptor desc;
        if(shl_gethandle_r(handle_, &desc) == -1)
            assert(false);
        if(desc.ref_count == 1)
            shl_unload(handle_);
    }
#else
#  if !defined(__GNUC__) && !defined(_AIX)
    //
    // TODO: needs further investigation
    //
    // Unloading a dynamic module causes the watson demo server to crash after
    // main returns on Linux & Solaris (both with GCC-2.95.3) and on AIX
    //
    if(handle_ != 0)
        dlclose(handle_);
#  endif
#endif
}

bool
SharedLibrary::open(const char* name)
{
#ifdef HAVE_SHL_LOAD
    shl_t handle = shl_load(name, BIND_IMMEDIATE | DYNAMIC_PATH, 0L);
#else
    void* handle = dlopen(name, RTLD_NOW);
#endif

    if(handle != 0)
    {
        handle_ = handle;
        return true;
    }
    else
        return false;
}

SharedLibrary::proc_type
SharedLibrary::findsym(const char* sym)
{
    assert(handle_ != 0);

#ifdef HAVE_SHL_LOAD
    void* fcn;
    if(shl_findsym(&handle_, sym, TYPE_PROCEDURE, &fcn) < 0)
        fcn = 0;
#else
    void* fcn = dlsym(handle_, sym);
#endif

    return fcn;
}

char*
SharedLibrary::error() const
{
#ifdef HAVE_SHL_LOAD
    return string_dup("");
#else
    return string_dup(dlerror());
#endif
}

#endif
