// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/Timer.h>

#ifdef WIN32
#   include <sys/timeb.h>
#endif

using namespace OB;

// ----------------------------------------------------------------------
// Global timer utility functions
// ----------------------------------------------------------------------

int
OB::TimeCmp(const timeval& a, const timeval& b)
{
    assert(a.tv_sec >= 0);
    assert(a.tv_usec >= 0);
    assert(a.tv_usec < 1000000);

    assert(b.tv_sec >= 0);
    assert(b.tv_usec >= 0);
    assert(b.tv_usec < 1000000);

    if(a.tv_sec != b.tv_sec)
	return a.tv_sec > b.tv_sec ? 1 : -1;
    
    if(a.tv_usec != b.tv_usec)
	return a.tv_usec > b.tv_usec ? 1 : -1;
    
    return 0;
}

timeval
OB::TimeSub(const timeval& a, const timeval& b)
{
    assert(a.tv_sec >= 0);
    assert(a.tv_usec >= 0);
    assert(a.tv_usec < 1000000);

    assert(b.tv_sec >= 0);
    assert(b.tv_usec >= 0);
    assert(b.tv_usec < 1000000);

    timeval tv;

    tv.tv_sec = a.tv_sec - b.tv_sec;
    tv.tv_usec = a.tv_usec - b.tv_usec;

    tv.tv_sec += tv.tv_usec / 1000000;
    tv.tv_usec = tv.tv_usec % 1000000;

    if(tv.tv_usec < 0)
    {
	tv.tv_usec += 1000000;
	tv.tv_sec -= 1;
    }

    assert(tv.tv_sec >= 0);
    assert(tv.tv_usec >= 0);
    assert(tv.tv_usec < 1000000);

    return tv;
}

timeval
OB::TimeAdd(const timeval& a, const timeval& b)
{
    assert(a.tv_sec >= 0);
    assert(a.tv_usec >= 0);
    assert(a.tv_usec < 1000000);

    assert(b.tv_sec >= 0);
    assert(b.tv_usec >= 0);
    assert(b.tv_usec < 1000000);

    timeval tv;

    tv.tv_sec = a.tv_sec + b.tv_sec;
    tv.tv_usec = a.tv_usec + b.tv_usec;
    
    tv.tv_sec += tv.tv_usec / 1000000;
    tv.tv_usec = tv.tv_usec % 1000000;

    if(tv.tv_usec < 0)
    {
	tv.tv_usec += 1000000;
	tv.tv_sec -= 1;
    }

    assert(tv.tv_sec >= 0);
    assert(tv.tv_usec >= 0);
    assert(tv.tv_usec < 1000000);

    return tv;
}

timeval
OB::TimeNow()
{
    timeval tv;
#ifdef WIN32
    struct _timeb timebuffer;
    _ftime(&timebuffer);
    tv.tv_sec = timebuffer.time;
    tv.tv_usec = timebuffer.millitm * 1000;
#else
    gettimeofday(&tv, 0);
#endif
    return tv;
}
