// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_TYPE_CODE_FACTORY_H
#define OB_TYPE_CODE_FACTORY_H

#include <OB/MarshalStubImpl.h>
#include <OB/IFR1.h>
#include <OB/TCKind.h>

namespace OB
{

//
// The TypeCodeFactory class
//
class TypeCodeFactory
{
    static bool checkId(const char*);
    static bool checkName(const char*);
    static bool checkType(CORBA::TypeCode_ptr);
    static bool compareLabels(CORBA::TCKind, const CORBA::Any&,
                              const CORBA::Any&);

public:

    static CORBA::TypeCode_ptr
    createPrimitiveTC(CORBA::TCKind, bool = false);

    static CORBA::TypeCode_ptr
    createStructTC(const char*, const char*, const CORBA::StructMemberSeq&);
    
    static CORBA::TypeCode_ptr
    createUnionTC(const char*, const char*, CORBA::TypeCode_ptr,
		  const CORBA::UnionMemberSeq&);
    
    static CORBA::TypeCode_ptr
    createEnumTC(const char*, const char*, const CORBA::EnumMemberSeq&);
    
    static CORBA::TypeCode_ptr
    createAliasTC(const char*, const char*, CORBA::TypeCode_ptr);
    
    static CORBA::TypeCode_ptr
    createExceptionTC(const char*, const char*, const CORBA::StructMemberSeq&);
    
    static CORBA::TypeCode_ptr
    createInterfaceTC(const char*, const char*);
    
    static CORBA::TypeCode_ptr
    createStringTC(CORBA::ULong);
    
    static CORBA::TypeCode_ptr
    createWstringTC(CORBA::ULong);
    
    static CORBA::TypeCode_ptr
    createFixedTC(CORBA::UShort, CORBA::Short);
    
    static CORBA::TypeCode_ptr
    createSequenceTC(CORBA::ULong, CORBA::TypeCode_ptr);
    
    static CORBA::TypeCode_ptr
    createRecursiveSequenceTC(CORBA::ULong, CORBA::ULong);
    
    static CORBA::TypeCode_ptr
    createArrayTC(CORBA::ULong, CORBA::TypeCode_ptr);
    
    static CORBA::TypeCode_ptr
    createValueTC(const char*, const char*, CORBA::ValueModifier,
		  CORBA::TypeCode_ptr, const CORBA::ValueMemberSeq&);
    
    static CORBA::TypeCode_ptr
    createValueBoxTC(const char*, const char*, CORBA::TypeCode_ptr);
    
    static CORBA::TypeCode_ptr
    createNativeTC(const char*, const char*);
    
    static CORBA::TypeCode_ptr
    createRecursiveTC(const char*);
    
    static CORBA::TypeCode_ptr
    createAbstractInterfaceTC(const char*, const char*);

    static CORBA::TypeCode_ptr
    createLocalInterfaceTC(const char*, const char*);
};

} // End of namespace OB

#endif
