// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_URL_IMPL_H
#define OB_URL_IMPL_H

#include <OB/ORBInstance_fwd.h>
#include <OB/OBORB_fwd.h>
#include <OB/Hashtable.h>
#include <OB/URL.h>

namespace OB
{

class URLRegistry_impl : public URLRegistry,
			 public OBCORBA::RefCountLocalObject
{
    URLRegistry_impl(const URLRegistry_impl&);
    void operator=(const URLRegistry_impl&);

    typedef OB::Hashtable< CORBA::String_var,
    			   OB::URLScheme_var,
    			   OB::StringHasher> SchemeTable;
    SchemeTable schemes_; 

public:

    URLRegistry_impl();
    ~URLRegistry_impl();

    virtual void add_scheme(URLScheme_ptr)
        throw(SchemeAlreadyExists,
              CORBA::SystemException);

    virtual URLScheme_ptr find_scheme(const char*)
        throw(CORBA::SystemException);

    virtual CORBA::Object_ptr parse_url(const char*)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);
};

class IORURLScheme_impl : public URLScheme,
			  public OBCORBA::RefCountLocalObject
{
    IORURLScheme_impl(const IORURLScheme_impl&);
    void operator=(const IORURLScheme_impl&);

    ORBInstance_var orbInstance_;

public:

    IORURLScheme_impl(ORBInstance_ptr);
    ~IORURLScheme_impl();

    virtual char* name()
        throw(CORBA::SystemException);

    virtual CORBA::Object_ptr parse_url(const char*)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);
};

class FileURLScheme_impl : public URLScheme,
			   public OBCORBA::RefCountLocalObject
{
    FileURLScheme_impl(const FileURLScheme_impl&);
    void operator=(const FileURLScheme_impl&);

    bool relative_;
    URLRegistry_var registry_;

public:

    FileURLScheme_impl(bool, URLRegistry_ptr);
    ~FileURLScheme_impl();

    virtual char* name()
        throw(CORBA::SystemException);

    virtual CORBA::Object_ptr parse_url(const char*)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);
};

class CorbalocURLScheme_impl : public CorbalocURLScheme,
			       public OBCORBA::RefCountLocalObject
{
    CorbalocURLScheme_impl(const CorbalocURLScheme_impl&);
    void operator=(const CorbalocURLScheme_impl&);

    ORBInstance_var orbInstance_;
    typedef OB::Hashtable< CORBA::String_var,
    			   OB::CorbalocProtocol_var,
    			   OB::StringHasher> ProtocolTable;
    ProtocolTable protocols_; 

    CORBA::Object_ptr parse_addresses(const char*, const char*, char*&);

public:

    CorbalocURLScheme_impl(ORBInstance_ptr);
    ~CorbalocURLScheme_impl();

    virtual char* name()
        throw(CORBA::SystemException);

    virtual CORBA::Object_ptr parse_url(const char*)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);

    virtual void add_protocol(CorbalocProtocol_ptr)
        throw(CorbalocURLScheme::ProtocolAlreadyExists,
              CORBA::SystemException);

    virtual CorbalocProtocol_ptr find_protocol(const char*)
        throw(CORBA::SystemException);
};

class CorbanameURLScheme_impl : public URLScheme,
			        public OBCORBA::RefCountLocalObject
{
    CorbanameURLScheme_impl(const CorbanameURLScheme_impl&);
    void operator=(const CorbanameURLScheme_impl&);

    CORBA::ORB_var orb_;
    CorbalocURLScheme_var corbaloc_;

public:

    CorbanameURLScheme_impl(CORBA::ORB_ptr, URLRegistry_ptr);
    ~CorbanameURLScheme_impl();

    virtual char* name()
        throw(CORBA::SystemException);

    virtual CORBA::Object_ptr parse_url(const char*)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);
};

} // End of namespace OB

#endif
