// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/NativeTypes.h>
#include <OB/Object.h>
#include <OB/MarshalStubImpl.h>
#include <OB/ValueBase.h>
#include <OB/AbstractBase.h>
#include <OB/ValueFactoryBase.h>
#include <OB/ValueFactoryManager.h>
#include <OB/OBORB.h>
#include <OB/ORB_init.h>
#include <OB/IFR4.h>
#include <OB/Stream.h>
#include <OB/ORBInstance.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ValueBaseInfo public member functions
// ----------------------------------------------------------------------

void
ValueBaseInfo::free(void* v) const
{
    CORBA::remove_ref((ValueBase*)v);
}

void*
ValueBaseInfo::dup(const void* v) const
{
    ValueBase* vb = (ValueBase*)v;
    CORBA::add_ref(vb);
    return vb;
}

void
ValueBaseInfo::marshal(const void* v, OutputStreamImpl* out) const
{
    out -> write_value((ValueBase*)v);
}

void
ValueBaseInfo::unmarshal(void* v, InputStreamImpl* in) const
{
    ValueBase* vb = (ValueBase*)v;
    if(vb)
        vb -> _OB_unmarshal(in);
}

// ----------------------------------------------------------------------
// ValueBase static members
// ----------------------------------------------------------------------

const char* ValueBase::_ob_id_ = "IDL:omg.org/CORBA/ValueBase:1.0";
const char* ValueBase::_ob_truncatable_ids_[] =
{
    0
};

ValueBaseInfo ValueBase::_OB_valueBaseInfo;

// ----------------------------------------------------------------------
// ValueBase constructor and destructor
// ----------------------------------------------------------------------

ValueBase::ValueBase()
{
}

ValueBase::ValueBase(const ValueBase&)
{
}
    
ValueBase::~ValueBase()
{
}

// ----------------------------------------------------------------------
// ValueBase public member functions
// ----------------------------------------------------------------------

ValueDef_ptr
ValueBase::_get_value_def()
{
    Object_var obj;

    try
    {
        //
	// IMPORTANT!!!! Don't use ORB_init() anywhere else. It is
	// only used here, because ValueBase doesn't have an ORB
	// associated with it, so we have to use the "default" ORB.
	//
	int argcDummy = 0;
	char* argvDummy[1];
	argvDummy[0] = 0;
	ORB_var orb = ORB_init(argcDummy, argvDummy, "");
	obj = orb -> resolve_initial_references("InterfaceRepository");
    }
    catch(const ORB::InvalidName&)
    {
        throw INTF_REPOS(MinorNoIntfRepos, COMPLETED_NO);
    }

    if(is_nil(obj))
    {
        throw INTF_REPOS(MinorNoIntfRepos, COMPLETED_NO);
    }

    Repository_var repository = Repository::_narrow(obj);

    if(is_nil(repository))
    {
        throw INTF_REPOS(MinorNoIntfRepos, COMPLETED_NO);
    }

    //
    // Obtain our dynamic type & attempt to find the Contained object
    //
    const char* id = _OB_typeId();
    Contained_var contained = repository -> lookup_id(id);

    if(is_nil(contained))
        return ValueDef::_nil();

    return ValueDef::_narrow(contained);
}

ValueBase*
ValueBase::_downcast(ValueBase* v)
{
    return v;
}

ValueBase*
ValueBase::_downcast(AbstractBase_ptr a)
{
    if(!is_nil(a))
    {
        ValueBase_var vb = a -> _to_value();
        return vb.in();
    }
    return 0;
}

const char*
ValueBase::_OB_typeId() const
{
    return _ob_id_;
}

const char*
ValueBase::_OB_id()
{
    return _ob_id_;
}

const char**
ValueBase::_OB_truncatableIds() const
{
    return _ob_truncatable_ids_;
}

Boolean
ValueBase::_OB_isCustom() const
{
    return false;
}

Boolean
ValueBase::_OB_isTruncatable() const
{
    return false;
}

//
// This will be overridden by valuetypes which support an abstract interface
//
AbstractBase_ptr
ValueBase::_OB_toAbstractBase()
{
    return AbstractBase::_nil();
}

void
ValueBase::_OB_marshal(OutputStreamImpl*)
{
    // This should not be called
    assert(false);
}

void
ValueBase::_OB_unmarshal(InputStreamImpl*)
{
    // This should not be called
    assert(false);
}

void
OBMarshal(ValueBase* v, OB::OutputStreamImpl* _ob_out)
{
    _ob_out -> write_value(v);
}

void
OBUnmarshal(ValueBase*& v, OB::InputStreamImpl* _ob_in)
{
    v = _ob_in -> read_value();
}

ValueBase*
ValueBase::_OB_createValue(ORBInstance_ptr orbInstance,
                           const OB::StrSeq< int >& ids,
                           const char* id)
{
    ValueFactoryManager_var manager = orbInstance -> getValueFactoryManager();

    //
    // If an array of repository IDs is provided, it represents the
    // set of "truncatable" IDs
    //
    if(ids.length() > 0)
    {
        for(ULong i = 0 ; i < ids.length() ; i++)
        {
            ValueFactoryBase_var factory =
                manager -> lookupValueFactory(ids[i]);
            if(factory.in() != 0)
                return factory -> create_for_unmarshal();

            //
            // If we have a formal ID, and we haven't found
            // a factory yet, then give up
            //
            if(id != 0 && strcmp(ids[i], id) == 0)
                break;
        }
    }
    else if(id != 0)
    {
        ValueFactoryBase_var factory = manager -> lookupValueFactory(id);
        if(factory.in() != 0)
            return factory -> create_for_unmarshal();
    }

    throw MARSHAL(MinorNoValueFactory, COMPLETED_NO);
    return 0; // some compilers need this
}

ValueBase*
ValueBase::_OB_createValue(ValueFactory factory)
{
    return factory -> create_for_unmarshal();
}

void
OBRelease(CORBA::ValueBase* v)
{
    remove_ref(v);
}

void
OBDuplicate(CORBA::ValueBase* v)
{
    add_ref(v);
}

// ----------------------------------------------------------------------
// DefaultValueRefCountBase constructor and destructor
// ----------------------------------------------------------------------

DefaultValueRefCountBase::DefaultValueRefCountBase()
{
}

DefaultValueRefCountBase::DefaultValueRefCountBase(
    const DefaultValueRefCountBase&)
{
}
    
DefaultValueRefCountBase::~DefaultValueRefCountBase()
{
}

// ----------------------------------------------------------------------
// DefaultValueRefCountBase public member functions
// ----------------------------------------------------------------------

ValueBase*
DefaultValueRefCountBase::_add_ref()
{
    _OB_incRef();
    return this;
}

void
DefaultValueRefCountBase::_remove_ref()
{
    _OB_decRef();
}

ULong
DefaultValueRefCountBase::_refcount_value()
{
    return _OB_getRef();
}

// ----------------------------------------------------------------------
// ValueRefCountBase private member implementation
// ----------------------------------------------------------------------

void
PortableServer::ValueRefCountBase::servantAddRefImpl()
{
    RefCountServantBase::_add_ref();
}

// ----------------------------------------------------------------------
// ValueRefCountBase constructor and destructor
// ----------------------------------------------------------------------

PortableServer::ValueRefCountBase::ValueRefCountBase()
{
}

PortableServer::ValueRefCountBase::ValueRefCountBase(const ValueRefCountBase&)
{
}
    
PortableServer::ValueRefCountBase::~ValueRefCountBase()
{
}

// ----------------------------------------------------------------------
// ValueRefCountBase public member functions
// ----------------------------------------------------------------------

void
PortableServer::ValueRefCountBase::_remove_ref()
{
    RefCountServantBase::_remove_ref();
}

ULong
PortableServer::ValueRefCountBase::_refcount_value()
{
    return _OB_getRef();
}
