// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Hashtable.h>
#include <OB/IFR.h>

#include <stdlib.h>
#include <stdio.h>

using namespace CORBA;
using namespace OB;

static void
usage(const char* progName)
{
    OB_CERR << "Usage:\n";
    OB_CERR << progName << " [options] scoped-name ...\n"
	"\n"
	"Options:\n"
	"-h, --help             Show this message.\n"
	"-v, --version          Show ORBacus version.\n"
	;
}

int
run(ORB_ptr orb, int argc, char* argv[])
{
    //
    // Set program name
    //
    const char* progName = argv[0];

    //
    // Get options
    //
    Long i;
    for(i = 1 ; i < argc && *argv[i] == '-' ; i++)
    {
	if(strcmp(argv[i], "--help") == 0 ||
	   strcmp(argv[i], "-h") == 0)
	{
	    usage(progName);
	    return 0;
	}
	else if(strcmp(argv[i], "--version") == 0 ||
		strcmp(argv[i], "-v") == 0)
	{
	    OB_CERR << "ORBacus " << Version << OB_ENDL;
	    return 0;
	}
	else
	{
	    OB_CERR << progName << ": unknown option `"
		    << argv[i] << "'\n" << OB_ENDL;
	    usage(progName);
	    return 1;
	}
    }

    if(i == argc)
    {
	OB_CERR << progName << ": no scoped name" << OB_ENDL;
	usage(progName);
	return 1;
    }

    Object_var obj;
    try
    {
        obj = orb -> resolve_initial_references("InterfaceRepository");
    }
    catch(ORB::InvalidName&)
    {
    }
    if(is_nil(obj))
    {
	OB_CERR << "No `InterfaceRepository' initial reference." << OB_ENDL;
	return 1;
    }

    Repository_var repository = Repository::_narrow(obj);
    if(is_nil(repository))
    {
	OB_CERR
	    << "`InterfaceRepository' initial reference is not a repository."
	    << OB_ENDL;
	return 1;
    }

    for( ; i < argc ; i++)
    {
	const char* scopedName = argv[i];

	Contained_var contained = repository -> lookup(scopedName);

	if(is_nil(contained))
	{
	    OB_CERR << progName << ": warning: `" << scopedName
		    << "' does not exist" << OB_ENDL;
	}
	else
	{
	    contained -> destroy();
	}
    }

    return 0;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
	//
	// Initialize the ORB
	//
	orb = ORB_init(argc, argv);

	//
	// Generate code
	//
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
