// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#include <Util.h>
#include <GenUtil.h>
#include <GenCPP.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Generate code for the example source file
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplC(Container_ptr container, bool all,
                                   IdlPrettyPrint& out)
{
    //
    // Get container contents descriptions
    //
    ScopedName_var scope;
    // No _var for seq! See comments for the implementation of "describe"
    Container::DescriptionSeq* seq = describe(container, scope.out());

    ULong i, j, k;

    for(i = 0 ; i < seq -> length() ; i++)
    {
	//
	// Get element description
	//
	Container::Description& seqElem = (*seq)[i];

	//
	// Get container
	//
	Container_var cont = Container::_narrow(seqElem.contained_object);

        switch(seqElem.kind)
        {
        case dk_Module:
        {
            //
            // Module
            //
            const ModuleDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
		continue;

            comment(desc -> id, out);

            out.sep();

            assert(!is_nil(cont));
            genCPlusPlusImplC(cont, all, out);

            break;
        }

        case dk_AbstractInterface:
            //
            // No sample implementation for abstract interface
            //
            break;

	case dk_LocalInterface:
        case dk_Interface:
        {
            //
            // Interface
            //
            const InterfaceDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
                continue;

            Identifier_var name = fixKwd(desc -> name);
            ScopedName_var absolute = getAbsolute(desc -> id);

            comment(desc -> id, out);

	    if(seqElem.kind == dk_LocalInterface)
            {
                out.sep();
                out << '\n' << absolute << "_impl::" << name << "_impl()";
                out.start();
                out.end();
            }
            else
            {
                out.sep();
                out << '\n' << absolute << "_impl::" << name << "_impl("
                    << "PortableServer::POA_ptr poa)";
                out.inc();
                out << "\n: poa_(PortableServer::POA::_duplicate(poa))";
                out.dec();
                out.start();
                out.end();
            }
            out.sep();
            out << '\n' << absolute << "_impl::~" << name << "_impl()";
            out.start();
            out.end();

	    if(seqElem.kind != dk_LocalInterface)
            {
                //
                // _default_POA
                //
                out.sep();
                out << "\nPortableServer::POA_ptr"
                    << '\n' << absolute << "_impl::_default_POA()";
                out.start();
                out << "\nreturn PortableServer::POA::_duplicate(poa_);";
                out.end();
            }

            //
            // Generate attributes and operations for any base abstract
            // interfaces
            //
            if(!all)
            {
                InterfaceDef_var intf = InterfaceDef::_narrow(cont);
                assert(!is_nil(intf));
                IdlInterfaceInfoSeq baseInfo;
                IdlGetBaseInterfaces(intf, baseInfo, false);

                StringSeq ids;

                for(j = 0 ; j < baseInfo.length() ; j++)
                {
                    if(!baseInfo[j].is_abstract)
                        continue;

		    ScopedName_var dummy;
		    // No _var for seq! See comments for the
		    // implementation of "describe"
		    Container::DescriptionSeq* contents =
			describe(baseInfo[j].intf, dummy.out());

                    for(k = 0 ; k < contents -> length() ; k++)
                    {
                        if((*contents)[k].kind == dk_Operation)
                        {
                            const OperationDescription* opDesc;
                            Boolean b = ((*contents)[k].value) >>= opDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], opDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplCOperation(absolute, opDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = opDesc -> id;
                            }
                        }
                        else if((*contents)[k].kind == dk_Attribute)
                        {
                            const AttributeDescription* atDesc;
                            Boolean b = ((*contents)[k].value) >>= atDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], atDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplCAttribute(absolute, atDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = atDesc -> id;
                            }
                        }
                    }
                }
            }

            //
            // Attributes and operations
            //

            Container::DescriptionSeq_var contents =
                cont -> describe_contents(dk_all, !all, -1);

            for(j = 0 ; j < contents -> length() ; j++)
            {
                if(contents[j].kind == dk_Attribute)
                {
                    const AttributeDescription* atDesc;
                    Boolean b = (contents[j].value >>= atDesc);
                    assert(b);
                    genCPlusPlusImplCAttribute(absolute, atDesc, true, out);
                }
                else if(contents[j].kind == dk_Operation)
                {
                    const OperationDescription* opDesc;
                    Boolean b = (contents[j].value >>= opDesc);
                    assert(b);
                    genCPlusPlusImplCOperation(absolute, opDesc, true, out);
                }
            }

            break;
        }

        case dk_Value:
        {
            //
            // Valuetype
            //
            const ValueDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
                continue;

            if(desc -> is_abstract)
                continue;

            ValueDef_var value = ValueDef::_narrow(cont);
            assert(! is_nil(value));

            Identifier_var name = fixKwd(desc -> name);
            ScopedName_var absolute = getAbsolute(desc -> id);

            IdlSupportedInterfaceInfoSeq info;
            ULong numRegular = 0;
            IdlGetAllSupportedInterfaces(value, info, numRegular);

            comment(desc -> id, out);

            out.sep();
            out << '\n' << absolute << "_impl::" << name << "_impl()";
            out.start();
            out.end();
            out.sep();
            out << '\n' << absolute << "_impl::~" << name << "_impl()";
            out.start();
            out.end();

            out.sep();
            out << "\nCORBA::ValueBase*\n" << absolute
                << "_impl::_copy_value()";
            out.start();
            out << '\n' << name << "_impl* _r = new " << name << "_impl;";
            out << "\n// TODO: Implementation";
            out << "\nreturn _r;";
            out.end();

            //
            // Custom valuetype
            //
            if(desc -> is_custom)
            {
                out.sep();
                out << "\nvoid\n" << absolute
                    << "_impl::marshal(CORBA::DataOutputStream* os)";
                out.start();
                out << "\n// TODO: Implementation";
                out.end();
                out.sep();
                out << "\nvoid\n" << absolute
                    << "_impl::unmarshal(CORBA::DataInputStream* is)";
                out.start();
                out << "\n// TODO: Implementation";
                out.end();
            }

            //
            // Valuetype attributes and operations
            //
            StringSeq ids;
            genCPlusPlusImplCValueOpAtt(value, absolute, all, ids, out);

            //
            // Supported interface attributes and operations
            //
            ids.length(0);
            for(j = 0 ; j < info.length() ; j++)
            {
                if(all || info[j].inheritance_level == 0)
                    genCPlusPlusImplCValueSupport(
                        absolute, info[j].interface_info.intf, ids,
                        !info[j].interface_info.is_abstract, out);
            }

            //
            // Value factory
            //

            out.sep();
            out << "\n" << absolute << "Factory_impl::" << name
                << "Factory_impl()";
            out.start();
            out.end();

            out.sep();
            out << "\n" << absolute << "Factory_impl::~" << name
                << "Factory_impl()";
            out.start();
            out.end();

            out.sep();
            out << "\nCORBA::ValueBase*\n" << absolute
                << "Factory_impl::create_for_unmarshal()";
            out.start();
            out << "\n// TODO: Implementation";
            out << "\nreturn new " << absolute << "_impl;";
            out.end();

            InitializerSeq_var inits = value -> initializers();
            for(j = 0 ; j < inits -> length() ; j++)
            {
                String_var factoryName = fixKwd(inits[j].name);
                StructMemberSeq members = inits[j].members;
                out.sep();
                out << '\n' << absolute << "*\n" << absolute
                    << "Factory_impl::" << factoryName << '(';

                unsigned int indent = out.getIndent();
                out.setIndent(out.getPos());

                for(k = 0 ; k < members.length() ; k++)
                {
                    TypeCode_var origMemberType =
                        GetOrigType(members[k].type);
                    String_var argName = fixKwd(members[k].name);
                    String_var type;

                    if(k > 0)
                        out << ",\n";

                    if(origMemberType -> kind() == tk_array)
                    {
                        type = getTypeString("", members[k].type,
                                             GetTypeNormal);
                        out << type << ' ' << argName;
                    }
                    //
                    // Fix ambiguous types
                    //
                    else if(origMemberType -> kind() == tk_boolean)
                    {
                        out << "CORBA::ValueFactoryBase::BooleanValue "
                            << argName;
                    }
                    else if(origMemberType -> kind() == tk_char)
                    {
                        out << "CORBA::ValueFactoryBase::CharValue "
                            << argName;
                    }
                    else if(origMemberType -> kind() == tk_wchar)
                    {
                        out << "CORBA::ValueFactoryBase::WCharValue "
                            << argName;
                    }
                    else if(origMemberType -> kind() == tk_octet)
                    {
                        out << "CORBA::ValueFactoryBase::OctetValue "
                            << argName;
                    }
                    else
                    {
                        type = getTypeString("", members[k].type, GetTypeIn);
                        out << type << ' ' << argName;
                    }
                }

                out << ")";
                out.setIndent(indent);
                out.start();
                out << "\n// TODO: Implementation";
                out << "\nreturn new " << absolute << "_impl;";
                out.end();
            }

            break;
        }

        default:
            break;
        }
    }
}

// ----------------------------------------------------------------------
// Generate operations and attributes for supported interfaces
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplCValueSupport(const char* absolute,
                                               InterfaceDef_ptr intf,
                                               StringSeq& ids,
                                               bool ex,
                                               IdlPrettyPrint& out)
{
    ULong i, j;

    InterfaceDefSeq_var bases = intf -> base_interfaces();
    for(i = 0 ; i < bases -> length() ; i++)
    {
        RepositoryId_var id = bases[i] -> id();
        for(j = 0 ; j < ids.length() ; j++)
            if(strcmp(id, ids[j]) == 0)
                break;
        if(j >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplCValueSupport(absolute, bases[i], ids, ex, out);
        }
    }

    ScopedName_var dummy;
    // No _var for seq! See comments for the implementation of
    // "describe"
    Container::DescriptionSeq* contents = describe(intf, dummy.out());

    for(i = 0 ; i < contents ->  length() ; i++)
    {
        if((*contents)[i].kind == dk_Attribute)
        {
            const AttributeDescription* desc;
            Boolean b = ((*contents)[i].value >>= desc);
            assert(b);
            genCPlusPlusImplCAttribute(absolute, desc, ex, out);
        }
        else if((*contents)[i].kind == dk_Operation)
        {
            const OperationDescription* desc;
            Boolean b = ((*contents)[i].value >>= desc);
            assert(b);
            genCPlusPlusImplCOperation(absolute, desc, ex, out);
        }
    }
}

// ----------------------------------------------------------------------
// Generate valuetype operations and attributes, including inherited ones
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplCValueOpAtt(ValueDef_ptr value,
                                             const char* absolute,
                                             bool all,
                                             StringSeq& ids,
                                             IdlPrettyPrint& out)
{
    ULong i, j;

    //
    // Only generate operations and attributes for the base value if
    // all == true
    //
    ValueDef_var baseValue = value -> base_value();
    if(all && !is_nil(baseValue))
    {
        RepositoryId_var id = baseValue -> id();
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(ids[i], id) == 0)
                break;
        if(i >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplCValueOpAtt(baseValue, absolute, all, ids, out);
        }
    }

    //
    // Always generate operations and attributes for abstract base values
    //
    ValueDefSeq_var abstractBaseValues = value -> abstract_base_values();
    for(i = 0 ; i < abstractBaseValues -> length() ; i++)
    {
        RepositoryId_var id = abstractBaseValues[i] -> id();
        for(j = 0 ; j < ids.length() ; j++)
            if(strcmp(ids[j], id) == 0)
                break;
        if(j >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplCValueOpAtt(abstractBaseValues[i], absolute, all,
                                        ids, out);
        }
    }

    ScopedName_var dummy;
    // No _var for seq! See comments for the implementation of
    // "describe"
    Container::DescriptionSeq* contents = describe(value, dummy.out());

    for(i = 0 ; i < contents -> length() ; i++)
    {
        if((*contents)[i].kind == dk_Attribute)
        {
            const AttributeDescription* atDesc;
            Boolean b = ((*contents)[i].value >>= atDesc);
            assert(b);
            genCPlusPlusImplCAttribute(absolute, atDesc, false, out);
        }
        else if((*contents)[i].kind == dk_Operation)
        {
            const OperationDescription* opDesc;
            Boolean b = ((*contents)[i].value >>= opDesc);
            assert(b);
            genCPlusPlusImplCOperation(absolute, opDesc, false, out);
        }
    }
}

// ----------------------------------------------------------------------
// Generate code for an example attribute implementation
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplCAttribute(const char* absolute,
                                            const AttributeDescription* desc,
                                            bool ex,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);
    out.sep();

    TypeCode_var origType = GetOrigType(desc -> type);
    String_var ret = getTypeString("", desc -> type, GetTypeReturn);
    out << '\n' << ret << '\n' << absolute << "_impl::" << name << "()";
    if(ex)
    {
        out.inc();
        out << "\nthrow(CORBA::SystemException)";
        out.dec();
    }
    out.start();
    out << "\n// TODO: Implementation";
    genCPlusPlusImplCReturn(desc -> type, "_r", out);
    out << "\nreturn _r;";
    out.end();

    if(desc -> mode == ATTR_NORMAL)
    {
        out.sep();

        String_var in = getTypeString("", desc -> type, GetTypeIn);

        out << "\nvoid\n" << absolute << "_impl::" << name << '(' << in
            << " a)";
        if(ex)
        {
            out.inc();
            out << "\nthrow(CORBA::SystemException)";
            out.dec();
        }
        out.start();
        out << "\n// TODO: Implementation";
        out.end();
    }
}

// ----------------------------------------------------------------------
// Generate code for an example operation implementation
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplCOperation(const char* absolute,
                                            const OperationDescription* desc,
                                            bool ex,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);
    out.sep();

    String_var ret = getTypeString("", desc -> result, GetTypeReturn);

    out << '\n' << ret << '\n' << absolute << "_impl::" << name << '(';

    unsigned int indent = out.getIndent();
    out.setIndent(out.getPos());

    ULong i;

    for(i = 0 ; i < desc -> parameters.length() ; i++)
    {
        GetType tt;
        switch(desc -> parameters[i].mode)
        {
        case PARAM_IN:
            tt = GetTypeIn;
            break;

        case PARAM_INOUT:
            tt = GetTypeInOut;
            break;

        case PARAM_OUT:
            tt = GetTypeOut;
            break;
        }

        String_var argName = fixKwd(desc -> parameters[i].name);

        String_var s = getTypeString("", desc -> parameters[i].type,
                                           tt, false, argName);

        if(i > 0)
            out << ",\n";

        out << s;
    }

    if(desc -> contexts.length() > 0)
    {
        if(desc -> parameters.length() > 0)
            out << ",\n";
    
        out << "CORBA::Context_ptr context";
    }

    out << ')';

    out.setIndent(indent);

    if(ex)
    {
        out.inc();
        out << "\nthrow(";

        indent = out.getIndent();
        out.setIndent(out.getPos());

        for(i = 0 ; i < desc -> exceptions.length() ; i++)
        {
            Identifier_var exName = getAbsolute(desc -> exceptions[i].id);
            out << exName << ",\n";
        }

        out << "CORBA::SystemException)";
        out.setIndent(indent);
        out.dec();
    }

    out.start();
    out << "\n// TODO: Implementation";

    if(desc -> result -> kind() != tk_void)
        genCPlusPlusImplCReturn(desc -> result, "_r", out);

    //
    // Show example code for those out and inout parameters requiring
    // allocation
    //
    for(i = 0 ; i < desc -> parameters.length() ; i++)
    {
        if(desc -> parameters[i].mode == PARAM_IN)
            continue;

        String_var argName = fixKwd(desc -> parameters[i].name);

        //
        // Get original type
        //
        TypeCode_var origType =
            GetOrigType(desc -> parameters[i].type);

        //
        // Check if original type is fixed-sized or variable-sized
        //
        bool variable = IdlIsVariable(origType);

        String_var norm =
            getTypeString("", desc -> parameters[i].type, GetTypeNormal);
        String_var ret =
            getTypeString("", desc -> parameters[i].type, GetTypeReturn);

        TCKind kind = origType -> kind();
        switch(kind)
        {
        case tk_void:
        case tk_null:
        case tk_alias:
        case tk_except:
            assert(false);
            break;

        case tk_boolean:
        case tk_short:
        case tk_long:
        case tk_longlong:
        case tk_ushort:
        case tk_ulong:
        case tk_ulonglong:
        case tk_octet:
        case tk_float:
        case tk_double:
        case tk_longdouble:
        case tk_char:
        case tk_wchar:
        case tk_enum:
        case tk_fixed:
            // nothing to do
            break;

        case tk_string:
            if(desc -> parameters[i].mode == PARAM_INOUT)
                out << "\nCORBA::string_free(" << argName << ");";
            out << '\n' << argName << " = CORBA::string_dup(\"\");";
            break;

        case tk_wstring:
            if(desc -> parameters[i].mode == PARAM_INOUT)
                out << "\nCORBA::wstring_free(" << argName << ");";
            out << '\n' << argName << " = CORBA::wstring_dup(L\"\");";
            break;

        case tk_any:
            if(desc -> parameters[i].mode == PARAM_OUT)
                out << '\n' << argName << " = new " << norm << ';';
            break;

        case tk_TypeCode:
        case tk_Principal:
        case tk_objref:
        case tk_abstract_interface:
	case tk_local_interface:
            if(desc -> parameters[i].mode == PARAM_INOUT)
                out << "\nCORBA::release(" << argName << ");";
            out << '\n' << argName << " = " << norm << "::_nil();";
            break;

        case tk_struct:
        case tk_union:
            if(desc -> parameters[i].mode == PARAM_OUT && variable)
                out << '\n' << argName << " = new " << norm << ';';
            break;

        case tk_sequence:
            if(desc -> parameters[i].mode == PARAM_OUT)
                out << '\n' << argName << " = new " << norm << ';';
            break;

        case tk_array:
            if(desc -> parameters[i].mode == PARAM_OUT && variable)
            {
                String_var arr =
                    getTypeString("", desc -> parameters[i].type,
                                  GetTypeArray);
                out << '\n' << argName << " = " << arr << "_alloc();";
            }
            break;

        case tk_native:
            out << '\n' << argName << " = (" << norm << ")0;";
            break;

        case tk_value:
        case tk_value_box:
            out << '\n' << argName << " = 0;";
            break;
        }
    }

    if(desc -> result -> kind() != tk_void)
        out << "\nreturn _r;";

    out.end();
}

// ----------------------------------------------------------------------
// Generate example return statement
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplCReturn(TypeCode_ptr type,
                                         const char* name,
                                         IdlPrettyPrint& out)
{
    //
    // Get original type
    //
    TypeCode_var origType = GetOrigType(type);

    //
    // Check if original type is fixed-sized or variable-sized
    //
    bool variable = IdlIsVariable(origType);

    String_var norm = getTypeString("", type, GetTypeNormal);
    String_var ret = getTypeString("", type, GetTypeReturn);

    //
    // Get result
    //
    TCKind kind = origType -> kind();
    switch(kind)
    {
    case tk_void:
    case tk_null:
    case tk_alias:
    case tk_except:
        assert(false);
        break;

    case tk_boolean:
        out << '\n' << ret << ' ' << name << " = false;";
        break;

    case tk_short:
    case tk_long:
    case tk_longlong:
    case tk_ushort:
    case tk_ulong:
    case tk_ulonglong:
    case tk_octet:
        out << '\n' << ret << ' ' << name << " = 0;";
        break;

    case tk_float:
    case tk_double:
    case tk_longdouble:
        out << '\n' << ret << ' ' << name << " = 0.0;";
        break;

    case tk_char:
        out << '\n' << ret << ' ' << name << " = '\\0';";
        break;

    case tk_wchar:
        out << '\n' << ret << ' ' << name << " = L'\\0';";
        break;

    case tk_enum:
        out << '\n' << ret << ' ' << name << " = (" << ret << ")0;";
        break;

    case tk_fixed:
        out << '\n' << ret << ' ' << name << "(\"0\");";
        break;

    case tk_string:
        out << '\n' << ret << ' ' << name << " = CORBA::string_dup(\"\");";
        break;

    case tk_wstring:
        out << '\n' << ret << ' ' << name << " = CORBA::wstring_dup(L\"\");";
        break;

    case tk_any:
        out << '\n' << ret << ' ' << name << " = new " << norm << ';';
        break;

    case tk_TypeCode:
    case tk_Principal:
    case tk_objref:
    case tk_abstract_interface:
    case tk_local_interface:
        out << '\n' << ret << ' ' << name << " = " << norm << "::_nil();";
        break;

    case tk_struct:
    case tk_union:
        if(variable)
            out << '\n' << ret << ' ' << name << " = new " << norm << ';';
        else
            out << '\n' << ret << ' ' << name << ';';
        break;

    case tk_sequence:
        out << '\n' << ret << ' ' << name << " = new " << norm << ';';
        break;

    case tk_array:
    {
        String_var arr = getTypeString("", type, GetTypeArray);
        out << '\n' << ret << ' ' << name << " = " << arr << "_alloc();";
        break;
    }

    case tk_native:
        out << '\n' << ret << ' ' << name << " = (" << norm << ")0;";
        break;

    case tk_value:
    case tk_value_box:
        out << '\n' << ret << ' ' << name << " = 0;";
        break;
    }
}
