// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#include <Util.h>
#include <GenUtil.h>
#include <GenTextBase.h>
#include <GenHTML.h>

#include <stdlib.h>
#include <ctype.h>
#include <errno.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Special HTML "Pretty-Print" class member implementation
// ----------------------------------------------------------------------

IdlHTMLGenerator::PrettyPrint::PrettyPrint(const char* prog,
                                           const char* fullName,
                                           const char* dir,
                                           unsigned int num)
    : IdlPrettyPrint(out_, num)
{
    //
    // Create complete file name path
    //
    path_ = string_dup(dir);
    if(strlen(path_) > 0)
        path_ += '/';
    path_ += fullName;
    path_ += ".html";

    //
    // Create the file
    //
    out_.open(path_);

    //
    // Write header
    //
    if(good())
    {
        direct("<!DOCTYPE html "
               "PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n");

        direct("\n<html>\n<head>\n\n");

        direct("<!-- Generated by the ORBacus IDL-to-HTML translator -->\n"
               "\n"
               "<!-- Copyright (c) 2002 -->\n"
               "<!-- IONA Technologies, Inc. -->\n"
               "<!-- Waltham, MA, USA -->\n"
               "\n"
               "<!-- All Rights Reserved -->\n"
               "\n"
               "<!-- Version: ");
        *this << Version;
        direct(" -->\n");

        direct("\n<title>IDL Documentation for &quot;");
        *this << fullName;
        direct("&quot;</title>\n");

        direct("\n</head>\n<body>\n");
    }
    else
    {
        OB_CERR << prog << ": can't open \"" << path_
                << "\": " << strerror(errno) << OB_ENDL;
    }
}

IdlHTMLGenerator::PrettyPrint::~PrettyPrint()
{
    if(good())
    {
        direct("\n<hr />");

        direct("\n<p /><em>");

        direct("\nGenerated by the "
               "<a href=\"http://www.ooc.com/ob/\">"
               "ORBacus</a> IDL-to-HTML translator");

        direct("\n</em>");

        direct("\n</body>\n</html>\n");
    }
}

// ----------------------------------------------------------------------
// Pretty-Print private and protected members
// ----------------------------------------------------------------------

void
IdlHTMLGenerator::PrettyPrint::printChar(char c)
{
    switch(c)
    {
    case '>':
        direct("&gt;");
        pos_++;
        break;

    case '<':
        direct("&lt;");
        pos_++;
        break;

    case '&':
        direct("&amp;");
        pos_++;
        break;

    default:
        IdlPrettyPrint::printChar(c);
    }
}
