// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#include <Util.h>
#include <GenUtil.h>

#ifdef WIN32
#   include <direct.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#include <GenJava.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

using namespace CORBA;
using namespace OB;

#define OBEVersion "2.0.1"

// ----------------------------------------------------------------------
// Special Java "Pretty-Print" class member implementation
// ----------------------------------------------------------------------

const char* IdlJavaGenerator::PrettyPrint::header_ =
"// **********************************************************************\n"
"//\n"
"// Generated by the ORBacus IDL to Java Translator\n"
"//\n"
"// Copyright (c) 2002\n"
"// IONA Technologies, Inc.\n"
"// Waltham, MA, USA\n"
"//\n"
"// All Rights Reserved\n"
"//\n"
"// **********************************************************************";

const char* IdlJavaGenerator::PrettyPrint::embeddedHeader_ =
"// **********************************************************************\n"
"//\n"
"// Generated by the Orbix/E IDL to Java Translator\n"
"//\n"
"// Copyright (c) 2002\n"
"// IONA Technologies, Inc.\n"
"// Waltham, MA, USA\n"
"//\n"
"// All Rights Reserved\n"
"//\n"
"// **********************************************************************";

IdlJavaGenerator::PrettyPrint::PrettyPrint(StringSeq& fileNameSeq,
					   const char* prog,
					   const char* name,
					   const char* package,
					   const char* dir,
					   bool embedded,
					   unsigned int num)
    : IdlPrettyPrint(out_, num), embedded_(embedded)
{
    //
    // Create directories and construct the directory path
    //
    path_ = string_dup(dir);
    String_var str = string_dup(package);
    char* s = str.inout();
    char* tok = 0;
    const char* sep = ".";
    while((tok = strtok(s, sep)))
    {
	s = 0;

	if(strlen(tok))
	{
	    if(strlen(path_) > 0)
		path_ += '/';
	    
	    path_ += tok;

#ifdef WIN32
	    _mkdir((const char*)path_);
#else
	    mkdir((const char*)path_, S_IRWXU | S_IRWXG | S_IRWXO);
#endif
	}
    }

    //
    // Create complete file name path
    //
    if(strlen(path_) > 0)
	path_ += '/';
    path_ += name;
    path_ += ".java";

    //
    // Create the file
    //
    out_.open(path_);

    //
    // Write header and package
    //
    if(good())
    {
        fileNameSeq.length(fileNameSeq.length() + 1);
        fileNameSeq[fileNameSeq.length() - 1] = path_;

	if(embedded_)
	{
	    *this << embeddedHeader_;
	    *this << '\n';
	    *this << "\n// Version: " << OBEVersion;
	}
	else
	{
	    *this << header_;
	    *this << '\n';
	    *this << "\n// Version: " << Version;
	}

	if(package && strlen(package) > 0)
	{
	    this -> sep();
	    *this << "\npackage " << package << ';';
	}
    }
    else
    {
	OB_CERR << prog << ": can't open \"" << path_ << "\": "
		<< strerror(errno) << OB_ENDL;
    }
}

// ----------------------------------------------------------------------
// Print header comment
// ----------------------------------------------------------------------

void
IdlJavaGenerator::comment(const char* s, IdlPrettyPrint& out)
{
    out.sep();
    out << "\n//";
    out << "\n// " << s;
    out << "\n//";
    out.ignoreNextSep();
}

// ----------------------------------------------------------------------
// Print documentation comment
// ----------------------------------------------------------------------

void
IdlJavaGenerator::docComment(const char* id, IdlPrettyPrint& out)
{
    if(!noComments_)
    {
	String_var comment;
	bool getResult = idHash_.get(id, comment);
	if(getResult)
	{
	    out << "\n/**";
	    
	    const char* s = comment;
	    
	    while(*s)
	    {
		static const char* ex = "@exception";
		if(strncmp(s, ex, strlen(ex)) == 0)
		{
		    out << ex;
		    s += strlen(ex);
		    
		    while(*s == ' ' || *s == '\t')
		    {
			out << *s;
			s++;
		    }
		    
		    const char* s2 = s;

		    while(*s2 != ' ' && *s2 != '\t' &&
			  *s2 != '\n' && *s2 != '\0')
		    {
			s2++;
		    }
		    
		    if((*s2 == ' ' || *s2 == '\t') && (s2 > s))
		    {
			String_var name = string_alloc(s2 - s);
			strncpy(name.inout(), s, s2 - s);
			name[s2 - s] = '\0';
			
			Contained_var contained = repository_ -> lookup_id(id);
			
			if(!is_nil(contained))
			{
			    Container_var container =
				contained -> defined_in();
			    
			    if(!is_nil(container))
			    {
				do
				{
				    try
				    {
					contained = container -> lookup(name);
				    }
				    catch(const INTF_REPOS&)
				    {
					contained = Contained::_nil();
					
					break;
				    }
				    
				    if(is_nil(contained))
				    {
					Contained_var c =
					    Contained::_narrow(container);
					
					if(is_nil(c))
					    break;
					
					container = c -> defined_in();
				    }
				}
				while(is_nil(contained));
				
				if(!is_nil(contained))
				    name = getAbsolute(contained);
			    }
			}
			
			out << name;
			
			out << *s2;
			s = s2 + 1;
		    }
		    
		    continue;
		}
		
		if(*s == '\n')
		    out << "\n *";
		else
		    out << *s;
		
		s++;
	    }
	    
	    out << "*/";
	}
    }
}

// ----------------------------------------------------------------------
// Precede Java keywords and reserved names by an underscore
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::fixName(const char* name, bool checkReserved)
{
    assert_nca(name != 0, NCANullString);
    assert(strchr(name, '.') == 0); // Not for absolute names

    if(*name == '\0')
	return string_dup("");

    ULong nameLen = strlen(name);

    static const char* kwds[] =
    {
	//
	// Note: This list must be sorted alphabetically
	//
	"abstract", "assert", "boolean", "break", "byte",
	"case", "catch", "char", "class",
	"clone", "const", "continue", "default",
	"do", "double", "else", "equals",
	"extends", "false", "final", "finalize",
	"finally", "float", "for", "getClass",
	"goto", "hashCode", "if", "implements",
	"import", "instanceof", "int", "interface",
	"long", "native", "new", "notify",
	"notifyAll", "null", "package", "private",
	"protected", "public", "return", "short",
	"static", "super", "switch", "synchronized",
	"this", "throw", "throws", "toString",
	"transient", "true", "try", "void",
	"volatile", "wait", "while"
    };

    ULong l = 0;
    ULong r = sizeof(kwds) / sizeof(const char*);

    while(l < r)
    {
        ULong m = (l + r) / 2;
	int res = strcmp(kwds[m], name);
        if(res == 0)
	{
	    String_var tmp = string_alloc(1 + nameLen);
            strcpy(tmp.inout(), "_");
	    strcat(tmp.inout(), name);
	    return tmp._retn();
	}
        else if(res > 0)
            r = m;
        else
            l = m + 1;
    }

    if(checkReserved)
    {
        //
        // Prepend an underscore for each of the reserved suffixes below
        //
        static const char* reserved[] =
        {
            "Helper", "Holder", "Operations", "POA", "POATie", "Package",
            "ValueFactory"
        };

        String_var result = name;
        String_var curr = name;

        bool match;
        do
        {
            ULong currLen = strlen(curr);

            match = false;
            for(ULong i = 0 ; i < sizeof(reserved) / sizeof(const char*) ; i++)
            {
                ULong resLen = strlen(reserved[i]);
                if(currLen > resLen &&
                   strncmp(reserved[i], &curr[currLen - resLen], resLen) == 0)
                {
                    String_var tmp;

                    //
                    // Prepend an underscore to result
                    //
                    tmp = string_alloc(1 + strlen(result));
                    strcpy(tmp.inout(), "_");
                    strcat(tmp.inout(), result);
                    result = tmp._retn();

                    //
                    // Remove suffix from curr
                    //
                    curr[currLen - resLen] = '\0';

                    match = true;
                    break;
                }
            }
        }
        while(match);

        return result._retn();
    }

    return string_dup(name);
}

// ----------------------------------------------------------------------
// Create type string
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::getTypeString(const char* package, TypeCode_ptr type,
				GetType gt)
{
    //
    // Get result
    //
    String_var result = string_dup("???");
    if(gt == GetTypeHelper && type -> kind() == tk_alias)
    {
	//
	// Helper classes are available for all aliases
	//
	result = getAbsolute(type -> id());
	result += "Helper";
	removePackage(package, result.inout());
	return result._retn();
    }
    else
    {
	TypeCode_var origType = GetOrigType(type);
	TCKind kind = origType -> kind();
	switch(kind)
	{
	case tk_void:
	case tk_short:
	case tk_long:
	case tk_ushort:
	case tk_ulong:
	case tk_longlong:
	case tk_ulonglong:
	case tk_float:
	case tk_double:
	case tk_fixed:
	case tk_boolean:
	case tk_char:
	case tk_wchar:
	case tk_octet:
	case tk_string:
	case tk_wstring:
	{
	    // **************************************************
	    // Basic types
	    // **************************************************
	    
	    switch(gt)
	    {
	    case GetTypeHolder:
	    {
		static const char* names[] =
		{
		    "???", "???", // null, void
		    "org.omg.CORBA.ShortHolder", "org.omg.CORBA.IntHolder",
		    "org.omg.CORBA.ShortHolder", "org.omg.CORBA.IntHolder",
		    "org.omg.CORBA.FloatHolder", "org.omg.CORBA.DoubleHolder",
		    "org.omg.CORBA.BooleanHolder", "org.omg.CORBA.CharHolder",
		    "org.omg.CORBA.ByteHolder",
		    "???", "???", "???", "???", "???", "???", "???",
		    "org.omg.CORBA.StringHolder",
		    "???", "???", "???", "???",
		    "org.omg.CORBA.LongHolder", "org.omg.CORBA.LongHolder",
		    "???", "org.omg.CORBA.CharHolder",
		    "org.omg.CORBA.StringHolder", "org.omg.CORBA.FixedHolder",
		    "???", "???", // value, value_box
		    "???", "???" // native, abstract_interface
		};
		result = names[(int)kind];
		break;
	    }
	    
	    case GetTypeHelper:
	    case GetTypeStub:
		result = string_dup("???");
		break;

	    case GetTypeNormal:
	    {
		static const char* names[] =
		{
		    "???", // null
		    "void",
		    "short", "int",
		    "short", "int",
		    "float", "double",
		    "boolean",
		    "char",
		    "byte",
		    "???", "???", "???", "???", "???", "???", "???",
		    "String",
		    "???", "???", "???", "???",
		    "long", "long",
		    "???",
		    "char",
		    "String",
		    "java.math.BigDecimal",
		    "???", "???", // value, value_box
		    "???", // native
                    "???" // abstract_interface
		};
		result = names[(int)kind];
		break;
	    }
	    }
	    
	    break;
	}
	
	case tk_any:
	{
	    // **************************************************
	    // The any type
	    // **************************************************

	    switch(gt)
	    {
	    case GetTypeHolder:
		result = string_dup("org.omg.CORBA.AnyHolder");
		break;
		
	    case GetTypeHelper:
	    case GetTypeStub:
		result = string_dup("???");
		break;

	    case GetTypeNormal:
		result = string_dup("org.omg.CORBA.Any");
		break;
	    }
	    
	    break;
	}
	
	case tk_TypeCode:
	{
            switch(gt)
            {
            case GetTypeHelper:
            case GetTypeStub:
                result = string_dup("???");
                break;

            case GetTypeNormal:
                result = string_dup("org.omg.CORBA.TypeCode");
                break;

            case GetTypeHolder:
                result = string_dup("org.omg.CORBA.TypeCodeHolder");
                break;
            }

            break;
        }

	case tk_Principal:
        {
            switch(gt)
            {
            case GetTypeHelper:
            case GetTypeStub:
                result = string_dup("???");
                break;

            case GetTypeNormal:
                result = string_dup("org.omg.CORBA.Principal");
                break;

            case GetTypeHolder:
                result = string_dup("org.omg.CORBA.PrincipalHolder");
                break;
            }

            break;
        }

	case tk_objref:
	{
            if(strcmp(origType -> id(), "IDL:omg.org/CORBA/Object:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeHelper:
                case GetTypeStub:
                    result = string_dup("???");
                    break;
                    
                case GetTypeNormal:
                    result = string_dup("org.omg.CORBA.Object");
                    break;

                case GetTypeHolder:
                    result = string_dup("org.omg.CORBA.ObjectHolder");
                    break;
                }
            }
            else
            {
                String_var s = getAbsolute(origType -> id());
                removePackage(package, s.inout());
                
                switch(gt)
                {
                case GetTypeHelper:
                    result = s;
                    result += "Helper";
                    break;

                case GetTypeStub:
                {
                    const char* p = strrchr(s.inout(), '.');

                    if(p)
                    {
                        p++;
                        assert(*p);
                        result = string_alloc(p - s.in());
                        strncpy(result.inout(), s, p - s.in());
                        result[p - s.in()] = '\0';

                        result += '_';
                        result += p;
                        result += "Stub";
                    }
                    else
                    {
                        result = string_dup("_");
                        result += s;
                        result += "Stub";
                    }

                    break;
                }

                case GetTypeNormal:
                    result = s;
                    break;

                case GetTypeHolder:
                    result = s;
                    result += "Holder";
                    break;
                }
            }

	    break;
	}

        case tk_struct:
	case tk_union:
	case tk_except:
	case tk_enum:
	{
	    // **************************************************
	    // Structs, unions, exceptions, enums
	    // **************************************************

            const char* id = origType -> id();
            String_var s = getAbsolute(id);
            removePackage(package, s.inout());
            result = s;

            //
            // Special case for org.omg.CORBA_2_4.TCKind
            //
            if(strcmp(id, "IDL:omg.org/CORBA/TCKind:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeHelper:
                    result = string_dup("org.omg.CORBA_2_4.TCKindHelper");
                    break;

                case GetTypeHolder:
                    result = string_dup("org.omg.CORBA_2_4.TCKindHolder");
                    break;

                case GetTypeStub:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    break;
                }
            }
            else
            {
                switch(gt)
                {
                case GetTypeHelper:
                    result += "Helper";
                    break;

                case GetTypeHolder:
                    result += "Holder";
                    break;

                case GetTypeStub:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    break;
                }
            }

	    break;
	}
	
	case tk_sequence:
	case tk_array:
	{
	    // **************************************************
	    // Sequences and arrays
	    // **************************************************
	    
	    switch(gt)
	    {
	    case GetTypeHelper:
		if(type -> kind() == tk_alias)
		{
		    String_var s = getAbsolute(type -> id());
		    removePackage(package, s.inout());
		    s += "Helper";
		    result = s;
		}
		break;

	    case GetTypeHolder:
		if(type -> kind() == tk_alias)
		{
                    //
                    // Unwind TypeCode to first alias
                    //
                    TypeCode_var tc = TypeCode::_duplicate(type);
                    while(true)
                    {
                        TypeCode_var content = tc -> content_type();
                        if(content -> kind() != tk_alias)
                            break;
                        tc = content;
                    }

		    String_var s = getAbsolute(tc -> id());
		    removePackage(package, s.inout());
		    s += "Holder";
		    result = s;
		}
		break;

	    case GetTypeStub:
		result = string_dup("???");
		break;
			
	    case GetTypeNormal:
	    {
		TypeCode_var contentType = origType -> content_type();
		result = getTypeString(package, contentType, gt);
		result += "[]";
		break;
	    }
	    }
	    
	    break;
	}

        case tk_value:
        {
            if(strcmp(origType -> id(),
                      "IDL:omg.org/CORBA/ValueBase:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeStub:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    result = string_dup("java.io.Serializable");
                    break;

                case GetTypeHelper:
                    result = string_dup("org.omg.CORBA.ValueBaseHelper");
                    break;

                case GetTypeHolder:
                    result = string_dup("org.omg.CORBA.ValueBaseHolder");
                    break;
                }
            }
            else
            {
                String_var s = getAbsolute(origType -> id());
                removePackage(package, s.inout());

                switch(gt)
                {
                case GetTypeStub:
                    result = string_dup("???");
                    break;

                case GetTypeHelper:
                    result = s;
                    result += "Helper";
                    break;

                case GetTypeNormal:
                    result = s;
                    break;

                case GetTypeHolder:
                    result = s;
                    result += "Holder";
                    break;
                }
            }

            break;
        }

        case tk_value_box:
        {
            String_var s = getAbsolute(origType -> id());
            removePackage(package, s.inout());

            switch(gt)
            {
            case GetTypeStub:
                result = string_dup("???");
                break;

            case GetTypeHelper:
                result = s;
                result += "Helper";
                break;

            case GetTypeNormal:
                if(isBasicBoxType(origType))
                    result = s;
                else
                {
                    TypeCode_var contentType = origType -> content_type();
                    result = getTypeString(package, contentType,
                                           GetTypeNormal);
                }
                break;

            case GetTypeHolder:
                result = s;
                result += "Holder";
                break;
            }

            break;
        }

        case tk_abstract_interface:
        {
            //
            // NOTE: CORBA::AbstractBase is a native type
            //

            String_var s = getAbsolute(origType -> id());
            removePackage(package, s.inout());

            switch(gt)
            {
            case GetTypeHelper:
                result = s;
                result += "Helper";
                break;

            case GetTypeStub:
            {
                const char* p = strrchr(s.inout(), '.');

                if(p)
                {
                    p++;
                    assert(*p);
                    result = string_alloc(p - s.in());
                    strncpy(result.inout(), s, p - s.in());
                    result[p - s.in()] = '\0';

                    result += '_';
                    result += p;
                    result += "Stub";
                }
                else
                {
                    result = string_dup("_");
                    result += s;
                    result += "Stub";
                }

                break;
            }

            case GetTypeNormal:
                result = s;
                break;

            case GetTypeHolder:
                result = s;
                result += "Holder";
                break;
            }

            break;
        }

        case tk_native:
        {
            String_var s = getAbsolute(origType -> id());
            removePackage(package, s.inout());

            //
            // CORBA::AbstractBase
            //
            if(strcmp(origType -> id(),
                      "IDL:omg.org/CORBA/AbstractBase:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeStub:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    result = string_dup("java.lang.Object");
                    break;

                case GetTypeHelper:
                    result = s;
                    result += "Helper";
                    break;

                case GetTypeHolder:
                    result = s;
                    result += "Holder";
                    break;
                }
            }
            //
            // CORBA::ValueFactory
            //
            else if(strcmp(origType -> id(),
                           "IDL:omg.org/CORBA/ValueFactory:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeStub:
                case GetTypeHelper:
                case GetTypeHolder:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    result = string_dup("org.omg.CORBA.portable.ValueFactory");
                    break;
                }
            }
            //
            // OB::PropertySet
            //
            else if(strcmp(origType -> id(),
                           "IDL:ooc.com/OB/PropertySet:1.0") == 0)
            {
                switch(gt)
                {
                case GetTypeStub:
                case GetTypeHelper:
                case GetTypeHolder:
                    result = string_dup("???");
                    break;

                case GetTypeNormal:
                    result = string_dup("java.util.Properties");
                    break;
                }
            }
            else
            {
                switch(gt)
                {
                case GetTypeStub:
                case GetTypeHelper:
                    result = string_dup("???");
                    break;

                    // TODO: This is a hack so that the Cookie type compiles
                case GetTypeNormal:
                    if(strcmp(s, "org.omg.PortableServer.ServantLocator."
			      "Cookie") == 0)
                        result = string_dup("java.lang.Object");
                    else
                        result = s;
                    break;

                    // TODO: This is a hack so that the Cookie type compiles
                case GetTypeHolder:
                    result = string_dup("org.omg.PortableServer."
					"ServantLocatorPackage.CookieHolder");
                    break;
                }
            }

            break;
        }

        case tk_local_interface:
        {
            String_var s = getAbsolute(origType -> id());
            removePackage(package, s.inout());

            switch(gt)
            {
            case GetTypeHelper:
                result = s;
                result += "Helper";
                break;

            case GetTypeNormal:
                result = s;
                break;

            case GetTypeHolder:
                result = s;
                result += "Holder";
                break;

            case GetTypeStub:
                result = string_dup("???");
                break;
            }

            break;
        }

	case tk_alias:
	case tk_null:
        case tk_longdouble:
	    assert(false);
	    break;
	}
    }

    return result._retn();
}

// ----------------------------------------------------------------------
// Determine if TypeCode is recursive
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::isRecursive(const char* id, TypeCode_ptr type,
                              StringSeq& ids)
{
    TypeCode_var origType = GetOrigType(type);
    switch(origType -> kind())
    {
    case tk_null:
    case tk_void:
    case tk_short:
    case tk_long:
    case tk_ushort:
    case tk_ulong:
    case tk_longlong:
    case tk_ulonglong:
    case tk_float:
    case tk_double:
    case tk_fixed:
    case tk_boolean:
    case tk_char:
    case tk_wchar:
    case tk_octet:
    case tk_string:
    case tk_wstring:
    case tk_any:
    case tk_TypeCode:
    case tk_Principal:
    case tk_objref:
    case tk_enum:
    case tk_abstract_interface:
    case tk_native:
    case tk_local_interface:
        break;

    case tk_struct:
    case tk_union:
    case tk_except:
    {
        ULong i;

        const char* repId = origType -> id();
        if(strcmp(repId, id) == 0)
            return true;

        //
        // Avoid infinite loops in certain cases, for example:
        //
        // struct S1
        // {
        //     sequence< S1> seq;
        // };
        //
        // struct S2
        // {
        //     S1 data;
        // };
        //
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(ids[i], repId) == 0)
                return false;

        ids.length(ids.length() + 1);
        ids[ids.length() - 1] = repId;

        for(i = 0 ; i < origType -> member_count() ; i++)
        {
            TypeCode_var memberType = origType -> member_type(i);
            if(isRecursive(id, memberType, ids))
                return true;
        }

        ids.length(ids.length() - 1);

        break;
    }
    
    case tk_sequence:
    case tk_array:
    case tk_value_box:
    {
        TypeCode_var contentType = origType -> content_type();
        return isRecursive(id, contentType, ids);
    }

    case tk_value:
    {
        ULong i;

        //
        // Check concrete base valuetype
        //
        TypeCode_var base = origType -> concrete_base_type();
        if(!is_nil(base) && isRecursive(id, base, ids))
            return true;

        const char* repId = origType -> id();
        if(strcmp(repId, id) == 0)
            return true;

        //
        // Avoid infinite loops in certain cases
        //
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(ids[i], repId) == 0)
                return false;

        ids.length(ids.length() + 1);
        ids[ids.length() - 1] = repId;

        for(i = 0 ; i < origType -> member_count() ; i++)
        {
            TypeCode_var memberType = origType -> member_type(i);
            if(isRecursive(id, memberType, ids))
                return true;
        }

        ids.length(ids.length() - 1);

        break;
    }

    case tk_alias:
    case tk_longdouble:
        assert(false);
        break;
    }

    return false;
}

// ----------------------------------------------------------------------
// Check if code needs to be generated for a specific repository id
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::check(const char* id)
{
    return all_ || idHash_.containsKey(id);
}

// ----------------------------------------------------------------------
// Describe a container
// ----------------------------------------------------------------------

// IMPORTANT NOTE: For efficiency reasons, this operation does not
// follow the regular C++ mapping rules in that the returned sequence
// is not a duplicate. So do not store the return result in a
// DescriptionSeq_var!
Container::DescriptionSeq*
IdlJavaGenerator::describe(Container_ptr container)
{
    RepositoryId_var id;
    Contained_var contained = Contained::_narrow(container);
    if(is_nil(contained))
    {
        id = string_dup("");
    }
    else
    {
        id = contained -> id();
    }

    Container::DescriptionSeq* seq;
    if(!descriptionMap_.get(id, seq))
    {
	seq = container -> describe_contents(dk_all, true, -1);
        assert(seq);
        descriptionMap_.put(id, seq);
    }
    return seq;
}

// ----------------------------------------------------------------------
// Get package from package options
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::getPackageFromOptions(const char* id)
{
    String_var package = string_dup("");

    //
    // First handle "--prefix-package"
    //
    if(strncmp(id, "IDL:", 4) == 0)
    {
        String_var str = id;
        while(package.in() == 0 || strlen(package) == 0)
        {
            char* s = strrchr(str, '/');
            if(s != 0)
            {
                *s = '\0';
                prefixPackageHash_.get(str.in() + 4, package);
            }
            else
                break;
        }
    }
    
    //
    // Now handle "--package"
    //
    if(package.in() == 0 || strlen(package) == 0)
	if(strlen(package_))
	    package = package_;
    
    //
    // Handle automatic package generation
    //
    if(package.in() == 0 || strlen(package) == 0)
	if(autoPackage_)
	    package = getPackageFromPrefix(id);

    return package._retn();
}

// ----------------------------------------------------------------------
// Get absolute name
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::getAbsolute(const char* id)
{
    String_var rc;

    if(!absMap_.get(id, rc))
    {
        Contained_var contained = repository_ -> lookup_id(id);
        assert(!is_nil(contained));
        
        rc = getAbsolute(contained);
        absMap_.put(id, string_dup(rc));
    }

    return rc._retn();
}

char*
IdlJavaGenerator::getAbsolute(Contained_ptr contained)
{
    RepositoryId_var id = contained -> id();
    String_var package = getPackageFromOptions(id);
    return getAbsolutePackage(contained, package);
}

char*
IdlJavaGenerator::getAbsolutePackage(Contained_ptr contained,
				     const char* package)
{
    RepositoryId_var id = contained -> id();
    Identifier_var name;

    //
    // Special treatment for CORBA::completion_status, which must map
    // to CORBA::CompletionStatus
    //
    if(strcmp(id, "IDL:omg.org/CORBA/completion_status:1.0") == 0)
        name = string_dup("CompletionStatus");
    else
    {
        name = contained -> name();
        name = fixName(name, true);
    }

    Container_var container = contained -> defined_in();
    Contained_var contained2 = Contained::_narrow(container);

    if(is_nil(contained2))
    {
 	if(package && strlen(package))
	{
	    char* result = string_alloc(strlen(package) + 1 + strlen(name));
	    strcpy(result, package);
	    strcat(result, ".");
	    strcat(result, name);
	    return result;
	}
	else
            return name._retn();
    }
    else
    {
	String_var s = getAbsolutePackage(contained2, package);

        DefinitionKind dk = contained2 -> def_kind();
	if(dk == dk_Interface || dk == dk_AbstractInterface ||
           dk == dk_LocalInterface || dk == dk_Value || dk == dk_Struct ||
           dk == dk_Exception || dk == dk_Union)
	{
	    DefinitionKind k = contained -> def_kind();
	    if(k == dk_Alias || k == dk_Enum ||
	       k == dk_Struct || k == dk_Exception ||
	       k == dk_Union)
	    {
		s += "Package";
	    }
	}
	
	char* result = string_alloc(strlen(s) + 1 + strlen(name));
	strcpy(result, s);
	strcat(result, ".");
	strcat(result, name);
	return result;
    }
}

// ----------------------------------------------------------------------
// Get package name
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::getPackage(const char* id)
{
    String_var package;

    if(!pkgMap_.get(id, package))
    {
	Contained_var contained = repository_ -> lookup_id(id);
	assert(!is_nil(contained));

	package = getPackage(contained);
	pkgMap_.put(id, package);
    }

    return package._retn();
}

char*
IdlJavaGenerator::getPackage(Contained_ptr contained)
{
    RepositoryId_var id = contained -> id();
    String_var package = getPackageFromOptions(id);
    return getPackagePackage(contained, package);
}

char*
IdlJavaGenerator::getPackagePackage(Contained_ptr contained,
				    const char* package)
{
    Container_var container = contained -> defined_in();
    Contained_var contained2 = Contained::_narrow(container);

    if(is_nil(contained2))
    {
 	if(package && strlen(package))
	    return string_dup(package);
	else
            return string_dup("");
    }
    else
    {
        DefinitionKind dk = contained2 -> def_kind();

	if(dk == dk_Module)
	{
	    return getAbsolutePackage(contained2, package);
	}
	else if(dk == dk_Interface || dk == dk_AbstractInterface ||
                dk == dk_LocalInterface || dk == dk_Value ||
                dk == dk_Struct || dk == dk_Exception || dk == dk_Union)
	{
	    DefinitionKind k = contained -> def_kind();
	    if(k == dk_Alias || k == dk_Enum ||
	       k == dk_Struct || k == dk_Exception ||
	       k == dk_Union)
	    {
		String_var s = getAbsolutePackage(contained2, package);
		s += "Package";
		return s._retn();
	    }
	    else
	    {
		return getPackagePackage(contained2, package);
	    }
	}
	else
	{
	    return getPackagePackage(contained2, package);
	}
    }
}

// ----------------------------------------------------------------------
// Build package name from prefix in repository id
// ----------------------------------------------------------------------

char*
IdlJavaGenerator::getPackageFromPrefix(const char* id)
{
    String_var package = string_dup("");

    if(strchr(id, '/') && strncmp(id, "IDL:", 4) == 0)
    {
	String_var str = string_dup(id + 4);
	*strchr(str.inout(), '/') = '\0';
	char* s = str.inout();
	const char* tok;

	while((tok = strtok(s, ".")))
	{
	    s = 0;
	    if(strlen(package) > 0)
	    {
		String_var newPackage =
		    string_alloc(strlen(tok) + 1 + strlen(package));
		strcpy(newPackage.inout(), tok);
		strcat(newPackage.inout(), ".");
		strcat(newPackage.inout(), package);
		package = newPackage;
	    }
	    else
		package = string_dup(tok);
	}
    }

    return package._retn();
}

// ----------------------------------------------------------------------
// Convert strings with escape sequences
// ----------------------------------------------------------------------

void
IdlJavaGenerator::addEscapes(char*& str, bool addQuotes)
{
    char* s = str;
    String_var result = string_dup("");

    if(addQuotes)
	result += '"';

    while(*s)
    {
	switch(*s)
	{
	case '\n':
	    result += "\\n";
	    break;

	case '\t':
	    result += "\\t";
	    break;

	case '\b':
	    result += "\\b";
	    break;

	case '\r':
	    result += "\\r";
	    break;

	case '\f':
	    result += "\\f";
	    break;

	case '\\':
	    result += "\\\\";
	    break;

	case '\'':
	    result += "\\'";
	    break;

	case '"':
	    result += "\\\"";
	    break;

	default:
	    if(*s >= 32 && *s < 127)
		result += *s;
	    else
	    {
		Char c = *s;
		char str[5];
		sprintf(str, "\\%.3o", (unsigned char)c);
		result += str;
	    }
	    break;
	}
	
	s++;
    }

    if(addQuotes)
	result += '"';

    string_free(str);
    str = result._retn();
}

char*
IdlJavaGenerator::addEscapes(const wchar_t* str, bool addQuotes)
{
    const wchar_t* s = str;
    String_var result = string_dup("");

    if(addQuotes)
	result += '"';

    while(*s)
    {
	switch(*s)
	{
	case L'\n':
	    result += "\\n";
	    break;

	case L'\t':
	    result += "\\t";
	    break;

	case L'\b':
	    result += "\\b";
	    break;

	case L'\r':
	    result += "\\r";
	    break;

	case L'\f':
	    result += "\\f";
	    break;

	case L'\\':
	    result += "\\\\";
	    break;

	case L'\'':
	    result += "\\'";
	    break;

	case L'"':
	    result += "\\\"";
	    break;

	default:
            if(*s < 256)
            {
                if(*s >= 32 && *s < 127)
                    result += (Char)*s;
                else
                {
                    Char c = (Char)*s;
                    char str[5];
                    sprintf(str, "\\%.3o", (unsigned char)c);
                    result += str;
                }
            }
            else
            {
                char str[10];
                sprintf(str, "\\u%lx", (ULong)*s);
                result += str;
            }
	    break;
	}
	
	s++;
    }

    if(addQuotes)
	result += '"';

    return result._retn();
}

// ----------------------------------------------------------------------
// Remove package from absolute name
// ----------------------------------------------------------------------

void
IdlJavaGenerator::removePackage(const char* package, char*& absolute)
{
    assert_nca(package != 0, NCANullString);
    assert_nca(absolute != 0, NCANullString);

/*
    if(strlen(absolute) > strlen(package) && strlen(package) > 0)
    {
	assert(package[strlen(package) - 1] != '.');

	if(strncmp(package, absolute, strlen(package)) == 0 &&
	   absolute[strlen(package)] == '.')
	{
	    char* a = string_dup(absolute + strlen(package) + 1);
	    string_free(absolute);
	    absolute = a;
	}
    }
*/
    if(strlen(absolute) > strlen(package) && strlen(package) > 0)
    {
	assert(package[strlen(package) - 1] != '.');

	const char* s = strrchr(absolute, '.');
	if(s)
	{
	    unsigned int len = s - absolute;
	    if(strlen(package) == len && strncmp(package, absolute, len) == 0)
	    {
		char* a = string_dup(s + 1);
		string_free(absolute);
		absolute = a;
	    }
	}
    }
}

// ----------------------------------------------------------------------
// Get union label sequence
// ----------------------------------------------------------------------

StringSeq*
IdlJavaGenerator::getUnionLabelSeq(const char* package,
				   TypeCode_ptr discType,
				   const IdlUnionMemberInfo& info)
{
    TypeCode_var origDiscType = GetOrigType(discType);
    StringSeq_var seq = new StringSeq;
    ULong i;

    switch(origDiscType -> kind())
    {
    case tk_short:
    case tk_ushort:

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    String_var label = string_dup("(short)");
	    label += info.pLabels[i];
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	for(i = 0 ; i < info.nLabels.length() ; i++)
	{
	    String_var label = string_dup("(short)");
	    label += '-';
	    label += info.nLabels[i];
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	break;
	
    case tk_long:
    case tk_ulong:

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    String_var label = string_dup("");
	    label += info.pLabels[i];
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	for(i = 0 ; i < info.nLabels.length() ; i++)
	{
	    String_var label = string_dup("-");
	    label += info.nLabels[i];
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	break;

    case tk_longlong:
    case tk_ulonglong:

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    String_var label = string_dup("");
	    label += info.pLabels[i];
            label += "L";
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	for(i = 0 ; i < info.nLabels.length() ; i++)
	{
	    String_var label = string_dup("-");
	    label += info.nLabels[i];
            label += "L";
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	break;
	
    case tk_char:

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    String_var s;
	    if((char)info.pLabels[i] == '\0')
		s = string_dup("\\0");
	    else
	    {
		s = string_alloc(1);
		s[0] = (char)info.pLabels[i];
		s[1] = '\0';
		addEscapes(s.inout(), false);
	    }

	    String_var label = string_dup("'");
	    label += s;
	    label += '\'';
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	assert(info.isDefault || info.nLabels.length() == 0);
	
	break;
	
    case tk_boolean:

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    String_var label = string_dup("");
	    label += info.pLabels[i];
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	assert(info.nLabels.length() == 0);
	
	break;
	
    case tk_enum:
    {
	String_var disc = getTypeString(package, origDiscType);

	for(i = 0 ; i < info.pLabels.length() ; i++)
	{
	    Identifier_var memberName =
		origDiscType -> member_name((ULong)info.pLabels[i]);
	    memberName = fixName(memberName, false);

	    String_var label = disc;
	    label += "._";
	    label += memberName;
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = label;
	}

	assert(info.nLabels.length() == 0);

	break;
    }

    default:
	assert(false);
	break;
    }

    return seq._retn();
}

// ----------------------------------------------------------------------
// Get holder class for a union member
// ----------------------------------------------------------------------

char *
IdlJavaGenerator::getUnionMemberHolder(TypeCode_ptr tc)
{
    String_var result;

    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
	assert(false);
	break;

    case tk_short:
    case tk_ushort:
	result = string_dup("org.omg.CORBA.ShortHolder");
	break;

    case tk_long:
    case tk_ulong:
	result = string_dup("org.omg.CORBA.IntHolder");
	break;

    case tk_longlong:
    case tk_ulonglong:
	result = string_dup("org.omg.CORBA.LongHolder");
	break;

    case tk_float:
	result = string_dup("org.omg.CORBA.FloatHolder");
	break;

    case tk_double:
	result = string_dup("org.omg.CORBA.DoubleHolder");
	break;

    case tk_fixed:
	result = string_dup("org.omg.CORBA.FixedHolder");
	break;

    case tk_boolean:
	result = string_dup("org.omg.CORBA.BooleanHolder");
	break;

    case tk_char:
    case tk_wchar:
	result = string_dup("org.omg.CORBA.CharHolder");
	break;

    case tk_octet:
	result = string_dup("org.omg.CORBA.ByteHolder");
	break;

    case tk_alias:
	assert(false);
	break;

    case tk_longdouble:
        assert(false);
        break;

    default:
	result = string_dup("");
	break;
    }

    return result._retn();
}

// ----------------------------------------------------------------------
// Determines if the typecode is a primitive (or immutable) type
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::isPrimitiveMemberType(TypeCode_ptr tc)
{
    bool result;

    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
	assert(false);
	break;

    case tk_short:
    case tk_ushort:
    case tk_long:
    case tk_ulong:
    case tk_longlong:
    case tk_ulonglong:
    case tk_float:
    case tk_double:
    case tk_longdouble:
    case tk_fixed:
    case tk_boolean:
    case tk_char:
    case tk_wchar:
    case tk_octet:
    case tk_string:
    case tk_wstring:
    case tk_TypeCode:
    case tk_Principal:
    case tk_objref:
    case tk_local_interface:
	result = true;
	break;

    default:
	result = false;
	break;
    }

    return result;
}

// ----------------------------------------------------------------------
// Determines if the TypeCode is a basic valuebox type
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::isBasicBoxType(TypeCode_ptr tc)
{
    bool result;

    TypeCode_var origType = GetOrigType(tc);
    assert(origType -> kind() == tk_value_box);

    TypeCode_var contentType = origType -> content_type();
    TypeCode_var origContentType = GetOrigType(contentType);

    switch(origContentType -> kind())
    {
        case tk_short:
        case tk_long:
        case tk_ushort:
        case tk_ulong:
        case tk_longlong:
        case tk_ulonglong:
        case tk_float:
        case tk_double:
        case tk_boolean:
        case tk_char:
        case tk_wchar:
        case tk_octet:
            result = true;
            break;

        default:
            result = false;
            break;
    }

    return result;
}

// ----------------------------------------------------------------------
// Determines if the typecode is marshallable
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::isMarshallableType(TypeCode_ptr tc)
{
    StringSeq ids;
    return isMarshallableType(tc, ids);
}

bool
IdlJavaGenerator::isMarshallableType(TypeCode_ptr tc, StringSeq& ids)
{
    bool result;

    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
    case tk_short:
    case tk_long:
    case tk_ushort:
    case tk_ulong:
    case tk_longlong:
    case tk_ulonglong:
    case tk_float:
    case tk_double:
    case tk_fixed:
    case tk_boolean:
    case tk_char:
    case tk_wchar:
    case tk_octet:
    case tk_string:
    case tk_wstring:
    case tk_any:
    case tk_TypeCode:
    case tk_Principal:
    case tk_objref:
    case tk_enum:
    case tk_abstract_interface:
        result = true;
        break;

    case tk_struct:
    case tk_union:
    case tk_except:
    {
        ULong i;

        const char* id = tc -> id();
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(id, ids[i]) == 0)
                return true;

        ids.length(ids.length() + 1);
        ids[ids.length() - 1] = id;

        result = true;
        for(i = 0 ; i < tc -> member_count() && result ; i++)
        {
            TypeCode_var memberType = tc -> member_type(i);
            result = isMarshallableType(memberType, ids);
        }

        ids.length(ids.length() - 1);

        break;
    }

    case tk_value:
    {
        ULong i;

        const char* id = tc -> id();
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(id, ids[i]) == 0)
                return true;

        ids.length(ids.length() + 1);
        ids[ids.length() - 1] = id;

        result = true;
        for(i = 0 ; i < tc -> member_count() && result ; i++)
        {
            TypeCode_var memberType = tc -> member_type(i);
            result = isMarshallableType(memberType, ids);
        }

        ids.length(ids.length() - 1);

        break;
    }

    case tk_sequence:
    case tk_array:
    case tk_value_box:
    case tk_alias:
    {
        TypeCode_var contentType = tc -> content_type();
        result = isMarshallableType(contentType, ids);
        break;
    }

    case tk_native:
    case tk_longdouble:
    case tk_local_interface:
        result = false;
        break;
    }

    return result;
}

// ----------------------------------------------------------------------
// Get the name of the class to import for a repository id (if necessary)
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::getImportId(const char* package, const char* id, char*& name)
{
    //
    // Only consider importing a name if the importer's package is
    // not empty and if the name doesn't contain a period. In other
    // words, there's no need to import a top-level class if the
    // importer is also a top-level class.
    //
    if(strlen(package) > 0)
    {
        //
        // Check for well-known IDs
        //
        if(strcmp(id, "IDL:omg.org/CORBA/Object:1.0") == 0 ||
           strcmp(id, "IDL:omg.org/CORBA/ValueBase:1.0") == 0)
            return false;

        String_var absolute = getAbsolute(id);
        if(strchr(absolute, '.') == 0)
        {
            name = absolute._retn();
            return true;
        }
    }
    return false;
}

// ----------------------------------------------------------------------
// Get the name of the class to import for a type (if necessary)
// ----------------------------------------------------------------------

bool
IdlJavaGenerator::getImportType(const char* package, TypeCode_ptr type,
                                char*& name, GetImport mode)
{
    bool result = false;

    if(mode == GetImportHelper && type -> kind() == tk_alias)
    {
        result = getImportId(package, type -> id(), name);
    }
    else
    {
        TypeCode_ptr tc = GetOrigType_nodup(type);

        switch(tc -> kind())
        {
        case tk_null:
        case tk_void:
        case tk_short:
        case tk_long:
        case tk_ushort:
        case tk_ulong:
        case tk_longlong:
        case tk_ulonglong:
        case tk_float:
        case tk_double:
        case tk_fixed:
        case tk_boolean:
        case tk_char:
        case tk_wchar:
        case tk_octet:
        case tk_string:
        case tk_wstring:
        case tk_any:
        case tk_TypeCode:
        case tk_Principal:
        case tk_longdouble:
            break;

        case tk_sequence:
        case tk_array:
        {
            if(mode == GetImportHolder)
            {
                //
                // If a holder for a sequence or array is necessary,
                // then the type *must* be a typedef for a sequence
                // or array. In this case, we need to unwind the
                // type to the last alias (the unaliased alias :).
                //
                assert(type -> kind() == tk_alias);
                TypeCode_ptr p = type;
                TypeCode_ptr alias = type;
                while(p -> kind() == tk_alias)
                {
                    alias = p;
                    p = p -> _OB_contentType_nodup();
                }
                result = getImportId(package, alias -> id(), name);
            }
            else
            {
                //
                // Check the content type
                //
                TypeCode_ptr contentType = tc -> _OB_contentType_nodup();
                result = getImportType(package, contentType, name);
            }
            break;
        }

        case tk_struct:
        case tk_union:
        case tk_except:
        case tk_enum:
        case tk_objref:
        case tk_local_interface:
        case tk_abstract_interface:
        case tk_value:
        case tk_native:
            result = getImportId(package, tc -> id(), name);
            break;

        case tk_value_box:
        {
            //
            // Determine the boxed type
            //
            TypeCode_ptr contentType = tc -> _OB_contentType_nodup();
            TypeCode_ptr origContentType = GetOrigType_nodup(contentType);

            switch(origContentType -> kind())
            {
                case tk_null:
                case tk_void:
                case tk_Principal:
                case tk_except:
                case tk_value:
                case tk_value_box:
                case tk_abstract_interface:
                case tk_longdouble:
                case tk_native:
                    //
                    // These types cannot be boxed
                    //
                    assert(false);
                    break;

                case tk_short:
                case tk_long:
                case tk_ushort:
                case tk_ulong:
                case tk_longlong:
                case tk_ulonglong:
                case tk_float:
                case tk_double:
                case tk_boolean:
                case tk_char:
                case tk_wchar:
                case tk_octet:
                    //
                    // A new class is generated for these boxed types
                    //
                    result = getImportId(package, tc -> id(), name);
                    break;

                case tk_fixed:
                case tk_string:
                case tk_wstring:
                case tk_any:
                case tk_TypeCode:
                    //
                    // Only a helper class is generated for these boxed
                    // types. For GetImportNormal or GetImportHolder, the
                    // class names do not need to be imported because they
                    // are already fully scoped (for example,
                    // org.omg.CORBA.StringHolder).
                    //
                    if(mode == GetImportHelper)
                        result = getImportId(package, tc -> id(), name);
                    break;

                case tk_objref:
                case tk_local_interface:
                case tk_struct:
                case tk_union:
                case tk_enum:
                    //
                    // Only a helper class is generated for these boxed
                    // types. For GetImportNormal or GetImportHolder,
                    // check the repository id of the boxed type.
                    //
                    if(mode == GetImportHelper)
                        result = getImportId(package, tc -> id(), name);
                    else
                        result = getImportId(package, origContentType -> id(),
                                             name);
                    break;

                case tk_sequence:
                case tk_array:
                    //
                    // A helper and a holder are generated for these
                    // boxed types
                    //
                    if(mode == GetImportNormal)
                    {
                        TypeCode_ptr cont =
                            origContentType -> _OB_contentType_nodup();
                        result = getImportType(package, cont, name,
                                               GetImportNormal);
                    }
                    else
                        result = getImportId(package, tc -> id(), name);
                    break;

                case tk_alias:
                    assert(false);
                    break;
            }

            break;
        }

        case tk_alias:
            assert(false);
            break;
        }
    }

    //
    // Append a suffix if necessary
    //
    if(result)
    {
        switch(mode)
        {
        case GetImportNormal:
            break;

        case GetImportHelper:
        {
            String_var tmp = name;
            tmp += "Helper";
            name = tmp._retn();
            break;
        }

        case GetImportHolder:
        {
            String_var tmp = name;
            tmp += "Holder";
            name = tmp._retn();
            break;
        }
        }
    }

    return result;
}

// ----------------------------------------------------------------------
// Collect classes which need to be imported
// ----------------------------------------------------------------------

void
IdlJavaGenerator::collectImports(const char* package, Container_ptr container,
                                 StringSeq& imports, ImportMode mode)
{
    //
    // Get container contents descriptions
    //
    Container::DescriptionSeq_var seqVar;
    Container::DescriptionSeq* seq;
    if(mode == ImportModeStubSkel || mode == ImportModeTie ||
       mode == ImportModeImpl)
    {
        //
        // Get all operations and attributes
        //
        Container::DescriptionSeq_var ops =
            container -> describe_contents(dk_Operation, false, -1);
        Container::DescriptionSeq_var attribs =
            container -> describe_contents(dk_Attribute, false, -1);
        seqVar = new Container::DescriptionSeq;
        seqVar -> length(ops -> length() + attribs -> length());
        ULong i;
        for(i = 0 ; i < ops -> length() ; i++)
            seqVar[i] = ops[i];
        for(i = 0 ; i < attribs -> length() ; i++)
            seqVar[i + ops -> length()] = attribs[i];
        seq = &seqVar.inout();
    }
    else
    {
        //
        // No _var for seq! See comments for the implementation of
        // "describe"
        //
        seq = describe(container);
    }

    ULong i, j;

    for(i = 0 ; i < seq -> length() ; i++)
    {
        //
        // Get element description
        //
        Container::Description& seqElem = (*seq)[i];

        if(mode == ImportModeOperations)
        {
            if(seqElem.kind != dk_Operation &&
               seqElem.kind != dk_Attribute &&
               seqElem.kind != dk_Constant)
                continue;
        }
        else if(mode == ImportModeSignatures)
        {
            if(seqElem.kind == dk_Operation ||
               seqElem.kind == dk_Attribute)
                continue;
        }

        switch(seqElem.kind)
        {
        case dk_Constant:
        {
            //
            // Constant
            //
            const ConstantDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            String_var name;
            if(getImportType(package, desc -> type, name.out()))
                addImport(imports, name);

            break;
        }

        case dk_ValueMember:
        {
            //
            // Valuetype data member
            //
            const ValueMember* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            String_var name;
            if(getImportType(package, desc -> type, name.out()))
            {
                addImport(imports, name);
                if(mode == ImportModeStreamable)
                {
                    if(getImportType(package, desc -> type, name.out(),
                                     GetImportHelper))
                    {
                        addImport(imports, name);
                    }
                }
            }

            break;
        }

        case dk_Attribute:
        {
            //
            // Attribute
            //
            const AttributeDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            String_var name;
            if(getImportType(package, desc -> type, name.out()))
            {
                addImport(imports, name);
                if(mode == ImportModeStubSkel)
                {
                    if(getImportType(package, desc -> type, name.out(),
                                     GetImportHelper))
                    {
                        addImport(imports, name);
                    }
                }
            }

            break;
        }

        case dk_Operation:
        {
            //
            // Operation
            //
            const OperationDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            String_var name;

            if(getImportType(package, desc -> result, name.out()))
            {
                addImport(imports, name);
                if(mode == ImportModeStubSkel)
                {
                    if(getImportType(package, desc -> result, name.out(),
                                     GetImportHelper))
                    {
                        addImport(imports, name);
                    }
                }
            }

            for(j = 0 ; j < desc -> parameters.length() ; j++)
                if(getImportType(package, desc -> parameters[j].type,
                                 name.out()))
                {
                    addImport(imports, name);
                    if(desc -> parameters[j].mode != PARAM_IN)
                    {
                        String_var holder;
                        if(getImportType(package, desc -> parameters[j].type,
                                         holder.out(), GetImportHolder))
                        {
                            addImport(imports, holder);
                        }
                    }
                    if(mode == ImportModeStubSkel)
                    {
                        String_var helper;
                        if(getImportType(package, desc -> parameters[j].type,
                                         helper.out(), GetImportHelper))
                        {
                            addImport(imports, helper);
                        }
                    }
                }

            for(j = 0 ; j < desc -> exceptions.length() ; j++)
                if(getImportId(package, desc -> exceptions[j].id, name.out()))
                {
                    addImport(imports, name);
                    if(mode == ImportModeStubSkel)
                    {
                        name += "Helper";
                        addImport(imports, name);
                    }
                }

            break;
        }

        default:
            break;
        }
    }
}

// ----------------------------------------------------------------------
// Add a class to the import list, ignoring duplicates
// ----------------------------------------------------------------------

void
IdlJavaGenerator::addImport(StringSeq& imports, const char* name)
{
    for(ULong i = 0 ; i < imports.length() ; i++)
        if(strcmp(imports[i], name) == 0)
            return;

    imports.length(imports.length() + 1);
    imports[imports.length() - 1] = name;
}
