// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef GEN_RTF_H
#define GEN_RTF_H

#include <GenRTFUtil.h>

//
// The RTF code generator class
//
class IdlRTFGenerator : public GenTextBase
{
    //
    // The top-level file name (without ".rtf")
    //
    static const char* top_;

    //
    // The program name
    //
    const char* prog_;

    //
    // If true, create index
    //
    bool withIndex_;

    //
    // If set, write .rtf files to this path
    //
    const char* outDir_;

    //
    // If set, write .rtf code to this file
    //
    const char* singleFile_;

    //
    // RTF font settings
    //
    RTFFontDescriptor** fonts_;

    //
    // Indices for nested tables and lists
    //
    unsigned int tblndx_;
    unsigned int lstndx_;

    //
    // Pointers to current formatting objects
    //
    RTFParagraph* style_;
    RTFDisplayList* lists_[MAXLISTS];
    RTFTable* table_[MAXTABLE];

    //
    // Output the start or end of a code block
    //
    virtual void startBlock(IdlPrettyPrint&);
    virtual void endBlock(IdlPrettyPrint&);

    //
    // Output the start or end of extra clauses
    //
    virtual void startExtra(IdlPrettyPrint&);
    virtual void endExtra(IdlPrettyPrint&);

    //
    // Output the start or end of an index
    //
    virtual void startIndex(IdlPrettyPrint&);
    virtual void endIndex(IdlPrettyPrint&);

    //
    // Output the start or end of an object header
    //
    virtual void startObject(const char*, IdlPrettyPrint&, bool);
    virtual void endObject(const char*, IdlPrettyPrint&,
                           CORBA::Container_ptr = 0, const char* = 0);

    //
    // Output various text formatting properties
    //
    virtual void textBold(bool, IdlPrettyPrint&);
    virtual void textItalic(bool, IdlPrettyPrint&);
    virtual void textLiteral(bool, IdlPrettyPrint&);
    virtual void textUnderline(bool, IdlPrettyPrint&);
    virtual void textLarge(bool, IdlPrettyPrint&);
    virtual void textSmall(bool, IdlPrettyPrint&);

    //
    // Output the master definition (level 1 heading)
    //
    virtual void putLevel1(const char*, const char*, IdlPrettyPrint&);

    //
    // Output a group association (level 2 heading)
    //
    virtual void putLevel2(const char*, const char*, IdlPrettyPrint&);

    //
    // Output an index entry
    //
    virtual void putEntry(const char*, const char*, IdlPrettyPrint&, bool,
                          const char*);

    //
    // Output a single @ clause from a comment.
    //
    virtual void putClause(const char*, const char*, const char*, const char*,
                           IdlPrettyPrint&, bool, bool);

    //
    // Output the comment for an object.  Do this as an arbitrary
    // paragraph, the introductory sentence, or the whole thing.
    //
    virtual void putParagraph(const char*, IdlPrettyPrint&);
    virtual void putComment(const char*, IdlPrettyPrint&, CORBA::Container_ptr,
                            const char* = 0);

    //
    // Generate body for a particular type
    //
    virtual void putInitializers(CORBA::Container_ptr,
                                 const CORBA::InitializerSeq_var&,
                                 IdlPrettyPrint&, const char*, const char*,
                                 unsigned int);

    virtual int formatModule(CORBA::Container_ptr, const char*,
                             const CORBA::Container::DescriptionSeq&,
                             IdlPrettyPrint&);
    virtual int formatInterface(CORBA::Container_ptr, const char*,
                                const CORBA::Container::DescriptionSeq&,
                                IdlPrettyPrint&);
    virtual int formatValuetype(CORBA::Container_ptr, const char*,
                                const CORBA::Container::DescriptionSeq&,
                                IdlPrettyPrint&);
    virtual int formatOperation(CORBA::Container_ptr, const char*,
                                const CORBA::Container::DescriptionSeq&,
                                IdlPrettyPrint&);

    //
    // Paragraph style management
    //
    RTFParagraph* getStyle() const { return style_; }
    virtual void setStyle(RTFParagraph*, IdlPrettyPrint&, bool = true);

    //
    // Indented display list management
    //
    RTFDisplayList* getList() const
    { return lstndx_ ? lists_[lstndx_-1] : NULL; }
    virtual void pushList(RTFDisplayList*, IdlPrettyPrint&, bool = true);
    virtual void popList(IdlPrettyPrint&);

    //
    // Formatted table management
    //
    RTFTable* getTable() const { return tblndx_ ? table_[tblndx_-1] : NULL; }
    virtual void pushTable(RTFTable*, IdlPrettyPrint&);
    virtual void popTable(IdlPrettyPrint&);

    RTFTable* prescanTable(char*, bool);
    unsigned int prescanRow(char*, RTFTable* = 0, char** = 0);

public:

    IdlRTFGenerator(const char*, CORBA::Repository_ptr, bool,
                    const IdlStringHash&, const char*, const char*,
                    bool, bool, bool, bool, bool, RTFFontDescriptor**);

    //
    // Generate RTF code
    //
    int gen();
};

#endif
