// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Hashtable.h>

#include <IFR1_impl.h>
#include <IFR3_impl.h>

using namespace CORBA;
using namespace OB;
using namespace PortableServer;

// ----------------------------------------------------------------------
// IRObject_impl member implementations
// ----------------------------------------------------------------------

IRObject_impl::IRObject_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* rep)
    : destroyed_(false), orb_(ORB::_duplicate(orb)),
      poa_(PortableServer::POA::_duplicate(poa)),
      repository_(rep)
{
#ifndef NDEBUG
    if(repository_)
	repository_ -> incIRObjectCount();
#endif
}

IRObject_impl::~IRObject_impl()
{
    assert(destroyed_);

#ifndef NDEBUG
    if(repository_)
	repository_ -> decIRObjectCount();
#endif
}

void
IRObject_impl::destroy()
    throw(SystemException)
{
    assert(!destroyed_);
    destroyed_ = true;

    try
    {
        POA_var poa = _default_POA();
        ObjectId_var oid = poa -> servant_to_id(this);
        poa -> deactivate_object(oid);
    }
    catch(const POA::ServantNotActive&)
    {
        // ignore
    }
    catch(const POA::ObjectNotActive&)
    {
        // ignore
    }
}

PortableServer::POA_ptr
IRObject_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

// ----------------------------------------------------------------------
// IDLType_impl member implementations
// ----------------------------------------------------------------------

IDLType_impl::IDLType_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* r)
    : IRObject_impl(orb, poa, r)
{
}

IDLType_impl::~IDLType_impl()
{
}
