// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <URLTest_impl.h>

#include <stdlib.h>
#include <errno.h>
#include <ctype.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_STDLIB_H
#   include <stdlib.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

//
// IDL:URLTest:1.0
//

//
// IDL:URLTest/IIOPAddress:1.0
//

URLTest::IIOPAddress_impl::IIOPAddress_impl(CORBA::ORB_ptr orb,
                                            const char* host,
                                            CORBA::Long port,
                                            const char* key,
                                            const char* str) :
    orb_(CORBA::ORB::_duplicate(orb)), host_(host), 
    port_(port), key_(key), str_(str)
{
}

URLTest::IIOPAddress_impl::~IIOPAddress_impl()
{
}

//
// IDL:URLTest/IIOPAddress/getKey:1.0
//
char*
URLTest::IIOPAddress_impl::getKey()
    throw(CORBA::SystemException)
{
    char* result = CORBA::string_dup(key_);
    return result;
}

//
// IDL:URLTest/IIOPAddress/getPort:1.0
//
CORBA::Long
URLTest::IIOPAddress_impl::getPort()
    throw(CORBA::SystemException)
{
    CORBA::Long result = port_;
    return result;
}

//
// IDL:URLTest/IIOPAddress/getHost:1.0
//
char*
URLTest::IIOPAddress_impl::getHost()
    throw(CORBA::SystemException)
{
    char* result = CORBA::string_dup(host_.in());
    return result;
}

//
// IDL:URLTest/IIOPAddress/getIIOPAddress:1.0
//
char*
URLTest::IIOPAddress_impl::getIIOPAddress()
    throw(CORBA::SystemException)
{
    CORBA::String_var result = CORBA::string_dup("iiop:");
    result += host_;
    result += ":";
    result += port_;

    return result._retn();
}

//
// IDL:URLTest/IIOPAddress/getCorbalocURL:1.0
//
char*
URLTest::IIOPAddress_impl::getCorbalocURL()
    throw(CORBA::SystemException)
{
    CORBA::String_var addr = getIIOPAddress();
    CORBA::String_var result = CORBA::string_dup("corbaloc:");
    result += addr;
    result += "/";
    result += key_;

    return result._retn();
}

//
// IDL:URLTest/IIOPAddress/destroy:1.0
//
void
URLTest::IIOPAddress_impl::destroy()
    throw(CORBA::SystemException)
{
    // 
    // TODO: implement tests involving destroy for fault tolerance
    // 
    CORBA::Object_var poaCurObj = 
        orb_ -> resolve_initial_references("POACurrent");
    PortableServer::Current_var poaCur = 
        PortableServer::Current::_narrow(poaCurObj);
    PortableServer::POA_var thePOA = poaCur -> get_POA();

    PortableServer::ObjectId corbaKey;
    
    thePOA -> deactivate_object(strToKey(key_.inout(), 0, corbaKey));

    _remove_ref();
}

//
// IDL:URLTest/IIOPAddress/setString:1.0
//
void
URLTest::IIOPAddress_impl::setString(const char* textStr)
    throw(CORBA::SystemException)
{
    str_ = textStr;
}

//
// IDL:URLTest/IIOPAddress/getString:1.0
//
char*
URLTest::IIOPAddress_impl::getString()
    throw(CORBA::SystemException)
{
    char* result = CORBA::string_dup(str_);
    return result;
}

//
// IDL:URLTest/IIOPAddress/deactivate:1.0
//
void
URLTest::IIOPAddress_impl::deactivate()
    throw(CORBA::SystemException)
{
    orb_ -> shutdown(false);
}

PortableServer::ObjectId&
URLTest::strToKey(const char* start, const char* end, 
		  PortableServer::ObjectId& key)
{
    if(start == 0)
        end = 0;
    else if(end == 0)
        end = start + strlen(start);

    CORBA::ULong len = end - start;
    key.length(len);

    if(start)
        memcpy((char* )key.get_buffer(), start, len);
    
    return key;
}


char
URLTest::EncodeHex(int i)
{
    assert(i >= 0 && i < 16);
    if(i >= 0 && i <= 9)
	return i + '0';
    return (i - 10) + 'A';
}

char*
URLTest::keyToStr(const OCI::ObjectKey& key, bool urlEscapeNulls)
{
    CORBA::ULong klen = key.length();
    CORBA::ULong len = klen + 1;

    char* strBuf = CORBA::string_alloc(len * 3);
    CORBA::String_var keyStr(strBuf);
    
    CORBA::ULong wr;
    CORBA::ULong rd;
    
    for(wr = 0, rd = 0; rd < klen; ++rd, ++wr)
    {
        char s = key[rd];
        if(s== '\0' &&
            urlEscapeNulls == false)
        {
            throw CORBA::BAD_PARAM("URL: embedded null character", 
                                   OB::MinorBadSchemeSpecificPart, 
                                   CORBA::COMPLETED_NO);
        }
        else if(isalpha(s) || isdigit(s))
            strBuf[wr] = s;
        else
        {
            strBuf[wr] = '%';
            strBuf[++wr] = EncodeHex((s >> 4) & 0xF);
            strBuf[++wr] = EncodeHex(s & 0xF);
        }
    }
    
    strBuf[wr]='\0';
    return keyStr._retn();
}

URLTest::TempIORFile::TempIORFile(CORBA::ORB_ptr orb, CORBA::Object_ptr obj,
                                  const char*iorFile, int ltor)
{
    fName_ = iorFile;
    OB_OFSTREAM out(iorFile);
    
    if(out.fail())
    {
        OB_ERROR(" can't open `" << iorFile << "': " << strerror(errno));
        throw "TemporaryRefFile: bad file";
    }
    
    CORBA::String_var ior = orb -> object_to_string(obj);
    out << ior;

    switch(ltor)
    {
    case NoEOL:
    default:
        break;
    case UnixEOL: 
        out << '\n';
        break;
    case Win32EOL:
        out << "\r\n" << OB_ENDL;
        break;
    case UnixEOLExtraText:
        out << '\n' <<  "/* This should be ignored */" << OB_ENDL;
        break;
    case Win32EOLExtraText:
        out << "\r\n" << "/*This should be ignored */" << OB_ENDL;
    }
    out.close();
}

//
// unlink the ior file 
//
URLTest::TempIORFile::~TempIORFile()
{
#ifdef HAVE_UNLINK
    _unlink(fName_.in());
#else
    unlink(fName_.in());
#endif
}
