// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Codec.h>

#include <Test.h>
#include <Test_impl.h>
#include <TestMacro.h>

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get "test" object
    //
    CORBA::Object_var obj;

    try
    {
        obj  = orb -> string_to_object("relfile:/Test.ref");
    }
    catch(const CORBA::BAD_PARAM&)
    {
    }

    if(CORBA::is_nil(obj))
    {
	OB_ERROR(argv[0] << ": cannot read IOR from Test.ref");
	return EXIT_FAILURE;
    }
    
    Test_var test = Test::_narrow(obj);
    TEST(!CORBA::is_nil(test));

    Test_var localTest = new LocalTest_impl();

    IOP::CodecFactory_var factory = IOP::CodecFactory::_narrow(
        CORBA::Object_var(orb -> resolve_initial_references("CodecFactory")));
    TEST(!CORBA::is_nil(factory));

    IOP::Encoding how;
    how.major_version = 0;
    how.minor_version = 0;
    how.format = IOP::ENCODING_CDR_ENCAPS;

    IOP::Codec_var codec = factory -> create_codec(how);
    TEST(!CORBA::is_nil(codec));

    OB_INFO("Testing Codec... ");
    try
    {
	CORBA::Any a;
	a <<= test;
	CORBA::OctetSeq_var data = codec -> encode_value(a);
    }
    catch(const CORBA::SystemException&)
    {
	TEST(false);
    }
    try
    {
	CORBA::Any a;
	a <<= localTest;
	CORBA::OctetSeq_var data = codec -> encode_value(a);
	TEST(false);
    }
    catch(const CORBA::MARSHAL&)
    {
    }
    OB_INFO_LN("Done!");
    
    OB_INFO("Testing simple RPC call... ");
    test -> say("Hi");
    OB_INFO_LN("Done!");

    OB_INFO("Testing passing non-local object... ");
    try
    {
	test -> intest(test);
    }
    catch(const CORBA::SystemException&)
    {
	TEST(false);
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing passing local object... ");
    try
    {
	test -> intest(localTest);
	TEST(false);
    }
    catch(const CORBA::MARSHAL&)
    {
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing passing non-local object in any... ");
    try
    {
	CORBA::Any a;
	a <<= test;
	test -> inany(a);
    }
    catch(const CORBA::SystemException&)
    {
	TEST(false);
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing insertion of local object in any... ");
    try
    {
	CORBA::Any a;
	a <<= localTest;
	Test_ptr t;
	TEST(a >>= t);
	TEST(t == localTest.in());
    }
    catch(const CORBA::SystemException&)
    {
	TEST(false);
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing passing local object in any... ");
    try
    {
	CORBA::Any a;
	a <<= localTest;
	test -> inany(a);
	TEST(false);
    }
    catch(const CORBA::MARSHAL&)
    {
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing returning local object... ");
    try
    {
	Test_var t = test -> returntest();
	TEST(false);
    }
    catch(const CORBA::MARSHAL&)
    {
    }
    OB_INFO_LN("Done!");

    OB_INFO("Testing returning local object in any... ");
    try
    {
	CORBA::Any_var a;
	test -> outany(a);
	TEST(false);
    }
    catch(const CORBA::MARSHAL&)
    {
    }
    OB_INFO_LN("Done!");

    test -> shutdown();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
