// **********************************************************************
//
// Copyright (c) 2001
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/OCI_IIOP_init.h>

#include <stdlib.h>

#ifdef HAVE_X11_INTRINSIC_H
#   include <OB/X11.h>
#endif

#ifdef WIN32
#   include <OB/OBWindows.h>
#endif

using namespace CORBA;

int
ReactorInit(
    int argc,
    char* argv[],
    OB::Reactor_out reactor
)
{
    OB::Properties_var properties = OB::Properties::getDefaultProperties();
    OB::ParseArgs(argc, argv, properties);
    OCI_parse_args_iiop(argc, argv, properties);

    for (int i = 1; i < argc; i++)
    {
#if defined(WIN32)
	if (strcmp(argv[i], "--with-windows-reactor") == 0)
	{
	    HINSTANCE hInstance = GetModuleHandle(0);
	    reactor = OB::GetWindowsReactor(hInstance);
	}
	else
	{
	    OB_ERROR(argv[0] << ": unknown option `" << argv[i] << "'\n");
	    OB_ERROR("Usage:");
	    OB_ERROR(argv[0] << " [--with-windows-reactor]");
	    return EXIT_FAILURE;
	}
#elif defined(HAVE_X11_INTRINSIC_H)
	if (strcmp(argv[i], "--with-x11-reactor") == 0)
	{
	    XtAppContext appContext;
	    XtAppInitialize(&appContext, "Test", 0, 0, &argc, argv, 0, 0, 0);
	    reactor = OB::GetX11Reactor(appContext);
	}
	else
	{
	    OB_ERROR(argv[0] << ": unknown option `" << argv[i] << "'\n");
	    OB_ERROR("Usage:");
	    OB_ERROR(argv[0] << " [--with-x11-reactor]");
	    return EXIT_FAILURE;
	}
#else
	OB_ERROR(argv[0] << ": unknown option `" << argv[i] << "'\n");
	return EXIT_FAILURE;
#endif
    }

    return EXIT_SUCCESS;
}
