// **********************************************************************
//
// Copyright (c) 2001
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntfStubTimeoutDSI_impl.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;

TestIntfStubTimeoutDSI_impl::TestIntfStubTimeoutDSI_impl(
    ORB_ptr orb,
    ORBTest_StubTimeout::Intf_ptr ti
)
    : m_orb(ORB::_duplicate(orb)),
      m_ti(ORBTest_StubTimeout::Intf::_duplicate(ti))
{
}

RepositoryId
TestIntfStubTimeoutDSI_impl::_primary_interface(
    const PortableServer::ObjectId& /* oid */,
    PortableServer::POA_ptr /* poa */
)
{
    return string_dup("IDL:ORBTest_StubTimeout/Intf:1.0");
}

CORBA::Boolean
TestIntfStubTimeoutDSI_impl::_is_a(
    const char* name
)
    throw (CORBA::SystemException)
{
    if (strcmp(name, "IDL:ORBTest_StubTimeout/Intf:1.0") == 0)
    {
	return true;
    }
    else
    {
#ifdef HAVE_VCPLUSPLUS_BUGS
	return ServantBase::_is_a(name);
#else
	return PortableServer::ServantBase::_is_a(name);
#endif
    }
}

void
TestIntfStubTimeoutDSI_impl::invoke(
    ServerRequest_ptr request
)
    throw ()
{
    String_var name = request->operation();

    bool ex;
    if (
	strlen(name) > 2
	&& strcmp((const char*)name + strlen(name) - 2, "Ex") == 0
    )
    {
	name[strlen(name) - 2] = '\0';
	ex = true;
    }
    else
	ex = false;

    if (strcmp(name, "sleep_oneway") == 0)
    {
	NVList_ptr list;
	Any* any;

	m_orb->create_list(0, list);
	list->add(ARG_IN)->value()->replace(_tc_ulong, 0);
	request->arguments(list);

	any = list->item(0)->value();
	ULong arg;
	*any >>= arg;

	m_ti->sleep_oneway(arg);

	return;
    }

    if (strcmp(name, "sleep_twoway") == 0)
    {
	NVList_ptr list;
	Any* any;

	m_orb->create_list(0, list);
	list->add(ARG_IN)->value()->replace(_tc_ulong, 0);
	request->arguments(list);

	any = list->item(0)->value();
	ULong arg;
	*any >>= arg;

	m_ti->sleep_twoway(arg);

	return;
    }

    // Unknown operation
    //

    OB_ERROR("DSI implementation: unknown operation: " << name);

    NVList_ptr list;
    m_orb->create_list(0, list);
    request->arguments(list);
    
    Any exAny;
    exAny <<= new BAD_OPERATION();
    request->set_exception(exAny);
}
