// **********************************************************************
//
// Copyright (c) 2001
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef TEST_INTF_IMPL_H
#define TEST_INTF_IMPL_H

#include <TestIntf_skel.h>

#include <TestIntfBasic_skel_tie.h>
#include <TestIntfBasic.h>
#include <TestIntfBasic_impl.h>
#include <TestIntfBasicDSI_impl.h>

#include <TestIntfContext_skel_tie.h>
#include <TestIntfContext.h>
#include <TestIntfContext_impl.h>
#include <TestIntfContextDSI_impl.h>

#include <TestIntfExceptions_skel_tie.h>
#include <TestIntfExceptions.h>
#include <TestIntfExceptions_impl.h>
#include <TestIntfExceptionsDSI_impl.h>

#include <TestIntfExceptionsExt_2_0_skel_tie.h>
#include <TestIntfExceptionsExt_2_0.h>
#include <TestIntfExceptionsExt_2_0_impl.h>
#include <TestIntfExceptionsExt_2_0DSI_impl.h>

#include <TestIntfWChar_skel_tie.h>
#include <TestIntfWChar.h>
#include <TestIntfWChar_impl.h>
#include <TestIntfWCharDSI_impl.h>

#include <TestIntfFixed_skel_tie.h>
#include <TestIntfFixed.h>
#include <TestIntfFixed_impl.h>
#include <TestIntfFixedDSI_impl.h>

#include <TestIntfLongDouble_skel_tie.h>
#include <TestIntfLongDouble.h>
#include <TestIntfLongDouble_impl.h>
#include <TestIntfLongDoubleDSI_impl.h>

#include <TestIntfLongLong_skel_tie.h>
#include <TestIntfLongLong.h>
#include <TestIntfLongLong_impl.h>
#include <TestIntfLongLongDSI_impl.h>

#include <TestIntfExceptionsExt_2_3_skel_tie.h>
#include <TestIntfExceptionsExt_2_3.h>
#include <TestIntfExceptionsExt_2_3_impl.h>
#include <TestIntfExceptionsExt_2_3DSI_impl.h>

#include <TestIntfStubTimeout_skel_tie.h>
#include <TestIntfStubTimeout.h>
#include <TestIntfStubTimeout_impl.h>
#include <TestIntfStubTimeoutDSI_impl.h>

class ImplType
{
public:
    enum ImplTypeEnum
    {
	SSI,
	Tie,
	DSI
    };

private:

    ImplTypeEnum m_type;

public:
    ImplType(
	ImplTypeEnum type
    )
    {
	m_type = type;
    }

    const char*
    to_string()
    {
	switch (m_type)
	{
	case SSI: return "SSI";
	case Tie: return "Tie";
	case DSI: return "DSI";
	}

	return "Unknown";
    }

    bool
    equals(
	ImplTypeEnum type
    )
    {
	return m_type == type;
    }
};

class TestIntf_impl
    : virtual public POA_ORBTest::Intf,
      public JTCMutex
{
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;

    TestIntfBasic_impl* m_test_intf_basic_impl;
    POA_ORBTest_Basic::Intf_tie< TestIntfBasic_impl >*
	m_test_intf_basic_tie_impl;
    TestIntfBasicDSI_impl* m_test_intf_basic_dsi_impl;
    ORBTest_Basic::Intf_var m_test_intf_basic;
    ORBTest_Basic::Intf_var m_test_intf_basic_tie;
    ORBTest_Basic::Intf_var m_test_intf_basic_dsi;

    TestIntfContext_impl* m_test_intf_context_impl;
    POA_ORBTest_Context::Intf_tie< TestIntfContext_impl >*
	m_test_intf_context_tie_impl;
    TestIntfContextDSI_impl* m_test_intf_context_dsi_impl;
    ORBTest_Context::Intf_var m_test_intf_context;
    ORBTest_Context::Intf_var m_test_intf_context_tie;
    ORBTest_Context::Intf_var m_test_intf_context_dsi;

    TestIntfExceptions_impl* m_test_intf_exceptions_impl;
    POA_ORBTest_Exceptions::Intf_tie< TestIntfExceptions_impl >*
	m_test_intf_exceptions_tie_impl;
    TestIntfExceptionsDSI_impl* m_test_intf_exceptions_dsi_impl;
    ORBTest_Exceptions::Intf_var m_test_intf_exceptions;
    ORBTest_Exceptions::Intf_var m_test_intf_exceptions_tie;
    ORBTest_Exceptions::Intf_var m_test_intf_exceptions_dsi;

    TestIntfExceptionsExt_2_0_impl* m_test_intf_exceptions_ext_2_0_impl;
    POA_ORBTest_ExceptionsExt_2_0::Intf_tie< TestIntfExceptionsExt_2_0_impl >*
	m_test_intf_exceptions_ext_2_0_tie_impl;
    TestIntfExceptionsExt_2_0DSI_impl* m_test_intf_exceptions_ext_2_0_dsi_impl;
    ORBTest_ExceptionsExt_2_0::Intf_var m_test_intf_exceptions_ext_2_0;
    ORBTest_ExceptionsExt_2_0::Intf_var m_test_intf_exceptions_ext_2_0_tie;
    ORBTest_ExceptionsExt_2_0::Intf_var m_test_intf_exceptions_ext_2_0_dsi;

    TestIntfWChar_impl* m_test_intf_wchar_impl;
    POA_ORBTest_WChar::Intf_tie< TestIntfWChar_impl >*
	m_test_intf_wchar_tie_impl;
    TestIntfWCharDSI_impl* m_test_intf_wchar_dsi_impl;
    ORBTest_WChar::Intf_var m_test_intf_wchar;
    ORBTest_WChar::Intf_var m_test_intf_wchar_tie;
    ORBTest_WChar::Intf_var m_test_intf_wchar_dsi;

    TestIntfFixed_impl* m_test_intf_fixed_impl;
    POA_ORBTest_Fixed::Intf_tie< TestIntfFixed_impl >*
	m_test_intf_fixed_tie_impl;
    TestIntfFixedDSI_impl* m_test_intf_fixed_dsi_impl;
    ORBTest_Fixed::Intf_var m_test_intf_fixed;
    ORBTest_Fixed::Intf_var m_test_intf_fixed_tie;
    ORBTest_Fixed::Intf_var m_test_intf_fixed_dsi;

    TestIntfLongDouble_impl* m_test_intf_long_double_impl;
    POA_ORBTest_LongDouble::Intf_tie< TestIntfLongDouble_impl >*
	m_test_intf_long_double_tie_impl;
    TestIntfLongDoubleDSI_impl* m_test_intf_long_double_dsi_impl;
    ORBTest_LongDouble::Intf_var m_test_intf_long_double;
    ORBTest_LongDouble::Intf_var m_test_intf_long_double_tie;
    ORBTest_LongDouble::Intf_var m_test_intf_long_double_dsi;

    TestIntfLongLong_impl* m_test_intf_long_long_impl;
    POA_ORBTest_LongLong::Intf_tie< TestIntfLongLong_impl >*
	m_test_intf_long_long_tie_impl;
    TestIntfLongLongDSI_impl* m_test_intf_long_long_dsi_impl;
    ORBTest_LongLong::Intf_var m_test_intf_long_long;
    ORBTest_LongLong::Intf_var m_test_intf_long_long_tie;
    ORBTest_LongLong::Intf_var m_test_intf_long_long_dsi;

    TestIntfExceptionsExt_2_3_impl* m_test_intf_exceptions_ext_2_3_impl;
    POA_ORBTest_ExceptionsExt_2_3::Intf_tie< TestIntfExceptionsExt_2_3_impl >*
	m_test_intf_exceptions_ext_2_3_tie_impl;
    TestIntfExceptionsExt_2_3DSI_impl* m_test_intf_exceptions_ext_2_3_dsi_impl;
    ORBTest_ExceptionsExt_2_3::Intf_var m_test_intf_exceptions_ext_2_3;
    ORBTest_ExceptionsExt_2_3::Intf_var m_test_intf_exceptions_ext_2_3_tie;
    ORBTest_ExceptionsExt_2_3::Intf_var m_test_intf_exceptions_ext_2_3_dsi;

    TestIntfStubTimeout_impl* m_test_intf_stub_timeout_impl;
    POA_ORBTest_StubTimeout::Intf_tie< TestIntfStubTimeout_impl >*
	m_test_intf_stub_timeout_tie_impl;
    TestIntfStubTimeoutDSI_impl* m_test_intf_stub_timeout_dsi_impl;
    ORBTest_StubTimeout::Intf_var m_test_intf_stub_timeout;
    ORBTest_StubTimeout::Intf_var m_test_intf_stub_timeout_tie;
    ORBTest_StubTimeout::Intf_var m_test_intf_stub_timeout_dsi;

    class TestCaseInitializer
    {
    public:
	virtual void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	) = 0;
    };

    class TestIntfBasicInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfBasicInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfBasicInitializer;

    class TestIntfContextInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfContextInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfContextInitializer;

    class TestIntfExceptionsInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfExceptionsInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfExceptionsInitializer;

    class TestIntfExceptionsExt_2_0Initializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfExceptionsExt_2_0Initializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfExceptionsExt_2_0Initializer;

    class TestIntfWCharInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfWCharInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfWCharInitializer;

    class TestIntfFixedInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfFixedInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfFixedInitializer;

    class TestIntfLongDoubleInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfLongDoubleInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfLongDoubleInitializer;

    class TestIntfLongLongInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfLongLongInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfLongLongInitializer;

    class TestIntfExceptionsExt_2_3Initializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfExceptionsExt_2_3Initializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfExceptionsExt_2_3Initializer;

    class TestIntfStubTimeoutInitializer
	: public TestCaseInitializer
    {
	ImplType m_impl_type;

    public:
	TestIntfStubTimeoutInitializer(
	    ImplType impl_type
	);

	void
	init(
	    TestIntf_impl& impl,
	    ORBTest::TestCase& test_case
	);
    };
    friend class TestIntfStubTimeoutInitializer;

    static TestCaseInitializer* test_case_initializers[];

public:

    TestIntf_impl(
	CORBA::ORB_ptr orb,
	PortableServer::POA_ptr poa
    );
    ~TestIntf_impl();

    virtual ORBTest::ORBType
    get_ORB_type()
        throw (CORBA::SystemException);

    virtual void
    deactivate()
        throw (CORBA::SystemException);

    virtual CORBA::Boolean
    concurrent_request_execution()
	throw (CORBA::SystemException);

    virtual ORBTest::TestCaseList*
    get_test_case_list()
	throw (CORBA::SystemException);
};

#endif
