// **********************************************************************
//
// Copyright (c) 2001
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OBPolicies.h>

#include "TestIntfStubTimeout.h"
#include "TestMacro.h"

#include "TestObjectStubTimeout.h"

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_TASK_LIB
#   include <taskLib.h>
#endif

TestObjectStubTimeout::TestObjectStubTimeout(
    CORBA::ORB_ptr orb,
    ORBTest::Intf_ptr test_intf,
    bool local
)
    : TestObject(orb, test_intf),
      m_local(local)
{
}

TestObjectStubTimeout::~TestObjectStubTimeout()
{
}

bool
TestObjectStubTimeout::is_supported(
    CORBA::Object_ptr obj
)
{
    //
    // Timeout tests are not for collocated client and servers
    //
    if (m_local)
	return false;

    ORBTest_StubTimeout::Intf_var t = (
	ORBTest_StubTimeout::Intf::_narrow(obj)
    );
    return !CORBA::is_nil(t);
}

void
TestObjectStubTimeout::test_SII(
    CORBA::Object_ptr obj
)
{
    ORBTest_StubTimeout::Intf_var ti_original = (
	ORBTest_StubTimeout::Intf::_narrow(obj)
    );

    //
    // Create policy list containing new timeout policy
    //
    CORBA::PolicyList pl(1);
    pl.length(1);
    CORBA::Any any;
    any <<= (CORBA::ULong)2000;
    pl[0] = m_orb->create_policy(OB::TIMEOUT_POLICY_ID, any);

    //
    // _set_policy_overrides returns new proxy with the policies applied
    //
    CORBA::Object_var new_obj = ti_original->_set_policy_overrides(
	pl,
	CORBA::ADD_OVERRIDE
    );

    //
    // _narrow to correct type
    //
    ORBTest_StubTimeout::Intf_var ti = (
	ORBTest_StubTimeout::Intf::_narrow(new_obj)
    );
    TEST(!CORBA::is_nil(ti));
    
    try
    {
	ti->sleep_twoway(3);
	TEST(false);
    }
    catch(const CORBA::NO_RESPONSE&)
    {
    }

#ifdef WIN32
    Sleep(2000);
#elif VXWORKS
    taskDelay(200);
#else
    sleep(2);
#endif

    //
    // The following test doesn't work, because there is a race
    // between the sleepOneway and the sleepTwoway if the server is
    // running in thread pool or thread-per-request mode. (This isn't
    // such an exiting test anyway, so perhaps we should permanently
    // remove it in a future version.)
    //
/*
    try
    {
	ti->sleep_oneway(3);
    }
    catch (const CORBA::NO_RESPONSE&)
    {
	TEST(false);
    }

    try
    {
	ti->sleep_twoway(0);
	TEST(false);
    }
    catch (const CORBA::NO_RESPONSE&)
    {
    }

#ifdef WIN32
    Sleep(2000);
#else
    sleep(2);
#endif
*/

    try
    {
	ti->sleep_twoway(0);
    }
    catch (const CORBA::NO_RESPONSE&)
    {
	TEST(false);
    }
}

void
TestObjectStubTimeout::test_DII(
    CORBA::Object_ptr /* obj */
)
{
}
