// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OBPolicies.h>

#include <TestMacro.h>
#include <Multi.h>

#include <stdlib.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;

int
run(ORB_ptr orb, int /* argc */, char* argv[])
{
    try
    {
	//
	// Get "multi" object
	//
	Object_var obj = orb -> string_to_object("relfile:/Multi.ref");
	if(is_nil(obj))
	{
	    OB_ERROR(argv[0] << ": cannot read IOR from Multi.ref");
	    return EXIT_FAILURE;
	}
	
	Multi_var multi = Multi::_narrow(obj);
	assert(!is_nil(multi));

	//
	// 5 servers are running. When the 6th is deactivated, I
	// should get an exception.
	//
	ULong oldpid = 0;
  	for(ULong i = 0 ; i < 5 ; i++)
  	{
	    OB_INFO("Getting process ID of server #" << i + 1 << "... ");

	    while(true)
	    {
		ULong pid = multi -> getpid();
		if(oldpid != pid)
		{
		    oldpid = pid;
		    break;
		}
	    }

	    OB_INFO_LN("Done!");
	    
	    OB_INFO("Deactivating server... ");
	    multi -> deactivate();
	    OB_INFO_LN("Done!");
  	}

#ifdef WIN32
        Sleep(1000);
#elif VXWORKS
        taskDelay(100);
#else
        sleep(1);
#endif

	OB_INFO("Checking whether I now get an exception... ");
	try
	{
	    while(true)
	    {
		ULong pid = multi -> getpid();
		TEST(pid == oldpid);
	    }
	}
	catch(const TRANSIENT& ex)
	{
	    OB_INFO_LN("Done!");
	    OB_INFO("Must be COMPLETED_NO... ");
	    TEST(ex.completed() == COMPLETED_NO);
	    OB_INFO_LN("Done!");
	}
    }
    catch(const Exception& ex)
    {
	OB_ERROR(ex);
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}


int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
        OB_ERROR(ex);
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
