// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Multi_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;

int
run(ORB_ptr orb, int argc, char* argv[])
{
    //
    // Get number of repetitions
    //
    if(argc < 2)
    {
	OB_ERROR("Usage: " << argv[0] << " server-number");
	return EXIT_FAILURE;
    }
    
    int num = atoi(argv[1]);
    if(num <= 0)
    {
	OB_ERROR(argv[0] << ": server number must be larger than 0");
	return EXIT_FAILURE;
    }
    
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);
    
    //
    // Activate the POA manager
    //
    PortableServer::POAManager_var manager = poa -> the_POAManager();
    manager -> activate();

    //
    // Create implementation object
    //
    Multi_impl multiImpl(orb);
#ifdef __linux__
    multiImpl.pid_ = (CORBA::ULong)::getpid();
#endif
    Multi_var multi = multiImpl._this();
    
    //
    // Save reference. This must be done after POA manager activation,
    // otherwise there is a potential for a race condition between the
    // client sending request and the server not being ready yet.
    //
    String_var s = orb -> object_to_string(multi);
    
    String_var refFile = string_dup("Multi.ref.");
    refFile += num;
    
    OB_OFSTREAM out(refFile);
    if(out.fail())
    {
	OB_ERROR(argv[0] << ": can't open `" << refFile << "': "
		 << strerror(errno));
	return EXIT_FAILURE;
    }
    
    out << s << OB_ENDL;
    out.close();
    
    //
    // Give up control to the ORB
    //
    orb -> run();
    
    //
    // Clean up
    //
#ifdef HAVE__UNLINK
    _unlink(refFile);
#else
    unlink(refFile);
#endif
    
    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_ERROR(ex);
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
