// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Test.h>

#include <stdlib.h>

#define REQS_PER_OBJ 4

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	//
	// Create ORB
	//
	orb = CORBA::ORB_init(argc, argv);
	CORBA::Object_var obj;

	//
	// Get test server object reference
	//
	obj = orb -> string_to_object("relfile:/Test.ref");
	if(CORBA::is_nil(obj))
	{
	    OB_ERROR(argv[0] << ": cannot read IOR from Test.ref");
	    return EXIT_FAILURE;
	}
	
	TestServer_var server = TestServer::_narrow(obj);
	assert(!CORBA::is_nil(server));
	TestInfoSeq_var info = server -> get_info();

	//
	// Test the thread dispatch strategies
	//
	CORBA::Request_var req;

	unsigned int idx;
	for(idx = 0; idx < info -> length(); ++idx)
	{
	    assert(!CORBA::is_nil(info[idx].obj));

	    for(unsigned int idx2 = 0; idx2 < REQS_PER_OBJ; ++idx2)
	    {
	        req = info[idx].obj -> _request("aMethod");
	        req -> send_deferred();
	    }
	}

	idx = info -> length() * REQS_PER_OBJ;
	do
	{
	    orb -> get_next_response(req);
	} while(--idx);

	server -> deactivate();
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	return status;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
