// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Test.h>

#include <stdlib.h>

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;
    try
    {
	//
	// Create ORB
	//
	orb = CORBA::ORB_init(argc, argv);
	CORBA::Object_var obj;

	//
	// Get test servers object references
	//
	obj = orb -> string_to_object("relfile:/Test.ref");
	if(CORBA::is_nil(obj))
	{
	    OB_ERROR(argv[0] << ": cannot read IOR from Test.ref");
	    return EXIT_FAILURE;
	}
	TestServer_var server = TestServer::_narrow(obj);
	assert(!CORBA::is_nil(server));

	//
	// Get object references from servers
	//
	TestInfoSeq_var info = server -> get_info();

	//
	// Call a few methods on each object
	//
	for(int idx = 0; idx < 3; ++idx)
	{
	    for(unsigned int idx2 = 0; idx2 < info -> length(); ++idx2)
	    {
		try
		{
		    info[idx2].obj -> aMethod();
		}
		catch(const CORBA::TRANSIENT&)
		{
		    //TODO: for now since shutdown can cause a
		    //TRANSIENT exception on the client.
		    assert(idx == 2);
		}
	    }
	}

	//
	// Deactivate servers
	//
	server -> deactivate();
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
