// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <Test_skel.h>
#include <TestRef_impl.h>
#include <TestUtil.h>

#include <stdlib.h>

using namespace CORBA;
using namespace PortableServer;

void
TestRefCount(ORB_ptr orb, POA_ptr root)
{
    Object_var obj;
    POA_var unique, multiple, implicit;
    ObjectId_var id1, id2;
    PolicyList policies;
    ServantBase_var servant;
    TestRef_impl* impl;

    POAManager_var manager = root -> the_POAManager();

    //
    // Create POAs
    //

    policies.length(2);
    policies[0] = root -> create_id_assignment_policy(PortableServer::USER_ID);
    policies[1] =
        root -> create_id_uniqueness_policy(PortableServer::UNIQUE_ID);
    unique = root -> create_POA("unique_id", manager, policies);

    policies.length(2);
    policies[0] = root -> create_id_assignment_policy(PortableServer::USER_ID);
    policies[1] =
        root -> create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
    multiple = root -> create_POA("multiple_id", manager, policies);

    policies.length(3);
    policies[0] =
        root -> create_id_assignment_policy(PortableServer::SYSTEM_ID);
    policies[1] =
        root -> create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
    policies[2] = root ->
        create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
    implicit = root -> create_POA("implicit", manager, policies);

    //
    // Start tests
    //

    id1 = string_to_ObjectId("obj1");
    id2 = string_to_ObjectId("obj2");
    impl = new TestRef_impl(orb, "obj1");
    servant = impl;

    //
    // Test w/ UNIQUE_ID POA
    //

    //OB_INFO_LN("Activating object with UNIQUE_ID POA... ");
    unique -> activate_object_with_id(id1, servant);

    {
        //OB_INFO_LN("Invoking reference_to_servant... ");
        obj = unique -> create_reference_with_id(id1, "IDL:Test:1.0");
        ServantBase_var s = unique -> reference_to_servant(obj);
    }

    {
        //OB_INFO_LN("Invoking id_to_servant... ");
        ServantBase_var s = unique -> id_to_servant(id1);
    }

    //OB_INFO_LN("Deactivating servant... ");
    unique -> deactivate_object(id1);

    //
    // Test w/ MULTIPLE_ID POA
    //

    //OB_INFO_LN("Activating object with MULTIPLE_ID POA... ");
    multiple -> activate_object_with_id(id1, servant);
    multiple -> activate_object_with_id(id2, servant);

    //OB_INFO_LN("Deactivating servant... ");
    multiple -> deactivate_object(id2);
    multiple -> deactivate_object(id1);

    //
    // Test w/ IMPLICIT_ACTIVATION POA
    //

    //OB_INFO_LN("Activating object with IMPLICIT_ACTIVATION POA... ");
    id1 = implicit -> servant_to_id(servant);
    id2 = implicit -> servant_to_id(servant);

    {
        //OB_INFO_LN("Invoking reference_to_servant... ");
        obj = implicit -> create_reference_with_id(id1, "IDL:Test:1.0");
        ServantBase_var s = implicit -> reference_to_servant(obj);
    }

    {
        //OB_INFO_LN("Invoking id_to_servant... ");
        ServantBase_var s = implicit -> id_to_servant(id1);
    }

    //OB_INFO_LN("Deactivating servant... ");
    implicit -> deactivate_object(id1);
    implicit -> deactivate_object(id2);

    //OB_INFO_LN("Destroying POAs... ");
    unique -> destroy(true, true);
    multiple -> destroy(true, true);
    implicit -> destroy(true, true);

    //OB_COUT << OB_ENDL
    //     << "Leaving TestRefCount - servant should be destroyed here... "
    //     << OB_ENDL;
    TEST(impl -> _OB_getRef() == 1);
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;

    ORB_var orb;
    try
    {
        orb = ORB_init(argc, argv);

        POA_var root = TestUtil_GetRootPOA(orb);

        OB_INFO("Testing collocated reference-counted servant... ");
        TestRefCount(orb, root);
        OB_INFO_LN("Done!");
    }
    catch(const Exception& ex)
    {
	OB_ERROR(ex);
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
