// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CosProperty.h>

//
// Tests for unconstrained PropertySetDef
//
void
TestUnconstrained(CosPropertyService::PropertySetDefFactory_ptr factory)
{
    CosPropertyService::PropertySetDef_var set;
    CORBA::ULong i, j;

    set = factory-> create_propertysetdef();

    {
	OB_INFO("  Testing define_property()... ");

	for(i = 0 ; i < 10 ; i++)
	{
	    CORBA::Any any;
	    any <<= i;

	    try
	    {
		CORBA::String_var s = CORBA::string_dup("property_");
		s += i;
		set -> define_property(s.in(), any);
	    }
	    catch(const CosPropertyService::ReadOnlyProperty&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::ConflictingProperty&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::UnsupportedProperty&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::UnsupportedTypeCode&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::InvalidPropertyName&)
	    {
		assert(false);
	    }
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing define_properties()... ");

	CosPropertyService::Properties nproperties;
	nproperties.length(10);

	for(i = 10 ; i < 20 ; i++)
	{
	    CORBA::Any any;
	    any <<= i;
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    CosPropertyService::Property property;
	    property.property_name = s;
	    property.property_value = any;
	    nproperties[i - 10] = property;
	}

	try
	{
	    set -> define_properties(nproperties);
	}
	catch(const CosPropertyService::MultipleExceptions&)
	{
	    assert(false);
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_all_property_names() without iterator... ");

	CosPropertyService::PropertyNames_var names;
	CosPropertyService::PropertyNamesIterator_var iterator;

	CORBA::ULong n = set -> get_number_of_properties();
	set -> get_all_property_names(n, names.out(), iterator.out());

	assert(names -> length() == n);

	CosPropertyService::PropertyNames v;
	v.length(n);

	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    v[i] = s;
	}

	//
	// Check if each property name exists only once
	//
	for(i = 0 ; i < n ; i++)
	{
	    for(j = 0 ; j < v.length() ; j++)
		if(!strcmp(names[i].in(), v[j].in()))
		    break;

            assert(j < v.length());
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_all_property_names() with iterator... ");

	CosPropertyService::PropertyNames_var names;
	CosPropertyService::PropertyNamesIterator_var iterator;

	//
	// Get one name without iterator
	//
	set -> get_all_property_names(1, names.out(), iterator.out());
	assert(names -> length() == 1);

	CORBA::ULong n = set -> get_number_of_properties();

	names -> length(n);

	//
	// Get remaining names with iterator
	//
	CosPropertyService::PropertyName_var name;
	i = 1;
	while(iterator -> next_one(name.out()))
	    names[i++] = name;

	CosPropertyService::PropertyNames v;
	v.length(n);

	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    v[i] = s;
	}

	//
	// Check if each property name exists only once
	//
	for(i = 0 ; i < n ; i++)
	{
	    for(j = 0 ; j < v.length() ; j++)
		if(!strcmp(names[i].in(), v[j].in()))
		    break;

            assert(j < v.length());
	}

	iterator -> destroy();

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_property_value()... ");

	CORBA::ULong n = set -> get_number_of_properties();
	for(i = 0 ; i < n ; i++)
	{
	    CORBA::Any_var any;

	    try
	    {
		CORBA::String_var s = CORBA::string_dup("property_");
		s += i;
		any = set -> get_property_value(s.in());
	    }
	    catch(const CosPropertyService::PropertyNotFound&)
	    {
		assert(false);
	    }

	    CORBA::ULong value;
	    any >>= value;
	    assert(value == i);
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_properties()... ");

	CosPropertyService::PropertyNames names;
	CORBA::ULong n = set -> get_number_of_properties();
	names.length(n);
	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    names[i] = s;
	}

	CosPropertyService::Properties_var nproperties;
	set -> get_properties(names, nproperties.out());

	CosPropertyService::PropertyNames vs;
	vs.length(n);
	CORBA::ULong vi[20];
	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    vs[i] = s;
	    vi[i] = i;
	}

	//
	// Check if each property name exists only once
	//
	CORBA::ULong match = 0;
	for(i = 0 ; i < n ; i++)
	{
	    for(j = 0 ; j < n ; j++)
	    {
		CORBA::String_var s = nproperties[i].property_name;
		if(strcmp(s.in(), vs[j].in()))
		    continue;

		CORBA::Any any = nproperties[i].property_value;
		CORBA::ULong value;
		any >>= value;
		if(value != vi[j])
		    continue;

		//
		// Mark current element as invalid, it has passed the test
		//
		vs[j] = CORBA::string_dup("");
		vi[j] = 9999;

		match++;
	    }
	}

	assert(match == n);

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_all_properties() without iterator... ");

	CosPropertyService::Properties_var properties;
	CosPropertyService::PropertiesIterator_var iterator;
	CORBA::ULong n = set -> get_number_of_properties();
	set -> get_all_properties(n, properties.out(), iterator.out());

	assert(properties -> length() == n);

	CosPropertyService::PropertyNames vs;
	vs.length(n);
	CORBA::ULong vi[20];

	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    vs[i] = s;
	    vi[i] = i;
	}

	//
	// Check if each property exists only once
	//
	CORBA::ULong match = 0;
	for(i = 0 ; i < n ; i++)
	{
	    for(j = 0 ; j < n ; j++)
	    {
		CORBA::String_var s = properties[i].property_name;
		if(strcmp(s.in(), vs[j].in()))
		    continue;

		CORBA::Any any = properties[i].property_value;
		CORBA::ULong value;
		any >>= value;
		if(value != vi[j])
		    continue;

		//
		// Mark current element as invalid, it has passed the test
		//
		vs[j] = CORBA::string_dup("");
		vi[j] = 9999;

		match++;
	    }
	}

	assert(match == n);

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing get_all_properties() with iterator... ");

	CosPropertyService::Properties_var properties;
	CosPropertyService::PropertiesIterator_var iterator;

	//
	// Get one property without iterator
	//
	set -> get_all_properties(1, properties.out(), iterator.out());
	assert(properties -> length() == 1);

	CORBA::ULong n = set -> get_number_of_properties();
	properties -> length(n);

	//
	// Get remaining properties with iterator
	//
	CosPropertyService::Property_var name;
	i = 1;
	while(iterator -> next_one(name.out()))
	    properties[i++] = name.in();

	CosPropertyService::PropertyNames vs;
	vs.length(n);
	CORBA::ULong vi[20];
	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;
	    vs[i] = s;
	    vi[i] = i;
	}

	//
	// Check if each property exists only once
	//
	CORBA::ULong match = 0;
	for(i = 0 ; i < n ; i++)
	{
	    for(j = 0 ; j < n ; j++)
	    {
		CORBA::String_var s = properties[i].property_name;
		if(strcmp(s.in(), vs[j].in()))
		    continue;

		CORBA::Any any = properties[i].property_value;
		CORBA::ULong value;
		any >>= value;
		if(value != vi[j])
		    continue;

		//
		// Mark current element as invalid, it has passed the test
		//
		vs[j] = CORBA::string_dup("");
		vi[j] = 9999;

		match++;
	    }
	}

	assert(match == n);

	iterator -> destroy();

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing is_property_defined()... ");

	CORBA::ULong n = set -> get_number_of_properties();
	for(i = 0 ; i < n ; i++)
	{
	    CORBA::String_var name = CORBA::string_dup("property_");
	    name += i;

	    try
	    {
		assert(set -> is_property_defined(name.in()));
	    }
	    catch(const CosPropertyService::InvalidPropertyName&)
	    {
		assert(false);
	    }
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing delete_property()... ");

	//
	// Delete first 5 properties, 15 remain
	//
	for(i = 0 ; i < 5 ; i++)
	{
	    CORBA::String_var name = CORBA::string_dup("property_");
	    name += i;

	    try
	    {
		set -> delete_property(name.in());
	    }
	    catch(const CosPropertyService::InvalidPropertyName&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::PropertyNotFound&)
	    {
		assert(false);
	    }
	    catch(const CosPropertyService::FixedProperty&)
	    {
		assert(false);
	    }
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing delete_properties()... ");

	//
	// Delete next 10 properties, 5 remain
	//
	CosPropertyService::PropertyNames names;
	names.length(10);

	for(i = 5 ; i < 15 ; i++)
	{
	    CORBA::String_var s = CORBA::string_dup("property_");
	    s += i;

	    names[i - 5] = s;
	}

	try
	{
	    set -> delete_properties(names);
	}
	catch(const CosPropertyService::MultipleExceptions&)
	{
	    assert(false);
	}

	CosPropertyService::PropertyNames_var names2;
	CosPropertyService::PropertyNamesIterator_var iterator;

	CORBA::ULong n = set -> get_number_of_properties();
	assert(n == 5);
	set -> get_all_property_names(n, names2.out(), iterator.out());

	for(i = 0 ; i < n ; i++)
	{
	    if(strcmp(names2[i], "property_15") &&
	       strcmp(names2[i], "property_16") &&
	       strcmp(names2[i], "property_17") &&
	       strcmp(names2[i], "property_18") &&
	       strcmp(names2[i], "property_19"))
	    {
		assert(false);
	    }
	}

	OB_INFO_LN("Done!");
    }

    {
	OB_INFO("  Testing delete_all_properties()... ");

	//
	// 10 properties remain and are deleted now
	//
	assert(set -> delete_all_properties());

	assert(!set -> get_number_of_properties());

	OB_INFO_LN("Done!");
    }
}
