// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/CosTime.h>
#include <OB/TimeHelper.h>

#include <TestMacro.h>

#include <time.h>

using namespace CORBA;
using namespace TimeBase;

void
TestUTO(CosTime::TimeService_ptr ts)
{
    CosTime::UTO_var uto;
    CosTime::UTO_var uto1;
    CosTime::UTO_var uto2;
    TimeBase::UtcT utc;

    //
    // TEST: CosTime::UTO creation
    //

    try
    {
	uto = ts -> new_universal_time(OB_ULONGLONG(0),
				       OB_ULONGLONG(0x1fffffffffffffff), 0);
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    try
    {
	uto = ts -> new_universal_time(OB_ULONGLONG(0),
				       OB_ULONGLONG(0x8fffffffffffffff), 0);
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    uto = ts -> universal_time();

    //
    // Visually display uto2
    //
    String_var str = OB::TimeHelper::toString(uto -> utc_time());
    TimeBase::TdfT tdf = uto -> tdf();
    OB_COUT << "(" << str << ", "
    	    << (double)tdf/60
	    << ((tdf <= 0) ? " west" : " east")
	    << " of GMT)";

    //
    // Source #2 for DCE -> POSIX offset is a test program for the
    // HPUX DCE library.
    //
    ULongLong offset = OB_ULONGLONG(122192928000000000);

    //
    // First check that the time is approximately the same as the
    // value returned from time(0) (that is +-1 second)
    //
    TimeBase::TimeT dceTime = uto -> time();
    ULongLong posixTime = ::time(0); // Don't use time_t to avoid warning

    dceTime -= offset;
    dceTime /= 10000000;

    //
    // posixTime -1 <= dceTime <= posixTime + 1
    //
    TEST(dceTime >= posixTime - 1 && dceTime <= posixTime +1);

    //
    // TEST: readonly attribute TimeBase::TimeT time;
    //

    uto = ts -> new_universal_time(OB::TimeHelper::MaxTimeT,
    				   OB::TimeHelper::MaxInaccuracyT, 123);
    TEST(uto -> time() == OB::TimeHelper::MaxTimeT);

    //
    // TEST: readonly attribute TimeBase::InaccuracyT inaccuracy;
    //

    TEST(uto -> inaccuracy() == OB::TimeHelper::MaxInaccuracyT);

    //
    // TEST: readonly attribute TimeBase::TdfT tdf;
    //

    TEST(uto -> tdf() == 123);

    //
    // TEST: readonly attribute TimeBase::UtcT utc_time;
    //

    uto = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000000),
    				   OB_ULONGLONG(0x8f542393FF56), 0);
    utc = uto -> utc_time();
    
    TEST(utc.time == uto -> time());
    TEST(utc.tdf == uto -> tdf());
    TEST(utc.inacchi == (UShort)0x8f54 &&
	 utc.inacclo == (ULong)0x2393FF56);
    
    //
    // TEST: TimeComparison compare_time(in ComparisonType comparison_type,
    //                                   in UTO uto);
    //

    //
    // TCEqualTo
    //

    //
    // Times the same and inaccuracy zero
    //
    uto = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000000), 0, 0);
    uto2 = ts -> new_universal_time(uto -> time(), 0, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto2) == CosTime::TCEqualTo);
    TEST(uto2 -> compare_time(CosTime::MidC, uto2) == CosTime::TCEqualTo);

    //
    // Times and inaccuracy the same
    //
    uto2 = ts -> new_universal_time(uto -> time(), 10, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto2) ==
         CosTime::TCIndeterminate);
    TEST(uto2 -> compare_time(CosTime::MidC, uto2) == CosTime::TCEqualTo);
    
    //
    // Times the same, inaccuracy not the same
    //
    CosTime::UTO_var uto3 =
        ts -> new_universal_time(uto2 -> time(), 30, uto2 -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto3) ==
	 CosTime::TCIndeterminate);
    TEST(uto2 -> compare_time(CosTime::MidC, uto3) == CosTime::TCEqualTo);

    //
    // TCLessThan
    //

    //
    // Compare time less than, no inaccuracy overlap
    //
    uto2 = ts -> new_universal_time(OB_ULONGLONG(0x7fffffffffffffce),
    				    25, uto -> tdf());
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000032),
    				    25, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto3) == CosTime::TCLessThan);
    TEST(uto2 -> compare_time(CosTime::MidC, uto3) == CosTime::TCLessThan);

    //
    // Compare time less than, inaccuracy overlap
    //
    uto2 = ts -> new_universal_time(OB_ULONGLONG(0x7fffffffffffffce),
    				    75, uto -> tdf());
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000032),
    				    75, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto3) == 
         CosTime::TCIndeterminate);
    TEST(uto2 -> compare_time(CosTime::MidC, uto3) == CosTime::TCLessThan);

    //
    // TCGreaterThan
    //

    //
    // Compare time greater than, no inaccuracy overlap
    //
    uto2 = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000032),
    				    25, uto -> tdf());
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x7fffffffffffffce),
    				    25, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto3) == 
         CosTime::TCGreaterThan);
    TEST(uto2 -> compare_time(CosTime::MidC, uto3) == CosTime::TCGreaterThan);

    //
    // Compare time greater than, inaccuracy overlap
    //
    uto2 = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000032),
    				    75, uto -> tdf());
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x7fffffffffffffce),
    				    75, uto -> tdf());

    TEST(uto2 -> compare_time(CosTime::IntervalC, uto3) == 
         CosTime::TCIndeterminate);
    TEST(uto2 -> compare_time(CosTime::MidC, uto3) == CosTime::TCGreaterThan);

    //
    // TEST: UTO absolute_time()
    //

    //
    // Test that time returned is within a couple of seconds. Cannot
    // be exact since method uses utcNow()
    //
    uto2 = ts -> new_universal_time(1000000000, 0, uto -> tdf());
    utc = OB::TimeHelper::utcNow();
    uto3 = uto2 -> absolute_time();
    TimeBase::UtcT utc2 = uto3 -> utc_time();
    TEST(utc2.time >= utc.time + 1000000000 &&
         utc2.time < utc.time + 1000000000 + 20000000);

    //
    // Test overflow check
    //
    offset = OB_ULONGLONG(0xffffffffffff0000);
    uto2 = ts -> new_universal_time(offset, 0, uto -> tdf());

    try
    {
        uto2 -> absolute_time();
	TEST(false);
    }
    catch(const DATA_CONVERSION&)
    {
    }

    //
    // TEST: TIO time_to_interval(in UTO uto);
    //

    //
    // Times the same
    //
    uto2 = ts -> new_universal_time(uto -> time(), 0, uto -> tdf());
    CosTime::TIO_var tio = uto2 -> time_to_interval(uto2);
    TimeBase::IntervalT interval = tio -> time_interval();
    TEST(interval.lower_bound == uto -> time() &&
         interval.upper_bound == uto -> time());

    //
    // Interval UTO greater than
    //
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000032),
    				    0, uto -> tdf());
    tio = uto2 -> time_to_interval(uto3);
    interval = tio -> time_interval();
    TEST(interval.lower_bound == uto -> time() &&
         interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval UTO less than
    //
    uto3 = ts -> new_universal_time(OB_ULONGLONG(0x7fffffffffffffce),
    				    0, uto -> tdf());
    tio = uto2 -> time_to_interval(uto3);
    interval = tio -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
         interval.upper_bound == uto -> time());

    //
    // TEST: TIO interval();
    //

    uto2 = ts -> new_universal_time(uto -> time(), 50, uto -> tdf());
    tio = uto2 -> interval();
    interval = tio -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));
}
