// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef SERVER_PACKET_HANDLER_H
#define SERVER_PACKET_HANDLER_H

#include <OB/Hashtable.h>
#include <OB/Net.h>
#include <OB/UDPIOP.h>

namespace OCI
{
    
namespace UDP
{
	    
//
// The ServerPacketHandler class
//
class ServerPacketHandler : public JTCThread
{
    //
    // The acceptor info
    //
    OCI::AcceptorInfo_var acceptorInfo_;

    //
    // The host
    //
    CORBA::String_var host_;

    //
    // The port
    //
    CORBA::UShort port_;

    //
    // The address
    //
    struct in_addr addr_;

    //
    // The socket
    //
    int fd_;

    //
    // The trace level
    //
    int trace_;

    //
    // The message timeout
    //
    int messageTimeout_;

    //
    // The transport timeout
    //
    int transportTimeout_;

    //
    // The logger
    //
    OB::Logger_var logger_;

    struct TransportKey
    {
	CORBA::ULong clientID;
	struct in_addr remoteAddr;
	CORBA::UShort remotePort;
	
	static bool comp(const TransportKey& obj1, 
			 const TransportKey& obj2)
	{
	    return obj1.clientID == obj2.clientID &&
		obj1.remoteAddr.s_addr == obj2.remoteAddr.s_addr &&
		obj1.remotePort == obj2.remotePort;
	}
	
	static CORBA::ULong hash(const TransportKey& value)
	{
            return value.clientID ^
                value.remoteAddr.s_addr ^ value.remotePort;
	}
    };

    //
    // The active transports
    //
    typedef OB::SynchronizedHashtable<TransportKey, OCI::Transport_var,
	TransportKey> TransportHashTable;
    TransportHashTable transports_;

    //
    // Transports yet to be accepted
    //
    typedef OB_STL::list<OCI::Transport_var> TransportList;
    TransportList newTransports_;
    JTCMonitor newTransportsMonitor_;

public:
    
    ServerPacketHandler(OCI::AcceptorInfo_ptr, const char*, CORBA::UShort,
                        const struct in_addr&, int, int, int, int,
                        OB::Logger_ptr);
    virtual ~ServerPacketHandler();
    
    virtual void run();
    void removeTransport(CORBA::ULong, struct in_addr, CORBA::UShort);
    OCI::Transport_ptr getNewTransport(bool);
    void connectSelf(OCI::Transport_ptr);
    void close();
    void shutdown();
    const char* getHost() const;
    CORBA::UShort getPort() const;
    const struct in_addr& getAddress() const;
};

typedef JTCHandleT<ServerPacketHandler> ServerPacketHandlerHandle;

} // End of namespace UDP

} // End of namespace OCI

#endif
