// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Codec.h>

#include <CorbaOut.h>

#include <stdio.h>

using namespace CORBA;
using namespace DynamicAny;
using namespace IOP;

using namespace OB;


CorbaOut::CorbaOut(DynamicAny::DynAnyFactory_ptr factory, WString_var& str)
    : factory_(DynamicAny::DynAnyFactory::_duplicate(factory)),
      str_(str),
      level_(0)
{
}

CorbaOut::~CorbaOut()
{
}

void
CorbaOut::skip()
{
    for(int i = 0 ; i < level_ ; i++)
        str_ += "   ";
}

// ----------------------------------------------------------------------
// Display CORBA::TypeCode
// ----------------------------------------------------------------------

static const char* NameTypeIDL [] =
{
    "null", "void",
    "short", "long",
    "unsigned short", "unsigned long",
    "float", "double",
    "boolean",
    "char",
    "octet",
    "any",
    "TypeCode",
    "Principal",
    "objref",
    "struct", "union",
    "enum",
    "string",
    "sequence", "array",
    "alias",
    "exception",
    "long long", "unsigned long long",
    "long double",
    "wchar", "wstring",
    "fixed",
    "value", "valuebox",
    "native",
    "abstract_interface"
};

void
CorbaOut::display_IDL_TypeCode(TypeCode_ptr tc)
{
    ULong i;
    TCKind kind = tc -> kind();

    switch(kind)
    {
    case tk_objref:
    {
        skip();
	str_ += "interface ";
	str_ += tc -> name();
	str_ += ';';
    }
        break;

    case tk_struct:
    case tk_except:
    {
        skip();
	str_ += NameTypeIDL[kind];
	str_ += ' ';
	str_ += tc -> name();
	str_ += " {\n";

        down();
        for(i = 0; i < tc -> member_count() ; i++)
        {
            skip();
            TypeCode_var member_type = tc -> member_type(i);
            show_name_TypeCode(member_type);
            str_ += ' ';
	    str_ += tc -> member_name(i);
	    str_ += ";\n";
        }
        up();
        skip();
	str_ += "};";

        break;
    }

    case tk_union:
    {
        skip();
	str_ += "union ";
	str_ += tc -> name();
	str_ += " switch(";
        TypeCode_var discriminator_type = tc -> discriminator_type();
        show_name_TypeCode(discriminator_type);
        str_ += ") {\n";

        down();
        for(i = 0 ; i < tc -> member_count() ; i++)
        {
            Any_var any_member_label = tc -> member_label(i);
            TypeCode_var type_label = any_member_label -> type();
            if(type_label -> kind() == tk_octet)
            {
                skip();
		str_ += "default";
            }
            else
            {
                skip();
		str_ += "case ";
                DynAny_var dyn_any_member_label =
                    factory_ -> create_dyn_any(any_member_label);
                show_DynAny_without_type(dyn_any_member_label);
            }
            str_ += ": ";
            TypeCode_var member_type = tc -> member_type(i);
            show_name_TypeCode(member_type);
            str_ += ' ';
	    str_ += tc -> member_name(i);
	    str_ += ";\n";
        }
        up();
        skip();
	str_ += "};";

        break;
    }

    case tk_enum:
    {
        skip();
	str_ += "enum ";
	str_ += tc -> name();
	str_ += " {";
        for(i = 0 ; i < tc -> member_count() ; i++) {
            if(i != 0)
                str_ += ", ";
            str_ += tc -> member_name(i);
        }
        str_ += "};";

        break;
    }

    case tk_string:
    {
        skip();
	str_ += "string";
        ULong length = tc -> length();
        if(length != 0)
	{
            str_ += '<';
	    str_ += length;
	    str_ += '>';
	}

        break;
    }

    case tk_sequence:
    {
        skip();
	str_ += "sequence<";
        TypeCode_var content_type = tc -> content_type();
        show_name_TypeCode(content_type);
        ULong length = tc -> length();
        if(length != 0)
	{
            str_ += ", ";
	    str_ += length;
	}
        str_ += '>';

        break;
    }

    case tk_array:
    {
        skip();
        TypeCode_var content_type = tc -> content_type();
        show_name_TypeCode(content_type);
        str_ += '[';
	str_ += tc -> length();
	str_ += ']';

        break;
    }

    case tk_alias:
    {
        skip();
	str_ += "typedef ";
        TypeCode_var content_type = tc -> content_type();
        show_name_TypeCode(content_type);
        str_ += ' ';
	str_ += tc -> name();
	str_ += ';';

        break;
    }

    default:
        skip();
	str_ += NameTypeIDL[kind];

        break;
    }

    str_ += '\n';
}

void
CorbaOut::show_name_TypeCode(TypeCode_ptr tc)
{
    TCKind kind = tc -> kind();

    switch(kind)
    {
    case tk_objref:
    case tk_enum:
    case tk_union:
    case tk_struct:
    case tk_except:
    case tk_alias:
    {
        str_ += tc -> name();

        break;
    }

    case tk_string:
    {
        str_ += "string";
        ULong length = tc -> length();
        if(length != 0)
	{
	    str_ += '<';
	    str_ += length;
	    str_ += '>';
	}

        break;
    }

    case tk_sequence:
    {
        str_ += "sequence<";
        TypeCode_var content_type = tc -> content_type();
        show_name_TypeCode(content_type);
        ULong length = tc -> length();
        if(length != 0)
	{
	    str_ += ", ";
	    str_ += length;
	}
        str_ += '>';

        break;
    }

    case tk_array:
    {
        TypeCode_var content_type = tc -> content_type();
        show_name_TypeCode(content_type);
        str_ += '[';
	str_ += tc -> length();
	str_ += ']';

        break;
    }

    default:
    {
        str_ += NameTypeIDL[kind];

        break;
    }
    }
}

// ----------------------------------------------------------------------
// Show CORBA::DynAny
// ----------------------------------------------------------------------

void
CorbaOut::display_DynAny(DynAny_ptr dyn_any)
{
    skip();
    show_DynAny(dyn_any);
    str_ += '\n';
}

void
CorbaOut::show_DynAny(DynAny_ptr dyn_any)
{
    TypeCode_var tc = dyn_any -> type();

    show_name_TypeCode(tc);
    str_ += '(';
    show_DynAny_without_type(dyn_any);
    str_ += ')';
}

void
CorbaOut::show_DynAny_without_type(DynAny_ptr dyn_any)
{
    ULong i;

    dyn_any -> rewind();

    TypeCode_var tc = dyn_any -> type();
    tc = OB::GetOrigType(tc);

    TCKind kind = tc -> kind();

    switch(kind)
    {
    case tk_null:
    case tk_void:
    {
        break;
    }

    case tk_short:
    {
        Short v_short = dyn_any -> get_short();
        str_ += v_short;

        break;
    }

    case tk_long:
    {
        Long v_long = dyn_any -> get_long();
        str_ += v_long;

        break;
    }

    case tk_ushort:
    {
        UShort v_ushort = dyn_any -> get_ushort();
	//
	// COMPILERFIX: Cast is required because of VC++ 6,
	// see HAVE_WIN32_WCHAR_TYPE
	//
        str_ += (ULongLong)v_ushort;

        break;
    }

    case tk_ulong:
    {
        ULong v_ulong = dyn_any -> get_ulong();
        str_ += v_ulong;

        break;
    }

    case tk_float:
    {
        char str[25];

        Float v_float = dyn_any -> get_float();
        sprintf(str, "%f", v_float);
        str_ += str;

        break;
    }

    case tk_double:
    {
        char str[25];

        Double v_double = dyn_any -> get_double();
        sprintf(str, "%f", v_double);
        str_ += str;

        break;
    }

    case tk_boolean:
    {
        Boolean v_boolean = dyn_any -> get_boolean();
        str_ += v_boolean ? "TRUE" : "FALSE";

        break;
    }

    case tk_char:
    {
        Char v_char = dyn_any -> get_char();
        str_ += "'";
	str_ += v_char;
	str_ += "'";

        break;
    }

    case tk_octet:
    {
        Octet v_octet = dyn_any -> get_octet();
        str_ += int(v_octet);

        break;
    }

    case tk_any:
    {
        Any_var v_any = dyn_any -> get_any();
        show_Any(v_any);

        break;
    }

    case tk_TypeCode: {
        TypeCode_var v_tc = dyn_any -> get_typecode();
        show_name_TypeCode(v_tc);

        break;
    }

    case tk_Principal:
    {
        str_ += "<Principal>";

        break;
    }

    case tk_objref:
    {
        str_ += tc -> id();
        Object_ptr v_obj = dyn_any -> get_reference();
        if(CORBA::is_nil(v_obj))
            str_ += "<NULL>";

        break;
    }

    case tk_struct:
    {
        DynStruct_var dyn_struct = DynStruct::_narrow(dyn_any);

        str_ += '\n';
        down();

        ULong member_count = tc -> member_count();
        for(i = 0 ; i < member_count ; i++)
        {
            String_var member_name = dyn_struct -> current_member_name();
            skip();
	    str_ += member_name;
	    str_ += " = ";
            DynAny_var component = dyn_struct -> current_component();
            show_DynAny(component);
            str_ += '\n';
            dyn_struct -> next();
        }

        up();
        skip();

        break;
    }

    case tk_union:
    {
        DynUnion_var dyn_union = DynUnion::_narrow(dyn_any);

        str_ += '\n';
        down();

        DynAny_var component = dyn_union -> current_component();

        skip();
	str_ += "discriminator = ";
        show_DynAny(component);
        str_ += '\n';

        if(dyn_union -> component_count() == 2)
        {
            dyn_union -> next();

            component = dyn_union -> current_component();

            String_var member_name = dyn_union -> member_name();
            skip();
	    str_ += member_name;
	    str_ += " = ";
            show_DynAny(component);
            str_ += '\n';
        }

        up();
        skip();

        break;
    }

    case tk_enum:
    {
         DynEnum_var dyn_enum = DynEnum::_narrow(dyn_any);
         String_var v_string = dyn_enum -> get_as_string();
         str_ += v_string;

         break;
    }

    case tk_string:
    {
        String_var v_string = dyn_any -> get_string();
        str_ += '"';
	str_ += v_string;
	str_ += '"';

        break;
    }

    case tk_sequence:
    {
        DynSequence_var dyn_sequence = DynSequence::_narrow(dyn_any);
        for(i = 0 ; i < dyn_sequence -> get_length() ; i++)
        {
            if(i != 0)
		str_ += ", ";
            DynAny_var component = dyn_sequence -> current_component();
            show_DynAny(component);
            dyn_sequence -> next();
        }

        break;
    }

    case tk_array:
    {
        DynArray_var dyn_array = DynArray::_narrow(dyn_any);
        for(i = 0 ; i < tc -> length() ; i++)
        {
            if(i != 0)
		str_ += ", ";
            DynAny_var component = dyn_array -> current_component();
            show_DynAny(component);
            dyn_array -> next();
        }

        break;
    }

    case tk_alias:
    {
        str_ += "<alias>";

        break;
    }

    case tk_except:
    {
        str_ += "<except>";

        break;
    }

    case tk_longlong:
    {
        LongLong v_longlong = dyn_any -> get_longlong();
        String_var str = OB::LongLongToString(v_longlong);
        str_ += str;

        break;
    }

    case tk_ulonglong:
    {
        ULongLong v_ulonglong = dyn_any -> get_ulonglong();
        String_var str = OB::ULongLongToString(v_ulonglong);
        str_ += str;

        break;
    }

#if SIZEOF_LONG_DOUBLE >= 12
    case tk_longdouble:
    {
        LongDouble v_longdouble = dyn_any -> get_longdouble();
        //
        // Not all platforms support this
        //
        //str_ += v_longdouble;
        char str[25];
        sprintf(str, "%31Lg", v_longdouble);
        str_ += str;

        break;
    }
#endif

    case tk_wchar:
    {
        WChar v_wchar = dyn_any -> get_wchar();
	str_ += "'";
	str_ += v_wchar;
	str_ += "'";

        break;
    }

    case tk_wstring:
    {
        WString_var v_wstring = dyn_any -> get_wstring();
	str_ += '"';
	str_ += v_wstring;
	str_ += '"';

        break;
    }

    case tk_fixed:
    {
        DynFixed_var dyn_fixed = DynFixed::_narrow(dyn_any);

        str_ += '\n';
        down();

        String_var str = dyn_fixed -> get_value();
        skip();
	str_ += str;
        str_ += '\n';

        up();
        skip();

        break;
    }

    case tk_value:
    {
        str_ += "<value>";

        break;
    }

    case tk_value_box:
    {
        str_ += "<value_box>";

        break;
    }

    case tk_native:
    {
        str_ += "<native>";

        break;
    }

    case tk_abstract_interface:
    {
        str_ += "<abstract_interface>";

        break;
    }

    default:
    {
        str_ += "Unknown DynAny: ";
	str_ += (int)kind;

        break;
    }
    }

    dyn_any -> rewind();
}

// ----------------------------------------------------------------------
// Show CORBA::Any
// ----------------------------------------------------------------------

void
CorbaOut::display_Any(const Any& any)
{
    DynAny_var dyn_any = factory_ -> create_dyn_any(any);
    display_DynAny(dyn_any);
}

void
CorbaOut::show_Any(const Any& any)
{
    DynAny_var dyn_any = factory_ -> create_dyn_any(any);
    show_DynAny(dyn_any);
}

