// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_LoadAlertInterceptor_h__
#define __OB_LoadAlertInterceptor_h__

#include <OB/LocalObject.h>
#include <OB/PIServer.h>

#include <UtilLogger_fwd.h>

#ifdef HAVE_SYS_TIME_H
#    include <sys/time.h>
#endif

#if !defined(WIN32)
#    include <unistd.h>
#endif

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// IDL:ooc.com/LoadBalancing/Util:1.0
//
namespace Util
{

//
// Server Request Interceptor which redirects requests back to the
// Balancer.
//
class LoadAlertInterceptor :
    public PortableInterceptor::ServerRequestInterceptor,
    public OBCORBA::RefCountLocalObject,
    public JTCMutex
{
    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // The Logger
    //
    Logger_var logger_;

    //
    // The POACurrent object
    //
    PortableServer::Current_var poaCurrent_;

    //
    // The alert expire time
    //
    CORBA::ULong expire_;

    //
    // The time when the alert started
    //
    timeval start_;

    //
    // Forward the next request?
    //
    bool forwardNext_;

    //
    // Get the current time.
    //
    timeval timeNow();

public:

    LoadAlertInterceptor(CORBA::ORB_ptr, Logger_ptr, CORBA::ULong expire);
    ~LoadAlertInterceptor();

    //
    // Redirects the next request back to the Balancer.
    //
    void alert();

    //
    // Retrieve the alert expire time.
    //
    CORBA::ULong getExpire();

    //
    // Set the alert expire time.
    //
    void setExpire(CORBA::ULong millis);

    //
    // IDL to C++ Mapping
    //

    virtual char*
    name();

    virtual void
    destroy()
	throw(CORBA::SystemException);

    virtual void
    receive_request_service_contexts(
    	PortableInterceptor::ServerRequestInfo_ptr)
	throw(PortableInterceptor::ForwardRequest, CORBA::SystemException);

    virtual void
    receive_request(PortableInterceptor::ServerRequestInfo_ptr)
	throw(PortableInterceptor::ForwardRequest, CORBA::SystemException);
    
    virtual void
    send_reply(PortableInterceptor::ServerRequestInfo_ptr)
	throw(CORBA::SystemException);
    
    virtual void
    send_exception(PortableInterceptor::ServerRequestInfo_ptr)
	throw(PortableInterceptor::ForwardRequest, CORBA::SystemException);

    virtual void
    send_other(PortableInterceptor::ServerRequestInfo_ptr)
	throw(PortableInterceptor::ForwardRequest, CORBA::SystemException);
};

} // End of namespace Util

} // End of namespace LoadBalancing

#endif
