// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_DYNAMIC_ANY_IMPL_H
#define OB_DYNAMIC_ANY_IMPL_H

#include <OB/ValueBase_fwd.h>

#include <OB/LocalObject.h>
#include <OB/Except.h>
#include <OB/DynamicAny.h>

namespace DynamicAny
{

class DynAny_impl : virtual public DynAny, public OBCORBA::RefCountLocalObject
{
    DynAny_impl(const DynAny_impl&);
    void operator=(const DynAny_impl&);

protected:

    DynAnyFactory_var factory_;
    OB::ORBInstance_var orbInstance_;
    CORBA::TypeCode_var type_;
    CORBA::TypeCode_ptr origType_;
    DynAny_ptr parent_;
    bool destroyed_;

    friend class DynAnyFactory_impl;

    DynAny_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		CORBA::TypeCode_ptr);
    virtual ~DynAny_impl();

    void checkValue(CORBA::Any*, CORBA::TypeCode_ptr);

    DynAny_ptr create(const CORBA::Any&, bool);
    DynAny_ptr create(CORBA::TypeCode_ptr, bool);

    void adoptChild(DynAny_ptr);

    //
    // Inform parent (if any) about a change to our state
    //
    virtual void notifyParent();

    //
    // Called when a child component has been modified
    //
    virtual void childModified(DynAny_ptr);

public:

    virtual CORBA::TypeCode_ptr type()
        throw(CORBA::SystemException);

    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException) = 0;

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException) = 0;

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException) = 0;

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException) = 0;

    virtual void destroy()
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException) = 0;

    virtual void insert_boolean(CORBA::Boolean)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_octet(CORBA::Octet)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_char(CORBA::Char)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_short(CORBA::Short)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ushort(CORBA::UShort)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_long(CORBA::Long)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulong(CORBA::ULong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_float(CORBA::Float)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_double(CORBA::Double)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_string(const char*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_reference(CORBA::Object_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_typecode(CORBA::TypeCode_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longlong(CORBA::LongLong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulonglong(CORBA::ULongLong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longdouble(CORBA::LongDouble)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_wchar(CORBA::WChar)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_wstring(const wchar_t*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_dyn_any(DynAny_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_val(CORBA::ValueBase*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_abstract(CORBA::AbstractBase_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_boolean_seq(const CORBA::BooleanSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_octet_seq(const CORBA::OctetSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_char_seq(const CORBA::CharSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_wchar_seq(const CORBA::WCharSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_short_seq(const CORBA::ShortSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ushort_seq(const CORBA::UShortSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_long_seq(const CORBA::LongSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulong_seq(const CORBA::ULongSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longlong_seq(const CORBA::LongLongSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulonglong_seq(const CORBA::ULongLongSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_float_seq(const CORBA::FloatSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_double_seq(const CORBA::DoubleSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longdouble_seq(const CORBA::LongDoubleSeq& value)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Boolean get_boolean()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Octet get_octet()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Char get_char()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Short get_short()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::UShort get_ushort()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Long get_long()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULong get_ulong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Float get_float()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Double get_double()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual char* get_string()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Object_ptr get_reference()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::TypeCode_ptr get_typecode()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongLong get_longlong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULongLong get_ulonglong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongDouble get_longdouble()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::WChar get_wchar()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual wchar_t* get_wstring()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* get_any()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual DynAny_ptr get_dyn_any()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ValueBase* get_val()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::AbstractBase_ptr get_abstract()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::BooleanSeq* get_boolean_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::OctetSeq* get_octet_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::CharSeq* get_char_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::WCharSeq* get_wchar_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ShortSeq* get_short_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::UShortSeq* get_ushort_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongSeq* get_long_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULongSeq* get_ulong_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongLongSeq* get_longlong_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULongLongSeq* get_ulonglong_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::FloatSeq* get_float_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::DoubleSeq* get_double_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongDoubleSeq* get_longdouble_seq()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException) = 0;

    virtual void rewind()
        throw(CORBA::SystemException) = 0;

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException) = 0;

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException) = 0;

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException) = 0;

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*) = 0;
    virtual void _OB_unmarshal(OB::InputStreamImpl*) = 0;
    virtual CORBA::Any* _OB_currentAny() = 0;
    virtual CORBA::Any* _OB_currentAnyValue() = 0;
    virtual DynAny_ptr _OB_getDynAny();
    virtual bool _OB_insertDynAny(DynAny_ptr);

};

class DynBasic_impl : public DynAny_impl
{
    DynBasic_impl(const DynBasic_impl&);
    void operator=(const DynBasic_impl&);

    CORBA::Any_var any_;

    //
    // This object needs a component when the type is tk_any
    //
    DynAny_var comp_;

protected:

    friend class DynAnyFactory_impl;

    DynBasic_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		  CORBA::TypeCode_ptr);
    virtual ~DynBasic_impl();

    //
    // Intercept changes to our state to synchronize our component
    //
    virtual void notifyParent();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
    virtual DynAny_ptr _OB_getDynAny();
    virtual bool _OB_insertDynAny(DynAny_ptr);
};

class DynFixed_impl : public DynFixed,
                      public DynAny_impl
{
    DynFixed_impl(const DynFixed_impl&);
    void operator=(const DynFixed_impl&);

    CORBA::Fixed value_;

protected:

    friend class DynAnyFactory_impl;

    DynFixed_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		  CORBA::TypeCode_ptr);
    virtual ~DynFixed_impl();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);


    //
    // Standard IDL-to-C++ mapping - DynFixed
    //
    virtual char* get_value()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean set_value(const char* val)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynEnum_impl : public DynEnum,
                     public DynAny_impl
{
    DynEnum_impl(const DynEnum_impl&);
    void operator=(const DynEnum_impl&);

    CORBA::ULong value_;

protected:

    friend class DynAnyFactory_impl;

    DynEnum_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		 CORBA::TypeCode_ptr);
    virtual ~DynEnum_impl();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);


    //
    // Standard IDL-to-C++ mapping - DynEnum
    //
    virtual char* get_as_string()
        throw(CORBA::SystemException);

    virtual void set_as_string(const char*)
        throw(InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULong get_as_ulong()
        throw(CORBA::SystemException);

    virtual void set_as_ulong(CORBA::ULong)
        throw(InvalidValue,
              CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynStruct_impl : public DynStruct,
                       public DynAny_impl
{
    DynStruct_impl(const DynStruct_impl&);
    void operator=(const DynStruct_impl&);

    DynAnySeq components_;
    CORBA::Long index_;

protected:

    friend class DynAnyFactory_impl;

    DynStruct_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		   CORBA::TypeCode_ptr);
    virtual ~DynStruct_impl();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);


    //
    // Standard IDL-to-C++ mapping - DynStruct
    //
    virtual FieldName current_member_name()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::TCKind current_member_kind()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual NameValuePairSeq* get_members()
        throw(CORBA::SystemException);

    virtual void set_members(const NameValuePairSeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual NameDynAnyPairSeq* get_members_as_dyn_any()
        throw(CORBA::SystemException);

    virtual void set_members_as_dyn_any(const NameDynAnyPairSeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);


    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynUnion_impl : public DynamicAny::DynUnion,
                      public DynAny_impl
{
    DynUnion_impl(const DynUnion_impl&);
    void operator=(const DynUnion_impl&);

    DynAny_var disc_;
    DynAny_var member_;
    CORBA::Long index_;
    CORBA::Long defaultMember_;
    CORBA::ULong selectedMember_;
    OB::FixSeq< CORBA::LongLong, int > memberLabels_;
    CORBA::TypeCode_ptr origDiscTC_;
    bool ignoreDiscChange_;

protected:

    friend class DynAnyFactory_impl;

    DynUnion_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
		  CORBA::TypeCode_ptr);
    virtual ~DynUnion_impl();

    virtual void childModified(DynAny_ptr);

    void initMember();
    void resetDiscriminator(CORBA::LongLong);
    CORBA::LongLong getDiscriminatorValue(const CORBA::Any&);
    CORBA::LongLong findUnusedDiscriminator();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);


    //
    // Standard IDL-to-C++ mapping - DynUnion
    //
    virtual DynAny_ptr get_discriminator()
        throw(CORBA::SystemException);

    virtual void set_discriminator(DynAny_ptr d)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void set_to_default_member()
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void set_to_no_active_member()
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual CORBA::Boolean has_no_active_member()
        throw(CORBA::SystemException);

    virtual CORBA::TCKind discriminator_kind()
        throw(CORBA::SystemException);

    virtual DynAny_ptr member()
        throw(InvalidValue,
              CORBA::SystemException);

    virtual FieldName member_name()
        throw(InvalidValue,
              CORBA::SystemException);

    virtual CORBA::TCKind member_kind()
        throw(InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Boolean is_set_to_default_member()
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

//
// This class serves as a common base class for DynSequence_impl
// and DynArray_impl
//
class DynSeqBase_impl : public DynAny_impl
{
    DynSeqBase_impl(const DynSeqBase_impl&);
    void operator=(const DynSeqBase_impl&);

protected:

    CORBA::TypeCode_ptr contentType_;
    CORBA::TCKind contentKind_;
    DynAnySeq components_;
    CORBA::Long index_;
    CORBA::Long length_;
    CORBA::Long max_;
    void* buf_;
    bool primitive_;
    bool ignoreChild_;

    DynSeqBase_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                    CORBA::TypeCode_ptr);
    virtual ~DynSeqBase_impl();

    virtual void childModified(DynAny_ptr);

    void validate(CORBA::TCKind);
    void getValue(CORBA::ULong, CORBA::Any&);
    void setValue(CORBA::ULong, const CORBA::Any&);
    void setValue(CORBA::ULong, DynAny_ptr);
    void updateComponent(CORBA::ULong, bool);
    void resize(CORBA::ULong, bool);

    AnySeq* getElements();
    DynAnySeq* getElementsAsDynAny();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual void insert_boolean(CORBA::Boolean)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_octet(CORBA::Octet)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_char(CORBA::Char)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_short(CORBA::Short)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ushort(CORBA::UShort)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_long(CORBA::Long)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulong(CORBA::ULong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_float(CORBA::Float)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_double(CORBA::Double)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_string(const char*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_reference(CORBA::Object_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_typecode(CORBA::TypeCode_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longlong(CORBA::LongLong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_ulonglong(CORBA::ULongLong)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_longdouble(CORBA::LongDouble)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_wchar(CORBA::WChar)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_wstring(const wchar_t*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_dyn_any(DynAny_ptr)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual void insert_val(CORBA::ValueBase*)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Boolean get_boolean()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Octet get_octet()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Char get_char()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Short get_short()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::UShort get_ushort()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Long get_long()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULong get_ulong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Float get_float()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Double get_double()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual char* get_string()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Object_ptr get_reference()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::TypeCode_ptr get_typecode()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongLong get_longlong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ULongLong get_ulonglong()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::LongDouble get_longdouble()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::WChar get_wchar()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual wchar_t* get_wstring()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* get_any()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual DynAny_ptr get_dyn_any()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::ValueBase* get_val()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynSequence_impl : public DynSequence,
                         public DynSeqBase_impl
{
    DynSequence_impl(const DynSequence_impl&);
    void operator=(const DynSequence_impl&);

protected:

    friend class DynAnyFactory_impl;

    DynSequence_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                     CORBA::TypeCode_ptr);
    virtual ~DynSequence_impl();

public:

    //
    // Standard IDL-to-C++ mapping - DynSequence
    //
    virtual CORBA::ULong get_length()
        throw(CORBA::SystemException);

    virtual void set_length(CORBA::ULong len)
        throw(InvalidValue,
              CORBA::SystemException);

    virtual AnySeq* get_elements()
        throw(CORBA::SystemException);

    virtual void set_elements(const AnySeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual DynAnySeq* get_elements_as_dyn_any()
        throw(CORBA::SystemException);

    virtual void set_elements_as_dyn_any(const DynAnySeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);
};

class DynArray_impl : public DynArray,
                      public DynSeqBase_impl
{
    DynArray_impl(const DynArray_impl&);
    void operator=(const DynArray_impl&);

protected:

    friend class DynAnyFactory_impl;

    DynArray_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                  CORBA::TypeCode_ptr);
    virtual ~DynArray_impl();

public:

    //
    // Standard IDL-to-C++ mapping - DynArray
    //
    virtual AnySeq* get_elements()
        throw(CORBA::SystemException);

    virtual void set_elements(const AnySeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual DynAnySeq* get_elements_as_dyn_any()
        throw(CORBA::SystemException);

    virtual void set_elements_as_dyn_any(const DynAnySeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);
};

class DynValueCommon_impl : virtual public DynAny_impl,
                            virtual public DynValueCommon
{
    DynValueCommon_impl(const DynValueCommon_impl&);
    void operator=(const DynValueCommon_impl&);

    bool isNull_;

protected:

    DynValueCommon_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                        CORBA::TypeCode_ptr);
    virtual ~DynValueCommon_impl();

    virtual void createComponents() = 0;
    virtual void destroyComponents() = 0;

public:

    //
    // Standard IDL-to-C++ mapping - DynValueCommon
    //
    virtual CORBA::Boolean is_null()
        throw(CORBA::SystemException);

    virtual void set_to_null()
        throw(CORBA::SystemException);

    virtual void set_to_value()
        throw(CORBA::SystemException);
};

class DynValue_impl : virtual public DynValueCommon_impl,
                      virtual public DynValue
{
    DynValue_impl(const DynValue_impl&);
    void operator=(const DynValue_impl&);

    DynAnySeq components_;
    OB::StrSeq< int > names_;
    OB::ObjSeq< CORBA::TypeCode, int > types_;
    OB::StrSeq< int > ids_;
    CORBA::Long index_;

    static void getMembers(CORBA::TypeCode_ptr, OB::StrSeq< int >&,
                           OB::ObjSeq< CORBA::TypeCode, int >&);

protected:

    friend class DynAnyFactory_impl;

    DynValue_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                  CORBA::TypeCode_ptr);
    virtual ~DynValue_impl();

    virtual void createComponents();
    virtual void destroyComponents();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);


    //
    // Standard IDL-to-C++ mapping - DynValue
    //
    virtual FieldName current_member_name()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::TCKind current_member_kind()
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual NameValuePairSeq* get_members()
        throw(CORBA::SystemException);

    virtual void set_members(const NameValuePairSeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual NameDynAnyPairSeq* get_members_as_dyn_any()
        throw(CORBA::SystemException);

    virtual void set_members_as_dyn_any(const NameDynAnyPairSeq&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);


    //
    // ORBacus internal functions
    //
    static void _OB_isNull(const CORBA::Any&);
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynValueBox_impl : virtual public DynValueCommon_impl,
                         virtual public DynValueBox
{
    DynValueBox_impl(const DynValueBox_impl&);
    void operator=(const DynValueBox_impl&);

    DynAny_var component_;
    CORBA::TypeCode_var boxedType_;
    CORBA::Long index_;

protected:

    friend class DynAnyFactory_impl;

    DynValueBox_impl(DynAnyFactory_ptr, OB::ORBInstance_ptr,
                     CORBA::TypeCode_ptr);
    virtual ~DynValueBox_impl();

    virtual void createComponents();
    virtual void destroyComponents();

public:

    //
    // Standard IDL-to-C++ mapping - DynAny
    //
    virtual void assign(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    virtual void from_any(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual CORBA::Any* to_any()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(DynAny_ptr)
        throw(CORBA::SystemException);

    virtual DynAny_ptr copy()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean seek(CORBA::Long)
        throw(CORBA::SystemException);

    virtual void rewind()
        throw(CORBA::SystemException);

    virtual CORBA::Boolean next()
        throw(CORBA::SystemException);

    virtual CORBA::ULong component_count()
        throw(CORBA::SystemException);

    virtual DynAny_ptr current_component()
        throw(TypeMismatch,
              CORBA::SystemException);

    //
    // Standard IDL-to-C++ mapping - DynValueBox
    //
    virtual CORBA::Any* get_boxed_value()
        throw(InvalidValue,
              CORBA::SystemException);

    virtual void set_boxed_value(const CORBA::Any&)
        throw(TypeMismatch,
              InvalidValue,
              CORBA::SystemException);

    virtual DynAny_ptr get_boxed_value_as_dyn_any()
        throw(InvalidValue,
              CORBA::SystemException);

    virtual void set_boxed_value_as_dyn_any(DynAny_ptr)
        throw(TypeMismatch,
              CORBA::SystemException);

    //
    // ORBacus internal functions
    //
    virtual void _OB_marshal(OB::OutputStreamImpl*);
    virtual void _OB_unmarshal(OB::InputStreamImpl*);
    virtual CORBA::Any* _OB_currentAny();
    virtual CORBA::Any* _OB_currentAnyValue();
};

class DynAnyFactory_impl : public DynAnyFactory,
			   public OBCORBA::RefCountLocalObject
{
    DynAnyFactory_impl(const DynAnyFactory_impl&);
    void operator=(const DynAnyFactory_impl&);

    OB::ORBInstance_var orbInstance_;

public:

    DynAnyFactory_impl(OB::ORBInstance_ptr);
    ~DynAnyFactory_impl();

    //
    // Standard IDL-to-C++ mapping - DynAnyFactory
    //
    virtual DynAny_ptr create_dyn_any(const CORBA::Any&)
        throw(InconsistentTypeCode,
              CORBA::SystemException);

    virtual DynAny_ptr create_dyn_any_from_type_code(CORBA::TypeCode_ptr type)
        throw(InconsistentTypeCode,
              CORBA::SystemException);
};

} // End of namespace DynamicAny

#endif
