// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef MESSAGING_IMPL_H
#define MESSAGING_IMPL_H

#include <OB/LocalObject.h>
#include <OB/TypeCode.h>
#include <OB/Messaging.h>

namespace Messaging
{

//
// Rebind Policy
//
class RebindPolicy_impl : public RebindPolicy,
			  public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RebindPolicy_impl(const RebindPolicy_impl&);
    void operator=(const RebindPolicy_impl&);

    RebindMode value_;

public:

    RebindPolicy_impl(RebindMode);
    ~RebindPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual RebindMode rebind_mode()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

//
// Synchronization Policy
//
class SyncScopePolicy_impl : public SyncScopePolicy,
			     public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    SyncScopePolicy_impl(const SyncScopePolicy_impl&);
    void operator=(const SyncScopePolicy_impl&);

    SyncScope value_;

public:

    SyncScopePolicy_impl(SyncScope);
    ~SyncScopePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual SyncScope synchronization()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

//
// Timeout Policies
//
class RequestStartTimePolicy_impl : public RequestStartTimePolicy,
			            public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RequestStartTimePolicy_impl(const RequestStartTimePolicy_impl&);
    void operator=(const RequestStartTimePolicy_impl&);

    TimeBase::UtcT value_;

public:

    RequestStartTimePolicy_impl(TimeBase::UtcT);
    ~RequestStartTimePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::UtcT start_time()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RequestEndTimePolicy_impl : public RequestEndTimePolicy,
			          public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RequestEndTimePolicy_impl(const RequestEndTimePolicy_impl&);
    void operator=(const RequestEndTimePolicy_impl&);

    TimeBase::UtcT value_;

public:

    RequestEndTimePolicy_impl(TimeBase::UtcT);
    ~RequestEndTimePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::UtcT end_time()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ReplyStartTimePolicy_impl : public ReplyStartTimePolicy,
		                  public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ReplyStartTimePolicy_impl(const ReplyStartTimePolicy_impl&);
    void operator=(const ReplyStartTimePolicy_impl&);

    TimeBase::UtcT value_;

public:

    ReplyStartTimePolicy_impl(TimeBase::UtcT);
    ~ReplyStartTimePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::UtcT start_time()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ReplyEndTimePolicy_impl : public ReplyEndTimePolicy,
			        public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ReplyEndTimePolicy_impl(const ReplyEndTimePolicy_impl&);
    void operator=(const ReplyEndTimePolicy_impl&);

    TimeBase::UtcT value_;

public:

    ReplyEndTimePolicy_impl(TimeBase::UtcT);
    ~ReplyEndTimePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::UtcT end_time()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RelativeRequestTimeoutPolicy_impl : public RelativeRequestTimeoutPolicy,
			                  public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RelativeRequestTimeoutPolicy_impl(
        const RelativeRequestTimeoutPolicy_impl&);
    void operator=(const RelativeRequestTimeoutPolicy_impl&);

    TimeBase::TimeT value_;

public:

    RelativeRequestTimeoutPolicy_impl(TimeBase::TimeT);
    ~RelativeRequestTimeoutPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::TimeT relative_expiry()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RelativeRoundTripTimeoutPolicy_impl :
    public RelativeRoundTripTimeoutPolicy,
    public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RelativeRoundTripTimeoutPolicy_impl(
        const RelativeRoundTripTimeoutPolicy_impl&);
    void operator=(const RelativeRoundTripTimeoutPolicy_impl&);

    TimeBase::TimeT value_;

public:

    RelativeRoundTripTimeoutPolicy_impl(TimeBase::TimeT);
    ~RelativeRoundTripTimeoutPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual TimeBase::TimeT relative_expiry()
        throw(CORBA::SystemException);
    
    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

} // End of namespace Messaging

#endif
