// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_ORB_INIT_INFO_IMPL_H
#define OB_ORB_INIT_INFO_IMPL_H

#include <OB/OBORBInit.h>
#include <OB/PIManager_fwd.h>
#include <OB/InitialServiceManager_fwd.h>
#include <OB/Codec_fwd.h>

#include <OB/LocalObject.h>

namespace OBPortableInterceptor
{

//
// An ORBacus-specific derivation of PortableInterceptor::ORBInitInfo
//
class ORBInitInfo_impl : public ORBInitInfo,
			 public OBCORBA::RefCountLocalObject
{
    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // Arguments
    //
    int ac_;
    char** av_;

    //
    // The ORB id
    //
    const char* id_;

    //
    // The PIManager
    //
    OB::PIManager_var manager_;

    //
    // The initial service mgr
    //
    OB::InitialServiceManager_var initServiceManager_;

    //
    // The Codec Factory
    //
    IOP::CodecFactory_var codecFactory_;

    bool destroy_;

public:

    ORBInitInfo_impl(CORBA::ORB_ptr, int, char**, const char*,
                     OB::PIManager_ptr,
                     OB::InitialServiceManager_ptr,
                     IOP::CodecFactory_ptr);
    virtual ~ORBInitInfo_impl();

    //
    // IDL to C++ Mapping
    //
    virtual CORBA::StringSeq*
    arguments()
	throw(CORBA::SystemException);

    virtual char*
    orb_id()
	throw(CORBA::SystemException);

    virtual IOP::CodecFactory_ptr
    codec_factory()
	throw(CORBA::SystemException);

    virtual void
    register_initial_reference(const char*, CORBA::Object_ptr)
	throw(CORBA::SystemException,
	      PortableInterceptor::ORBInitInfo::InvalidName);

    virtual CORBA::Object_ptr
    resolve_initial_references(const char*)
	throw(CORBA::SystemException,
	      PortableInterceptor::ORBInitInfo::InvalidName);

    virtual void
    add_client_request_interceptor(
        PortableInterceptor::ClientRequestInterceptor_ptr)
	throw(CORBA::SystemException,
	      PortableInterceptor::ORBInitInfo::DuplicateName);

    virtual void
    add_server_request_interceptor(
        PortableInterceptor::ServerRequestInterceptor_ptr)
	throw(CORBA::SystemException,
	      PortableInterceptor::ORBInitInfo::DuplicateName);

    virtual void
    add_ior_interceptor(PortableInterceptor::IORInterceptor_ptr)
	throw(CORBA::SystemException,
	      PortableInterceptor::ORBInitInfo::DuplicateName);

    virtual PortableInterceptor::SlotId
    allocate_slot_id()
	throw(CORBA::SystemException);

    virtual void
    register_policy_factory(CORBA::PolicyType,
                            PortableInterceptor::PolicyFactory_ptr)
	throw(CORBA::SystemException);

    virtual CORBA::ORB_ptr
    orb()
	throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_destroy();
};

} // End of namespace OBPortableInterceptor

#endif
