// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POLICIES_IMPL_H
#define OB_POLICIES_IMPL_H

#include <OB/Except.h>
#include <OB/LocalObject.h>
#include <OB/Any.h>
#include <OB/OBPolicies.h>

namespace OB
{

class ProtocolPolicy_impl : public ProtocolPolicy,
			    public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ProtocolPolicy_impl(const ProtocolPolicy_impl&);
    void operator=(const ProtocolPolicy_impl&);

    OCI::PluginIdSeq value_;

public:

    ProtocolPolicy_impl(const OCI::PluginIdSeq&);
    ~ProtocolPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::PluginIdSeq* value()
        throw(CORBA::SystemException);
    virtual CORBA::Boolean contains(const char*)
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ConnectionReusePolicy_impl : public ConnectionReusePolicy,
				   public OBCORBA::RefCountLocalObject

{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectionReusePolicy_impl(const ConnectionReusePolicy_impl&);
    void operator=(const ConnectionReusePolicy_impl&);

    CORBA::Boolean value_;

public:

    ConnectionReusePolicy_impl(CORBA::Boolean);
    ~ConnectionReusePolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::Boolean value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RetryPolicy_impl : public RetryPolicy,
			 public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RetryPolicy_impl(const RetryPolicy_impl&);
    void operator=(const RetryPolicy_impl&);

    CORBA::Short mode_;
    CORBA::ULong interval_;
    CORBA::ULong max_;
    bool remote_;

public:

    RetryPolicy_impl(CORBA::Short, CORBA::ULong, CORBA::ULong, bool);
    ~RetryPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::Short retry_mode()
        throw(CORBA::SystemException);
    virtual CORBA::ULong retry_interval()
        throw(CORBA::SystemException);
    virtual CORBA::ULong retry_max()
        throw(CORBA::SystemException);
    virtual CORBA::Boolean retry_remote()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class TimeoutPolicy_impl : public TimeoutPolicy,
			   public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    TimeoutPolicy_impl(const TimeoutPolicy_impl&);
    void operator=(const TimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    TimeoutPolicy_impl(CORBA::ULong);
    ~TimeoutPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ConnectTimeoutPolicy_impl : public ConnectTimeoutPolicy,
				  public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectTimeoutPolicy_impl(const ConnectTimeoutPolicy_impl&);
    void operator=(const ConnectTimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    ConnectTimeoutPolicy_impl(CORBA::ULong);
    ~ConnectTimeoutPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RequestTimeoutPolicy_impl : public RequestTimeoutPolicy,
				  public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    RequestTimeoutPolicy_impl(const RequestTimeoutPolicy_impl&);
    void operator=(const RequestTimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    RequestTimeoutPolicy_impl(CORBA::ULong);
    ~RequestTimeoutPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class LocationTransparencyPolicy_impl : public LocationTransparencyPolicy,
					public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    LocationTransparencyPolicy_impl(const LocationTransparencyPolicy_impl&);
    void operator=(const LocationTransparencyPolicy_impl&);

    CORBA::Short value_;

public:

    LocationTransparencyPolicy_impl(CORBA::Short);
    ~LocationTransparencyPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::Short value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class InterceptorPolicy_impl : public InterceptorPolicy,
			       public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    InterceptorPolicy_impl(const InterceptorPolicy_impl&);
    void operator=(const InterceptorPolicy_impl&);

    bool value_;

public:

    InterceptorPolicy_impl(bool);
    ~InterceptorPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::Boolean value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class LocateRequestPolicy_impl : public LocateRequestPolicy,
                                 public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    LocateRequestPolicy_impl(const LocateRequestPolicy_impl&);
    void operator=(const LocateRequestPolicy_impl&);

    bool value_;

public:

    LocateRequestPolicy_impl(bool);
    ~LocateRequestPolicy_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual CORBA::Boolean value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

} // End of namespace OB

#endif
