// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OCI_IMPL_H
#define OB_OCI_IMPL_H

#include <OB/Object.h>
#include <OB/Except.h>
#include <OB/Any.h>
#include <OB/OCI.h>
#include <OB/LocalObject.h>
#include <OB/list.h>

namespace OBCORBA
{

class ORB_impl; // For the friend declaration

} // End of namespace CORBA

namespace OBPortableServer
{

class POAManager_impl; // For the friend declaration

} // End of namespace PortableServer

namespace OCI
{

//
// The AccFactoryRegistry_impl class
//
class AccFactoryRegistry_impl :
    public AccFactoryRegistry,
    public OBCORBA::RefCountLocalObject,
    public JTCMutex
{
    //
    // All acceptor factories
    //
    OB_STL::list<AccFactory_var> factories_;

    //
    // Hide copy-constructor and assignment operator
    //
    AccFactoryRegistry_impl(const AccFactoryRegistry_impl&);
    void operator=(const AccFactoryRegistry_impl&);

    AccFactoryRegistry_impl() { }
    virtual ~AccFactoryRegistry_impl() { }

    // ORB_impl creates AccFactoryRegistry_impl
    friend class ::OBCORBA::ORB_impl;

public:

    //
    // Standard IDL to C++ Mapping
    //
    virtual void add_factory(AccFactory_ptr)
        throw(FactoryAlreadyExists,
              CORBA::SystemException);

    virtual AccFactory_ptr get_factory(const char*)
        throw(NoSuchFactory,
              CORBA::SystemException);

    virtual AccFactorySeq* get_factories()
        throw(CORBA::SystemException);
};

//
// The ConFactoryRegistry_impl class
//
class ConFactoryRegistry_impl :
    public ConFactoryRegistry,
    public OBCORBA::RefCountLocalObject,
    public JTCMutex
{
    //
    // All connector factories
    //
    OB_STL::list<ConFactory_var> factories_;

    //
    // Hide copy-constructor and assignment operator
    //
    ConFactoryRegistry_impl(const ConFactoryRegistry_impl&);
    void operator=(const ConFactoryRegistry_impl&);

    ConFactoryRegistry_impl() { }
    virtual ~ConFactoryRegistry_impl() { }

    // ORB_impl creates ConFactoryRegistry_impl
    friend class ::OBCORBA::ORB_impl;

public:

    //
    // Standard IDL to C++ Mapping
    //
    virtual void add_factory(ConFactory_ptr)
        throw(FactoryAlreadyExists,
              CORBA::SystemException);

    virtual ConFactory_ptr get_factory(const char*)
        throw(NoSuchFactory,
              CORBA::SystemException);

    virtual ConFactorySeq* get_factories()
        throw(CORBA::SystemException);
};

//
// The Current_impl class
//
struct CurrentStateHolder;

class Current_impl : public Current, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    Current_impl(const Current_impl&);
    void operator=(const Current_impl&);

    //
    // The state key, or the data itself if no multi thread support
    //
    JTCThreadKey stateKey_;

    CurrentStateHolder* getStateHolder();

public:

    Current_impl();
    virtual ~Current_impl();
    
    //
    // Standard IDL to C++ Mapping
    //
    virtual TransportInfo_ptr get_oci_transport_info()
        throw(CORBA::SystemException);

    virtual AcceptorInfo_ptr get_oci_acceptor_info()
        throw(CORBA::SystemException);
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_preinvoke(TransportInfo_ptr);
    void _OB_postinvoke();
};

} // End of namespace OCI

#endif
