// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_IOR_INFO_IMPL_H
#define PI_IOR_INFO_IMPL_H

#include <OB/PIManager.h> // For ComponentTable (TODO)
#include <OB/RefCounted_fwd.h>
#include <OB/IOP.h>
#include <OB/ValueBase.h>
#include <OB/PIIOR.h>
#include <OB/OCI.h>
#include <OB/GIOP.h>
#include <OB/LocalObject.h>

namespace PortableInterceptor
{

//
// IORInfo implementation
//
class IORInfo_impl : public IORInfo,
		     public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    IORInfo_impl(const IORInfo_impl&);
    void operator=(const IORInfo_impl&);

    OCI::AcceptorSeq acceptors_;

    OB::ComponentTable table_;

    IOP::TaggedComponentSeq all_;

    OB::RefCountPolicyList_var policies_;

    ObjectReferenceTemplate_var adapterTemplate_;

    ObjectReferenceFactory_var currentFactory_;

    AdapterManagerId id_;
    
    AdapterState state_;

public:

    IORInfo_impl(const OCI::AcceptorSeq&,
		 OB::RefCountPolicyList_ptr,
		 AdapterManagerId,
		 AdapterState);
    ~IORInfo_impl();

    //
    // IDL to C++ Mapping
    //
    virtual CORBA::Policy_ptr
    get_effective_policy(CORBA::PolicyType)
	throw(CORBA::SystemException);

    virtual void
    add_ior_component(const IOP::TaggedComponent&)
	throw(CORBA::SystemException);

    virtual void
    add_ior_component_to_profile(const IOP::TaggedComponent&, IOP::ProfileId)
	throw(CORBA::SystemException);

    virtual ObjectReferenceTemplate* adapter_template()
	throw(CORBA::SystemException);

    virtual ObjectReferenceFactory* current_factory()
	throw(CORBA::SystemException);

    virtual void current_factory(ObjectReferenceFactory*)
	throw(CORBA::SystemException);

    virtual AdapterManagerId manager_id()
	throw(CORBA::SystemException);

    virtual AdapterState state()
	throw(CORBA::SystemException);

    //
    // Internal methods
    //
    void _OB_adapterTemplate(ObjectReferenceTemplate*);

    void _OB_state(AdapterState);

    void _OB_addComponents(IOP::IOR&, const GIOP::Version&);

};

} // end namespace PortableInterceptor

#endif // PI_IOR_INFO_IMPL_H
