// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_INFO_IMPL_H
#define PI_INFO_IMPL_H

#include <OB/Except.h>
#include <OB/LocalObject.h>
#include <OB/PIRequest.h>

#include <PIArgStrategy.h>

namespace PortableInterceptor
{

class Current_impl;

//
// ORBacus proprietary status flags
//
enum OBReplyStatus
{
    NO_REPLY_SC = -2,
    NO_REPLY = -1
};

//
// Implementation of RequestInfo
//
class RequestInfo_impl : virtual public RequestInfo,
			 public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy constructor and assignment operator
    //
    RequestInfo_impl(const RequestInfo_impl&);
    void operator=(const RequestInfo_impl&);

protected:

    //
    // The Request ID
    //
    CORBA::ULong id_;

    //
    // The operation name
    //
    CORBA::String_var op_;

    //
    // Is this method oneway?
    //
    CORBA::Boolean responseExpected_;

    //
    // The reply status
    //
    ReplyStatus status_;

    //
    // The forward reference (if status_ == LOCATION_FORWARD[_PERM]
    //
    OB::RefCountIOR_var forwardReference_;

    //
    // The ORB instance
    //
    OB::ORBInstance_var orbInstance_;

    //
    // The policies
    //
    OB::RefCountPolicyList_var policies_;

    //
    // The argument strategy
    //
    ArgumentStrategy* argStrategy_;

    //
    // The Request and Reply service context lists
    //
    IOP::ServiceContextList& requestSCL_;
    IOP::ServiceContextList& replySCL_;

    //
    // ReceivedException (status_ == [SYSTEM|USER]_EXCEPTION)
    //
    CORBA::Exception* receivedException_;
    CORBA::String_var receivedId_;

    //
    // Slot data for the request
    //
    CORBA::AnySeq_var slots_;

    //
    // A pointer to the PortableInterceptor::Current implementation
    //
    Current_impl* current_;

    //
    // Does the slot data need to be popped in the current
    // implementation?
    //
    bool popCurrent_;

    //
    // Retrieve a service context
    //
    IOP::ServiceContext*
    getServiceContext(const IOP::ServiceContextList&,
                      IOP::ServiceId)
	throw(CORBA::SystemException);

    //
    // Add a service context
    //
    void addServiceContext(IOP::ServiceContextList&,
                           const IOP::ServiceContext&,
                           bool)
	throw(CORBA::SystemException);

public:

    //
    // No argument information available
    //
    RequestInfo_impl(CORBA::ULong, const char*, bool,
                     IOP::ServiceContextList&,
                     IOP::ServiceContextList&,
                     OB::ORBInstance_ptr,
		     OB::RefCountPolicyList_ptr,
                     Current_impl*);

    virtual ~RequestInfo_impl();


    //
    // IDL to C++ Mapping
    //
    virtual CORBA::ULong request_id()
	throw(CORBA::SystemException);

    virtual char* operation()
	throw(CORBA::SystemException);

    virtual Dynamic::ParameterList* arguments()
	throw(CORBA::SystemException);

    virtual Dynamic::ExceptionList* exceptions()
	throw(CORBA::SystemException);
    
    virtual Dynamic::ContextList* contexts()
	throw(CORBA::SystemException);

    virtual Dynamic::RequestContext* operation_context()
	throw(CORBA::SystemException);
    
    virtual CORBA::Any* result()
	throw(CORBA::SystemException);

    virtual CORBA::Boolean response_expected()
	throw(CORBA::SystemException);

    virtual Messaging::SyncScope sync_scope()
	throw(CORBA::SystemException);
    
    virtual ReplyStatus reply_status()
	throw(CORBA::SystemException);
    
    virtual CORBA::Object_ptr forward_reference()
	throw(CORBA::SystemException);

    virtual CORBA::Any* get_slot(SlotId)
	throw(CORBA::SystemException, InvalidSlot);

    virtual IOP::ServiceContext* get_request_service_context(IOP::ServiceId)
	throw(CORBA::SystemException);
    
    virtual IOP::ServiceContext* get_reply_service_context(IOP::ServiceId)
	throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_setReplyStatus(ReplyStatus);
    void _OB_setForwardReference(OB::RefCountIOR_ptr);
    void _OB_setReceivedException(const CORBA::Exception&, const char*);
};

} // End namespace PortableInterceptor

#endif
