// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POA_MANAGER_IMPL_H
#define OB_POA_MANAGER_IMPL_H

#include <OB/POAServantBase_fwd.h>
#include <OB/POAInterface_fwd.h>
#include <OB/OAInterface_fwd.h>
#include <OB/DirectStubImpl_fwd.h>
#include <OB/ORBInstance_fwd.h>
#include <OB/Properties_fwd.h>
#include <OB/BootManager_fwd.h>

#include <Server_fwd.h>
#include <CollocatedServer_fwd.h>

#include <OB/LocalObject.h>
#include <OB/StubImpl.h>
#include <OB/OBPOAManager.h>
#include <OB/Hashtable.h>
#include <OB/GIOP.h>
#include <OB/PICommon.h>

namespace OB
{

//
// Other declarations
// 
struct ObjectKeyData;
class BootManager_impl;

} // End of namespace OB

namespace OBPortableServer
{

//
// Set of associated POAs
//
struct POANameHasher;
typedef OB::SynchronizedHashtable< CORBA::StringSeq, PortableServer::POA_var,
		                   POANameHasher> POASyncHashTable;

//
// Forward declarations
//
class POALocator;
class POA_impl;

//
// Implementation of PortableServer::POAManager
//
class POAManager_impl : public POAManager,
			public OBCORBA::RefCountLocalObject,
			public JTCMonitor
{
    //
    // Hide copy constructor/assignment operator
    //
    POAManager_impl(const POAManager_impl&);
    void operator=(const POAManager_impl&);

    //
    // The ORBInstance
    //
    OB::ORBInstance_var orbInstance_;

    //
    // Set of connected POAs
    //
    POASyncHashTable poas_;

    //
    // The POAManager state
    //
    POAManager::State state_;

    //
    // The OCI acceptors
    //
    OCI::AcceptorSeq acceptors_;

    //
    // The server manager
    //
    OB::ServerManager_var serverManager_;

    //
    // The POAManager name
    //
    CORBA::String_var name_;

    //
    // The POAManager id
    //
    PortableInterceptor::AdapterManagerId id_;

    //
    // The OAInterface
    //
    OB::OAInterface_var oaInterface_;

    //
    // The GIOP version
    //
    GIOP::Version version_;

    //
    // The boot manager and implementation
    //
    OB::BootManager_var bootManager_;
    OB::BootManager_impl* bootManagerImpl_;

    //
    // The POALocator
    //
    POALocator* poaLocator_;

    //
    // The Server id
    //
    CORBA::String_var serverId_;

    //
    // Wait for all pending requests to terminate
    //
    void waitPendingRequests()
	throw(CORBA::SystemException);

    //
    // Check if we are dispatching a request on this POAManager's ORB.
    //
    bool isInORBUpcall();

    //
    // Etherealize servants associated to this POA's POAManager
    //
    void etherealizePOAs();

    POAManager_impl(OB::ORBInstance_ptr, POALocator*, const char*,
    		    CORBA::Long, const OCI::AcceptorSeq&)
	throw(CORBA::SystemException);

    //
    // Only POAManagerFactory_impl is permitted to create new
    // instances
    //
    friend class POAManagerFactory_impl;

public:

    ~POAManager_impl();

    //
    // Standard IDL to C++ Mapping
    //
    
    //
    // Mapping for PortableServer::POAManager
    //
    virtual void activate()
        throw(POAManager::AdapterInactive,
              CORBA::SystemException);
    
    virtual void hold_requests(CORBA::Boolean)
        throw(POAManager::AdapterInactive,
              CORBA::SystemException);
    
    virtual void discard_requests(CORBA::Boolean)
        throw(POAManager::AdapterInactive,
              CORBA::SystemException);
    
    virtual void deactivate(CORBA::Boolean, CORBA::Boolean)
        throw(POAManager::AdapterInactive,
              CORBA::SystemException);
    
    virtual POAManager::State get_state()
        throw(CORBA::SystemException);

    //
    // Mapping for OBPortableServer::POAManager
    //
    virtual char* name()
	throw(CORBA::SystemException);

    virtual OCI::AcceptorSeq*
    get_acceptors()
	throw(POAManager::AdapterInactive,
	      CORBA::SystemException);
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //

    //
    // Add a POA to this manager
    //
    void _OB_addPOA(PortableServer::POA_ptr, const CORBA::StringSeq&)
	throw();

    //
    // Remove a POA from this manager
    //
    void _OB_removePOA(const CORBA::StringSeq&)
	throw(CORBA::SystemException);

    //
    // Create a DirectStubImpl on the relevant managed POA. If there
    // is no POA for the servant OBJECT_NOT_EXIST is thrown.
    //
    OB::DirectStubImpl_ptr _OB_createDirectStubImpl(const OCI::ObjectKey&,
    						    OB::RefCountPolicyList_ptr)
	throw(PortableServer::POAManager::AdapterInactive,
	      CORBA::SystemException,
	      OB::LocationForward);

    //
    // Locate the POA for this object key. If the POA cannot be
    // located a _nil POA is returned.
    //
    PortableServer::POA_ptr _OB_locatePOA(const OB::ObjectKeyData&)
        throw(OB::LocationForward, CORBA::SystemException);

    //
    // Retrieve the CollocatedServer for this POAManager
    //
    OB::CollocatedServer_ptr _OB_getCollocatedServer()
	throw();

    //
    // Validate the POAManager state. If the state is not valid an
    // exception is raised
    //
    void _OB_validateState()
	throw(CORBA::SystemException);

    //
    // Get the protocol version
    //
    const GIOP::Version& _OB_getGIOPVersion() const
        throw();

    //
    // Get the manager id
    //
    PortableInterceptor::AdapterManagerId _OB_getAdapterManagerId()
	throw(CORBA::SystemException);

    //
    // Get the adapter state
    //
    PortableInterceptor::AdapterState _OB_getAdapterState()
	throw();

    //
    // Get the manager acceptors
    //
    OCI::AcceptorSeq* _OB_getAcceptors() const
	throw();
};

} // End of namespace OBPortableServer

#endif
