// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POA_IMPL_H
#define OB_POA_IMPL_H

#include <OB/Upcall_fwd.h>
#include <OB/DirectStubImpl_fwd.h>

#include <OB/Upcall_fwd.h>
#include <OB/PIIOR_fwd.h>
#include <OB/PIORT_fwd.h>

#include <OB/ValueBase.h>
#include <OB/NativeTypes.h>
#include <OB/MarshalStubImpl.h>
#include <OB/OBPOAInterface.h>
#include <OB/POAServantLocator.h>
#include <OB/IMRTypes.h>
#include <OB/ObjectKey.h>
#include <OB/Hashtable.h>
#include <OB/LocalObject.h>

namespace OCI
{

class Current_impl;

} // End of namespace OCI

namespace PortableServer
{

class Current_impl; // Implementation of PortableServer::Current

} // end of namespace PortableServer

namespace OBPortableServer
{

//
// Forward declarations
//
class POAControl;
class AdapterActivatorHolder;
class ServantLocationStrategy;
class IdGenerationStrategy;
class POAPolicies;

//
// The POA_impl class
//
class POA_impl : public POA,
		 public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy constructor/assignment operator
    //
    POA_impl(const POA_impl&);
    void operator=(const POA_impl&);

    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    //
    // The name of this POA and the name of the root POA
    //
    CORBA::String_var name_;

    //
    // The adapter id
    //
    CORBA::OctetSeq_var adapterId_;

    //
    // The Server ID (or "_RootPOA" if not set)
    //
    CORBA::String_var serverId_;

    //
    // The POA name. This is the full path to the POA.
    //
    CORBA::StringSeq poaId_;

    //
    // My parent POA
    //
    // We have the var so parent will not go away while child still exists. 
    //
    POA_var parent_;

    //
    // My POA manager
    //
    OBPortableServer::POAManager_var manager_;

    //
    // My servant location strategy
    //
    ServantLocationStrategy* servantLocationStrategy_;

    //
    // My POAControl object
    //
    POAControl* poaControl_;

    //
    // The associated AdapterActivator
    //
    AdapterActivatorHolder* adapterActivator_;
    
    //
    // All children of this POA
    //
    typedef OB::SynchronizedHashtable<CORBA::String_var, 
	PortableServer::POA_var, OB::StringHasher> POAHashTable;
    POAHashTable children_;

    //
    // My policies
    //
    POAPolicies* policies_;
    
    //
    // Used for generating object ids
    //
    IdGenerationStrategy* idGenerationStrategy_;
    
    //
    // The POA creation time, if not persistent
    //
    time_t poaCreateTime_;

    //
    // If this POA has the single thread policy set then this mutex is
    // used ensure this policy is met
    //
    JTCRecursiveMutex poaSyncMutex_;
    static JTCRecursiveMutex orbSyncMutex_;

    OB::DispatchStrategy_var dispatchStrategy_;

    //                                                    
    // The Current implementation. The RootPOA is responsible for the
    // creation and deletion of the one and only PortableServer::Current.
    //                                               
    PortableServer::Current_impl* poaCurrent_;

    //
    // The OCI::Current implementation. The RootPOA is responsible for
    // the creation and deletion of the one and only OCI::Current.
    //
    OCI::Current_impl* ociCurrent_;

    //
    // This is the set of policies that are not POA policies and are
    // registered with a valid PolicyFactory
    //
    OB::RefCountPolicyList_var rawPolicies_;

    //
    // The primary, secondary and current ObjectReferenceTemplate
    //
    PortableInterceptor::ObjectReferenceTemplate_var adapterTemplate_;
    PortableInterceptor::ObjectReferenceFactory_var currentFactory_;
    PortableInterceptor::ObjectReferenceFactory_var ort_;

    //
    // The IORInfo
    //
    PortableInterceptor::IORInfo_var iorInfo_;

    //
    // The child ORTs required for adapterStateChange call when
    // POA is destroyed.
    //
    PortableInterceptor::ObjectReferenceTemplateSeq childTemplates_;

    //
    // Should POA call AdapterStateChange when destroyed
    //
    bool callAdapterStateChange_;

    //
    // Find the index of the given policy
    //
    bool findPolicyIndex(const CORBA::PolicyList&, CORBA::PolicyType,
			 CORBA::ULong&);

    //
    // Validate the set of provided POA policies
    //
    bool validatePolicies(const POAPolicies&, const CORBA::PolicyList&,
			  CORBA::ULong&);

    //
    // Handle unknown exceptions
    //
    void handleUnknownException(OB::Upcall_ptr);

    //
    // Complete the destroy of the POA
    //
    void completeDestroy();

    //
    // Common initialization code
    //
    void init()
	throw(CORBA::SystemException);

    //
    // Private, internal constructor, for non-root POAs
    //
    POA_impl(CORBA::ORB_ptr,
    	     OB::ORBInstance_ptr,
	     const char*,
	     const char*,
	     POA_impl*,
	     POAManager_ptr,
             PortableServer::Current_impl*,
             OCI::Current_impl*,
	     const POAPolicies&,
             const CORBA::PolicyList&)
	throw(CORBA::SystemException);

public:

    //
    // Public constructor for the root POA
    //
    POA_impl(CORBA::ORB_ptr, OB::ORBInstance_ptr, const char*, POAManager_ptr)
	throw(CORBA::SystemException);
    ~POA_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PortableServer::POA_ptr
    create_POA(const char* adapter_name,
	       PortableServer::POAManager_ptr a_POAManager,
               const CORBA::PolicyList& policies)
        throw(PortableServer::POA::AdapterAlreadyExists,
	      PortableServer::POA::InvalidPolicy,
              CORBA::SystemException);

    virtual PortableServer::POA_ptr
    find_POA(const char* adapter_name, CORBA::Boolean activate_it)
        throw(PortableServer::POA::AdapterNonExistent,
	      CORBA::SystemException);

    virtual void
    destroy(CORBA::Boolean etherealize_objects,
	    CORBA::Boolean wait_for_completion)
        throw(CORBA::SystemException);

    virtual PortableServer::ThreadPolicy_ptr
    create_thread_policy(PortableServer::ThreadPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::LifespanPolicy_ptr
    create_lifespan_policy(PortableServer::LifespanPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::IdUniquenessPolicy_ptr
    create_id_uniqueness_policy(PortableServer::IdUniquenessPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::IdAssignmentPolicy_ptr
    create_id_assignment_policy(PortableServer::IdAssignmentPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::ImplicitActivationPolicy_ptr
    create_implicit_activation_policy(
	PortableServer::ImplicitActivationPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::ServantRetentionPolicy_ptr
    create_servant_retention_policy(
	PortableServer::ServantRetentionPolicyValue value)
        throw(CORBA::SystemException);

    virtual PortableServer::RequestProcessingPolicy_ptr
    create_request_processing_policy(
	PortableServer::RequestProcessingPolicyValue value)
        throw(CORBA::SystemException);

    virtual SynchronizationPolicy_ptr
    create_synchronization_policy(
	SynchronizationPolicyValue value)
        throw(CORBA::SystemException);

    virtual DispatchStrategyPolicy_ptr
    create_dispatch_strategy_policy(OB::DispatchStrategy_ptr value)
        throw(CORBA::SystemException);

    virtual char* the_name()
        throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr the_parent()
        throw(CORBA::SystemException);

    virtual PortableServer::POAList* the_children()
        throw(CORBA::SystemException);
    
    virtual PortableServer::POAManager_ptr the_POAManager()
        throw(CORBA::SystemException);

    virtual CORBA::OctetSeq* id()
        throw(CORBA::SystemException);

    virtual PortableServer::AdapterActivator_ptr the_activator()
        throw(CORBA::SystemException);

    virtual void the_activator(PortableServer::AdapterActivator_ptr)
        throw(CORBA::SystemException);

    virtual CORBA::PolicyList* the_policies()
        throw(CORBA::SystemException);

    virtual OB::DispatchStrategy_ptr the_dispatch_strategy()
        throw(CORBA::SystemException);

    virtual CORBA::ORB_ptr the_ORB()
        throw(CORBA::SystemException);

    virtual PortableInterceptor::ObjectReferenceTemplate* adapter_template()
        throw(CORBA::SystemException);

    virtual PortableInterceptor::ObjectReferenceFactory* current_factory()
        throw(CORBA::SystemException);

    virtual PortableServer::ServantManager_ptr get_servant_manager()
        throw(PortableServer::POA::WrongPolicy, CORBA::SystemException);

    virtual void set_servant_manager(PortableServer::ServantManager_ptr imgr)
        throw(PortableServer::POA::WrongPolicy, CORBA::SystemException);

    virtual PortableServer::Servant get_servant()
        throw(PortableServer::POA::NoServant,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual void set_servant(PortableServer::Servant p_servant)
        throw(PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual PortableServer::ObjectId*
    activate_object(PortableServer::Servant p_servant)
        throw(PortableServer::POA::ServantAlreadyActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual void activate_object_with_id(const PortableServer::ObjectId& id,
					 PortableServer::Servant p_servant)
        throw(PortableServer::POA::ServantAlreadyActive,
	      PortableServer::POA::ObjectAlreadyActive,
              PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual void deactivate_object(const PortableServer::ObjectId& oid)
        throw(PortableServer::POA::ObjectNotActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual CORBA::Object_ptr create_reference(const char* intf)
        throw(PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual CORBA::Object_ptr
    create_reference_with_id(const PortableServer::ObjectId& oid,
			     const char* intf)
        throw(CORBA::SystemException);

    virtual PortableServer::ObjectId*
    servant_to_id(PortableServer::Servant p_servant)
        throw(PortableServer::POA::ServantNotActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual CORBA::Object_ptr
    servant_to_reference(PortableServer::Servant p_servant)
        throw(PortableServer::POA::ServantNotActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual PortableServer::Servant
    reference_to_servant(CORBA::Object_ptr reference)
        throw(PortableServer::POA::ObjectNotActive,
	      PortableServer::POA::WrongAdapter,
	      PortableServer::POA::WrongPolicy,
              CORBA::SystemException);

    virtual PortableServer::ObjectId*
    reference_to_id(CORBA::Object_ptr reference)
        throw(PortableServer::POA::WrongAdapter,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual PortableServer::Servant
    id_to_servant(const PortableServer::ObjectId& oid)
        throw(PortableServer::POA::ObjectNotActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    virtual CORBA::Object_ptr
    id_to_reference(const PortableServer::ObjectId& oid)
        throw(PortableServer::POA::ObjectNotActive,
	      PortableServer::POA::WrongPolicy,
	      CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    
    //
    // Called before a method invocation on a servant
    //
    void _OB_preinvoke(const char*, const PortableServer::ObjectId&,
		       PortableServer::ServantBase*,
		       PortableServer::ServantLocator::Cookie,
                       OCI::TransportInfo_ptr);

    //
    // Called after a method invocation on a servant
    //
    void _OB_postinvoke();

    //
    // Determine if a servant is present
    //
    void _OB_locateServant(const PortableServer::ObjectId&)
	throw(CORBA::SystemException, OB::LocationForward);

    //
    // Create an Upcall object. Return a _nil Upcall if the POA is in
    // the process of being destroyed.
    //
    // Note: For efficiency reasons, the Upcall constructor assumes
    // ownership for all parameter marked with "/**/"
    //
    OB::Upcall_ptr _OB_createUpcall(
	const PortableServer::ObjectId&,
	OB::UpcallReturn_ptr,
	/**/ OCI::ProfileInfo*,
	/**/ OCI::TransportInfo_ptr,
	CORBA::ULong,
	/**/ char*,
	/**/ OB::InputStream_ptr,
	/**/ IOP::ServiceContextList*)
	throw(CORBA::SystemException, OB::LocationForward);

    //
    // Dispatch a method invocation
    //
    void _OB_dispatch(const PortableServer::ObjectId&,
		      OB::Upcall_ptr)
	throw();

    //
    // Ask a servant to create the StubImplBase, if possible.
    //
    OB::DirectStubImpl_ptr _OB_createDirectStubImpl(
	const PortableServer::ObjectId&,
	OB::RefCountPolicyList_ptr)
	throw(CORBA::SystemException, OB::LocationForward);

    //
    // Remove a DirectStubImpl
    //
    void _OB_removeDirectStubImpl(const PortableServer::ObjectId&,
				  OB::DirectStubImpl*);

    //
    // Determine if this POA matches the provided object key
    //
    bool _OB_poaMatches(const OB::ObjectKeyData&, bool);

    //
    // Increment the outstanding number of requests
    //
    bool _OB_incrementRequestCount();

    //
    // Decrement the outstanding number of requests
    //
    void _OB_decrementRequestCount();

    //
    // Wait for all pending requests to terminate
    //
    void _OB_waitPendingRequests();

    //
    // Get the ORBInstance object
    //
    OB::ORBInstance_ptr _OB_ORBInstance();

    //
    // Add the policy factory for the POA Policies
    //
    void _OB_addPolicyFactory();

    //
    // Valid the associated POAManager state -- potentially blocking
    // until active.
    //
    void _OB_validateManagerState()
	throw(CORBA::SystemException);

    //
    // Etherealize all servants
    //
    void _OB_etherealize(POAManager_ptr);

    //
    //
    //
    void _OB_destroy(bool, bool, 
    		     PortableInterceptor::ObjectReferenceTemplateSeq*)
        throw(CORBA::SystemException);
};

} // End of namespace OBPortableServer

#endif
