// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_WIN_REACTOR_H
#define OB_WIN_REACTOR_H

#include <OB/EventHandlerBase_fwd.h>

#include <OB/NativeTypes.h>
#include <OB/LocalObject.h>
#include <OB/Reactor.h>
#include <OB/list.h>

#include <Logger_impl.h>

namespace OB
{

//
// The WinReactor class
//
class WinReactor : public Reactor,
		   public OBCORBA::RefCountLocalObject,
		   public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    WinReactor(const WinReactor&);
    void operator=(const WinReactor&);

    //
    // If set to true, interrupt event dispatching immediately
    //
    bool interrupt_;

    //
    // The window used for message notifications
    //
    HWND hWnd_;

    //
    // Struct for an event handler info
    //
    struct HandlerInfo
    {
	EventHandlerBase_var handler;
	Handle handle;
	Mask mask;
	TypeMask typeMask;
    };

    //
    // Our event handler info list
    //
    OB_STL::list<HandlerInfo> handlerInfoList_;

    //
    // Return the TypeMask for the registered event handlers
    //
    TypeMask evalTypeMasks() const;

public:

    WinReactor(HINSTANCE);
    virtual ~WinReactor();

    virtual void register_handler(EventHandlerBase*, Mask, TypeMask, Handle);
    virtual void unregister_handler(EventHandlerBase*);
    virtual CORBA::Boolean dispatch(TypeMask);
    virtual void interrupt_dispatch();
    virtual CORBA::Boolean dispatch_one_event(CORBA::Long);
    virtual CORBA::Boolean event_ready();

    //
    // Windows message processing function
    //
    LRESULT wndProc(HWND, UINT, WPARAM, LPARAM);
};

//
// The windows message logger class
//
class WinLogger_impl : public Logger_impl
{
    //
    // Hide copy-constructor and assignment operator
    //
    WinLogger_impl(const WinLogger_impl&);
    void operator=(const WinLogger_impl&);

    //
    // Print messages
    //
    virtual void message(const char*);

public:

    WinLogger_impl() { }
    virtual ~WinLogger_impl() { }

};

} // End of namespace OB

#endif
