// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#include <Util.h>
#include <GenUtil.h>
#include <GenCPP.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Generate code for the example header file
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplH(Container_ptr container, bool all,
                                   IdlPrettyPrint& out)
{
    //
    // Get container contents descriptions
    //
    ScopedName_var scope;
    // No _var for seq! See comments for the implementation of "describe"
    Container::DescriptionSeq* seq = describe(container, scope.out());

    for(ULong i = 0 ; i < seq -> length() ; i++)
    {
	//
	// Get element description
	//
	Container::Description& seqElem = (*seq)[i];

	//
	// Get container
	//
	Container_var cont = Container::_narrow(seqElem.contained_object);

        switch(seqElem.kind)
        {
        case dk_Module:
        {
            //
            // Module
            //
            const ModuleDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

	    if(!check(desc -> id))
		continue;

            Identifier_var name = fixKwd(desc -> name);

            comment(desc -> id, out);
            out << "\nnamespace " << name;
            out << "\n{";

            assert(!is_nil(cont));
            genCPlusPlusImplH(cont, all, out);

            out.sep();
            out << "\n} // End of namespace " << name;

            break;
        }

        case dk_AbstractInterface:
            //
            // No sample implementation for abstract interface
            //
            break;

	case dk_LocalInterface:
        case dk_Interface:
        {
            //
            // Interface
            //
            const InterfaceDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
                continue;

            Identifier_var name = fixKwd(desc -> name);
            ScopedName_var absolute = getAbsolute(desc -> id);

            comment(desc -> id, out);

            out.sep();
            out << "\nclass " << name << "_impl : ";

            unsigned int indent = out.getIndent();
            out.setIndent(out.getPos());

	    //
            // TODO: Is virtual inheritance mandatory if there are base
            //       interfaces and all == false?
	    //
            if(!noVirtual_)
                out << "virtual ";
	    if(seqElem.kind == dk_LocalInterface)
                out << "public " << absolute << ", public CORBA::LocalObject";
            else
                out << "public " << "POA_" << absolute;

            InterfaceDef_var intf = InterfaceDef::_narrow(cont);
            assert(!is_nil(intf));
            IdlInterfaceInfoSeq baseInfo;
            IdlGetBaseInterfaces(intf, baseInfo, false);
            ULong numAbstract = 0;

            if(!all)
            {
                for(ULong j = 0 ; j < baseInfo.length() ; j++)
                {
                    //
                    // Impl classes are not generated for abstract interfaces
                    //
                    if(!baseInfo[j].is_abstract)
                    {
                        ScopedName_var baseAbsolute =
                            getAbsolute(baseInfo[j].id, "");

                        out << ",\n";

                        if(!noVirtual_)
                            out << "virtual ";

                        out << "public " << baseAbsolute << "_impl";
                    }
                    else
                        numAbstract++;
                }
            }

            if(all || baseInfo.length() - numAbstract == 0)
            {
                out << ",\n";

                if(!noVirtual_)
                    out << "virtual ";

                out << "public PortableServer::RefCountServantBase";
            }

            out.setIndent(indent);

            out.start();

            out << '\n' << name << "_impl(const " << name
                << "_impl&);";
            out << "\nvoid operator=(const " << name << "_impl&);";

	    if(seqElem.kind != dk_LocalInterface)
            {
                out.sep();
                out << "\nPortableServer::POA_var poa_;";
            }

            out.sep();
            out.dec();
            out << "\npublic:";
            out.inc();
            out.sep();
	    if(seqElem.kind == dk_LocalInterface)
                out << '\n' << name << "_impl();";
            else
                out << '\n' << name << "_impl(PortableServer::POA_ptr);";
            out << "\n~" << name << "_impl();";

	    if(seqElem.kind != dk_LocalInterface)
            {
                //
                // _default_POA
                //
                out.sep();
                out << "\nvirtual PortableServer::POA_ptr _default_POA();";
            }

            //
            // Generate attributes and operations for any base abstract
            // interfaces
            //
            if(!all)
            {
                StringSeq ids;

                for(ULong j = 0 ; j < baseInfo.length() ; j++)
                {
                    if(!baseInfo[j].is_abstract)
                        continue;

		    ScopedName_var dummy;
		    // No _var for seq! See comments for the
		    // implementation of "describe"
		    Container::DescriptionSeq* contents =
			describe(baseInfo[j].intf, dummy.out());

                    for(ULong k = 0 ; k < contents -> length() ; k++)
                    {
                        if((*contents)[k].kind == dk_Operation)
                        {
                            const OperationDescription* opDesc;
                            Boolean b = ((*contents)[k].value) >>= opDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], opDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplHOperation(absolute, opDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = opDesc -> id;
                            }
                        }
                        else if((*contents)[k].kind == dk_Attribute)
                        {
                            const AttributeDescription* atDesc;
                            Boolean b = ((*contents)[k].value) >>= atDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], atDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplHAttribute(absolute, atDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = atDesc -> id;
                            }
                        }
                    }
                }
            }

            //
            // Attributes and operations
            //

            Container::DescriptionSeq_var contents =
                cont -> describe_contents(dk_all, !all, -1);

            for(ULong j = 0 ; j < contents -> length() ; j++)
            {
                if(contents[j].kind == dk_Attribute)
                {
                    const AttributeDescription* atDesc;
                    Boolean b = (contents[j].value >>= atDesc);
                    assert(b);
                    genCPlusPlusImplHAttribute(absolute, atDesc, true, out);
                }
                else if(contents[j].kind == dk_Operation)
                {
                    const OperationDescription* opDesc;
                    Boolean b = (contents[j].value >>= opDesc);
                    assert(b);
                    genCPlusPlusImplHOperation(absolute, opDesc, true, out);
                }
            }

            out.end();
            out << ';';

            break;
        }

        case dk_Value:
        {
            //
            // Valuetype
            //
            const ValueDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
                continue;

            if(desc -> is_abstract)
                continue;

            ValueDef_var value = ValueDef::_narrow(cont);
            assert(! is_nil(value));

            Identifier_var name = fixKwd(desc -> name);
            ScopedName_var absolute = getAbsolute(desc -> id);

            IdlSupportedInterfaceInfoSeq info;
            ULong numRegular = 0;
            IdlGetAllSupportedInterfaces(value, info, numRegular);

            ValueDef_var baseValue = value -> base_value();
            ScopedName_var baseAbsolute;
            if(!is_nil(baseValue))
                baseAbsolute = getAbsolute(desc -> base_value);

            comment(desc -> id, out);

            out.sep();
            out << "\nclass " << name << "_impl : ";

            unsigned int indent = out.getIndent();
            out.setIndent(out.getPos());

            //
            // Inherit from POA skeleton if the valuetype supports a
            // regular interface
            //
            if(numRegular > 0)
            {
                out << "virtual public " << "POA_" << absolute << ",";
                out << "\nvirtual public " << "OBV_" << absolute << ",";
                if(!is_nil(baseValue) && !all)
                    out << "\nvirtual public " << baseAbsolute << "_impl";
                else
                    out << "\nvirtual public "
                        << "PortableServer::ValueRefCountBase";
            }
            else
            {
                out << "virtual public " << "OBV_" << absolute << ",";
                if(!is_nil(baseValue) && !all)
                    out << "\nvirtual public " << baseAbsolute << "_impl";
                else
                    out << "\nvirtual public CORBA::DefaultValueRefCountBase";
            }

            out.setIndent(indent);

            out.start();

            out << '\n' << name << "_impl(const " << name << "_impl&);";
            out << "\nvoid operator=(const " << name << "_impl&);";

            out.sep();
            out.dec();
            out << "\npublic:";
            out.inc();
            out.sep();
            out << '\n' << name << "_impl();";
            out << "\n~" << name << "_impl();";

            //
            // _copy_value
            //
            out.sep();
            out << "\nvirtual CORBA::ValueBase* _copy_value();";

            //
            // Custom valuetype
            //
            if(desc -> is_custom)
            {
                out.sep();
                out << "\nvirtual void marshal(CORBA::DataOutputStream*);";
                out << "\nvirtual void unmarshal(CORBA::DataInputStream*);";
            }

            //
            // Valuetype attributes and operations
            //
            StringSeq ids;
            genCPlusPlusImplHValueOpAtt(value, absolute, all, ids, out);

            //
            // Supported interface attributes and operations
            //
            ids.length(0);
            for(ULong j = 0 ; j < info.length() ; j++)
            {
                //
                // Don't generate exception specifications if the supported
                // interface is abstract
                //
                if(all || info[j].inheritance_level == 0)
                    genCPlusPlusImplHValueSupport(
                        absolute, info[j].interface_info.intf, ids,
                        !info[j].interface_info.is_abstract, out);
            }

            out.end();
            out << ';';

            //
            // Value factory
            //

            out.sep();
            out << "\nclass " << name << "Factory_impl : virtual public ";
            InitializerSeq_var inits = value -> initializers();
            if(inits -> length() == 0)
                out << "CORBA::ValueFactoryBase";
            else
                out << name << "_init";
            out.start();

            //
            // create_for_unmarshal
            //
            out << "\nvirtual CORBA::ValueBase* create_for_unmarshal();";

            out.sep();
            out.dec();
            out << "\npublic:";
            out.inc();
            out << '\n' << name << "Factory_impl();";
            out << "\nvirtual ~" << name << "Factory_impl();";

            if(inits -> length() > 0)
            {
                for(ULong j = 0 ; j < inits -> length() ; j++)
                {
                    String_var factoryName = fixKwd(inits[j].name);
                    StructMemberSeq members = inits[j].members;
                    out.sep();
                    out << "\nvirtual " << name << "* " << factoryName << '(';

                    unsigned int indent = out.getIndent();
                    out.setIndent(out.getPos());

                    for(ULong k = 0 ; k < members.length() ; k++)
                    {
                        TypeCode_var origMemberType =
                            GetOrigType(members[k].type);
                        String_var argName = fixKwd(members[k].name);
                        String_var type;

                        if(k > 0)
                            out << ",\n";

                        if(origMemberType -> kind() == tk_array)
                        {
                            type = getTypeString("", members[k].type,
                                                 GetTypeNormal);
                            out << type << ' ' << argName;
                        }
                        //
                        // Fix ambiguous types
                        //
                        else if(origMemberType -> kind() == tk_boolean)
                        {
                            out << "CORBA::ValueFactoryBase::BooleanValue "
                                << argName;
                        }
                        else if(origMemberType -> kind() == tk_char)
                        {
                            out << "CORBA::ValueFactoryBase::CharValue "
                                << argName;
                        }
                        else if(origMemberType -> kind() == tk_wchar)
                        {
                            out << "CORBA::ValueFactoryBase::WCharValue "
                                << argName;
                        }
                        else if(origMemberType -> kind() == tk_octet)
                        {
                            out << "CORBA::ValueFactoryBase::OctetValue "
                                << argName;
                        }
                        else
                        {
                            type = getTypeString("", members[k].type,
                                                 GetTypeIn);
                            out << type << ' ' << argName;
                        }
                    }

                    out << ");";
                    out.setIndent(indent);
                }
            }

            out.end();
            out << ';';

            break;
        }

        default:
            break;
        }
    }
}

// ----------------------------------------------------------------------
// Generate valuetype operations and attributes, including inherited ones
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplHValueOpAtt(ValueDef_ptr value,
                                             const char* absolute,
                                             bool all,
                                             StringSeq& ids,
                                             IdlPrettyPrint& out)
{
    ULong i, j;

    //
    // Only generate operations and attributes for the base value if
    // all == true
    //
    ValueDef_var baseValue = value -> base_value();
    if(all && !is_nil(baseValue))
    {
        RepositoryId_var id = baseValue -> id();
        for(i = 0 ; i < ids.length() ; i++)
            if(strcmp(ids[i], id) == 0)
                break;
        if(i >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplHValueOpAtt(baseValue, absolute, all, ids, out);
        }
    }

    //
    // Always generate operations and attributes for abstract base values
    //
    ValueDefSeq_var abstractBaseValues = value -> abstract_base_values();
    for(i = 0 ; i < abstractBaseValues -> length() ; i++)
    {
        RepositoryId_var id = abstractBaseValues[i] -> id();
        for(j = 0 ; j < ids.length() ; j++)
            if(strcmp(ids[j], id) == 0)
                break;
        if(j >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplHValueOpAtt(abstractBaseValues[i], absolute, all,
                                        ids, out);
        }
    }
    
    ScopedName_var dummy;
    // No _var for seq! See comments for the implementation of
    // "describe"
    Container::DescriptionSeq* contents = describe(value, dummy.out());
    
    for(i = 0 ; i < contents -> length() ; i++)
    {
        if((*contents)[i].kind == dk_Attribute)
        {
            const AttributeDescription* atDesc;
            Boolean b = ((*contents)[i].value >>= atDesc);
            assert(b);
            genCPlusPlusImplHAttribute(absolute, atDesc, false, out);
        }
        else if((*contents)[i].kind == dk_Operation)
        {
            const OperationDescription* opDesc;
            Boolean b = ((*contents)[i].value >>= opDesc);
            assert(b);
            genCPlusPlusImplHOperation(absolute, opDesc, false, out);
        }
    }
}

// ----------------------------------------------------------------------
// Generate operations and attributes for supported interfaces
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplHValueSupport(const char* absolute,
                                               InterfaceDef_ptr intf,
                                               StringSeq& ids,
                                               bool ex,
                                               IdlPrettyPrint& out)
{
    ULong i, j;

    InterfaceDefSeq_var bases = intf -> base_interfaces();
    for(i = 0 ; i < bases -> length() ; i++)
    {
        RepositoryId_var id = bases[i] -> id();
        for(j = 0 ; j < ids.length() ; j++)
            if(strcmp(id, ids[j]) == 0)
                break;
        if(j >= ids.length())
        {
            ids.length(ids.length()+1);
            ids[ids.length()-1] = id;
            genCPlusPlusImplHValueSupport(absolute, bases[i], ids, ex, out);
        }
    }

    ScopedName_var dummy;
    // No _var for seq! See comments for the implementation of
    // "describe"
    Container::DescriptionSeq* contents = describe(intf, dummy.out());

    for(i = 0 ; i < contents ->  length() ; i++)
    {
        if((*contents)[i].kind == dk_Attribute)
        {
            const AttributeDescription* desc;
            Boolean b = ((*contents)[i].value >>= desc);
            assert(b);
            genCPlusPlusImplHAttribute(absolute, desc, ex, out);
        }
        else if((*contents)[i].kind == dk_Operation)
        {
            const OperationDescription* desc;
            Boolean b = ((*contents)[i].value >>= desc);
            assert(b);
            genCPlusPlusImplHOperation(absolute, desc, ex, out);
        }
    }
}

// ----------------------------------------------------------------------
// Generate header code for an example attribute
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplHAttribute(const char* /* absolute */,
                                            const AttributeDescription* desc,
                                            bool ex,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);

    String_var ret = getTypeString("", desc -> type, GetTypeReturn);

    out.sep();
    out << "\nvirtual " << ret << ' ' << name << "()";
    if(ex)
    {
        out.inc();
        out << "\nthrow(CORBA::SystemException)";
        out.dec();
    }
    out << ';';

    if(desc -> mode == ATTR_NORMAL)
    {
        String_var in = getTypeString("", desc -> type, GetTypeIn);
        out << "\nvirtual void " << name << '(' << in << ')';
        if(ex)
        {
            out.inc();
            out << "\nthrow(CORBA::SystemException)";
            out.dec();
        }
        out << ';';
    }
}

// ----------------------------------------------------------------------
// Generate header code for an example operation
// ----------------------------------------------------------------------

void
IdlCPPGenerator::genCPlusPlusImplHOperation(const char* /* absolute */,
                                            const OperationDescription* desc,
                                            bool ex,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);

    String_var ret = getTypeString("", desc -> result, GetTypeReturn);

    out.sep();
    out << "\nvirtual " << ret << ' ' << name << "(";

    unsigned int indent = out.getIndent();
    out.setIndent(out.getPos());

    for(ULong i = 0 ; i < desc -> parameters.length() ; i++)
    {
        GetType tt;
        switch(desc -> parameters[i].mode)
        {
        case PARAM_IN:
            tt = GetTypeIn;
            break;

        case PARAM_INOUT:
            tt = GetTypeInOut;
            break;

        case PARAM_OUT:
            tt = GetTypeOut;
            break;
        }

        String_var argName = fixKwd(desc -> parameters[i].name);

        String_var s =
            getTypeString("", desc -> parameters[i].type, tt, false, argName);

        if(i > 0)
            out << ",\n";

        out << s;
    }

    if(desc -> contexts.length() > 0)
    {
        if(desc -> parameters.length() > 0)
            out << ",\n";

        out << "CORBA::Context_ptr";
    }

    out << ")";

    out.setIndent(indent);

    if(ex)
    {
        //
        // Servants must declare throw() clauses
        //

        out.inc();
        out << "\nthrow(";

        indent = out.getIndent();
        out.setIndent(out.getPos());

        for(ULong i = 0 ; i < desc -> exceptions.length() ; i++)
        {
            Identifier_var exName = getAbsolute(desc -> exceptions[i].id);
            out << exName << ",\n";
        }

        out << "CORBA::SystemException);";
        out.setIndent(indent);
        out.dec();
    }
    else
        out << ';';
}
