// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_RILOGGER_H
#define OB_RILOGGER_H

#include <OB/PIRequest.h>
#include <OB/LocalObject.h>

#include <CorbaOut.h>


namespace OB
{

//
// Request interceptor logger base class
//
class RILogger : public OBCORBA::RefCountLocalObject
{
protected:

    //
    // Service context identifiers
    //
    const IOP::ServiceId REQUEST_CONTEXT_ID;
    const IOP::ServiceId REPLY_CONTEXT_ID;

    //
    // Used to encode/decode the call stack information so that
    // it can be used as the service context
    //
    IOP::Codec_var cdrCodec_;

    //
    // My slot id
    //
    PortableInterceptor::SlotId mySlotId_;

    //
    // The logging level
    //
    CORBA::ULong logLevel_;

    //
    // The logger
    //
    WLogger_ptr logger_;

    //
    // String used to buffer trace output
    //
    CORBA::WString_var str_;

    //
    // Used to display CORBA::Any
    //
    CorbaOut* out_;

    //
    // Dump octet sequence in readable format
    //
    const char* dumpOctetSeq(const char * prefix, 
                             const CORBA::OctetSeq& octets) const;

    //
    // Display PortableInterceptor::RequestInfo
    //
    void displayRequestInfo(PortableInterceptor::RequestInfo_ptr);

    //
    // Send cumulated trace message to logger
    //
    void logTraceOutput(const wchar_t* category);

public:

    //
    // Constructor/Destructor
    //
    RILogger(PortableInterceptor::ORBInitInfo_ptr,
             PortableInterceptor::SlotId,
	     CORBA::ULong,
	     OB::WLogger_ptr);

    virtual ~RILogger();

};

} // End of namespace OB

#endif // OB_RILOGGER_H

