// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/PortableInterceptor.h>
#include <OB/LocalObject.h>

#include <OB/OBORBInit.h>
#include <OB/Properties.h>

#include <CRILogger_impl.h>
#include <SRILogger_impl.h>

#include <RIORBInitializer_impl.h>

using namespace OB;

// 
// Constructor
//
RIORBInitializer_impl::RIORBInitializer_impl()
    : logLevelIn_(0), logLevelOut_(0), properties_(0), logger_(0)
{
}

// 
// Destructor
//
RIORBInitializer_impl::~RIORBInitializer_impl()
{
}

// ---------------------------------------------------------------------
// IDL to C++ Mapping
// ---------------------------------------------------------------------

void
RIORBInitializer_impl::pre_init(PortableInterceptor::ORBInitInfo_ptr info)
{
    //
    // Get properties set via ORB config file.
    //
    OBPortableInterceptor::ORBInitInfo_var orbInitInfo = 
	OBPortableInterceptor::ORBInitInfo::_narrow(info);
    CORBA::ORB_var orb = orbInitInfo -> orb();
    
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    assert(!CORBA::is_nil(oborb));
    
    //
    // Get the ORBs Logger
    //
    Logger_var logger = oborb -> logger();
    logger_ = WLogger::_narrow(logger);
    assert(!CORBA::is_nil(logger_));

    CORBA::String_var msg = (const char*) "module initialization for ORB ";
    msg += '"';
    msg += info -> orb_id();
    msg += '"';
    logger_ -> trace("Watson", msg.in());

    //
    // The Properties from the ORB configuration
    //
    properties_ = oborb -> properties();
    assert(!CORBA::is_nil(properties_));

    //
    // Set internal switches (logLevel ...)
    //
    const char* value;
    
    value = properties_ -> getProperty("ooc.watson.trace.requests");
    if (value != 0)
    {
	logLevelIn_ = atol(value);
	logLevelOut_ = atol(value);
	msg = (const char*) "ooc.watson.trace.requests     : ";
	msg += value;
	logger_ -> trace("Watson", msg.in());
    }

    value = properties_ -> getProperty("ooc.watson.trace.requests.in");
    if (value != 0)
    {
	logLevelIn_ = atol(value);

	msg = (const char*) "ooc.watson.trace.requests.in  : ";
	msg += value;
	logger_ -> trace("Watson", msg.in());
    }

    value = properties_ -> getProperty("ooc.watson.trace.requests.out");
    if (value != 0)
    {
	logLevelOut_ = atol(value);

	msg = (const char*) "ooc.watson.trace.requests.out : ";
	msg += value;
	logger_ -> trace("Watson", msg.in());
    }
}

void
RIORBInitializer_impl::post_init(PortableInterceptor::ORBInitInfo_ptr info)
{
    //
    // Allocate state slot for call stack tracing
    //
    PortableInterceptor::SlotId slotId = info -> allocate_slot_id();
    
    //
    // Create and register the interceptors
    //
    if (logLevelOut_ > 0)
    {
	PortableInterceptor::ClientRequestInterceptor_var 
	    clientInterceptor = new CRILogger_impl(info, 
						   slotId, 
						   logLevelOut_,
						   logger_);
	try
	{
	    info -> add_client_request_interceptor(clientInterceptor);
	}
	catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
	{
	    CORBA::String_var msg = (const char*) "Watson : ";
	    msg += "client_request_interceptor already registered for ORB ";
	    msg += '"';
	    msg += info -> orb_id();
	    msg += '"';
	    logger_ -> warning(msg.in());
	}
    }
    
    if (logLevelIn_ > 0)
    {
	PortableInterceptor::ServerRequestInterceptor_var
	    serverInterceptor = new SRILogger_impl(info, 
						   slotId,
						   logLevelIn_,
						   logger_);
	try
	{
	    info -> add_server_request_interceptor(serverInterceptor);
	}
	catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
	{
	    CORBA::String_var msg = (const char*) "Watson : ";
	    msg += "server_request_interceptor already registered for ORB ";
	    msg += '"';
	    msg += info -> orb_id();
	    msg += '"';
	    logger_ -> warning(msg.in());
	}
    }
    
}

